#include <QtGui>
#include "mainwindow.h"
#include "default.h"
#include "bottomwindow.h"
#include "viewcatalogimpl.h"
#include "bottomwindow.h"

BottomWindow::~BottomWindow()
{
    // no need to delete child widgets, Qt does it all for us
}
BottomWindow::BottomWindow  (QWidget * parent)
    : QDialog(0)
{
    setFont( parent->font() );
	setWindowTitle("Comments");
    tabWidget = new QTabWidget;
    comtable = new ComTable( 1,4,this);
    tabWidget->addTab(comtable,tr("Errors"));
    tabWidget->setTabIcon (0, QIcon(":/images/errortab.png"));

    messageTab = new MessageTab(this);
    tabWidget->addTab(messageTab,tr("Messages"));
    tabWidget->setTabIcon (1, QIcon(":/images/exectab.png"));

    correctionTab = new CorrectionTab(this);
    tabWidget->addTab(correctionTab,tr("Corrections"));
    tabWidget->setTabIcon (2, QIcon(":/images/editproc.png"));
    tabWidget->setTabPosition(QTabWidget::South);
	QGridLayout* gridLayout = new QGridLayout(this);
	gridLayout->setSpacing(6);
    gridLayout->setMargin(9);
    gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
	gridLayout->addWidget(tabWidget, 0, 0, 1, 1);
}

void BottomWindow::OpenProblemsProtocolPage(){
tabWidget->setCurrentIndex(0);

}

void BottomWindow::OpenMessagesProtocolPage(){
tabWidget->setCurrentIndex(1);
}

void BottomWindow::OpenCorrectionsProtocolPage(){
tabWidget->setCurrentIndex(2);
}
void BottomWindow::CleanAllEntries(){
messageTab->clear ();
correctionTab->clear ();
tabWidget->setCurrentIndex(0);
comtable->ClearComTable();
}
//---------------------------------------------------------------------------------
MessageTab::MessageTab(QWidget *parent)
    : QTextEdit()
{
    setFont( parent->font() );
    parentWindow = (BottomWindow*)parent;
}
//---------------------------------------------------------------------------------
CorrectionTab::CorrectionTab(QWidget *parent)
    : QTextEdit()
{
    setFont( parent->font() );
    parentWindow = (BottomWindow*)parent;
}
//---------------------------------------------------------------------------------
ComTable::~ComTable(){

}
ComTable::ComTable(int numRows,int numCols,QWidget* parent)
    : QTableView(),
    NumRows(numRows), NumCols( numCols)
{
    QFont f = parent->font();
    model = new ErrorModel(NumRows, numCols,this);
    setAlternatingRowColors (TRUE);
    setModel(model);
    ErrorDelegate *delegate = new ErrorDelegate;
    setItemDelegate(delegate);

    errorvec = 0;
    parentWindow = (BottomWindow*)parent;
   // setSelectionMode( QAbstractItemView::SingleSelection );
   // setSelectionBehavior ( QAbstractItemView::SelectItems);
    lastCheckedRow = -1;
    setEditTriggers(QAbstractItemView::NoEditTriggers);
    ResizeHorizontalHeader(horizontalHeader ());
    connect( this, SIGNAL( clicked(QModelIndex) ),
	     this, SLOT( clickedRowCol(QModelIndex ) ) );
 //   connect( verticalHeader (), SIGNAL(  sectionClicked(int)  ),
//	     this, SLOT( clickedVerticalHeader(int ) ) );
		 hideColumn (1);
    ResizeRowsCols();
}
void ComTable::ClearComTable(){
	if(NumRows > 1)
    model->removeRows(1,NumRows-1);	
	NumRows = 1;
    model->setData(model->index(0, 2, QModelIndex()), QVariant("No input catalog"),Qt::DisplayRole  );
    model->setHeaderData(0,Qt::Vertical, QString());
}
void ComTable::ResizeHorizontalHeader(QHeaderView *hd)
{
QFont fnOld = font();
QFont fn = fnOld;
fn.setBold(true); 
hd->resizeSection(numberErrorRecord(),fontMetrics().width(fontmetricErrorRecord()) );
hd->resizeSection(numberErrorLine(),fontMetrics().width(fontmetricErrorLine()));
hd->setStretchLastSection(TRUE);
setFont(fnOld);
}
void ComTable::clickedRowCol(QModelIndex index){
if (index.row() == 0) return;
if (type == ParamFirst || type == ParamAll)  {
  if (index.column() == 1) {
  bool checked = (model->data(index,Qt::CheckStateRole)).toBool();
  if(!checked)
  model->setData(index, QVariant(true),Qt::CheckStateRole );
  else
  model->setData(index, QVariant(false),Qt::CheckStateRole );
 }

   if (index.column() == 0 || index.column() == 2||index.column() == 3) 
   {
       // "@%1@%2"  "@%1@%2@%3@%4"
       ItrVector itv = errorvec->begin(); itv += index.row()-1;
       QStringList param = (*itv).split( "@");
       QStringList::Iterator itstart = param.begin();itstart +=2;
       int row = param[1].toInt(); int col = (*itstart).toInt();
       int numCol = param.count() - 2;
       MainWindow::self->viewcatalogimpl->cattable->MarkParameterErrorCatalog(row,col,numCol);
//-----------------------------------------------------------------------
/*
int k = 0;
       for ( QStringList::Iterator its = itstart; its != param.end(); ++its ) {	   
	      int roww = param[1].toInt(); int coll = (*its).toInt();
	      MainWindow::self->viewcatalogimpl->cattable->MarkParameterErrorCatalog(roww,coll);
k++;

      }
*/
//---------------------------------------------------------------------------------------------   
   }
}
if (type == SortAll || type == SortFirst) {
  if (index.column() == 1 ) {
    bool checked = (model->data(index,Qt::CheckStateRole)).toBool();
    if(!checked)
    model->setData(index, QVariant(true),Qt::CheckStateRole );
    else
    model->setData(index, QVariant(false),Qt::CheckStateRole );
  }
   if (index.column() == 0) {
     int n = model->data(index, Qt::DisplayRole).toInt();
     MainWindow::self->viewcatalogimpl->cattable->MarkParameterErrorCatalog(n,-1,-1);
   //  MainWindow::self->viewcatalogimpl->cattable->SearchNumber(n);
  }
   
   if (index.column() == 3 || index.column() == 2) {
    QString tmp  = model->data(index, Qt::DisplayRole).toString();
    tmp = tmp.right(tmp.length() -  tmp.lastIndexOf( ' ') -1);
    MainWindow::self->viewcatalogimpl->cattable->MarkParameterErrorCatalog(tmp.toInt(),-1,-1);
  //  MainWindow::self->viewcatalogimpl->cattable->SearchNumber(tmp.toInt());
   }
  }
}
/*
void ComTable::clickedVerticalHeader(int section){
  if (section == 0) return;
  if (type == ParamFirst || type == ParamAll)  {
//  QModelIndex index = model->index(section,2, QModelIndex());
       ItrVector itv = errorvec->begin(); itv += section-1;
       QStringList param = (*itv).split( "@");
       QStringList err = param[0].split( " ");
       QString comment = QString("Record - %1 .%2 ").arg(param[1]).arg(param[0]);
       if(err[1].indexOf(numberErrorMonth()) != -1) {
        QString temp = QString(" [%1,%2].").arg(minCatalogMonth()).arg(maxCatalogMonth());
        //MainWindow::self->GetSetDocument()->
         comment += temp;
      }

   // qWarning("'%s'",comment.toAscii().data());
     QMessageBox::information (this,"Error record field", comment,QMessageBox::Ok);
  }
}
*/
bool ComTable::InitCommentTable(ErrorVector *errorv,TypeError tp ){
    type = tp;
    parentWindow->show();
    errorvec = errorv;
	if(NumRows > 1)
    model->removeRows(1,NumRows-1);
   
    if (!errorvec->empty()) {
       ItrVector itvEnd = errorvec->end();
	     if (type == ParamFirst || type == SortFirst) {
           NumRows = errorvec->size();
           model->insertRows(1,NumRows-1);
           if (type == ParamFirst)
	         model->SetRow(0,2,QString("Range check completed. First %1 errors found.").arg(errorvec->size()-1));
           if (type == SortFirst)
           model->SetRow(0,2,QString("Order check completed. First %1 errors.").arg(errorvec->size()-1));
           itvEnd--;
           MainWindow::self->messageLabel->setText(" Catalog errors found");
       }
     if (type == ParamAll || type == SortAll)  {
         NumRows = errorvec->size()+1;
         model->insertRows(1,NumRows-1);
         if (type == ParamAll)
         model->SetRow(0,2,QString("Range check completed. %1 errors found.").arg(errorvec->size()));
         if (type == SortAll)
         model->SetRow(0,2,QString("Order check completed. %1 errors found.").arg(errorvec->size()));
         MainWindow::self->messageLabel->setText("Catalog errors found");
         }
      model->setHeaderData(0,Qt::Vertical, QString());
	     int i = 1;
       if (type == ParamFirst || type == ParamAll)
	     for ( ItrVector itv = errorvec->begin(); itv != errorvec->end(); ++itv ) {
	         QStringList param = (*itv).split( "@");
           model->SetRow(i, 0,  QString("%1").arg(param[1]));
           model->SetRow(i, 1,  QString("-!-"));
           model->SetRow(i, 2, QString("%1").arg(param[0]));
           model->SetRow(i, 3, QString());
           model->setHeaderData(i,Qt::Vertical, QString("%1").arg(i));
	         i++;
	     }
       if (type == SortFirst || type == SortAll)
	     for ( ItrVector itv = errorvec->begin(); itv != errorvec->end(); ++itv ) {
	         QStringList param = (*itv).split( "@");
           model->SetRow(i, 0,  QString("%1").arg(param[1]));
           model->SetRow(i, 1,  QString("-!-"));
           model->SetRow(i, 2, QString("%1").arg(param[0]));
           model->SetRow(i, 3,QString("%1").arg(param[2]));
           model->setHeaderData(i,Qt::Vertical, QString("%1").arg(i));
	         i++;
	     }

     } else {
	 NumRows = 1;
	// model->insertRows(0,1);
   model->SetRow(0,2,"Default catalog check completed.");
   model->setHeaderData(0,Qt::Vertical, QString());
   MainWindow::self->messageLabel->setText("");
  }
  ResizeRowsCols();
	return true;
}



void ComTable::ResizeRowsCols()
{

 //  resizeColumnToContents(0);
   resizeColumnToContents(1); //resizeColumnToContents(3);
   for (int row = 0; row < NumRows; ++row)  resizeRowToContents(row);
   QHeaderView *hd = horizontalHeader ();
  //hd->setStretchLastSection(TRUE);
   hd->resizeSection (2, (int)(parentWindow->width()* 0.6) );
}
//-------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ErrorModel::ErrorModel( int rows, int columns, QObject * parent )
    : QStandardItemModel(rows, columns,parent),NumRows(rows),NumCols(columns)
{

parentTable =   (QTableView*) parent;

SetHorizontalHeader();
setData(index(0, 2, QModelIndex()), QVariant("No input catalog"),Qt::DisplayRole  );
setHeaderData(0,Qt::Vertical, QString());
}
void ErrorModel::SetHorizontalHeader(){
setHeaderData(numberErrorRecord()    , Qt::Horizontal, headerErrorRecord());
setHeaderData(numberErrorCheck()    , Qt::Horizontal, headerErrorCheck());
setHeaderData(numberErrorDescriptionRecord()    , Qt::Horizontal, headerErrorDescriptionRecord());
setHeaderData(numberErrorLine()    , Qt::Horizontal, headerErrorLine());
}
void ErrorModel::SetRow(int row,int col,QString temp){
QModelIndex ind = index(row, col, QModelIndex());
if(ind.column() == 1)   setData(ind, QVariant(false),Qt::CheckStateRole );
else
  setData(ind, QVariant(temp),Qt::DisplayRole  );
}
