#include <QtGui>
#include <qmessagebox.h> 
#include "mainwindow.h"
#include "document.h"
#include "default.h"

///////////////////// STL classes & functions /////////////////////////////

void delAndNullify(PropertyValue* &pPropVal) {delete  pPropVal;  pPropVal = 0;}
RowValue::~RowValue(){}
RowValue::RowValue(int id,QString name):id(id),name( name){markDelete=false;}
RowValue::RowValue(int id,QString name,QString textcol1,QString dfile):id(id),name( name),
textCol1(textcol1),dfile(dfile){markDelete = false;}
///////////////////// PropertyValue Implemenation /////////////////////////////// 
PropertyValue::~PropertyValue(){
}
PropertyValue::PropertyValue( const int i,const QString defval,const QString name) : 
	id(i),value(defval),initialvalue(defval),resetvalue(defval),defaultvalue(defval),name(name),changed(FALSE){}
PropertyValue::PropertyValue(const int i, const QString resval,const QString defval,const QString name) :
	id(i),value(resval),initialvalue(resval),resetvalue(resval),defaultvalue(defval),name(name),changed(FALSE){}
PropertyValue::PropertyValue( const int i,const  QString val, const QString resval,const QString defval,const QString name):
	id(i),value(val),initialvalue(val),resetvalue(resval),defaultvalue(defval),name(name),changed(FALSE){}
void PropertyValue::setChanged( bool b) {
    if ( changed == b )
	return;
    changed = b;
    //repaint();
    
}

bool PropertyValue::isChanged() const
{
    return changed;
}
bool PropertyValue::isReseted() const {
    if(value == resetvalue) return FALSE; else return TRUE;
}
QDataStream &operator<< (QDataStream & out, const PropertyValue * prop){
    
out <<   prop->GetId() <<    prop->GetValue();
return out;
}
QDataStream &operator>>(QDataStream & in, PropertyValue & prop){
QString v;    int id;
in >>   id  >>  v;
 prop.SetId(id);   prop.SetResetvalue(v);
return in;
}
///////////////////// <: inherits from PropertyValue :> ////////////////////////// 

///////////////////// PropertyBoolValue Implemenation ///////////////////////////    
PropertyBoolValue::PropertyBoolValue(const int i, const QString defval,const QString name ) : 
	PropertyValue(i,defval ,name) {kind = PropertyValue::Property_Bool;}
PropertyBoolValue::PropertyBoolValue( const int i, const QString resval, const QString defval ,const QString name) : 
	PropertyValue(i, resval,defval ,name) {kind = PropertyValue::Property_Bool;}
PropertyBoolValue::PropertyBoolValue( const int i, const QString val,const  QString resval,const QString defval,const QString name) :
	PropertyValue(i, val,resval,defval,name ) {kind = PropertyValue::Property_Bool;}
///////////////////// PropertyIntValue Implemenation ///////////////////////////  
PropertyIntValue::PropertyIntValue( const int i,const QString defval ,const QString name,const int min,const int max, const bool s):
	PropertyValue(i,defval ,name),
	min(min),max(max), signedValue(s) {kind = PropertyValue::Property_Int;}
PropertyIntValue::PropertyIntValue( const int i,const  QString resval,const QString defval ,const QString name,const int min,const int max, const bool s):
	PropertyValue( i,resval,defval ,name),
	min(min),max(max), signedValue(s) {kind = PropertyValue::Property_Int;}
	
PropertyIntValue::PropertyIntValue( const int i, const  QString val, const  QString resval,const QString defval,const QString name,const int min,const int max, const bool s):
	PropertyValue( i,val,resval,defval,name ),
	min(min),max(max), signedValue(s) {kind = PropertyValue::Property_Int;} 

///////////////////// PropertyIntSpinValue Implemenation ///////////////////////////
PropertyIntSpinValue::PropertyIntSpinValue( const int i,const QString defval ,const QString name,const int min,const int max, const int s):
	PropertyValue(i,defval ,name),
	min(min),max(max), step(s) {kind = PropertyValue::Property_IntSpin;}
PropertyIntSpinValue::PropertyIntSpinValue( const int i,const  QString resval,const QString defval ,const QString name,const int min,const int max, const int s):
	PropertyValue( i,resval,defval ,name),
	min(min),max(max), step(s) {kind = PropertyValue::Property_IntSpin;}

PropertyIntSpinValue::PropertyIntSpinValue( const int i, const  QString val, const  QString resval,const QString defval,const QString name,const int min,const int max, const int s):
	PropertyValue( i,val,resval,defval,name ),
	min(min),max(max), step(s) {kind = PropertyValue::Property_IntSpin;}	
///////////////////// PropertyFloatValue Implemenation /////////////////////////// 

PropertyFloatValue::PropertyFloatValue( const int i,  const QString defval ,const QString name,
					const float min,const float max, const int dec) :
	PropertyValue(i,defval ,name),
	min(min),max(max), decimal(dec) {kind = PropertyValue::Property_Float;}

PropertyFloatValue::PropertyFloatValue(  const int i, const QString resval,const QString defval,
					const QString name,
					const float min,const float max, const int dec ) :
	PropertyValue(i, resval,defval ,name),
	min(min),max(max), decimal(dec) {kind = PropertyValue::Property_Float;}

PropertyFloatValue::PropertyFloatValue( const int i, const  QString val,const  QString resval,
					const QString defval,const QString name,
					const float min,const float max, const int dec):
	PropertyValue( i,val,resval,defval,name ),
	min(min),max(max), decimal(dec) {kind = PropertyValue::Property_Float;} 

///////////////////// PropertyEnumValue Implemenation /////////////////////////// 

PropertyEnumValue::PropertyEnumValue( const int i,  const QString defval ,const QString name,const QString lst) :
  PropertyValue(i,defval ,name) {
list = lst.split ('@'); kind = PropertyValue::Property_Enum;
}
PropertyEnumValue::PropertyEnumValue(  const int i,  const QString resval,const QString defval ,const QString name,const QString lst) :
  PropertyValue( i,resval,defval ,name) {
list = lst.split ('@');  kind = PropertyValue::Property_Enum;
}
PropertyEnumValue::PropertyEnumValue( const int i,  const  QString val, const QString resval,const QString defval,const QString name,const QString lst):
 PropertyValue( i,val,resval,defval,name ) {
list = lst.split ('@'); kind = PropertyValue::Property_Enum;
}
///////////////////// PropertyStringValue Implemenation /////////////////////////// 

PropertyStringValue::PropertyStringValue( const int i,  const QString defval ,const QString name) :
  PropertyValue(i,defval ,name) {
 kind = PropertyValue::Property_String;
}
PropertyStringValue::PropertyStringValue(  const int i,  const QString resval,const QString defval ,const QString name) :
  PropertyValue( i,resval,defval ,name) {
 kind = PropertyValue::Property_String;
}
PropertyStringValue::PropertyStringValue( const int i,  const  QString val, const QString resval,const QString defval,const QString name):
 PropertyValue( i,val,resval,defval,name ) {
 kind = PropertyValue::Property_String;
}
///////////////////// PropertyStringListValue Implemenation /////////////////////////// 

PropertyStringListValue::PropertyStringListValue( const int i,  const QString defval ,const QString name) :
  PropertyValue(i,defval ,name) {
if(!value.isEmpty()) list = value.split ('*'); else  list = QStringList(); kind = PropertyValue::Property_StringList;
}
PropertyStringListValue::PropertyStringListValue(  const int i,  const QString resval,const QString defval ,const QString name) :
  PropertyValue( i,resval,defval ,name) {
if(!value.isEmpty()) list = value.split ('*'); else  list = QStringList();  kind = PropertyValue::Property_StringList;
}
PropertyStringListValue::PropertyStringListValue( const int i,  const  QString val, const QString resval,const QString defval,const QString name):
 PropertyValue( i,val,resval,defval,name ) {
if(!value.isEmpty()) list = value.split ('*'); else  list = QStringList(); kind = PropertyValue::Property_StringList;
}
void PropertyStringListValue::SetListToValue(QStringList &lst) {
 if(!lst.empty ()) {
     value = lst.join("*"); list = lst;
 } else {
     value = QString(); list = QStringList(); 
 }
}
void PropertyStringListValue::SetValue  (const QString val)  { 
   if (!val.isEmpty()) {
      value = val; list = value.split ('*');
   } else {
     value = QString(); list = QStringList(); 
   }
}
void PropertyStringListValue::SetResetvalueToValue       ()  { 
   if (!resetvalue.isEmpty()) {
   value   =  resetvalue;   list = value.split ('*');
   } else {
     value = QString(); list = QStringList(); 
   }
}
void PropertyStringListValue::SetDefault                 ()  {
   if (!defaultvalue.isEmpty()) {
      value   =  defaultvalue; list = value.split ('*');
   } else {
     value = QString(); list = QStringList(); 
   }
}
void PropertyStringListValue::SetInitial                 ()  {
   if (!initialvalue.isEmpty()) {
    value   =  initialvalue; list = value.split ('*');
   } else {
     value = QString(); list = QStringList(); 
    }
}
//////////////////////// PropertyTransformFunction /////////////////////
PropertyTransformFunction::PropertyTransformFunction(const int i, const QString defval ,const QString name):
                           PropertyValue( i,defval ,name)
{
SetVal(defval);SetDef(defval); kind = PropertyValue::Property_TransformFunction;
};
PropertyTransformFunction::PropertyTransformFunction(const int i, const  QString resval,const QString defval,
                           const QString name):
                           PropertyValue( i,resval,defval ,name) 
{
SetVal(resval);SetDef(defval); kind = PropertyValue::Property_TransformFunction;
};
PropertyTransformFunction::PropertyTransformFunction(const int i, const  QString val, const  QString resval,
                            const QString defval,const QString name):
                            PropertyValue( i,val,resval,defval,name ) 
{
SetVal(val);SetDef(defval);  kind = PropertyValue::Property_TransformFunction;
};
void PropertyTransformFunction::SetValue(PropertyTransformFunction* val){
    valIsOn = val->valIsOn;
    valIndexMagnitude = val->valIndexMagnitude;
    valA = val->valA;
    valB = val->valB;
    valC = val->valC;
    valD = val->valD;
    defD = val->defD; defA  = val->defA; defB = val->defB; defC = val->defC;
    QString tmp = "";  // "1@0@0.0@1.0@0.0@0.0"
    if (valIsOn) tmp = "1"; else tmp = "0";
    value = QString("%1@%2@%3@%4@%5@%6").arg(tmp).arg(valIndexMagnitude).arg(valD).arg(valA).arg(valB).arg(valC);
//qWarning(" '%s' ",value.toAscii().data());
}
void PropertyTransformFunction::SetVal(const QString val){
// "1@0@0.0@1.0@0.0@0.0"
  QStringList param = val.split( '@');
  if(param[0].toInt() == 1) valIsOn = true; else valIsOn = false;
  valIndexMagnitude = param[1].toInt();
  valD = param[2]; valA  = param[3]; valB = param[4]; valC = param[5]; 
}

void PropertyTransformFunction::SetDef(const QString val){
// "1@0@0.0@1.0@0.0@0.0"
  QStringList param = val.split( '@');
  if(param[0].toInt() == 1) defIsOn = true; else defIsOn = false;
  defIndexMagnitude = param[1].toInt();
  defD = param[2]; defA  = param[3]; defB = param[4]; defC = param[5]; 
}

void PropertyTransformFunction::SetMagnitude(float m){
if (m >= valD.toFloat()) magnitude = m*valA.toFloat() + valB.toFloat();  else magnitude = valC.toFloat();
}
float PropertyTransformFunction::GetMagnitude(float m) {
SetMagnitude(m);return  magnitude;
}
void  PropertyTransformFunction::GetM(float inm,float &outm){
if (inm >= valD.toFloat()) outm = inm*valA.toFloat() + valB.toFloat();  else outm = valC.toFloat();

}
//--------------------------------------------------------------------------------------------------------
PropertyDateTimeValue::PropertyDateTimeValue( const int i,  const QString defval ,const QString name) :
	PropertyValue(i,defval ,name){kind = PropertyValue::Property_DateTime; }

PropertyDateTimeValue::PropertyDateTimeValue(  const int i, const QString resval,const QString defval,
					const QString name) :
	PropertyValue(i, resval,defval ,name){kind = PropertyValue::Property_DateTime; }

PropertyDateTimeValue::PropertyDateTimeValue( const int i, const  QString val,const  QString resval,
					const QString defval,const QString name):
	PropertyValue( i,val,resval,defval,name ){kind = PropertyValue::Property_DateTime; }

///////////////////// FDocument class implementation ////////////////////////////

FDocument::~FDocument() {
//QList<PropertyTransformFunction *> mb = GetTransformMagnitudeMb();

//qWarning("FDocument::~FDocument '%s'",mb.at(0)->valA.toAscii().data());
  for_each(values.begin(),values.end(),delAndNullify);

  values.erase(remove(values.begin(),values.end(),
               static_cast<PropertyValue*>(0)),
               values.end());

/*
PropValIterator it; int i = 0;
for (it = values.begin(); it!= values.end(); it++) {
qWarning(" i %d  id %d '%s'",i,(*it)->GetId(),(*it)->GetValue().toAscii().data());
delete *it; i++;
}
values.clear();
*/
//------------------------------ Transform Magnitude  --------------------------------------
TransformMagnitudeFunction.clear();

TransformMagnitudeMb.clear();

TransformMagnitudeMs.clear();

TransformMagnitudeMl.clear();

TransformMagnitudeMp.clear();

}
//================================================================================
FDocument::FDocument(FDocument *) {
/*
PropValIterator it; 

SetPropertyFloatValue(RangeLatitudeMin,idRangeLatitudeMin(),doc->GetRangeLatitudeMin()->GetValue(),
                      doc->GetRangeLatitudeMin()->GetResetvalue(),defaultRangeLatitudeMin(),
                      nameRangeLatitudeMin(),minRangeLongitude(),
                      maxRangeLongitude(),decimalRangeLongitude());

*/
}
//================================================================================
    FDocument::FDocument()
    {

///-----------------------------------------Check Range ----------------------------------------
SetPropertyFloatValue(RangeLatitudeMin,idRangeLatitudeMin(),
                      defaultRangeLatitudeMin(),defaultRangeLatitudeMin(),
                      nameRangeLatitudeMin(),minRangeLongitude(),
                      maxRangeLongitude(),decimalRangeLongitude());

SetPropertyFloatValue(RangeLatitudeMax,idRangeLatitudeMax(),
                      defaultRangeLatitudeMax(),defaultRangeLatitudeMax(),
                      nameRangeLatitudeMax(),minRangeLongitude(),
                      maxRangeLongitude(),decimalRangeLongitude());

SetPropertyFloatValue(RangeLongitudeMin,idRangeLongitudeMin(),
                      defaultRangeLongitudeMin(),defaultRangeLongitudeMin(),
                      nameRangeLongitudeMin(),minRangeLongitude(),
                      maxRangeLongitude(),decimalRangeLongitude());
SetPropertyFloatValue(RangeLongitudeMax,idRangeLongitudeMax(),
                      defaultRangeLongitudeMax(),defaultRangeLongitudeMax(),
                      nameRangeLongitudeMax(),minRangeLongitude(),
                      maxRangeLongitude(),decimalRangeLongitude());

SetPropertyFloatValue(RangeDepthMin,idRangeDepthMin(),
                      defaultRangeDepthMin(),defaultRangeDepthMin(),
                      nameRangeDepthMin(),minRangeDepth(),
                      maxRangeDepth(),decimalRangeDepth());
SetPropertyFloatValue(RangeDepthMax,idRangeDepthMax(),
                      defaultRangeDepthMax(),defaultRangeDepthMax(),
                      nameRangeDepthMax(),minRangeDepth(),
                      maxRangeDepth(),decimalRangeDepth());

SetPropertyFloatValue(RangeMbMin,idRangeMbMin(),
                      defaultRangeMbMin(),defaultRangeMbMin(),
                      nameRangeMbMin(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());
SetPropertyFloatValue(RangeMbMax,idRangeMbMax(),
                      defaultRangeMbMax(),defaultRangeMbMax(),
                      nameRangeMbMax(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());

SetPropertyFloatValue(RangeMsMin,idRangeMsMin(),
                      defaultRangeMsMin(),defaultRangeMsMin(),
                      nameRangeMsMin(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());
SetPropertyFloatValue(RangeMsMax,idRangeMsMax(),
                      defaultRangeMsMax(),defaultRangeMsMax(),
                      nameRangeMsMax(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());

SetPropertyFloatValue(RangeMlMin,idRangeMlMin(),
                      defaultRangeMlMin(),defaultRangeMlMin(),
                      nameRangeMlMin(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());
SetPropertyFloatValue(RangeMlMax,idRangeMlMax(),
                      defaultRangeMlMax(),defaultRangeMlMax(),
                      nameRangeMlMax(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());

SetPropertyFloatValue(RangeMpMin,idRangeMpMin(),
                      defaultRangeMpMin(),defaultRangeMpMin(),
                      nameRangeMpMin(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());
SetPropertyFloatValue(RangeMpMax,idRangeMpMax(),
                      defaultRangeMpMax(),defaultRangeMpMax(),
                      nameRangeMpMax(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());


///-----------------------------------------Identification of doubles ----------------------------------------
///------------------------ Define Delta----------------------------------------------
	deltaTime          = new     PropertyIntValue(idDeltaTime(),defaultDeltaTime(),nameDeltaTime(),
						      minDeltaTime(),maxDeltaTime(),signedDeltaTime()); 
	values.push_back(deltaTime); 
	
	deltaDepth        = new     PropertyIntValue(idDeltaDepth(),defaultDeltaDepth(),nameDeltaDepth(),
						      minDeltaDepth(),maxDeltaDepth(),signedDeltaDepth()); 
	values.push_back(deltaDepth);  
	
	deltaLatitude     = new PropertyFloatValue( idDeltaLatitude(),
						    defaultDeltaLatitude(),
						    nameDeltaLatitude(),
						    minDeltaLatitude(),
						    maxDeltaLatitude(),
						    decimalDeltaLatitude());
	 values.push_back(deltaLatitude);
	 
	deltaLongitude  =  new PropertyFloatValue( idDeltaLongitude(),
						       defaultDeltaLongitude(),
						       nameDeltaLongitude(),
						       minDeltaLongitude(),
						       maxDeltaLongitude(),
						       decimalDeltaLongitude());
	 values.push_back(deltaLongitude);
	 
  deltaMagnitude      =  new PropertyFloatValue( idDeltaMagnitude(),
						       defaultDeltaMagnitude(),
						       nameDeltaMagnitude(),
						       minDeltaMagnitude(),
						       maxDeltaMagnitude(),
						       decimalDeltaMagnitude());

	 values.push_back(deltaMagnitude);

SetPropertyEnumValue(DeltaRemoveAction,idDeltaRemoveAction(),
                     defaultDeltaRemoveAction(),defaultDeltaRemoveAction(),
                     nameDeltaRemoveAction(),enumDeltaRemoveAction());

SetPropertyEnumValue(DeltaRemoveEqual,idDeltaRemoveEqual(),
                     defaultDeltaRemoveEqual(),defaultDeltaRemoveEqual(),
                     nameDeltaRemoveEqual(),enumDeltaRemoveEqual());
 //---------------------- Functions entries ------------------------------------------
///----------------------Removing Functions  ----------------------------------------

   RemovingDuplicateFunction = new PropertyEnumValue(idRemovingDuplicateFunction(),
                    defaultRemovingDuplicateFunction(),
                    nameRemovingDuplicateFunction(),
                    enumRemovingDuplicateFunction());
   values.push_back(RemovingDuplicateFunction);
//------------ 1
   RemovingDuplicatePriorityFirst = new PropertyEnumValue(idRemovingFirst(),
                    defaultPriorityIsUsing(),
                    namePriorityIsUsing(),
                    enumPriorityList());
   values.push_back(RemovingDuplicatePriorityFirst);

   RemovingDuplicatePriorityFirstMagnitude = new PropertyEnumValue(idRemovingFirstMagnitude(),
                    defaultPriorityFirstMagnitude(),
                    namePriorityFirstMagnitude(),
                    enumComboBoxMagnitudeList());
   values.push_back(RemovingDuplicatePriorityFirstMagnitude);

//------------ 2
   RemovingDuplicatePrioritySecond = new PropertyEnumValue(idRemovingSecond(),
                    defaultPriorityIsUsing(),
                    namePriorityIsUsing(),
                    enumPriorityList());
   values.push_back(RemovingDuplicatePrioritySecond);

   RemovingDuplicatePrioritySecondMagnitude = new PropertyEnumValue(idRemovingSecondMagnitude(),
                    defaultPrioritySecondMagnitude(),
                    namePrioritySecondMagnitude(),
                    enumComboBoxMagnitudeList());
   values.push_back(RemovingDuplicatePrioritySecondMagnitude);

//------------ 3
   RemovingDuplicatePriorityThird = new PropertyEnumValue(idRemovingThird(),
                    defaultPriorityIsUsing(),
                    namePriorityIsUsing(),
                    enumPriorityList());
   values.push_back(RemovingDuplicatePriorityThird);

   RemovingDuplicatePriorityThirdMagnitude = new PropertyEnumValue(idRemovingThirdMagnitude(),
                    defaultPriorityThirdMagnitude(),
                    namePriorityThirdMagnitude(),
                    enumComboBoxMagnitudeList());
   values.push_back(RemovingDuplicatePriorityThirdMagnitude);

//------------ 4
   RemovingDuplicatePriorityFourth = new PropertyEnumValue(idRemovingFourth(),
                    defaultPriorityIsUsing(),
                    namePriorityIsUsing(),
                    enumPriorityList());
   values.push_back(RemovingDuplicatePriorityFourth);

   RemovingDuplicatePriorityFourthMagnitude = new PropertyEnumValue(idRemovingFourthMagnitude(),
                    defaultPriorityFourthMagnitude(),
                    namePriorityFourthMagnitude(),
                    enumComboBoxMagnitudeList());
   values.push_back(RemovingDuplicatePriorityFourthMagnitude);
///----------------------Delta Magnitude Functions  ----------------------------------------

   DeltaMagnitudeFunction = new PropertyEnumValue(idDeltaMagnitudeFunction(),
                    defaultDeltaMagnitudeFunction(),
                    nameDeltaMagnitudeFunction(),
                    enumDeltaMagnitudeFunction());
   values.push_back(DeltaMagnitudeFunction);
//----------- 1
   DefineDeltaFirstMagnitude = new PropertyTransformFunction(idDeltaMagnitudePriorityFirst(),
                    defaultMagnitudePriorityFirst(),
                    nameMagnitudePriorityFirst());
   values.push_back(DefineDeltaFirstMagnitude);
//----------- 2
   DefineDeltaSecondMagnitude = new PropertyTransformFunction(idDeltaMagnitudePrioritySecond(),
                    defaultMagnitudePrioritySecond(),
                    nameMagnitudePrioritySecond());
   values.push_back(DefineDeltaSecondMagnitude);

//----------- 3
   DefineDeltaThirdMagnitude = new PropertyTransformFunction(idDeltaMagnitudePriorityThird(),
                    defaultMagnitudePriorityThird(),
                    nameMagnitudePriorityThird());
   values.push_back(DefineDeltaThirdMagnitude);

//----------- 4
   DefineDeltaFourthMagnitude = new PropertyTransformFunction(idDeltaMagnitudePriorityFourth(),
                    defaultMagnitudePriorityFourth(),
                    nameMagnitudePriorityFourth());
   values.push_back(DefineDeltaFourthMagnitude);


//-------------------------------  Merge Catalogs -------------------------------------

 SetPropertyIntValue(deltaCatTime,idDeltaCatTime(),
                     defaultDeltaTime(),defaultDeltaTime(),nameDeltaTime(),
                     minDeltaTime(),maxDeltaTime(),signedDeltaTime());
 SetPropertyIntValue(deltaCatDepth,idDeltaCatDepth(),
                     defaultDeltaDepth(),defaultDeltaDepth(),nameDeltaDepth(),
                     minDeltaDepth(),maxDeltaDepth(),signedDeltaDepth());

 SetPropertyFloatValue(deltaCatLatitude,idDeltaCatLatitude(),
                      defaultDeltaLatitude(),defaultDeltaLatitude(),
                      nameDeltaLatitude(),minDeltaLatitude(),
                      maxDeltaLatitude(),decimalDeltaLatitude());

 SetPropertyFloatValue(deltaCatLongitude,idDeltaCatLongitude(),
                      defaultDeltaLongitude(),defaultDeltaLongitude(),
                      nameDeltaLongitude(),minDeltaLongitude(),
                      maxDeltaLongitude(),decimalDeltaLongitude());

 SetPropertyFloatValue(deltaCatMagnitude,idDeltaCatMagnitude(),
                      defaultDeltaMagnitude(),defaultDeltaMagnitude(),
                      nameDeltaMagnitude(),minDeltaMagnitude(),
                      maxDeltaMagnitude(),decimalDeltaMagnitude());

 SetPropertyEnumValue(DeltaCatMagnitudeFunction,idDeltaCatMagnitudeFunction(),
                     defaultDeltaMagnitudeFunction(),defaultDeltaMagnitudeFunction(),
                     nameDeltaMagnitudeFunction(),enumDeltaMagnitudeFunction());

 SetPropertyTransformFunction(DefineDeltaCatFirstMagnitude,
                    idDeltaCatMagnitudePriorityFirst(),
                    defaultMagnitudePriorityFirst(),defaultMagnitudePriorityFirst(),
                    nameMagnitudePriorityFirst());

 SetPropertyTransformFunction(DefineDeltaCatSecondMagnitude,
                    idDeltaCatMagnitudePrioritySecond(),
                    defaultMagnitudePrioritySecond(),defaultMagnitudePrioritySecond(),
                    nameMagnitudePrioritySecond());

 SetPropertyTransformFunction(DefineDeltaCatThirdMagnitude,
                    idDeltaCatMagnitudePriorityThird(),
                    defaultMagnitudePriorityThird(),defaultMagnitudePriorityThird(),
                    nameMagnitudePriorityThird());

 SetPropertyTransformFunction(DefineDeltaCatFourthMagnitude,
                    idDeltaCatMagnitudePriorityFourth(),
                    defaultMagnitudePriorityFourth(),defaultMagnitudePriorityFourth(),
                    nameMagnitudePriorityFourth());

//---------------------------- Duplicate Magnitudes ---------------------------------
   AssignDuplicateMagnitude = new PropertyEnumValue(idAssignDuplicateMagnitude(),
                    defaultAssignDuplicateMagnitude(),
                    nameAssignDuplicateMagnitude(),
                    enumPriorityList());
   values.push_back(AssignDuplicateMagnitude);

//------------------------------Selected Magnitude ---------------------------------------------------
   SelectedDuplicateMagnitude = new PropertyEnumValue(idSelectedDuplicateMagnitude(),
                    defaultSelectedDuplicateMagnitude(),
                    nameSelectedDuplicateMagnitude(),
                    enumComboBoxMagnitudeList());
   values.push_back(SelectedDuplicateMagnitude);

//------------------------------Removing Magnitude ---------------------------------------------------
   RemovingDuplicateMagnitude = new PropertyEnumValue(idRemovingDuplicateMagnitude(),
                    defaultRemovingDuplicateMagnitude(),
                    nameRemovingDuplicateMagnitude(),
                    enumComboBoxMagnitudeList());
   values.push_back(RemovingDuplicateMagnitude);

   SetPropertyStringListValue(ListMergeCatalog,idListMergeCatalog(),
                             defaultListMergeCatalog(),defaultListMergeCatalog(),
                             nameListMergeCatalog());

   SetPropertyStringValue(SecondCatalogName,idSecondCatalog(),
                          defaultSecondCatalog(),defaultSecondCatalog(),
                          nameSecondCatalog());
//----------------------------------------- Compare Action  -------------------
  SetPropertyEnumValue(CompareCatalogAction,idCompareCatalogAction(),
                     defaultCompareCatalogAction(),defaultCompareCatalogAction(),
                     nameCompareCatalogAction(),enumCompareCatalogAction());

//----------------------------------- ADD Catalogs-----------------------------
   SetPropertyStringListValue(ListAddCatalog,idListAddCatalog(),
                             defaultListAddCatalog(),defaultListAddCatalog(),
                             nameListAddCatalog());

//-----------------------------------Create subcatalog -----------------------------
//------------------------------------ Record ----------------------------------
   SampleRecordIs = new PropertyEnumValue(idSampleIsRecord(),
                    defaultSampleIsRecord(),
                    nameSampleIsRecord(),
                    enumPriorityList());
   values.push_back(SampleRecordIs);

   SampleRecordType = new PropertyEnumValue(idSampleRecordType(),
                    defaultSampleRecordType(),
                    nameSampleRecordType(),
                    enumSampleRecordType());
   values.push_back(SampleRecordType);

   SampleRecordDateMin = new PropertyDateTimeValue(idSampleRecordDateMin(),
                    defaultSampleRecordDateMin(),
                    nameSampleRecordDateMin());
   values.push_back(SampleRecordDateMin);

   SampleRecordDateMax = new PropertyDateTimeValue(idSampleRecordDateMax(),
                    defaultSampleRecordDateMax(),
                    nameSampleRecordDateMax());
   values.push_back(SampleRecordDateMax);

   SampleRecordNumberMin  = new PropertyIntValue(idSampleRecordNumberMin(),
                  defaultSampleRecordNumberMin(),
                  nameSampleRecordNumberMin(),
                  minCatalogRecord(),
                  maxCatalogRecord(),
                  signedCatalogRecord());
   values.push_back(SampleRecordNumberMin);

   SampleRecordNumberMax  = new PropertyIntValue(idSampleRecordNumberMax(),
                  defaultSampleRecordNumberMax(),
                  nameSampleRecordNumberMax(),
                  minCatalogRecord(),
                  maxCatalogRecord(),
                  signedCatalogRecord());
   values.push_back(SampleRecordNumberMax);

//------------------------------------ Hipocenter ----------------------------------
   SampleHipoIs = new PropertyEnumValue(idSampleIsHipo(),
                    defaultSampleIsHipo(),
                    nameSampleIsHipo(),
                    enumPriorityList());
   values.push_back(SampleHipoIs);

   SampleHipoType = new PropertyEnumValue(idSampleHipoType(),
                    defaultSampleHipoType(),
                    nameSampleHipoType(),
                    enumSampleHipoType());
   values.push_back(SampleHipoType);


//------------------------------------ Rectangle Hipocenter --------------------------
   SampleHipoRectLatitudeMin  = new PropertyFloatValue( idSampleHipoRectLatitudeMin(),
                defaultSampleHipoRectLatitudeMin(),
                nameSampleHipoRectLatitudeMin(),
                minCatalogLatitude(),
                maxCatalogLatitude(),
                decimalCatalogLatitude());
   values.push_back(SampleHipoRectLatitudeMin);

   SampleHipoRectLatitudeMax  = new PropertyFloatValue( idSampleHipoRectLatitudeMax(),
                defaultSampleHipoRectLatitudeMax(),
                nameSampleHipoRectLatitudeMax(),
                minCatalogLatitude(),
                maxCatalogLatitude(),
                decimalCatalogLatitude());
   values.push_back(SampleHipoRectLatitudeMax);

   SampleHipoRectLongitudeMin  = new PropertyFloatValue(idSampleHipoRectLongitudeMin(),
                defaultSampleHipoRectLongitudeMin(),
                nameSampleHipoRectLongitudeMin(),               
                minCatalogLongitude(),
                maxCatalogLongitude(),
                decimalCatalogLongitude());
   values.push_back(SampleHipoRectLongitudeMin);

   SampleHipoRectLongitudeMax  = new PropertyFloatValue(idSampleHipoRectLongitudeMax(),
                defaultSampleHipoRectLongitudeMax(),
                nameSampleHipoRectLongitudeMax(),
                minCatalogLongitude(),
                maxCatalogLongitude(),
                decimalCatalogLongitude());
   values.push_back(SampleHipoRectLongitudeMax);

   SampleHipoRectMinus180 = new PropertyEnumValue(idSampleHipoRectMinus180(),
                    defaultSampleHipoRectMinus180(),
                    nameSampleHipoRectMinus180(),
                    enumPriorityList());
   values.push_back(SampleHipoRectMinus180);

//------------------------------------ Circle Hipocenter --------------------------
   SampleHipoCircleLatitude  = new PropertyFloatValue( idSampleHipoCircleLatitude(),
                defaultSampleHipoCircleLatitude(),
                nameSampleHipoCircleLatitude(),
                minCatalogLatitude(),
                maxCatalogLatitude(),
                decimalCatalogLatitude());
   values.push_back(SampleHipoCircleLatitude);

   SampleHipoCircleLongitude  = new PropertyFloatValue(idSampleHipoCircleLongitude(),
                defaultSampleHipoCircleLongitude(),
                nameSampleHipoCircleLongitude(),
                minCatalogLongitude(),
                maxCatalogLongitude(),
                decimalCatalogLongitude());
   values.push_back(SampleHipoCircleLongitude);

   SampleHipoCircleRadius  = new PropertyFloatValue( idSampleHipoCircleRadius(),
                defaultSampleHipoCircleRadius(),
                nameSampleHipoCircleRadius(),
                minSampleHipoCircleRadius(),
                maxSampleHipoCircleRadius(),
                decimalSampleHipoCircleRadius());
   values.push_back(SampleHipoCircleRadius);


//------------------------------------ Polygon Hipocenter --------------------------
   SampleHipoPolygonVertice  = new PropertyIntValue(idSampleHipoPolygonVertice(),
                  defaultSampleHipoPolygonVertice(),
                  nameSampleHipoPolygonVertice(),
                  minSampleHipoPolygonVertice(),
                  maxSampleHipoPolygonVertice(),
                  signedSampleHipoPolygonVertice());
   values.push_back(SampleHipoPolygonVertice);

   SampleHipoPolygonMinus180 = new PropertyEnumValue(idSampleHipoPolygonMinus180(),
                    defaultSampleHipoPolygonMinus180(),
                    nameSampleHipoPolygonMinus180(),
                    enumPriorityList());
   values.push_back(SampleHipoPolygonMinus180);

   SetPropertyStringListValue(SampleListLatitude,idSampleListLatitude(),
                             defaultSampleListLatitude(),defaultSampleListLatitude(),
                             nameSampleListLatitude());

   SetPropertyStringListValue(SampleListLongitude,idSampleListLongitude(),
                             defaultSampleListLongitude(),defaultSampleListLongitude(),
                             nameSampleListLongitude());

//------------------------------ Depth Hipocenter --------------------------------------
   SampleDepthIs = new PropertyEnumValue(idSampleIsDepth(),
                    defaultSampleIsDepth(),
                    nameSampleIsDepth(),
                    enumPriorityList());
   values.push_back(SampleDepthIs);

   SampleDepthMin  = new PropertyFloatValue( idSampleDepthMin(),
                defaultSampleDepthMin(),
                nameSampleDepthMin(),
                minCatalogDepth(),
                maxCatalogDepth(),
                decimalCatalogDepth());
   values.push_back(SampleDepthMin);

   SampleDepthMax  = new PropertyFloatValue( idSampleDepthMax(),
                defaultSampleDepthMax(),
                nameSampleDepthMax(),
                minCatalogDepth(),
                maxCatalogDepth(),
                decimalCatalogDepth());
   values.push_back(SampleDepthMax);

//------------------------------ Magnitude Mb --------------------------------------
   SampleMbIs = new PropertyEnumValue(idSampleIsMb(),
                    defaultSampleIsMb(),
                    nameSampleIsMb(),
                    enumPriorityList());
   values.push_back(SampleMbIs);

   SampleMbMin  = new PropertyFloatValue( idSampleMbMin(),
                defaultSampleMbMin(),
                nameSampleMbMin(),
                minCatalogMb(),
                maxCatalogMb(),
                decimalCatalogMb());
   values.push_back(SampleMbMin);

   SampleMbMax  = new PropertyFloatValue( idSampleMbMax(),
                defaultSampleMbMax(),
                nameSampleMbMax(),
                minCatalogMb(),
                maxCatalogMb(),
                decimalCatalogMb());
   values.push_back(SampleMbMax);

//------------------------------ Magnitude Ms --------------------------------------
   SampleMsIs = new PropertyEnumValue(idSampleIsMs(),
                    defaultSampleIsMs(),
                    nameSampleIsMs(),
                    enumPriorityList());
   values.push_back(SampleMsIs);

   SampleMsMin  = new PropertyFloatValue( idSampleMsMin(),
                defaultSampleMsMin(),
                nameSampleMsMin(),
                minCatalogMs(),
                maxCatalogMs(),
                decimalCatalogMs());
   values.push_back(SampleMsMin);

   SampleMsMax  = new PropertyFloatValue( idSampleMsMax(),
                defaultSampleMsMax(),
                nameSampleMsMax(),
                minCatalogMs(),
                maxCatalogMs(),
                decimalCatalogMs());
   values.push_back(SampleMsMax);

//------------------------------ Magnitude Ml --------------------------------------
   SampleMlIs = new PropertyEnumValue(idSampleIsMl(),
                    defaultSampleIsMl(),
                    nameSampleIsMl(),
                    enumPriorityList());
   values.push_back(SampleMlIs);

   SampleMlMin  = new PropertyFloatValue( idSampleMlMin(),
                defaultSampleMlMin(),
                nameSampleMlMin(),
                minCatalogMl(),
                maxCatalogMl(),
                decimalCatalogMl());
   values.push_back(SampleMlMin);

   SampleMlMax  = new PropertyFloatValue( idSampleMlMax(),
                defaultSampleMlMax(),
                nameSampleMlMax(),
                minCatalogMl(),
                maxCatalogMl(),
                decimalCatalogMl());
   values.push_back(SampleMlMax);

//------------------------------ Magnitude Mp --------------------------------------
   SampleMpIs = new PropertyEnumValue(idSampleIsMp(),
                    defaultSampleIsMp(),
                    nameSampleIsMp(),
                    enumPriorityList());
   values.push_back(SampleMpIs);

   SampleMpMin  = new PropertyFloatValue( idSampleMpMin(),
                defaultSampleMpMin(),
                nameSampleMpMin(),
                minCatalogMp(),
                maxCatalogMp(),
                decimalCatalogMp());
   values.push_back(SampleMpMin);

   SampleMpMax  = new PropertyFloatValue( idSampleMpMax(),
                defaultSampleMpMax(),
                nameSampleMpMax(),
                minCatalogMp(),
                maxCatalogMp(),
                decimalCatalogMp());
   values.push_back(SampleMpMax);
//------------------------------ Intencity --------------------------------------
   SampleIntensityIs = new PropertyEnumValue(idSampleIsIntensity(),
                    defaultSampleIsIntensity(),
                    nameSampleIsIntensity(),
                    enumPriorityList());
   values.push_back(SampleIntensityIs);

   SampleIntensityMin  = new PropertyIntValue( idSampleIntensityMin(),
                defaultSampleIntensityMin(),
                nameSampleIntensityMin(),
                minCatalogIntensity(),
                maxCatalogIntensity(),
                signedCatalogIntensity());
   values.push_back(SampleIntensityMin);

   SampleIntensityMax  = new PropertyIntValue( idSampleIntensityMax(),
                defaultSampleIntensityMax(),
                nameSampleIntensityMax(),
                minCatalogIntensity(),
                maxCatalogIntensity(),
                signedCatalogIntensity());
   values.push_back(SampleIntensityMax);

SetPropertyEnumValue(SampleMagnitudeIs,idSampleIsMagFunction(),
                     defaultSampleIsMagFunction(),defaultSampleIsMagFunction(),
                     nameSampleIsMagFunction(),enumPriorityList());

SetPropertyEnumValue(SampleMagnitudeType,idSampleMagFunctionType(),
                     defaultSampleMagFunctionType(),defaultSampleMagFunctionType(),
                     nameSampleMagFunctionType(),enumPriorityList());

SetPropertyFloatValue(SampleMagFunctionMin,idSampleMagFunctionMin(),
                      defaultSampleMagFunctionMin(),defaultSampleMagFunctionMin(),
                      nameSampleMagFunctionMin(),minCatalogMb(),
                      maxCatalogMb(),decimalCatalogMb());

SetPropertyFloatValue(SampleMagFunctionMax,idSampleMagFunctionMax(),
                      defaultSampleMagFunctionMax(),defaultSampleMagFunctionMax(),
                      nameSampleMagFunctionMax(),minCatalogMb(),
                      maxCatalogMb(),decimalCatalogMb());

SetPropertyEnumValue(SampleMagFunction,idSampleMagFunction(),
                     defaultSampleMagFunction(),defaultSampleMagFunction(),
                     nameSampleMagFunction(),enumSampleMagFunction());

SetPropertyTransformFunction(SampleMagFunctionFirstMagnitude,
                    idSampleMagFunctionPriorityFirst(),
                    defaultMagnitudePriorityFirst(),defaultMagnitudePriorityFirst(),
                    nameMagnitudePriorityFirst());

SetPropertyTransformFunction(SampleMagFunctionSecondMagnitude,
                    idSampleMagFunctionPrioritySecond(),
                    defaultMagnitudePrioritySecond(),defaultMagnitudePrioritySecond(),
                    nameMagnitudePrioritySecond());

SetPropertyTransformFunction(SampleMagFunctionThirdMagnitude,
                    idSampleMagFunctionPriorityThird(),
                    defaultMagnitudePriorityThird(),defaultMagnitudePriorityThird(),
                    nameMagnitudePriorityThird());

SetPropertyTransformFunction(SampleMagFunctionFourthMagnitude,
                    idSampleMagFunctionPriorityFourth(),
                    defaultMagnitudePriorityFourth(),defaultMagnitudePriorityFourth(),
                    nameMagnitudePriorityFourth());

//------------------------------ Transform Magnitude  --------------------------------------

 PropertyEnumValue* pEnum = new PropertyEnumValue(1001,"0","",enumDeltaMagnitudeFunction());
 TransformMagnitudeFunction.append(pEnum); values.push_back(pEnum);

 pEnum = new PropertyEnumValue(1002,"0","",enumDeltaMagnitudeFunction());
 TransformMagnitudeFunction.append(pEnum); values.push_back(pEnum);

 pEnum = new PropertyEnumValue(1003,"0","",enumDeltaMagnitudeFunction());
 TransformMagnitudeFunction.append(pEnum); values.push_back(pEnum);

 pEnum = new PropertyEnumValue(1004,"0","",enumDeltaMagnitudeFunction());
 TransformMagnitudeFunction.append(pEnum); values.push_back(pEnum);

  //-------------------------- MB ---------------------------------------

 PropertyTransformFunction* pTrans =
                            new  PropertyTransformFunction(1005,"1@0@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMb.append(pTrans); values.push_back(pTrans);

 pTrans = new  PropertyTransformFunction(1006,"0@1@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMb.append(pTrans); values.push_back(pTrans);

 pTrans = new  PropertyTransformFunction(1007,"0@2@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMb.append(pTrans); values.push_back(pTrans);

 pTrans = new  PropertyTransformFunction(1008,"0@3@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMb.append(pTrans); values.push_back(pTrans);

  //-------------------------- MS ---------------------------------------

 pTrans = new  PropertyTransformFunction(1009,"0@0@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMs.append(pTrans); values.push_back(pTrans);

 pTrans = new  PropertyTransformFunction(1010,"1@1@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMs.append(pTrans); values.push_back(pTrans);

 pTrans = new  PropertyTransformFunction(1011,"0@2@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMs.append(pTrans); values.push_back(pTrans);

 pTrans = new  PropertyTransformFunction(1012,"0@3@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMs.append(pTrans); values.push_back(pTrans);

   //-------------------------- ML ---------------------------------------

 pTrans = new  PropertyTransformFunction(1013,"0@0@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMl.append(pTrans); values.push_back(pTrans);

 pTrans = new  PropertyTransformFunction(1014,"0@1@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMl.append(pTrans); values.push_back(pTrans);

 pTrans = new  PropertyTransformFunction(1015,"1@2@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMl.append(pTrans); values.push_back(pTrans);

 pTrans = new  PropertyTransformFunction(1016,"0@3@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMl.append(pTrans); values.push_back(pTrans);

  //-------------------------- MP ---------------------------------------

 pTrans = new  PropertyTransformFunction(1017,"0@0@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMp.append(pTrans); values.push_back(pTrans);

 pTrans = new  PropertyTransformFunction(1018,"0@1@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMp.append(pTrans); values.push_back(pTrans);

 pTrans = new  PropertyTransformFunction(1019,"0@2@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMp.append(pTrans); values.push_back(pTrans);

 pTrans = new  PropertyTransformFunction(1020,"1@3@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMp.append(pTrans); values.push_back(pTrans);

///---------------------- Project parameters -------------------------------------
   SetPropertyStringListValue(ProjectActionList,idProjectActionList(),
                             defaultProjectActionList(),defaultProjectActionList(),
                             projectActionList());

   SetPropertyStringValue(ProjectCatalogName,idProjectCatalogName(),
                          defaultProjectCatalogName(),defaultProjectCatalogName(),
                          nameProjectCatalogName());

  SetPropertyEnumValue(ProjectDocumentIn,idProjectDocumentIn(),
                     defaultProjectDocumentIn(),defaultProjectDocumentIn(),
                     nameProjectDocumentIn(),enumProjectDocumentIn());

  SetPropertyEnumValue(ProjectActionChoise,idProjectActionChoise(),
                     defaultProjectActionChoise(),defaultProjectActionChoise(),
                     nameProjectActionChoise(),enumProjectActionChoise());

}
void FDocument::SetPropertyIntValue(PropertyIntValue* &p, int id,QString reset,QString defaultvalue,
                        QString name,int min,int max,bool sign){
   p = new PropertyIntValue(id,reset,defaultvalue,name,min,max,sign);
   values.push_back(p);
}
void FDocument::SetPropertyTransformFunction(PropertyTransformFunction* &p, int id,
                        QString reset,QString defaultvalue,
                        QString name){
   p = new PropertyTransformFunction(id,reset,defaultvalue,name);
   values.push_back(p);
}
void FDocument::SetPropertyFloatValue(PropertyFloatValue* &p, int id,QString reset,QString defaultvalue,
                        QString name,float min,float max,int decimal){
   p = new PropertyFloatValue(id,reset,defaultvalue,name,min,max,decimal);
   values.push_back(p);
}

void FDocument::SetPropertyEnumValue(PropertyEnumValue* &p, int id,QString reset,QString defaultvalue,
                        QString name,QString enumeration){
   p = new PropertyEnumValue(id,reset,defaultvalue,name,enumeration);                    
   values.push_back(p);
}
void FDocument::SetPropertyStringValue(PropertyStringValue* &p, int id,QString reset,QString defaultvalue,
                        QString name){
   p = new PropertyStringValue(id,reset,defaultvalue,name);
   values.push_back(p);
}
void FDocument::SetPropertyStringListValue(PropertyStringListValue* &p, int id,QString reset,QString defaultvalue,
                        QString name){
   p = new PropertyStringListValue(id,reset,defaultvalue,name);                    
   values.push_back(p);
}
bool FDocument::CheckModified(){
PropValIterator i = values.begin();
while (i != values.end()) { (*i)->setChanged();i++;}
return  isModified();
}
bool FDocument::isModified() {
    PropValIterator i = find_if(values.begin(),values.end(),mem_fun(&PropertyValue::isChanged));
    if(i!=values.end()) return true;
    else return false;
}
/*
void    FDocument::printFDocument(){
    PrintDocument<PropValVector> P = 
	    for_each(values.begin(),values.end(),
		     PrintDocument<PropValVector>(std::cout)); 
    cout << "-------------  "<< P.count << " parameters are in document  -------------" << endl<< endl; 
    
    cout << "-------------  End of print document  ----------------" << endl<<endl; 
}
*/
FDocument::FDocument(const QString &fileName){
    
projectFileGood = true;
if (!loadBinary(fileName)) projectFileGood = false;

}

bool  FDocument::loadBinary(const QString &fileName){

    QFile file(fileName);  
    if (!file.open(QIODevice::ReadOnly)) {
	ioError(file,"Cannot open file %1 for reading") ;
	return false;
    }
    QDataStream in (&file);    
    quint32 magic;
    in >>  magic;
    if (magic != MagicNumber) {
    error(file,"File %1 is not a project file. Bad Format.");
    	return false;
    }
    int ecpVersion;
    
    in >>  ecpVersion;
    QString lst =  EcpVersionsForQt4();
    QStringList words =  lst.split(" ");
   int minForQt4 =  words[0].toInt();
   int maxForQt4 =  words[1].toInt();
   
   if( ecpVersion  >=  minForQt4 && ecpVersion  <=  maxForQt4 )	
       in.setVersion(QDataStream::Qt_4_0);
// Here for next    VersionQDataStream
   lst =  EcpVersionsForQt4_3();
   int minForQt4_3 =  words[0].toInt();
   int maxForQt4_3 =  words[1].toInt();  
   if( ecpVersion  >=  minForQt4_3 && ecpVersion  <=  maxForQt4_3 )	
	   in.setVersion(QDataStream::Qt_4_3);
 
// Here for next    VersionQDataStream  
    readFromStream(in);
   
    if ( file.error() != QFile::NoError  ) {
	ioError(file,"Error reading from file %1") ;
	return false;    
   }
    
    file.close() ;    
    return true;    
}

bool  FDocument::saveBinary(const QString &fileName){
    QFile file(fileName);  
    if (!file.open(QIODevice::WriteOnly)) {
	ioError(file,"Cannot open file %1 for writing") ;
	return false;
    }
    QDataStream out (&file);
     out  <<  (quint32)MagicNumber;
     int ecpVersion =  EcpCurrentVersion().toInt();
     out  <<  ecpVersion;
     writeToStream(out);
    if ( file.error() != QFile::NoError  ) {
	ioError(file,"Error writing to file %1") ;
	return false;
    }

    file.close() ;
return true;
}

void FDocument::error(const QFile &file ,const QString &message){
    QMessageBox::warning(0,"Project Parameters",message.arg(file.fileName()));    
}
void FDocument::ioError(const QFile &file ,const QString &message){
    error(file, message + ": " + file.errorString());
}
void FDocument::writeToStream(QDataStream &out){
    PropValIterator it = values.begin();
    while(it != values.end()) {  out << *it; ++it;  }
}
void FDocument::readFromStream(QDataStream &in){
	PropertyValue prop;	
   while (!in.atEnd()){
	in >> prop;

///-----------------------------------------Check Range ----------------------------------------
if ( prop.GetId() ==   idRangeLatitudeMin())
SetPropertyFloatValue(RangeLatitudeMin,idRangeLatitudeMin(),
                      prop.GetResetvalue(),defaultRangeLatitudeMin(),
                      nameRangeLatitudeMin(),minRangeLongitude(),
                      maxRangeLongitude(),decimalRangeLongitude());

if ( prop.GetId() ==   idRangeLatitudeMax())
SetPropertyFloatValue(RangeLatitudeMax,idRangeLatitudeMax(),
                      prop.GetResetvalue(),defaultRangeLatitudeMax(),
                      nameRangeLatitudeMax(),minRangeLongitude(),
                      maxRangeLongitude(),decimalRangeLongitude());

if ( prop.GetId() ==   idRangeLongitudeMin())
SetPropertyFloatValue(RangeLongitudeMin,idRangeLongitudeMin(),
                      prop.GetResetvalue(),defaultRangeLongitudeMin(),
                      nameRangeLongitudeMin(),minRangeLongitude(),
                      maxRangeLongitude(),decimalRangeLongitude());
if ( prop.GetId() ==   idRangeLongitudeMax())
SetPropertyFloatValue(RangeLongitudeMax,idRangeLongitudeMax(),
                      prop.GetResetvalue(),defaultRangeLongitudeMax(),
                      nameRangeLongitudeMax(),minRangeLongitude(),
                      maxRangeLongitude(),decimalRangeLongitude());

if ( prop.GetId() ==   idRangeDepthMin())
SetPropertyFloatValue(RangeDepthMin,idRangeDepthMin(),
                      prop.GetResetvalue(),defaultRangeDepthMin(),
                      nameRangeDepthMin(),minRangeDepth(),
                      maxRangeDepth(),decimalRangeDepth());
if ( prop.GetId() ==   idRangeDepthMax())
SetPropertyFloatValue(RangeDepthMax,idRangeDepthMax(),
                      prop.GetResetvalue(),defaultRangeDepthMax(),
                      nameRangeDepthMax(),minRangeDepth(),
                      maxRangeDepth(),decimalRangeDepth());

if ( prop.GetId() ==   idRangeMbMin())
SetPropertyFloatValue(RangeMbMin,idRangeMbMin(),
                      prop.GetResetvalue(),defaultRangeMbMin(),
                      nameRangeMbMin(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());

if ( prop.GetId() ==   idRangeMbMax())
SetPropertyFloatValue(RangeMbMax,idRangeMbMax(),
                      prop.GetResetvalue(),defaultRangeMbMax(),
                      nameRangeMbMax(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());

if ( prop.GetId() ==   idRangeMsMin())
SetPropertyFloatValue(RangeMsMin,idRangeMsMin(),
                      prop.GetResetvalue(),defaultRangeMsMin(),
                      nameRangeMsMin(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());
if ( prop.GetId() ==   idRangeMsMax())
SetPropertyFloatValue(RangeMsMax,idRangeMsMax(),
                      prop.GetResetvalue(),defaultRangeMsMax(),
                      nameRangeMsMax(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());

if ( prop.GetId() ==   idRangeMlMin())
SetPropertyFloatValue(RangeMlMin,idRangeMlMin(),
                      prop.GetResetvalue(),defaultRangeMlMin(),
                      nameRangeMlMin(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());
if ( prop.GetId() ==   idRangeMlMax())
SetPropertyFloatValue(RangeMlMax,idRangeMlMax(),
                      prop.GetResetvalue(),defaultRangeMlMax(),
                      nameRangeMlMax(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());

if ( prop.GetId() ==   idRangeMpMin())
SetPropertyFloatValue(RangeMpMin,idRangeMpMin(),
                      prop.GetResetvalue(),defaultRangeMpMin(),
                      nameRangeMpMin(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());
if ( prop.GetId() ==   idRangeMpMax())
SetPropertyFloatValue(RangeMpMax,idRangeMpMax(),
                      prop.GetResetvalue(),defaultRangeMpMax(),
                      nameRangeMpMax(),minRangeMagnitude(),
                      maxRangeMagnitude(),decimalRangeMagnitude());


	
 ///-----------------------------------------Identification of doubles -----------------------------------


///------------------------ Define Delta----------------------------------------------
	
	if ( prop.GetId() ==  idDeltaTime()) {
	    deltaTime   =  new  PropertyIntValue(prop.GetId(),
						  prop.GetResetvalue(), 
						  defaultDeltaTime(),
						  nameDeltaTime(),
						  minDeltaTime(),
						  maxDeltaTime(),
						  signedDeltaTime());
	    values.push_back(deltaTime); 
	}
	
	if ( prop.GetId() ==  idDeltaDepth()) {
	    deltaDepth   =  new  PropertyIntValue(prop.GetId(),
						  prop.GetResetvalue(), 
						  defaultDeltaDepth(),
						  nameDeltaDepth(),
						  minDeltaDepth(),
						  maxDeltaDepth(),
						  signedDeltaDepth());
	    values.push_back(deltaDepth); 
	}	
	
	if ( prop.GetId() ==   idDeltaLatitude()) {
	    deltaLatitude    = new PropertyFloatValue(prop.GetId(),
						prop.GetResetvalue(), 
						defaultDeltaLatitude(),
						nameDeltaLatitude(),
						minDeltaLatitude(),
						maxDeltaLatitude(),
						decimalDeltaLatitude());
	    values.push_back(deltaLatitude);
	}   

	if ( prop.GetId() ==   idDeltaLongitude()) {
	    deltaLongitude    = new PropertyFloatValue(prop.GetId(),
						prop.GetResetvalue(), 
						defaultDeltaLongitude(),
						nameDeltaLongitude(),
						minDeltaLongitude(),
						maxDeltaLongitude(),
						decimalDeltaLongitude());
	    values.push_back(deltaLongitude);
	}   
	
	if ( prop.GetId() ==   idDeltaMagnitude()) {
	    deltaMagnitude    = new PropertyFloatValue(prop.GetId(),
						prop.GetResetvalue(),
						defaultDeltaMagnitude(),
						nameDeltaMagnitude(),
						minDeltaMagnitude(),
						maxDeltaMagnitude(),
						decimalDeltaMagnitude());
	    values.push_back(deltaMagnitude);
	}

if ( prop.GetId() ==   idDeltaRemoveAction())
SetPropertyEnumValue(DeltaRemoveAction,idDeltaRemoveAction(),
                     prop.GetResetvalue(),defaultDeltaRemoveAction(),
                     nameDeltaRemoveAction(),enumDeltaRemoveAction());

if ( prop.GetId() ==   idDeltaRemoveEqual())
SetPropertyEnumValue(DeltaRemoveEqual,idDeltaRemoveEqual(),
                     prop.GetResetvalue(),defaultDeltaRemoveEqual(),
                     nameDeltaRemoveEqual(),enumDeltaRemoveEqual());


 //---------------------- Functions entries ------------------------------------------

///----------------------Removing Functions  ----------------------------------------

	if ( prop.GetId() ==   idRemovingDuplicateFunction()) {
	    RemovingDuplicateFunction    = new PropertyEnumValue(prop.GetId(),
						prop.GetResetvalue(),
						defaultRemovingDuplicateFunction(),
						nameRemovingDuplicateFunction(),
						enumRemovingDuplicateFunction());
	    values.push_back(RemovingDuplicateFunction);
	}  
 //------------ 1
 if ( prop.GetId() ==   idRemovingFirst()) {
      RemovingDuplicatePriorityFirst       = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultPriorityIsUsing(),
                    namePriorityIsUsing(),
                    enumPriorityList());
   values.push_back(RemovingDuplicatePriorityFirst);
 }
 if ( prop.GetId() ==   idRemovingFirstMagnitude()) {
      RemovingDuplicatePriorityFirstMagnitude = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultPriorityFirstMagnitude(),
                    namePriorityFirstMagnitude(),
                    enumComboBoxMagnitudeList());
   values.push_back(RemovingDuplicatePriorityFirstMagnitude);
 }
//------------ 2
 if ( prop.GetId() ==   idRemovingSecond()) {
      RemovingDuplicatePrioritySecond       = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultPriorityIsUsing(),
                    namePriorityIsUsing(),
                    enumPriorityList());
   values.push_back(RemovingDuplicatePrioritySecond);
 }
 if ( prop.GetId() ==   idRemovingSecondMagnitude()) {
      RemovingDuplicatePrioritySecondMagnitude = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultPrioritySecondMagnitude(),
                    namePrioritySecondMagnitude(),
                    enumComboBoxMagnitudeList());
   values.push_back(RemovingDuplicatePrioritySecondMagnitude);
 }
//------------ 3
 if ( prop.GetId() ==   idRemovingThird()) {
      RemovingDuplicatePriorityThird       = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultPriorityIsUsing(),
                    namePriorityIsUsing(),
                    enumPriorityList());
   values.push_back(RemovingDuplicatePriorityThird);
 }
 if ( prop.GetId() ==   idRemovingThirdMagnitude()) {
      RemovingDuplicatePriorityThirdMagnitude = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultPriorityThirdMagnitude(),
                    namePriorityThirdMagnitude(),
                    enumComboBoxMagnitudeList());
   values.push_back(RemovingDuplicatePriorityThirdMagnitude);
 }
//------------ 4
 if ( prop.GetId() ==   idRemovingFourth()) {
      RemovingDuplicatePriorityFourth       = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultPriorityIsUsing(),
                    namePriorityIsUsing(),
                    enumPriorityList());
   values.push_back(RemovingDuplicatePriorityFourth);
 }
 if ( prop.GetId() ==   idRemovingFourthMagnitude()) {
      RemovingDuplicatePriorityFourthMagnitude = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultPriorityFourthMagnitude(),
                    namePriorityFourthMagnitude(),
                    enumComboBoxMagnitudeList());
   values.push_back(RemovingDuplicatePriorityFourthMagnitude);
 }


///----------------------Delta Magnitude Functions  ----------------------------------------
	if ( prop.GetId() ==   idDeltaMagnitudeFunction()) {
	    DeltaMagnitudeFunction    = new PropertyEnumValue(prop.GetId(),
						prop.GetResetvalue(),
            defaultDeltaMagnitudeFunction(),
            nameDeltaMagnitudeFunction(),
            enumDeltaMagnitudeFunction());
	    values.push_back(DeltaMagnitudeFunction);
	}  

//----------- 1
 if ( prop.GetId() ==   idDeltaMagnitudePriorityFirst()) {
      DefineDeltaFirstMagnitude = new PropertyTransformFunction(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultMagnitudePriorityFirst(),
                    nameMagnitudePriorityFirst());
   values.push_back(DefineDeltaFirstMagnitude);
 }

//----------- 2
 if ( prop.GetId() ==   idDeltaMagnitudePrioritySecond()) {
      DefineDeltaSecondMagnitude = new PropertyTransformFunction(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultMagnitudePrioritySecond(),
                    nameMagnitudePrioritySecond());
   values.push_back(DefineDeltaSecondMagnitude);
 }

//----------- 3
 if ( prop.GetId() ==   idDeltaMagnitudePriorityThird()) {
      DefineDeltaThirdMagnitude = new PropertyTransformFunction(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultMagnitudePriorityThird(),
                    nameMagnitudePriorityThird());
   values.push_back(DefineDeltaThirdMagnitude);
 }

//----------- 4
 if ( prop.GetId() ==   idDeltaMagnitudePriorityFourth()) {
      DefineDeltaFourthMagnitude = new PropertyTransformFunction(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultMagnitudePriorityFourth(),
                    nameMagnitudePriorityFourth());
   values.push_back(DefineDeltaFourthMagnitude);
 }


//-------------------------------  Merge Two Catalog -------------------------------------
 if ( prop.GetId() ==   idDeltaCatTime())
 SetPropertyIntValue(deltaCatTime,idDeltaCatTime(),
                     prop.GetResetvalue(),defaultDeltaTime(),nameDeltaTime(),
                     minDeltaTime(),maxDeltaTime(),signedDeltaTime());

 if ( prop.GetId() ==   idDeltaCatDepth())
 SetPropertyIntValue(deltaCatDepth,idDeltaCatDepth(),
                     prop.GetResetvalue(),defaultDeltaDepth(),nameDeltaDepth(),
                     minDeltaDepth(),maxDeltaDepth(),signedDeltaDepth());

 if ( prop.GetId() ==   idDeltaCatLatitude())
 SetPropertyFloatValue(deltaCatLatitude,idDeltaCatLatitude(),
                      prop.GetResetvalue(),defaultDeltaLatitude(),
                      nameDeltaLatitude(),minDeltaLatitude(),
                      maxDeltaLatitude(),decimalDeltaLatitude());

  if ( prop.GetId() ==   idDeltaCatLongitude())
  SetPropertyFloatValue(deltaCatLongitude,idDeltaCatLongitude(),
                      prop.GetResetvalue(),defaultDeltaLongitude(),
                      nameDeltaLongitude(),minDeltaLongitude(),
                      maxDeltaLongitude(),decimalDeltaLongitude());

  if ( prop.GetId() ==   idDeltaCatMagnitude())
  SetPropertyFloatValue(deltaCatMagnitude,idDeltaCatMagnitude(),
                      prop.GetResetvalue(),defaultDeltaMagnitude(),
                      nameDeltaMagnitude(),minDeltaMagnitude(),
                      maxDeltaMagnitude(),decimalDeltaMagnitude());

  if ( prop.GetId() ==   idDeltaCatMagnitudeFunction())
  SetPropertyEnumValue(DeltaCatMagnitudeFunction,idDeltaCatMagnitudeFunction(),
                     prop.GetResetvalue(),defaultDeltaMagnitudeFunction(),
                     nameDeltaMagnitudeFunction(),enumDeltaMagnitudeFunction());

  if ( prop.GetId() ==   idDeltaCatMagnitudePriorityFirst())
  SetPropertyTransformFunction(DefineDeltaCatFirstMagnitude,
                    idDeltaCatMagnitudePriorityFirst(),
                    prop.GetResetvalue(),defaultMagnitudePriorityFirst(),
                    nameMagnitudePriorityFirst());

  if ( prop.GetId() ==   idDeltaCatMagnitudePrioritySecond())
  SetPropertyTransformFunction(DefineDeltaCatSecondMagnitude,
                    idDeltaCatMagnitudePrioritySecond(),
                    prop.GetResetvalue(),defaultMagnitudePrioritySecond(),
                    nameMagnitudePrioritySecond());

  if ( prop.GetId() ==   idDeltaCatMagnitudePriorityThird())
  SetPropertyTransformFunction(DefineDeltaCatThirdMagnitude,
                    idDeltaCatMagnitudePriorityThird(),
                    prop.GetResetvalue(),defaultMagnitudePriorityThird(),
                    nameMagnitudePriorityThird());

  if ( prop.GetId() ==   idDeltaCatMagnitudePriorityFourth())
  SetPropertyTransformFunction(DefineDeltaCatFourthMagnitude,
                    idDeltaCatMagnitudePriorityFourth(),
                    prop.GetResetvalue(),defaultMagnitudePriorityFourth(),
                    nameMagnitudePriorityFourth());


//---------------------------- Duplicate Magnitudes ---------------------------------
  if ( prop.GetId() ==   idAssignDuplicateMagnitude()) {
   AssignDuplicateMagnitude = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultAssignDuplicateMagnitude(),
                    nameAssignDuplicateMagnitude(),
                    enumPriorityList());
   values.push_back(AssignDuplicateMagnitude);
  }
//------------------------------Selected Magnitude ---------------------------------------------------
  if ( prop.GetId() ==   idSelectedDuplicateMagnitude()) {
   SelectedDuplicateMagnitude = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultSelectedDuplicateMagnitude(),
                    nameSelectedDuplicateMagnitude(),
                    enumComboBoxMagnitudeList());
   values.push_back(SelectedDuplicateMagnitude);
  }
//------------------------------Removing Magnitude ---------------------------------------------------
  if ( prop.GetId() ==   idRemovingDuplicateMagnitude()) {
   RemovingDuplicateMagnitude = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultRemovingDuplicateMagnitude(),
                    nameRemovingDuplicateMagnitude(),
                    enumComboBoxMagnitudeList());
   values.push_back(RemovingDuplicateMagnitude);
  }

 if ( prop.GetId() ==   idListMergeCatalog())
   SetPropertyStringListValue(ListMergeCatalog,idListMergeCatalog(),
                             prop.GetResetvalue(),defaultListMergeCatalog(),
                             nameListMergeCatalog());

 if ( prop.GetId() ==   idSecondCatalog())
   SetPropertyStringValue(SecondCatalogName,idSecondCatalog(),
                          prop.GetResetvalue(),defaultSecondCatalog(),
                          nameSecondCatalog());
//----------------------------------------- Compare Action  -------------------
 if ( prop.GetId() ==   idCompareCatalogAction()) {
  SetPropertyEnumValue(CompareCatalogAction,idCompareCatalogAction(),
                      prop.GetResetvalue(),defaultCompareCatalogAction(),
                     nameCompareCatalogAction(),enumCompareCatalogAction());
 }
///----------------------------------- Add Catalogs-----------------------------
 if ( prop.GetId() ==   idListAddCatalog()) {
   SetPropertyStringListValue(ListAddCatalog,idListAddCatalog(),
                             prop.GetResetvalue(),defaultListAddCatalog(),
                             nameListAddCatalog());
 }
///-----------------------------------Create subcatalog -----------------------------
//------------------------------------ Record ----------------------------------
  if ( prop.GetId() ==   idSampleIsRecord()) {
   SampleRecordIs = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultSampleIsRecord(),
                    nameSampleIsRecord(),
                    enumPriorityList());
   values.push_back(SampleRecordIs);
  }

  if ( prop.GetId() ==   idSampleRecordType()) {
   SampleRecordType = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultSampleRecordType(),
                    nameSampleRecordType(),
                    enumSampleRecordType());
   values.push_back(SampleRecordType);
  }

  if ( prop.GetId() ==   idSampleRecordDateMin()) {
   SampleRecordDateMin = new PropertyDateTimeValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultSampleRecordDateMin(),
                    nameSampleRecordDateMin());
   values.push_back(SampleRecordDateMin);
  }

 if ( prop.GetId() ==   idSampleRecordDateMax()) {
   SampleRecordDateMax = new PropertyDateTimeValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultSampleRecordDateMax(),
                    nameSampleRecordDateMax());
   values.push_back(SampleRecordDateMax);
  }

 if ( prop.GetId() ==   idSampleRecordNumberMin()) {
   SampleRecordNumberMin  = new PropertyIntValue(prop.GetId(),
                    prop.GetResetvalue(),
                  defaultSampleRecordNumberMin(),
                  nameSampleRecordNumberMin(),
                  minCatalogRecord(),
                  maxCatalogRecord(),
                  signedCatalogRecord());
   values.push_back(SampleRecordNumberMin);
  }
 if ( prop.GetId() ==   idSampleRecordNumberMax()) {
   SampleRecordNumberMax  = new PropertyIntValue(prop.GetId(),
                    prop.GetResetvalue(),
                  defaultSampleRecordNumberMin(),
                  nameSampleRecordNumberMax(),
                  minCatalogRecord(),
                  maxCatalogRecord(),
                  signedCatalogRecord());
   values.push_back(SampleRecordNumberMax);
  }
//------------------------------------ Hipocenter ----------------------------------
 
 if ( prop.GetId() ==   idSampleIsHipo()) {
   SampleHipoIs = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultSampleIsHipo(),
                    nameSampleIsHipo(),
                    enumPriorityList());
   values.push_back(SampleHipoIs);
  }

 if ( prop.GetId() ==   idSampleHipoType()) {
   SampleHipoType = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultSampleHipoType(),
                    nameSampleHipoType(),
                    enumSampleHipoType());
   values.push_back(SampleHipoType);
  }
//------------------------------------ Rectangle Hipocenter --------------------------
 if ( prop.GetId() ==   idSampleHipoRectLatitudeMin()) {
   SampleHipoRectLatitudeMin  = new PropertyFloatValue( prop.GetId(),
                    prop.GetResetvalue(),
                defaultSampleHipoRectLatitudeMin(),
                nameSampleHipoRectLatitudeMin(),
                minCatalogLatitude(),
                maxCatalogLatitude(),
                decimalCatalogLatitude());
   values.push_back(SampleHipoRectLatitudeMin);
  }

 if ( prop.GetId() ==   idSampleHipoRectLatitudeMax()) {
   SampleHipoRectLatitudeMax  = new PropertyFloatValue( prop.GetId(),
                    prop.GetResetvalue(),
                defaultSampleHipoRectLatitudeMax(),
                nameSampleHipoRectLatitudeMax(),
                minCatalogLatitude(),
                maxCatalogLatitude(),
                decimalCatalogLatitude());
   values.push_back(SampleHipoRectLatitudeMax);
  }

  if ( prop.GetId() ==   idSampleHipoRectLongitudeMin()) {
   SampleHipoRectLongitudeMin  = new PropertyFloatValue( prop.GetId(),
                    prop.GetResetvalue(),
                defaultSampleHipoRectLongitudeMin(),
                nameSampleHipoRectLongitudeMin(),
                minCatalogLongitude(),
                maxCatalogLongitude(),
                decimalCatalogLongitude());
   values.push_back(SampleHipoRectLongitudeMin);
  }

  if ( prop.GetId() ==   idSampleHipoRectLongitudeMax()) {
   SampleHipoRectLongitudeMax  = new PropertyFloatValue( prop.GetId(),
                    prop.GetResetvalue(),
                defaultSampleHipoRectLongitudeMax(),
                nameSampleHipoRectLongitudeMax(),
                minCatalogLongitude(),
                maxCatalogLongitude(),
                decimalCatalogLongitude());
   values.push_back(SampleHipoRectLongitudeMax);
  }

  if ( prop.GetId() ==   idSampleHipoRectMinus180()) {
   SampleHipoRectMinus180 = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultSampleHipoRectMinus180(),
                    nameSampleHipoRectMinus180(),
                    enumPriorityList());
   values.push_back(SampleHipoRectMinus180);
  }
//------------------------------------ Circle Hipocenter --------------------------
 
  if ( prop.GetId() ==   idSampleHipoCircleLatitude()) {
   SampleHipoCircleLatitude  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleHipoCircleLatitude(),
                nameSampleHipoCircleLatitude(),
                minCatalogLatitude(),
                maxCatalogLatitude(),
                decimalCatalogLatitude());
   values.push_back(SampleHipoCircleLatitude);
  }

  if ( prop.GetId() ==   idSampleHipoCircleLongitude()) {
   SampleHipoCircleLongitude  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleHipoCircleLongitude(),
                nameSampleHipoCircleLongitude(),
                minCatalogLongitude(),
                maxCatalogLongitude(),
                decimalCatalogLongitude());
   values.push_back(SampleHipoCircleLongitude);
  }

  if ( prop.GetId() ==   idSampleHipoCircleRadius()) {
   SampleHipoCircleRadius  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleHipoCircleRadius(),
                nameSampleHipoCircleRadius(),
                minSampleHipoCircleRadius(),
                maxSampleHipoCircleRadius(),
                decimalSampleHipoCircleRadius());
   values.push_back(SampleHipoCircleRadius);
  }
//------------------------------------ Polygon Hipocenter --------------------------

  if ( prop.GetId() ==   idSampleHipoPolygonVertice()) {
   SampleHipoPolygonVertice  = new PropertyIntValue(prop.GetId(),
                prop.GetResetvalue(),
                  defaultSampleHipoPolygonVertice(),
                  nameSampleHipoPolygonVertice(),
                  minSampleHipoPolygonVertice(),
                  maxSampleHipoPolygonVertice(),
                  signedSampleHipoPolygonVertice());
   values.push_back(SampleHipoPolygonVertice);
  }

  if ( prop.GetId() ==   idSampleHipoPolygonMinus180()) {
   SampleHipoPolygonMinus180 = new PropertyEnumValue(prop.GetId(),
                    prop.GetResetvalue(),
                    defaultSampleHipoPolygonMinus180(),
                    nameSampleHipoPolygonMinus180(),
                    enumPriorityList());
   values.push_back(SampleHipoPolygonMinus180);
  }

  if ( prop.GetId() ==  idSampleListLatitude()) {
   SetPropertyStringListValue(SampleListLatitude,idSampleListLatitude(),
                             prop.GetResetvalue(),defaultSampleListLatitude(),
                             nameSampleListLatitude());
}

  if ( prop.GetId() ==   idSampleListLongitude()) {
   SetPropertyStringListValue(SampleListLongitude,idSampleListLongitude(),
                             prop.GetResetvalue(),defaultSampleListLongitude(),
                             nameSampleListLongitude());
}


//------------------------------ Depth Hipocenter --------------------------------------
  if ( prop.GetId() ==   idSampleIsDepth()) {
   SampleDepthIs = new PropertyEnumValue(prop.GetId(),
                prop.GetResetvalue(),
                    defaultSampleIsDepth(),
                    nameSampleIsDepth(),
                    enumPriorityList());
   values.push_back(SampleDepthIs);
  } 

  if ( prop.GetId() ==   idSampleDepthMin()) {
   SampleDepthMin  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleDepthMin(),
                nameSampleDepthMin(),
                minCatalogDepth(),
                maxCatalogDepth(),
                decimalCatalogDepth());
   values.push_back(SampleDepthMin);
  } 

  if ( prop.GetId() ==   idSampleDepthMax()) {
   SampleDepthMax  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleDepthMax(),
                nameSampleDepthMax(),
                minCatalogDepth(),
                maxCatalogDepth(),
                decimalCatalogDepth());
   values.push_back(SampleDepthMax);
  }

//------------------------------ Magnitude Mb --------------------------------------
  if ( prop.GetId() ==   idSampleIsMb()) {
   SampleMbIs = new PropertyEnumValue(prop.GetId(),
                prop.GetResetvalue(),
                    defaultSampleIsMb(),
                    nameSampleIsMb(),
                    enumPriorityList());
   values.push_back(SampleMbIs);
  }

  if ( prop.GetId() ==   idSampleMbMin()) {
   SampleMbMin  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleMbMin(),
                nameSampleMbMin(),
                minCatalogMb(),
                maxCatalogMb(),
                decimalCatalogMb());
   values.push_back(SampleMbMin);
  }
  if ( prop.GetId() ==   idSampleMbMax()) {
   SampleMbMax  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleMbMax(),
                nameSampleMbMax(),
                minCatalogMb(),
                maxCatalogMb(),
                decimalCatalogMb());
   values.push_back(SampleMbMax);
  }
//------------------------------ Magnitude Ms --------------------------------------

  if ( prop.GetId() ==   idSampleIsMs()) {
   SampleMsIs = new PropertyEnumValue(prop.GetId(),
                prop.GetResetvalue(),
                    defaultSampleIsMs(),
                    nameSampleIsMs(),
                    enumPriorityList());
   values.push_back(SampleMsIs);
  }

  if ( prop.GetId() ==   idSampleMsMin()) {
   SampleMsMin  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleMsMin(),
                nameSampleMsMin(),
                minCatalogMs(),
                maxCatalogMs(),
                decimalCatalogMs());
   values.push_back(SampleMsMin);
  }
  if ( prop.GetId() ==   idSampleMsMax()) {
   SampleMsMax  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleMsMax(),
                nameSampleMsMax(),
                minCatalogMs(),
                maxCatalogMs(),
                decimalCatalogMs());
   values.push_back(SampleMsMax);
  }
//------------------------------ Magnitude Ml --------------------------------------

  if ( prop.GetId() ==   idSampleIsMl()) {
   SampleMlIs = new PropertyEnumValue(prop.GetId(),
                prop.GetResetvalue(),
                    defaultSampleIsMl(),
                    nameSampleIsMl(),
                    enumPriorityList());
   values.push_back(SampleMlIs);
  }

  if ( prop.GetId() ==   idSampleMlMin()) {
   SampleMlMin  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleMlMin(),
                nameSampleMlMin(),
                minCatalogMl(),
                maxCatalogMl(),
                decimalCatalogMl());
   values.push_back(SampleMlMin);
  }
  if ( prop.GetId() ==   idSampleMlMax()) {
   SampleMlMax  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleMlMax(),
                nameSampleMlMax(),
                minCatalogMl(),
                maxCatalogMl(),
                decimalCatalogMl());
   values.push_back(SampleMlMax);
  }

//------------------------------ Magnitude Mp --------------------------------------

  if ( prop.GetId() ==   idSampleIsMp()) {
   SampleMpIs = new PropertyEnumValue(prop.GetId(),
                prop.GetResetvalue(),
                    defaultSampleIsMp(),
                    nameSampleIsMp(),
                    enumPriorityList());
   values.push_back(SampleMpIs);
  }

  if ( prop.GetId() ==   idSampleMpMin()) {
   SampleMpMin  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleMpMin(),
                nameSampleMpMin(),
                minCatalogMp(),
                maxCatalogMp(),
                decimalCatalogMp());
   values.push_back(SampleMpMin);
  }
  if ( prop.GetId() ==   idSampleMpMax()) {
   SampleMpMax  = new PropertyFloatValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleMpMax(),
                nameSampleMpMax(),
                minCatalogMp(),
                maxCatalogMp(),
                decimalCatalogMp());
   values.push_back(SampleMpMax);
  }
//------------------------------ Intensity --------------------------------------
  if ( prop.GetId() ==   idSampleIsIntensity()) {
   SampleIntensityIs = new PropertyEnumValue(prop.GetId(),
                prop.GetResetvalue(),
                    defaultSampleIsIntensity(),
                    nameSampleIsIntensity(),
                    enumPriorityList());
   values.push_back(SampleIntensityIs);
  }

  if ( prop.GetId() ==   idSampleIntensityMin()) {
   SampleIntensityMin  = new PropertyIntValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleIntensityMin(),
                nameSampleIntensityMin(),
                minCatalogIntensity(),
                maxCatalogIntensity(),
                signedCatalogIntensity());
   values.push_back(SampleIntensityMin);
  }
  if ( prop.GetId() ==   idSampleIntensityMax()) {
   SampleIntensityMax  = new PropertyIntValue(prop.GetId(),
                prop.GetResetvalue(),
                defaultSampleIntensityMax(),
                nameSampleIntensityMax(),
                minCatalogIntensity(),
                maxCatalogIntensity(),
                signedCatalogIntensity());
   values.push_back(SampleIntensityMax);
  }
//----------------------------------------------------
if ( prop.GetId() ==   idSampleIsMagFunction())
SetPropertyEnumValue(SampleMagnitudeIs,idSampleIsMagFunction(),
                     prop.GetResetvalue(),defaultSampleIsMagFunction(),
                     nameSampleIsMagFunction(),enumPriorityList());

if ( prop.GetId() ==   idSampleMagFunctionType())
SetPropertyEnumValue(SampleMagnitudeType,idSampleMagFunctionType(),
                     prop.GetResetvalue(),defaultSampleMagFunctionType(),
                     nameSampleMagFunctionType(),enumPriorityList());

if ( prop.GetId() ==   idSampleMagFunctionMin())
SetPropertyFloatValue(SampleMagFunctionMin,idSampleMagFunctionMin(),
                      prop.GetResetvalue(),defaultSampleMagFunctionMin(),
                      nameSampleMagFunctionMin(),minCatalogMb(),
                      maxCatalogMb(),decimalCatalogMb());

if ( prop.GetId() ==   idSampleMagFunctionMax())
SetPropertyFloatValue(SampleMagFunctionMax,idSampleMagFunctionMax(),
                      prop.GetResetvalue(),defaultSampleMagFunctionMax(),
                      nameSampleMagFunctionMax(),minCatalogMb(),
                      maxCatalogMb(),decimalCatalogMb());

if ( prop.GetId() ==   idSampleMagFunction())
SetPropertyEnumValue(SampleMagFunction,idSampleMagFunction(),
                     prop.GetResetvalue(),defaultSampleMagFunction(),
                     nameSampleMagFunction(),enumSampleMagFunction());

if ( prop.GetId() ==   idSampleMagFunctionPriorityFirst())
SetPropertyTransformFunction(SampleMagFunctionFirstMagnitude,
                    idSampleMagFunctionPriorityFirst(),
                    prop.GetResetvalue(),defaultMagnitudePriorityFirst(),
                    nameMagnitudePriorityFirst());

if ( prop.GetId() ==   idSampleMagFunctionPrioritySecond())
SetPropertyTransformFunction(SampleMagFunctionSecondMagnitude,
                    idSampleMagFunctionPrioritySecond(),
                    prop.GetResetvalue(),defaultMagnitudePrioritySecond(),
                    nameMagnitudePrioritySecond());

if ( prop.GetId() ==   idSampleMagFunctionPriorityThird())
SetPropertyTransformFunction(SampleMagFunctionThirdMagnitude,
                    idSampleMagFunctionPriorityThird(),
                    prop.GetResetvalue(),defaultMagnitudePriorityThird(),
                    nameMagnitudePriorityThird());

if ( prop.GetId() ==   idSampleMagFunctionPriorityFourth())
SetPropertyTransformFunction(SampleMagFunctionFourthMagnitude,
                    idSampleMagFunctionPriorityFourth(),
                    prop.GetResetvalue(),defaultMagnitudePriorityFourth(),
                    nameMagnitudePriorityFourth());

///////////////////////////////////////////////////////////////////////////////////////////////////
//------------------------------ Transform Magnitude  --------------------------------------
 PropertyEnumValue* pEnum;
 if ( prop.GetId() ==   1001) {
 SetPropertyEnumValue(pEnum,1001,prop.GetResetvalue(),"0","",enumDeltaMagnitudeFunction());
 TransformMagnitudeFunction.append(pEnum);
}
 if ( prop.GetId() ==   1002){
 SetPropertyEnumValue(pEnum,1002,prop.GetResetvalue(),"0","",enumDeltaMagnitudeFunction());
 TransformMagnitudeFunction.append(pEnum);
}
 if ( prop.GetId() ==   1003){
 SetPropertyEnumValue(pEnum,1003,prop.GetResetvalue(),"0","",enumDeltaMagnitudeFunction());
 TransformMagnitudeFunction.append(pEnum);
}
 if ( prop.GetId() ==   1004){
 SetPropertyEnumValue(pEnum,1004,prop.GetResetvalue(),"0","",enumDeltaMagnitudeFunction());
 TransformMagnitudeFunction.append(pEnum);
}
 PropertyTransformFunction* pTrans;
  //-------------------------- MB ---------------------------------------
if ( prop.GetId() ==   1005) {
 SetPropertyTransformFunction(pTrans,1005,prop.GetResetvalue(),"1@0@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMb.append(pTrans);
}
if ( prop.GetId() ==   1006){
 SetPropertyTransformFunction(pTrans,1006,prop.GetResetvalue(),"1@1@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMb.append(pTrans);
}
if ( prop.GetId() ==   1007){
 SetPropertyTransformFunction(pTrans,1007,prop.GetResetvalue(),"1@2@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMb.append(pTrans);
}
if ( prop.GetId() ==   1008){
 SetPropertyTransformFunction(pTrans,1008,prop.GetResetvalue(),"1@3@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMb.append(pTrans);
}
  //-------------------------- MS ---------------------------------------

if ( prop.GetId() ==   1009){
 SetPropertyTransformFunction(pTrans,1009,prop.GetResetvalue(),"1@0@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMs.append(pTrans);
}
if ( prop.GetId() ==   1010){
 SetPropertyTransformFunction(pTrans,1010,prop.GetResetvalue(),"1@1@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMs.append(pTrans);
}
if ( prop.GetId() ==   1011){
 SetPropertyTransformFunction(pTrans,1011,prop.GetResetvalue(),"1@2@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMs.append(pTrans);
}
if ( prop.GetId() ==   1012){
 SetPropertyTransformFunction(pTrans,1012,prop.GetResetvalue(),"1@3@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMs.append(pTrans);
}
   //-------------------------- ML ---------------------------------------

 if ( prop.GetId() ==   1013){
 SetPropertyTransformFunction(pTrans,1013,prop.GetResetvalue(),"1@0@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMl.append(pTrans);
}
 if ( prop.GetId() ==   1014){
 SetPropertyTransformFunction(pTrans,1014,prop.GetResetvalue(),"1@1@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMl.append(pTrans);
}
if ( prop.GetId() ==   1015){
 SetPropertyTransformFunction(pTrans,1015,prop.GetResetvalue(),"1@2@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMl.append(pTrans);
}
if ( prop.GetId() ==   1016){
 SetPropertyTransformFunction(pTrans,1016,prop.GetResetvalue(),"1@3@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMl.append(pTrans);
}
  //-------------------------- MP ---------------------------------------

if ( prop.GetId() ==   1017){
 SetPropertyTransformFunction(pTrans,1017,prop.GetResetvalue(),"1@0@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMp.append(pTrans);
}
if ( prop.GetId() ==   1018){
 SetPropertyTransformFunction(pTrans,1018,prop.GetResetvalue(),"1@1@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMp.append(pTrans);
}
if ( prop.GetId() ==   1019){
 SetPropertyTransformFunction(pTrans,1019,prop.GetResetvalue(),"1@2@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMp.append(pTrans);
}
if ( prop.GetId() ==   1020){
 SetPropertyTransformFunction(pTrans,1020,prop.GetResetvalue(),"1@3@0.0@1.0@0.0@0.0","");
 TransformMagnitudeMp.append(pTrans);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
///---------------------- Project parameters -------------------------------------
if ( prop.GetId() ==   idProjectActionList())
   SetPropertyStringListValue(ProjectActionList,idProjectActionList(),
                             prop.GetResetvalue(),defaultProjectActionList(),
                             projectActionList());

if ( prop.GetId() ==   idProjectCatalogName())
   SetPropertyStringValue(ProjectCatalogName,idProjectCatalogName(),
                          prop.GetResetvalue(),defaultProjectCatalogName(),
                          nameProjectCatalogName());

if ( prop.GetId() ==   idProjectDocumentIn())
  SetPropertyEnumValue(ProjectDocumentIn,idProjectDocumentIn(),
                     prop.GetResetvalue(),defaultProjectDocumentIn(),
                     nameProjectDocumentIn(),enumProjectDocumentIn());

if ( prop.GetId() ==   idProjectActionChoise())
  SetPropertyEnumValue(ProjectActionChoise,idProjectActionChoise(),
                     prop.GetResetvalue(),defaultProjectActionChoise(),
                     nameProjectActionChoise(),enumProjectActionChoise());

} //while (!in.atEnd())
    
}
void FDocument::SetResetvalueToValue() {
   for_each(values.begin(),values.end(),mem_fun(&PropertyValue::SetResetvalueToValue));    
}
void FDocument::SetValueToResetvalue() {
   for_each(values.begin(),values.end(),mem_fun(&PropertyValue::SetValueToResetvalue));    
}
void FDocument::SetDefault() {
   for_each(values.begin(),values.end(),mem_fun(&PropertyValue::SetDefault));    
}
void FDocument::SetInitial() {
   for_each(values.begin(),values.end(),mem_fun(&PropertyValue::SetInitial));    
}
///===============================================================================
///===============================================================================
SettingDocument::~SettingDocument(){
 for_each(values.begin(),values.end(),delAndNullify);

  values.erase(remove(values.begin(),values.end(),
               static_cast<PropertyValue*>(0)),
               values.end());
}
SettingDocument::SettingDocument(){
    QSettings settings("mitp.ru", "EditCatalogSettings");
    settings.beginGroup("/LimitsParameters");
    QString spaceSize   = settings.value("/limits/spacesize",   defaultViewBufferCatalog()).toString();
    WorkSpaceSize = new PropertyIntSpinValue(idViewBufferCatalog(),
                       spaceSize,defaultViewBufferCatalog(),
                       nameViewBufferCatalog(),
                       minViewBufferCatalog() ,
                       maxViewBufferCatalog(),
                       stepViewBufferCatalog());
    values.push_back(WorkSpaceSize);

    QString errorLines   = settings.value("/limits/errorlines", defaultViewBufferError()).toString();
    MaxErrorLines = new PropertyIntSpinValue(idViewBufferError(),
                       errorLines,defaultViewBufferError(),
                       nameViewBufferError(),
                       minViewBufferError() ,
                       maxViewBufferError(),
                       stepViewBufferError());
    values.push_back(MaxErrorLines);


    QString correctLines   = settings.value("/limits/correctlines", defaultMaxLinesErrorsCorrectonPage()).toString();
    MaxCorrectionLines = new PropertyIntSpinValue(idMaxLinesErrorsCorrectonPage(),
                       correctLines,defaultMaxLinesErrorsCorrectonPage(),
                       nameMaxLinesErrorsCorrectonPage(),
                       minMaxLinesErrorsCorrectonPage() ,
                       maxMaxLinesErrorsCorrectonPage(),
                       stepMaxLinesErrorsCorrectonPage());
    values.push_back(MaxCorrectionLines);
    settings.endGroup();
	
///-----------------------------------  Histogram Parameters ----------------------------
    settings.beginGroup("/HistogramParameters");
	QString numInterval  = settings.value("/numberintervals/axisX", defaultNumberIntervalAxisX()).toString();
    NumberIntervalAxisX = new PropertyIntSpinValue(idNumberIntervalAxisX(),
                       numInterval,defaultNumberIntervalAxisX(),
                       nameNumberIntervalAxisX(),
                       minNumberIntervalAxisX() ,
                       maxNumberIntervalAxisX(),
                       stepNumberIntervalAxisX());
    values.push_back(NumberIntervalAxisX);

	numInterval  = settings.value("/numberintervals/axisY", defaultNumberIntervalAxisY()).toString();
    NumberIntervalAxisY = new PropertyIntSpinValue(idNumberIntervalAxisY(),
                       numInterval,defaultNumberIntervalAxisY(),
                       nameNumberIntervalAxisY(),
                       minNumberIntervalAxisY() ,
                       maxNumberIntervalAxisY(),
                       stepNumberIntervalAxisY());
    values.push_back(NumberIntervalAxisY);

	numInterval  = settings.value("/numberintervals/axisZ", defaultNumberIntervalAxisZ()).toString();
    NumberIntervalAxisZ = new PropertyIntSpinValue(idNumberIntervalAxisZ(),
                       numInterval,defaultNumberIntervalAxisZ(),
                       nameNumberIntervalAxisZ(),
                       minNumberIntervalAxisZ() ,
                       maxNumberIntervalAxisZ(),
                       stepNumberIntervalAxisZ());
    values.push_back(NumberIntervalAxisZ);
	
	settings.endGroup();
/*

    settings.beginGroup("/RangeParameters");
    QString min   = settings.value("/range/minLatitude", defaultMinimumPropertyLatitude()).toString();
    MinLatitude = new PropertyFloatValue(idMinimumPropertyLatitude(),
                       min,defaultMinimumPropertyLatitude(),
                       nameMinimumPropertyLatitude(),
                       minMinimumPropertyLatitude(),
                       maxMinimumPropertyLatitude(),
                       decimalMinimumPropertyLatitude());
    values.push_back(MinLatitude);

    QString max   = settings.value("/range/maxLatitude", defaultMaximumPropertyLatitude()).toString();
    MaxLatitude = new PropertyFloatValue(idMaximumPropertyLatitude(),
                       max,defaultMaximumPropertyLatitude(),
                       nameMaximumPropertyLatitude(),
                       minMaximumPropertyLatitude(),
                       maxMaximumPropertyLatitude(),
                       decimalMaximumPropertyLatitude());
    values.push_back(MaxLatitude);

    min   = settings.value("/range/minLongitude", defaultMinimumPropertyLongitude()).toString();
    MinLongitude = new PropertyFloatValue(idMinimumPropertyLongitude(),
                       min,defaultMinimumPropertyLongitude(),
                       nameMinimumPropertyLongitude(),
                       minMinimumPropertyLongitude(),
                       maxMinimumPropertyLongitude(),
                       decimalMinimumPropertyLongitude());
    values.push_back(MinLongitude);

    max   = settings.value("/range/maxLongitude", defaultMaximumPropertyLongitude()).toString();
    MaxLongitude = new PropertyFloatValue(idMaximumPropertyLongitude(),
                       max,defaultMaximumPropertyLongitude(),
                       nameMaximumPropertyLongitude(),
                       minMaximumPropertyLongitude(),
                       maxMaximumPropertyLongitude(),
                       decimalMaximumPropertyLongitude());
    values.push_back(MaxLongitude);

   min   = settings.value("/range/minDepth", defaultMinimumPropertyDepth()).toString();
    MinDepth = new PropertyFloatValue(idMinimumPropertyDepth(),
                       min,defaultMinimumPropertyDepth(),
                       nameMinimumPropertyDepth(),
                       minMinimumPropertyDepth(),
                       maxMinimumPropertyDepth(),
                       decimalMinimumPropertyDepth());
    values.push_back(MinDepth);

    max   = settings.value("/range/maxDepth", defaultMaximumPropertyDepth()).toString();
    MaxDepth = new PropertyFloatValue(idMaximumPropertyDepth(),
                       max,defaultMaximumPropertyDepth(),
                       nameMaximumPropertyDepth(),
                       minMaximumPropertyDepth(),
                       maxMaximumPropertyDepth(),
                       decimalMaximumPropertyDepth());
    values.push_back(MaxDepth);

    min   = settings.value("/range/minMb", defaultMinimumPropertyMb()).toString();
    MinMb = new PropertyFloatValue(idMinimumPropertyMb(),
                       min,defaultMinimumPropertyMb(),
                       nameMinimumPropertyMb(),
                       minMinimumPropertyMb(),
                       maxMinimumPropertyMb(),
                       decimalMinimumPropertyMb());
    values.push_back(MinMb);

    max   = settings.value("/range/maxMb", defaultMaximumPropertyMb()).toString();
    MaxMb = new PropertyFloatValue(idMaximumPropertyMb(),
                       max,defaultMaximumPropertyMb(),
                       nameMaximumPropertyMb(),
                       minMaximumPropertyMb(),
                       maxMaximumPropertyMb(),
                       decimalMaximumPropertyMb());
    values.push_back(MaxMb);

    min   = settings.value("/range/minMs", defaultMinimumPropertyMs()).toString();
    MinMs = new PropertyFloatValue(idMinimumPropertyMs(),
                       min,defaultMinimumPropertyMs(),
                       nameMinimumPropertyMs(),
                       minMinimumPropertyMs(),
                       maxMinimumPropertyMs(),
                       decimalMinimumPropertyMs());
    values.push_back(MinMs);

    max   = settings.value("/range/maxMs", defaultMaximumPropertyMs()).toString();
    MaxMs = new PropertyFloatValue(idMaximumPropertyMs(),
                       max,defaultMaximumPropertyMs(),
                       nameMaximumPropertyMs(),
                       minMaximumPropertyMs(),
                       maxMaximumPropertyMs(),
                       decimalMaximumPropertyMs());
    values.push_back(MaxMs);

    min   = settings.value("/range/minMl", defaultMinimumPropertyMl()).toString();
    MinMl = new PropertyFloatValue(idMinimumPropertyMl(),
                       min,defaultMinimumPropertyMl(),
                       nameMinimumPropertyMl(),
                       minMinimumPropertyMl(),
                       maxMinimumPropertyMl(),
                       decimalMinimumPropertyMl());
    values.push_back(MinMl);

    max   = settings.value("/range/maxMl", defaultMaximumPropertyMl()).toString();
    MaxMl = new PropertyFloatValue(idMaximumPropertyMl(),
                       max,defaultMaximumPropertyMl(),
                       nameMaximumPropertyMl(),
                       minMaximumPropertyMl(),
                       maxMaximumPropertyMl(),
                       decimalMaximumPropertyMl());
    values.push_back(MaxMl);

    min   = settings.value("/range/minMp", defaultMinimumPropertyMp()).toString();
    MinMp = new PropertyFloatValue(idMinimumPropertyMp(),
                       min,defaultMinimumPropertyMp(),
                       nameMinimumPropertyMp(),
                       minMinimumPropertyMp(),
                       maxMinimumPropertyMp(),
                       decimalMinimumPropertyMp());
    values.push_back(MinMp);

    max   = settings.value("/range/maxMp", defaultMaximumPropertyMp()).toString();
    MaxMp = new PropertyFloatValue(idMaximumPropertyMp(),
                       max,defaultMaximumPropertyMp(),
                       nameMaximumPropertyMp(),
                       minMaximumPropertyMp(),
                       maxMaximumPropertyMp(),
                       decimalMaximumPropertyMp());
    values.push_back(MaxMp);

    settings.endGroup();
*/
}
void SettingDocument::read(){
    QSettings settings("mitp.ru", "EditCatalogSettings");
    settings.beginGroup("/LimitsParameters");
	QString tmp = "";
	tmp = settings.value("/limits/spacesize",WorkSpaceSize->GetDefaultvalue()).toString();
	WorkSpaceSize->SetValue(tmp);
	tmp = settings.value("/limits/errorlines",MaxErrorLines->GetDefaultvalue()).toString();
	MaxErrorLines->SetValue(tmp);
	tmp = settings.value("/limits/correctlines",MaxCorrectionLines->GetDefaultvalue()).toString();
	MaxCorrectionLines->SetValue(tmp);
	settings.endGroup();
	
	settings.beginGroup("/HistogramParameters");
	
	tmp =settings.value("/numberintervals/axisX", defaultNumberIntervalAxisX()).toString();
	NumberIntervalAxisX->SetValue(tmp);
	
	tmp =settings.value("/numberintervals/axisY", defaultNumberIntervalAxisY()).toString();
	NumberIntervalAxisY->SetValue(tmp);
	
	tmp =settings.value("/numberintervals/axisZ", defaultNumberIntervalAxisZ()).toString();
	NumberIntervalAxisZ->SetValue(tmp);

	settings.endGroup();
}
void SettingDocument::write(){

    QSettings settings("mitp.ru", "EditCatalogSettings");

    settings.beginGroup("/LimitsParameters");

    settings.setValue("/limits/spacesize",     WorkSpaceSize->GetValue());
    settings.setValue("/limits/errorlines",    MaxErrorLines->GetValue());
    settings.setValue("/limits/correctlines",  MaxCorrectionLines->GetValue());

    settings.endGroup();
	
	settings.beginGroup("/HistogramParameters");

	settings.setValue("/numberintervals/axisX", NumberIntervalAxisX->GetValue());
	settings.setValue("/numberintervals/axisY", NumberIntervalAxisY->GetValue());
	settings.setValue("/numberintervals/axisZ", NumberIntervalAxisZ->GetValue());

	settings.endGroup();
	
/*
    settings.beginGroup("/RangeParameters");

    settings.setValue("/range/minLatitude",  MinLatitude->GetValue());
    settings.setValue("/range/maxLatitude",  MaxLatitude->GetValue());
    settings.setValue("/range/minLongitude", MinLongitude->GetValue());
    settings.setValue("/range/maxLongitude", MaxLongitude->GetValue());
    settings.setValue("/range/minDepth",     MinDepth->GetValue());
    settings.setValue("/range/maxDepth",     MaxDepth->GetValue());
    settings.setValue("/range/minMb",        MinMb->GetValue());
    settings.setValue("/range/maxMb",        MaxMb->GetValue());
    settings.setValue("/range/minMs",        MinMs->GetValue());
    settings.setValue("/range/maxMs",        MaxMs->GetValue());
    settings.setValue("/range/minMl",        MinMl->GetValue());
    settings.setValue("/range/maxMl",        MaxMl->GetValue());
    settings.setValue("/range/minMp",        MinMp->GetValue());
    settings.setValue("/range/maxMp",        MaxMp->GetValue());

    settings.endGroup();
*/
}

bool SettingDocument::isModified(){
bool mod = false;
PropValIterator i = values.begin();
  while (i != values.end()) {
      if((*i)->GetId() == 500 || (*i)->GetId() == 503) {i++;continue;}
      if( (*i)->GetValue() != (*i)->GetResetvalue()) {mod = true; (*i)->setChanged(true);  }
      else (*i)->setChanged(false);
      i++;
  }
return mod;

}
