#ifndef _ECP_CONFIG_H
#define _ECP_CONFIG_H
#include <QPolygon>
#include <QList>
#include <list>
#include <string>
#include <vector>
#include <map>
#include <set>
#include <iostream>
#include <fstream>
#include <functional>
#include <algorithm>
using namespace std;

class QDataStream;
class  QFile;
class PropertyValue; 
void delAndNullify(PropertyValue* &pPropVal);
class RowValue 
{ 
 
public:
    RowValue();
    RowValue(int id,QString name);
    RowValue(int id,QString name,QString textcol1,QString df);
    virtual     ~RowValue();
    const int GetId                               () const        { return id;                     } 
    const bool GetMarkDelete          () const        { return markDelete;    } 
    const QString GetName              () const        { return name;               } 
    const QString GetTextCol1         () const        { return textCol1;          }
    const QString GetDfile                 () const        { return  dfile;                }
    void SetId                      (const int i              )     { id = i;                           } 
    void SetMarkDelete     (const bool i              )     { markDelete = i;          } 
    void SetName               (const QString val)     { name = val;                } 
    void SetTextCol1           (const QString val)    {  textCol1 = val;          } 
    void SetDfile                  (const QString val)     { dfile = val;                  } 
private:  
    int id;
    QString name; 
    QString textCol1; 
    QString dfile; 
    bool markDelete; // false - Include   true- Exclude
};
//---------------------------------------------------------------------------------------------
class PropertyValue 
{
    friend QDataStream &operator<< (QDataStream &, const PropertyValue &); 
    friend QDataStream &operator >> (QDataStream &, PropertyValue &); 

public:
    enum Kind
    {
        Property_Bool,
        Property_Int,
        Property_IntSpin,
        Property_Float,
        Property_Enum,
        Property_TransformFunction,
        Property_DateTime,
        Property_String,
        Property_StringList
    };
    PropertyValue(){};
    PropertyValue(const int id,const QString defval,const QString name);
    PropertyValue(const int id,const QString resval,const QString defval,const QString name);
    PropertyValue(const int id,const QString val,const QString resval,const QString defval,const QString name);  
    virtual     ~PropertyValue();
    virtual void setChanged( bool b);
    PropertyValue::Kind GetKind() {return kind;}
    bool isChanged() const;
    bool isReseted() const;
    void SetValueToResetvalue       ()  { resetvalue = value;               }
    void SetValueToDefaultvalue     ()  { defaultvalue = value;             }
    void SetInitial                 ()  { value         =  initialvalue;    }
    void setChanged                 ()  { if (value  ==  initialvalue)   setChanged( FALSE);
                                           else setChanged( TRUE);}
    const int GetId                 () const       { return id;              } 
    const QString GetName           () const       { return name;            } 
    const QString GetInitialvalue   () const       { return initialvalue;    } 
    const QString GetDefaultvalue   () const       { return defaultvalue;    }  
    const QString GetResetvalue     ()  const      { return resetvalue;      }
    const QString GetValue          ()  const      { return value;           }
    void SetId             (const int i      )     { id = i;                 } 
    void SetName           (const QString val)     { name = val;             } 
    void SetResetvalue     (const QString val)     { resetvalue = val;       }
//---------------------------------------------------------------------------------------
    virtual void SetIntialvalue    (const QString ){}
    virtual void SetValue          (const QString ){}
    virtual void SetResetvalueToValue       () {}
    virtual void SetDefault                 () {}
//----------------------------------------------------------------------------------------
protected:
    int id;
    QString value;
    QString initialvalue;
    QString resetvalue;
    QString defaultvalue;
    QString name;
    PropertyValue::Kind kind;
private:
    bool changed;

};
//---------------------------------------------------------------------------------------------
class PropertyBoolValue : public PropertyValue
{
 
public:
    PropertyBoolValue( const int id,const QString defval ,const QString name);
    PropertyBoolValue(const int id,const  QString resval, const QString defval ,const QString name);
    PropertyBoolValue( const int id,const QString val,const  QString resval,const QString defval,const QString name);    
	    ~PropertyBoolValue(){};
    void SetIntialvalue    (const QString val)     { initialvalue= val;      } 
    void SetValue          (const QString val)     { value = val;            }
    void SetResetvalueToValue       ()  { value         = resetvalue ;      }
    void SetDefault                 ()  { value         =  defaultvalue ;   }
private:
  
};
//---------------------------------------------------------------------------------------------
class PropertyIntValue : public PropertyValue
{
 
public:
    PropertyIntValue( const int id,const QString defval ,const QString name,const int min,const int max, const bool s);
    PropertyIntValue( const int id,const  QString resval,const QString defval ,const QString name,const int min,const int max, const bool s);
    PropertyIntValue( const int id, const  QString val, const  QString resval,const QString defval,const QString name,const int min,const int max, const bool s);    
	    ~PropertyIntValue(){};
    const int  GetMin()          const     { return min;}
    const int  GetMax()          const     { return max;}
    const bool GetSignedValue() const     { return   signedValue;}

    void SetIntialvalue    (const QString val)     { initialvalue= val;      } 
    void SetValue          (const QString val)     { value = val;            }
    void SetResetvalueToValue       ()  { value         = resetvalue ;      }
    void SetDefault                 ()  { value         =  defaultvalue ;   }
private:
int min;
int max;
bool signedValue;
};
//---------------------------------------------------------------------------------------------
class PropertyIntSpinValue : public PropertyValue
{

public:
    PropertyIntSpinValue( const int id,const QString defval ,const QString name,const int min,const int max, const int s);
    PropertyIntSpinValue( const int id,const  QString resval,const QString defval ,const QString name,const int min,const int max, const int s);
    PropertyIntSpinValue( const int id, const  QString val, const  QString resval,const QString defval,const QString name,const int min,const int max, const int s);
	    ~PropertyIntSpinValue(){};
    const int  GetMin()          const     { return min;}
    const int  GetMax()          const     { return max;}
    const int GetStep()         const     { return  step;}
    void SetIntialvalue    (const QString val)     { initialvalue= val;      } 
    void SetValue          (const QString val)     { value = val;            }
    void SetResetvalueToValue       ()  { value         = resetvalue ;      }
    void SetDefault                 ()  { value         =  defaultvalue ;   }

private:
int min;
int max;
int step;
};
//---------------------------------------------------------------------------------------------
class PropertyFloatValue : public PropertyValue
{
 
public:
    PropertyFloatValue(const int id, const QString defval ,const QString name,const float min,const float max, const int dec);
    PropertyFloatValue(const int id, const  QString resval,const QString defval ,const QString name,const float min,const float max, const int dec);
    PropertyFloatValue(const int id, const  QString val, const  QString resval,const QString defval,const QString name,const float min,const float max, const int dec);    
	    ~PropertyFloatValue(){};
    const float GetMin()           const     { return min;}
    const float GetMax()          const     { return max;}
    void SetMin(float val)        { min = val;}
    void SetMax(float val)        { max = val;}
    const int GetDecimal()     const     { return  decimal;}
    void SetIntialvalue    (const QString val)     { initialvalue= val;      } 
    void SetValue          (const QString val)     { value = val;            }
    void SetResetvalueToValue       ()  { value         = resetvalue ;      }
    void SetDefault                 ()  { value         =  defaultvalue ;   }
    void SetDefault         (QString val) { defaultvalue  =  val; }
private:
float min;
float max;
int decimal;
};
//---------------------------------------------------------------------------------------------
class PropertyStringListValue : public PropertyValue
{
public:
    PropertyStringListValue(const int id, const QString defval ,const QString name);
    PropertyStringListValue(const int id, const  QString resval,const QString defval ,const QString name);
    PropertyStringListValue(const int id, const QString val,const QString resval,const QString defval,const QString name);    
    ~PropertyStringListValue() {list.clear();}
    void SetValue  (const QString val);
    void SetResetvalueToValue       ();
    void SetDefault                 ();
    void SetInitial                 ();
    QStringList &GetList   ()       { return list;}
    QStringList GetStringList ()    { return list;}
    void SetListToValue(QStringList &list);
private:
    QStringList list; 

};

//---------------------------------------------------------------------------------------------
class PropertyStringValue : public PropertyValue
{
public:
    PropertyStringValue(const int id, const QString defval ,const QString name);
    PropertyStringValue(const int id, const  QString resval,const QString defval ,const QString name);
    PropertyStringValue(const int id, const QString val,const QString resval,const QString defval,const QString name);    
    ~PropertyStringValue(){};

    void SetIntialvalue    (const QString val)     { initialvalue= val;      } 
    void SetValue          (const QString val)     { value = val;            }
    void SetResetvalueToValue       ()  { value         = resetvalue ;      }
    void SetDefault                 ()  { value         =  defaultvalue ;   }
private:


};

//---------------------------------------------------------------------------------------------
class PropertyEnumValue : public PropertyValue
{
public:
    PropertyEnumValue(const int id, const QString defval ,const QString name,const QString list);
    PropertyEnumValue(const int id, const  QString resval,const QString defval ,const QString name,const QString list);
    PropertyEnumValue(const int id, const QString val,const QString resval,const QString defval,const QString name,const QString list);    
    ~PropertyEnumValue(){list.clear();}
    QStringList &GetList   ()       { return list;}
    QStringList GetStringList ()    { return list;}
    void SetIntialvalue    (const QString val)     { initialvalue= val;      } 
    void SetValue          (const QString val)     { value = val;            }
    void SetResetvalueToValue       ()  { value         = resetvalue ;      }
    void SetDefault                 ()  { value         =  defaultvalue ;   }
private:
    QStringList list; 

};
//---------------------------------------------------------------------------------------------
class PropertyTransformFunction : public PropertyValue
{

public:
    PropertyTransformFunction(const int id, const QString defval ,const QString name);
    PropertyTransformFunction(const int id, const  QString resval,const QString defval ,const QString name);
    PropertyTransformFunction(const int id, const  QString val, const  QString resval,const QString defval,const QString name);
    ~PropertyTransformFunction(){};

    void SetValue  (const QString val)  { value = val;             SetVal(value);}
    void SetResetvalueToValue       ()  { value   =  resetvalue;   SetVal(value);}
    void SetDefault                 ()  { value   =  defaultvalue; SetVal(value);}
    void SetInitial                 ()  { value   =  initialvalue; SetVal(value);}
    void SetVal(const QString val);
    void SetDef(const QString val);
    void SetValue(PropertyTransformFunction* val);
    float GetMagnitude() { return  magnitude;}
    float GetMagnitude(float m);
    void  GetM(float inm,float &outm);
    void  SetMagnitude(float m);
    bool  valIsOn,defIsOn;
    int   valIndexMagnitude,defIndexMagnitude;
    QString valA,valB,valC,valD,defA,defB,defC,defD;

private:
float magnitude;
};
//---------------------------------------------------------------------------------------------
class PropertyDateTimeValue : public PropertyValue
{

public:
    PropertyDateTimeValue( const int id,const QString defval ,const QString name);
    PropertyDateTimeValue( const int id,const  QString resval,const QString defval ,const QString name);
    PropertyDateTimeValue( const int id, const  QString val, const  QString resval,const QString defval,const QString name);
	  ~PropertyDateTimeValue(){};
    void SetIntialvalue    (const QString val)     { initialvalue= val;      } 
    void SetValue          (const QString val)     { value = val;            }
    void SetResetvalueToValue       ()  { value         = resetvalue ;      }
    void SetDefault                 ()  { value         =  defaultvalue ;   }
};
//---------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------
class FDocument
{
    typedef vector<PropertyValue* > PropValVector;
    typedef vector<PropertyValue* >::iterator PropValIterator; 
public:
    enum ActionType {
        Value,
        Default,
        Reset,
        Initial
    };
    FDocument();
    FDocument(const QString &fileName);
    FDocument(FDocument *doc);
    ~FDocument(); 
    bool loadBinary(const QString &fileName);
    bool saveBinary(const QString &fileName);   
    bool ReadFileListCtachan();
    bool isModified();
    void SetResetvalueToValue();
    void SetValueToResetvalue();
    void SetDefault();
    void SetInitial();
///-----------------------------------------Check Range ----------------------------------------
PropertyFloatValue*  &GetRangeLatitudeMin  ()   {return RangeLatitudeMin;   }
PropertyFloatValue*  &GetRangeLatitudeMax  ()   {return RangeLatitudeMax;   }
PropertyFloatValue*  &GetRangeLongitudeMin ()   {return RangeLongitudeMin;  }
PropertyFloatValue*  &GetRangeLongitudeMax ()   {return RangeLongitudeMax;  }
PropertyFloatValue*  &GetRangeDepthMin     ()   {return RangeDepthMin;   }
PropertyFloatValue*  &GetRangeDepthMax     ()   {return RangeDepthMax;   }
PropertyFloatValue*  &GetRangeMbMin        ()   {return RangeMbMin;   }
PropertyFloatValue*  &GetRangeMbMax        ()   {return RangeMbMax;   }
PropertyFloatValue*  &GetRangeMsMin        ()   {return RangeMsMin;   }
PropertyFloatValue*  &GetRangeMsMax        ()   {return RangeMsMax;   }
PropertyFloatValue*  &GetRangeMlMin        ()   {return RangeMlMin;   }
PropertyFloatValue*  &GetRangeMlMax        ()   {return RangeMlMax;   }
PropertyFloatValue*  &GetRangeMpMin        ()   {return RangeMpMin;   }
PropertyFloatValue*  &GetRangeMpMax        ()   {return RangeMpMax;   }
    ///-----------------------------------------Identification of doubles: ----------------------------------------    
    PropertyIntValue*    &GetDeltaTime        ()   {return  deltaTime;       }        
    PropertyIntValue*    &GetDeltaDepth       ()   {return  deltaDepth;      }            
    PropertyFloatValue*  &GetDeltaLatitude    ()   {return deltaLatitude;    }      
    PropertyFloatValue*  &GetDeltaLongitude   ()   {return deltaLongitude;   }           
    PropertyFloatValue*  &GetDeltaMagnitude   ()   {return deltaMagnitude;   }
    PropertyEnumValue*   &GetDeltaRemoveAction()   {return DeltaRemoveAction;}
    PropertyEnumValue*   &GetDeltaRemoveEqual()    {return DeltaRemoveEqual;}

    PropertyEnumValue* &GetRemovingDuplicateFunction       () {return RemovingDuplicateFunction; }

    PropertyEnumValue* &GetRemovingDuplicatePriorityFirst          () {return RemovingDuplicatePriorityFirst; }
    PropertyEnumValue* &GetRemovingDuplicatePriorityFirstMagnitude () {return RemovingDuplicatePriorityFirstMagnitude; }
    PropertyEnumValue* &GetRemovingDuplicatePrioritySecond         () {return RemovingDuplicatePrioritySecond; }
    PropertyEnumValue* &GetRemovingDuplicatePrioritySecondMagnitude() {return RemovingDuplicatePrioritySecondMagnitude; }
    PropertyEnumValue* &GetRemovingDuplicatePriorityThird          () {return RemovingDuplicatePriorityThird; }
    PropertyEnumValue* &GetRemovingDuplicatePriorityThirdMagnitude () {return RemovingDuplicatePriorityThirdMagnitude; }
    PropertyEnumValue* &GetRemovingDuplicatePriorityFourth         () {return RemovingDuplicatePriorityFourth; }
    PropertyEnumValue* &GetRemovingDuplicatePriorityFourthMagnitude() {return RemovingDuplicatePriorityFourthMagnitude; }

    PropertyEnumValue* &GetDeltaMagnitudeFunction() {return  DeltaMagnitudeFunction; }
    PropertyTransformFunction* &GetDefineDeltaFirstMagnitude()  {return DefineDeltaFirstMagnitude;  }
    PropertyTransformFunction* &GetDefineDeltaSecondMagnitude() {return DefineDeltaSecondMagnitude; } 
    PropertyTransformFunction* &GetDefineDeltaThirdMagnitude()  {return DefineDeltaThirdMagnitude;  }
    PropertyTransformFunction* &GetDefineDeltaFourthMagnitude() {return DefineDeltaFourthMagnitude; }

//-------------------------------  Merge Catalogs -------------------------------------
    PropertyIntValue*    &GetDeltaCatTime        ()   {return  deltaCatTime;       }
    PropertyIntValue*    &GetDeltaCatDepth       ()   {return  deltaCatDepth;      }
    PropertyFloatValue*  &GetDeltaCatLatitude    ()   {return deltaCatLatitude;    }
    PropertyFloatValue*  &GetDeltaCatLongitude   ()   {return deltaCatLongitude;   }
    PropertyFloatValue*  &GetDeltaCatMagnitude   ()   {return deltaCatMagnitude;   }

    PropertyEnumValue* &GetDeltaCatMagnitudeFunction() {return  DeltaCatMagnitudeFunction; }
    PropertyTransformFunction* &GetDefineDeltaCatFirstMagnitude()  {return DefineDeltaCatFirstMagnitude;  }
    PropertyTransformFunction* &GetDefineDeltaCatSecondMagnitude() {return DefineDeltaCatSecondMagnitude; }
    PropertyTransformFunction* &GetDefineDeltaCatThirdMagnitude()  {return DefineDeltaCatThirdMagnitude;  }
    PropertyTransformFunction* &GetDefineDeltaCatFourthMagnitude() {return DefineDeltaCatFourthMagnitude; }

    PropertyEnumValue* &GetAssignDuplicateMagnitude()  {return AssignDuplicateMagnitude;  }
    PropertyEnumValue* &GetSelectedDuplicateMagnitude() {return SelectedDuplicateMagnitude; }
    PropertyEnumValue* &GetRemovingDuplicateMagnitude() {return RemovingDuplicateMagnitude; }
    PropertyStringListValue* &GetListMergeCatalog(){ return ListMergeCatalog;}
    PropertyStringValue*     &GetSecondCatalogName(){ return SecondCatalogName;}
//----------------------------------------- Compare Action  ----------------------------------------
    PropertyEnumValue* &GetCompareCatalogAction(){ return CompareCatalogAction;}
//----------------------------------- ADD Catalogs-----------------------------
    PropertyStringListValue* &GetListAddCatalog(){ return ListAddCatalog;}
//-----------------------------------Create subcatalog -----------------------------
//------------------------------------ Record ----------------------------------
PropertyEnumValue*  &GetSampleRecordIs ()  {return SampleRecordIs; }
PropertyEnumValue*  &GetSampleRecordType ()  {return SampleRecordType; }

PropertyDateTimeValue* &GetSampleRecordDateMin ()  {return SampleRecordDateMin; }
PropertyDateTimeValue* &GetSampleRecordDateMax ()  {return SampleRecordDateMax; }

PropertyIntValue* &GetSampleRecordNumberMin ()  {return SampleRecordNumberMin; }
PropertyIntValue* &GetSampleRecordNumberMax ()  {return SampleRecordNumberMax; }
//------------------------------------ Hipocenter ----------------------------------

PropertyEnumValue*  &GetSampleHipoIs ()  {return SampleHipoIs; }
PropertyEnumValue*  &GetSampleHipoType ()  {return SampleHipoType; }
//------------------------------------ Rectangle Hipocenter --------------------------
PropertyFloatValue* &GetSampleHipoRectLatitudeMin ()  {return SampleHipoRectLatitudeMin;  }
PropertyFloatValue* &GetSampleHipoRectLatitudeMax ()  {return SampleHipoRectLatitudeMax; }
PropertyFloatValue* &GetSampleHipoRectLongitudeMin ()  {return SampleHipoRectLongitudeMin; }
PropertyFloatValue* &GetSampleHipoRectLongitudeMax ()  {return SampleHipoRectLongitudeMax; }
PropertyEnumValue*  &GetSampleHipoRectMinus180 ()  {return SampleHipoRectMinus180; }
PropertyEnumValue*  &GetSampleHipoPolygonMinus180 ()  {return SampleHipoPolygonMinus180; }

//------------------------------------ Circle Hipocenter --------------------------

PropertyFloatValue* &GetSampleHipoCircleLatitude ()  {return SampleHipoCircleLatitude; }
PropertyFloatValue* &GetSampleHipoCircleLongitude ()  {return SampleHipoCircleLongitude; }
PropertyFloatValue* &GetSampleHipoCircleRadius ()  {return SampleHipoCircleRadius; }

//------------------------------------ Polygon Hipocenter --------------------------

PropertyIntValue* &GetSampleHipoPolygonVertice()    {return SampleHipoPolygonVertice; }
PropertyStringListValue*  &GetSampleListLatitude()  {return SampleListLatitude; }
PropertyStringListValue*  &GetSampleListLongitude() {return SampleListLongitude; }

//-----------------------------
PropertyEnumValue*  &GetSampleDepthIs ()  {return SampleDepthIs; }
PropertyFloatValue* &GetSampleDepthMin ()  {return SampleDepthMin; }
PropertyFloatValue* &GetSampleDepthMax ()  {return SampleDepthMax; }
//------------------------------ Magnitude Mb --------------------------------------
PropertyEnumValue*  &GetSampleMbIs ()  {return SampleMbIs; }
PropertyFloatValue* &GetSampleMbMin ()  {return SampleMbMin; }
PropertyFloatValue* &GetSampleMbMax ()  {return SampleMbMax; }
//------------------------------ Magnitude Ms --------------------------------------
PropertyEnumValue*  &GetSampleMsIs ()  {return SampleMsIs;  }
PropertyFloatValue* &GetSampleMsMin ()  {return SampleMsMin; }
PropertyFloatValue* &GetSampleMsMax ()  {return SampleMsMax; }
//------------------------------ Magnitude Ml --------------------------------------
PropertyEnumValue*  &GetSampleMlIs ()  {return SampleMlIs;  }
PropertyFloatValue* &GetSampleMlMin ()  {return SampleMlMin; }
PropertyFloatValue* &GetSampleMlMax ()  {return SampleMlMax; }
//------------------------------ Magnitude Mp --------------------------------------
PropertyEnumValue*  &GetSampleMpIs ()  {return SampleMpIs;  }
PropertyFloatValue* &GetSampleMpMin ()  {return SampleMpMin; }
PropertyFloatValue* &GetSampleMpMax ()  {return SampleMpMax; }
//------------------------------ Intencity --------------------------------------
PropertyEnumValue* &GetSampleIntensityIs ()  {return SampleIntensityIs; }
PropertyIntValue*  &GetSampleIntensityMin ()  {return SampleIntensityMin; }
PropertyIntValue*  &GetSampleIntensityMax ()  {return SampleIntensityMax; }

//------------------------------ Magnitude Fanction --------------------------------------
PropertyEnumValue*  &GetSampleMagnitudeIs ()  {return SampleMagnitudeIs; }
PropertyEnumValue*  &GetSampleMagnitudeType ()  {return SampleMagnitudeType; }

PropertyFloatValue* &GetSampleMagFunctionMin ()  {return SampleMagFunctionMin; }
PropertyFloatValue* &GetSampleMagFunctionMax ()  {return SampleMagFunctionMax; }

PropertyEnumValue* &GetSampleMagFunction ()  {return SampleMagFunction; }

PropertyTransformFunction* &GetSampleMagFunctionFirstMagnitude ()
                           {return SampleMagFunctionFirstMagnitude; }

PropertyTransformFunction* &GetSampleMagFunctionSecondMagnitude ()
                           {return SampleMagFunctionSecondMagnitude; }

PropertyTransformFunction* &GetSampleMagFunctionThirdMagnitude ()
                           {return SampleMagFunctionThirdMagnitude; }

PropertyTransformFunction* &GetSampleMagFunctionFourthMagnitude ()
                          {return SampleMagFunctionFourthMagnitude; }

//------------------------------ Transform Magnitude  --------------------------------------
QList<PropertyEnumValue*> &GetTransformMagnitudeFunction()   {return TransformMagnitudeFunction;}
QList<PropertyTransformFunction *> &GetTransformMagnitudeMb(){return TransformMagnitudeMb;}
QList<PropertyTransformFunction *> &GetTransformMagnitudeMs(){return TransformMagnitudeMs;}
QList<PropertyTransformFunction *> &GetTransformMagnitudeMl(){return TransformMagnitudeMl;}
QList<PropertyTransformFunction *> &GetTransformMagnitudeMp(){return TransformMagnitudeMp;}
///---------------------- Project parameters -------------------------------------
PropertyStringListValue* &GetProjectActionList()   {return ProjectActionList;}
PropertyStringValue*     &GetProjectCatalogName()  {return ProjectCatalogName;}
PropertyEnumValue*       &GetProjectDocumentIn()   {return ProjectDocumentIn;}
PropertyEnumValue*       &GetProjectActionChoise() {return ProjectActionChoise;}

    vector <PropertyValue* >  &GetValues(){return values;}
    bool GetProjectFileGood()  {return projectFileGood; }
   // void printFDocument();
    bool isChanged() const;
    bool CheckModified();
void SetPropertyEnumValue(PropertyEnumValue* &p, int id,QString reset,QString defaultvalue,
                        QString name,QString enumeration);
void SetPropertyIntValue(PropertyIntValue* &p, int id,QString reset,QString defaultvalue,
                        QString name,int min,int max,bool sign);
void SetPropertyFloatValue(PropertyFloatValue* &p, int id,QString reset,QString defaultvalue,
                        QString name,float min,float max,int decimal);
void SetPropertyTransformFunction(PropertyTransformFunction* &p, int id,
                        QString reset,QString defaultvalue, QString name);
void SetPropertyStringValue(PropertyStringValue* &p, int id,
                        QString reset,QString defaultvalue, QString name);
void SetPropertyStringListValue(PropertyStringListValue* &p, int id,
                        QString reset,QString defaultvalue, QString name);
private:
    bool projectFileGood;
    bool changed;
    enum{ MagicNumber = 0x98c58f10 };
    void writeToStream(QDataStream &out);
    void readFromStream(QDataStream &in);
    void error(const QFile &file ,const QString &message);
    void ioError(const QFile &file ,const QString &message);

///-----------------------------------------Check Range ----------------------------------------
PropertyFloatValue*  RangeLatitudeMin;
PropertyFloatValue*  RangeLatitudeMax;
PropertyFloatValue*  RangeLongitudeMin;
PropertyFloatValue*  RangeLongitudeMax;
PropertyFloatValue*  RangeDepthMin;
PropertyFloatValue*  RangeDepthMax;
PropertyFloatValue*  RangeMbMin;
PropertyFloatValue*  RangeMbMax;
PropertyFloatValue*  RangeMsMin;
PropertyFloatValue*  RangeMsMax;
PropertyFloatValue*  RangeMlMin;
PropertyFloatValue*  RangeMlMax;
PropertyFloatValue*  RangeMpMin;
PropertyFloatValue*  RangeMpMax;

///-----------------------------------------Identification of doubles: ----------------------------------------    
 PropertyIntValue*    deltaTime;
 PropertyIntValue*    deltaDepth;
 PropertyFloatValue*  deltaLatitude;
 PropertyFloatValue*  deltaLongitude;
 PropertyFloatValue*  deltaMagnitude;
 PropertyEnumValue*   DeltaRemoveAction;
 PropertyEnumValue*   DeltaRemoveEqual;

 PropertyEnumValue* RemovingDuplicateFunction;  
 PropertyEnumValue* RemovingDuplicatePriorityFirst;
 PropertyEnumValue* RemovingDuplicatePriorityFirstMagnitude;
 PropertyEnumValue* RemovingDuplicatePrioritySecond;
 PropertyEnumValue* RemovingDuplicatePrioritySecondMagnitude;
 PropertyEnumValue* RemovingDuplicatePriorityThird;
 PropertyEnumValue* RemovingDuplicatePriorityThirdMagnitude;
 PropertyEnumValue* RemovingDuplicatePriorityFourth;
 PropertyEnumValue* RemovingDuplicatePriorityFourthMagnitude;

 PropertyEnumValue* DeltaMagnitudeFunction;
 PropertyTransformFunction* DefineDeltaFirstMagnitude;
 PropertyTransformFunction* DefineDeltaSecondMagnitude;
 PropertyTransformFunction* DefineDeltaThirdMagnitude;
 PropertyTransformFunction* DefineDeltaFourthMagnitude;

//-------------------------------  Merge Catalogs -------------------------------------
 PropertyIntValue*    deltaCatTime;
 PropertyIntValue*    deltaCatDepth;
 PropertyFloatValue*  deltaCatLatitude;
 PropertyFloatValue*  deltaCatLongitude;
 PropertyFloatValue*  deltaCatMagnitude;

 PropertyEnumValue* DeltaCatMagnitudeFunction;
 PropertyTransformFunction* DefineDeltaCatFirstMagnitude;
 PropertyTransformFunction* DefineDeltaCatSecondMagnitude;
 PropertyTransformFunction* DefineDeltaCatThirdMagnitude;
 PropertyTransformFunction* DefineDeltaCatFourthMagnitude;

//---------------------------- Duplicate Magnitudes ---------------------------------
 PropertyEnumValue* AssignDuplicateMagnitude; 
 PropertyEnumValue* SelectedDuplicateMagnitude;
 PropertyEnumValue* RemovingDuplicateMagnitude;

PropertyStringValue* SecondCatalogName;
PropertyStringListValue* ListMergeCatalog;
//----------------------------------------- Compare Action  -------------------
PropertyEnumValue* CompareCatalogAction;
//----------------------------------- Add Catalogs-----------------------------
PropertyStringListValue* ListAddCatalog;
//-----------------------------------Create subcatalog -----------------------------
//------------------------------------ Record ----------------------------------
PropertyEnumValue*  SampleRecordIs;
PropertyEnumValue*  SampleRecordType;

PropertyDateTimeValue* SampleRecordDateMin;
PropertyDateTimeValue* SampleRecordDateMax;

PropertyIntValue* SampleRecordNumberMin;
PropertyIntValue* SampleRecordNumberMax;
//------------------------------------ Hipocenter ----------------------------------

PropertyEnumValue*  SampleHipoIs;
PropertyEnumValue*  SampleHipoType;
//------------------------------------ Rectangle Hipocenter --------------------------
PropertyFloatValue* SampleHipoRectLatitudeMin;
PropertyFloatValue* SampleHipoRectLatitudeMax;
PropertyFloatValue* SampleHipoRectLongitudeMin;
PropertyFloatValue* SampleHipoRectLongitudeMax;
PropertyEnumValue*  SampleHipoRectMinus180;
//------------------------------------ Circle Hipocenter --------------------------

PropertyFloatValue* SampleHipoCircleLatitude;
PropertyFloatValue* SampleHipoCircleLongitude;
PropertyFloatValue*   SampleHipoCircleRadius;

//------------------------------------ Polygon Hipocenter --------------------------
PropertyIntValue* SampleHipoPolygonVertice;
PropertyEnumValue*  SampleHipoPolygonMinus180;
PropertyStringListValue* SampleListLatitude;
PropertyStringListValue* SampleListLongitude;


//------------------------------ Depth Hipocenter --------------------------------------
PropertyEnumValue*  SampleDepthIs;
PropertyFloatValue* SampleDepthMin;
PropertyFloatValue* SampleDepthMax;
//------------------------------ Magnitude Mb --------------------------------------
PropertyEnumValue*  SampleMbIs;
PropertyFloatValue* SampleMbMin;
PropertyFloatValue* SampleMbMax;
//------------------------------ Magnitude Ms --------------------------------------
PropertyEnumValue*  SampleMsIs;
PropertyFloatValue* SampleMsMin;
PropertyFloatValue* SampleMsMax;
//------------------------------ Magnitude Ml --------------------------------------
PropertyEnumValue*  SampleMlIs;
PropertyFloatValue* SampleMlMin;
PropertyFloatValue* SampleMlMax;
//------------------------------ Magnitude Mp --------------------------------------
PropertyEnumValue*  SampleMpIs;
PropertyFloatValue* SampleMpMin;
PropertyFloatValue* SampleMpMax;
//------------------------------ Intencity --------------------------------------
PropertyEnumValue* SampleIntensityIs;
PropertyIntValue*  SampleIntensityMin;
PropertyIntValue*  SampleIntensityMax;

//------------------------------ Magnitude Function --------------------------------------
PropertyEnumValue*  SampleMagnitudeIs;
PropertyEnumValue*  SampleMagnitudeType;

PropertyFloatValue* SampleMagFunctionMin;
PropertyFloatValue* SampleMagFunctionMax;

PropertyEnumValue* SampleMagFunction;
PropertyTransformFunction* SampleMagFunctionFirstMagnitude;
PropertyTransformFunction* SampleMagFunctionSecondMagnitude;
PropertyTransformFunction* SampleMagFunctionThirdMagnitude;
PropertyTransformFunction* SampleMagFunctionFourthMagnitude;

//------------------------------ Transform Magnitude  --------------------------------------
QList<PropertyEnumValue*> TransformMagnitudeFunction;
QList<PropertyTransformFunction *> TransformMagnitudeMb;
QList<PropertyTransformFunction *> TransformMagnitudeMs;
QList<PropertyTransformFunction *> TransformMagnitudeMl;
QList<PropertyTransformFunction *> TransformMagnitudeMp;

///---------------------- Project parameters -------------------------------------
PropertyStringListValue* ProjectActionList;
PropertyStringValue* ProjectCatalogName;
PropertyEnumValue* ProjectDocumentIn;
PropertyEnumValue* ProjectActionChoise;

PropValVector values;
  
};
class SettingDocument
{
    typedef vector<PropertyValue* > PropValVector;
    typedef vector<PropertyValue* >::iterator PropValIterator; 
public:

    SettingDocument();
    ~SettingDocument();
    bool isModified();
    void write();
    void read();
    vector <PropertyValue* >  &GetValues(){return values;}


PropertyIntSpinValue* &GetWorkSpaceSize  ()    {return   WorkSpaceSize;}
PropertyIntSpinValue* &GetMaxErrorLines  ()    {return   MaxErrorLines;}
PropertyIntSpinValue* &GetMaxCorrectionLines() {return   MaxCorrectionLines;}

PropertyIntSpinValue* &GetNumberIntervalAxisX()    {return   NumberIntervalAxisX;}
PropertyIntSpinValue* &GetNumberIntervalAxisY()    {return   NumberIntervalAxisY;}
PropertyIntSpinValue* &GetNumberIntervalAxisZ()    {return   NumberIntervalAxisZ;}
/*
PropertyFloatValue* &GetMinLatitude () {return   MinLatitude;}
PropertyFloatValue* &GetMaxLatitude () {return   MaxLatitude;}

PropertyFloatValue* &GetMinLongitude () {return   MinLongitude;}
PropertyFloatValue* &GetMaxLongitude () {return   MaxLongitude;}

PropertyFloatValue* &GetMinDepth () {return   MinDepth;}
PropertyFloatValue* &GetMaxDepth () {return   MaxDepth;}

PropertyFloatValue* &GetMinMb () {return   MinMb;}
PropertyFloatValue* &GetMaxMb () {return   MaxMb;}

PropertyFloatValue* &GetMinMs () {return   MinMs;}
PropertyFloatValue* &GetMaxMs () {return   MaxMs;}

PropertyFloatValue* &GetMinMl () {return   MinMl;}
PropertyFloatValue* &GetMaxMl () {return   MaxMl;}

PropertyFloatValue* &GetMinMp () {return   MinMp;}
PropertyFloatValue* &GetMaxMp () {return   MaxMp;}
*/
private:
///---------------------- Settings parameters -------------------------------------
PropertyIntSpinValue* WorkSpaceSize;
PropertyIntSpinValue* MaxErrorLines;
PropertyIntSpinValue* MaxCorrectionLines;

///-----------------------------------  Histogram Parameters ----------------------------

PropertyIntSpinValue* NumberIntervalAxisX;
PropertyIntSpinValue* NumberIntervalAxisY;
PropertyIntSpinValue* NumberIntervalAxisZ;

/*
PropertyFloatValue* MinLatitude;
PropertyFloatValue* MaxLatitude;

PropertyFloatValue* MinLongitude;
PropertyFloatValue* MaxLongitude;

PropertyFloatValue* MinDepth;
PropertyFloatValue* MaxDepth;

PropertyFloatValue* MinMb;
PropertyFloatValue* MaxMb;

PropertyFloatValue* MinMs;
PropertyFloatValue* MaxMs;

PropertyFloatValue* MinMl;
PropertyFloatValue* MaxMl;

PropertyFloatValue* MinMp;
PropertyFloatValue* MaxMp;
*/
PropValVector values;
};
#endif
