#include <QtGui>
#include <qapplication.h>
#include "mainwindow.h" 
#include "default.h" 
#include "editline.h"
#include "document.h"
#include "float.h"
#include "viewcatalogimpl.h"
#include "catalog.h"
#include <qwt_color_map.h>
#include <qwt_plot_spectrogram.h>
#include <qwt_scale_engine.h>
#include <qwt_scale_widget.h>
#include <qwt_plot_grid.h>
#include <qwt_scale_draw.h>
#include <qwt_plot_zoomer.h>
#include <qwt_plot_panner.h>
#include <qwt_plot_layout.h>
#include <qwt_interval_data.h>
#include <qwt_legend.h>
#include <qwt_text.h>
#include "histogram_item.h"
#include "histogram.h"
MainWindowPlot::~MainWindowPlot(){
if (d_zoomer!= 0)   {delete d_zoomer; d_zoomer = 0;}
if (d_panner  != 0) {delete d_panner; d_panner = 0;}
if (d_picker  != 0) {delete d_picker; d_picker = 0;}
 
}
MainWindowPlot::MainWindowPlot(QWidget *parent):
    QMainWindow(parent),parent(parent)
{
	QFont f = parent->font();
    setFont( parent->font() );
d_plot = 0; //qwt auto new-delete
d_zoomer = 0;
d_panner = 0;
d_picker = 0;
isCommonScale = false;
(void)statusBar();
(void)statusBar()->setSizeGripEnabled ( FALSE );
}
void MainWindowPlot::GetHistogramX(DataPlot* dataPlot,LogScale scale){
	datPlot = dataPlot;
if (d_zoomer!= 0)   {delete d_zoomer; d_zoomer = 0;}
if (d_panner  != 0) {delete d_panner; d_panner = 0;}
if (d_picker  != 0) {delete d_picker; d_picker = 0;}

 
if (scale == Yes)
    h_plot = new HistPlot(this,dataPlot,HistPlot::HistX,HistPlot::Log);
if (scale == No)
    h_plot = new HistPlot(this,dataPlot,HistPlot::HistX,HistPlot::Normal);
	logscale = scale;
    h_plot->setMargin(5);

    setContextMenuPolicy(Qt::NoContextMenu);
	setCentralWidget(h_plot);
}
void MainWindowPlot::GetHistogramY(DataPlot* dataPlot,LogScale scale){
	datPlot = dataPlot;
if (d_zoomer!= 0)   {delete d_zoomer; d_zoomer = 0;}
if (d_panner  != 0) {delete d_panner; d_panner = 0;}
if (d_picker  != 0) {delete d_picker; d_picker = 0;}
if (scale == Yes)
    h_plot = new HistPlot(this,dataPlot,HistPlot::HistY,HistPlot::Log);
if (scale == No)
    h_plot = new HistPlot(this,dataPlot,HistPlot::HistY,HistPlot::Normal);
	logscale = scale;
    h_plot->setMargin(5);

    setContextMenuPolicy(Qt::NoContextMenu);
	setCentralWidget(h_plot);

}
void MainWindowPlot::GetSpectrogram(DataPlot* dataPlot,LogScale scale){
	datPlot = dataPlot;
if (d_zoomer != 0){ delete d_zoomer; d_zoomer = 0;}
if (d_panner  != 0) {delete d_panner; d_panner = 0;}
if (d_picker  != 0) {delete d_picker; d_picker = 0;}

if (scale == Yes)
    d_plot = new Plot(this,dataPlot,Plot::Log);
if (scale == No)
    d_plot = new Plot(this,dataPlot,Plot::Normal);

	logscale = scale;

    d_plot->setMargin(5);

    setContextMenuPolicy(Qt::NoContextMenu);

		
    d_zoomer = new QwtPlotZoomer( d_plot->canvas());
		
	d_zoomer->setMousePattern(QwtEventPattern::MouseSelect2,
        Qt::RightButton, Qt::ControlModifier);	
	d_zoomer->setMousePattern(QwtEventPattern::MouseSelect3,
        Qt::RightButton);	
	
  //  const QColor c(Qt::white);
  //  d_zoomer->setRubberBandPen(c);
  //  d_zoomer->setTrackerPen(c);
	
    d_zoomer->setRubberBand(QwtPicker::RectRubberBand);
    d_zoomer->setRubberBandPen(QColor(Qt::green));
    d_zoomer->setTrackerMode(QwtPicker::ActiveOnly);
    d_zoomer->setTrackerPen(QColor(Qt::white));

//    d_zoomer[1] = new Zoomer(QwtPlot::xTop, QwtPlot::yRight,
//         d_plot->canvas());

 //   d_zoomer[1] = new QwtPlotZoomer(d_plot->canvas());
    
	
    d_panner = new QwtPlotPanner(d_plot->canvas());
    d_panner->setMouseButton(Qt::MidButton);

    d_picker = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft,
        QwtPicker::PointSelection | QwtPicker::DragSelection, 
        QwtPlotPicker::CrossRubberBand, QwtPicker::AlwaysOn, 
        d_plot->canvas());
    d_picker->setRubberBandPen(QColor(Qt::green));
    d_picker->setRubberBand(QwtPicker::CrossRubberBand);
    d_picker->setTrackerPen(QColor(Qt::white));

    setCentralWidget(d_plot);

	enableZoomMode(false);
    showInfo(); 

    connect(d_picker, SIGNAL(moved(const QPoint &)),
            SLOT(moved(const QPoint &)));
    connect(d_picker, SIGNAL(selected(const QwtPolygon &)),
            SLOT(selected(const QwtPolygon &)));
	
	
}
///=========================================================================
void MainWindowPlot::print()
{

#if 1
    QPrinter printer;
#else
    QPrinter printer(QPrinter::HighResolution);

    printer.setOutputFileName("/tmp/test.ps");
#endif

    QString docName = "";
	HistogramSettings * wnd = (HistogramSettings* )parent;
	HistogramControl* wcntrl = (HistogramControl*) wnd->GetHistogramControl();
	if (wcntrl->IsSpectrogram()) docName = d_plot->title().text();
	if (wcntrl->IsHistogramX() || wcntrl->IsHistogramY()) docName = h_plot->title().text();
    if ( !docName.isEmpty() )
    {
        docName.replace (QRegExp (QString::fromLatin1 ("\n")), tr (" -- "));
        printer.setDocName (docName);
    }

    printer.setCreator("Printer panel");
    printer.setOrientation(QPrinter::Landscape);
    QPrintDialog dialog(&printer);
    if ( dialog.exec() )
    {
        QwtPlotPrintFilter filter;
        if ( printer.colorMode() == QPrinter::GrayScale )
        {
            filter.setOptions(QwtPlotPrintFilter::PrintAll 
                & ~QwtPlotPrintFilter::PrintCanvasBackground);
        }
        
	if (wcntrl->IsSpectrogram()) d_plot->print(printer, filter);
	if (wcntrl->IsHistogramX() || wcntrl->IsHistogramY()) h_plot->print(printer, filter);

    }
}

void MainWindowPlot::exportPDF()
{
 
    const QString fileName = QFileDialog::getSaveFileName(
        this, "Export File Name", QString(),
        "PDF Documents (*.pdf)");
    if ( !fileName.isEmpty() )
    {
        QPrinter printer;
        printer.setOutputFormat(QPrinter::PdfFormat);
        printer.setOrientation(QPrinter::Landscape);
        printer.setOutputFileName(fileName);

        printer.setCreator("Histogram PDF printer panel");
		HistogramSettings * wnd = (HistogramSettings* )parent;
		HistogramControl* wcntrl = (HistogramControl*) wnd->GetHistogramControl();
		if (wcntrl->IsSpectrogram()) d_plot->print(printer);
	    if (wcntrl->IsHistogramX() || wcntrl->IsHistogramY()) h_plot->print(printer);
        
    }

}
void MainWindowPlot::exportCSV(DataPlot* dataPlot)
{
    QString fileFlt = ".csv";
    QString fileName = QFileDialog::getSaveFileName(
        this, "Export File Name", QString(),
        QString("Microsoft Exel CSV Documents (*%1)").arg(fileFlt));
    if ( fileName.isEmpty() ) return;
	int ind = fileName.length () - fileFlt.length ()-1; if (ind < 0) ind =0;
    if ( ( fileName.indexOf(fileFlt,ind)) == -1 && ( fileName.indexOf(fileFlt.toUpper(),ind)) == -1) {
      fileName.append(fileFlt);

        if (QFile::exists(fileName)) {
        int ret = QMessageBox::warning(this, mainWindowCaption(),
                     QString("File %1 already exists.\n Do you want to overwrite it?")
                     .arg(QDir::toNativeSeparators(fileName)),
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No | QMessageBox::Escape);
        if (ret == QMessageBox::No)
            return;
       }
    }
	QDateTime dt = QDateTime::currentDateTime ();
    QString currentTime = dt.toString("dd.MM.yyyy.hh.mm.ss");
    QString printOutTime = dt.toString("dd/MM/yyyy hh:mm:ss");
	currentTime.remove (16, 3 );
	QFile file(fileName);
	
    if (!file.open(QFile::WriteOnly)) {
		QMessageBox::warning(this, "", QString ("Can't open for writing file\n '%1' ").arg(fileName));
		return;
	}
    QTextStream out(&file);
			  
		TableLimits *tbl = dataPlot->GetLimits();
		float *** table  = dataPlot->GetTable();
		float **  tableX = dataPlot->GetTableX();
		float **  tableY = dataPlot->GetTableY();
	    QString tmp =  "";
		HistogramSettings * wnd = (HistogramSettings* )parent;
		HistogramControl* hcontr = wnd->GetHistogramControl();
		
		QString LogScale = "Linear scale of values was used.";
		if (logscale == Yes) LogScale = "Logarithmic (base 10) scale of values was used.";
		
		QString ListParameter = "";
		if (tbl->numIntervalZ > 1) {
			float minZ = tbl->minZ;
			float maxZ = tbl->maxZ;
			int numInterval = 1;
			if (hcontr->IsSpectrogram() ) 
				numInterval = d_plot->GetCurrentIntervalZ();
			if (hcontr->IsHistogramX() || hcontr->IsHistogramY()) 
				numInterval = h_plot->GetCurrentIntervalZ();
			float step = (maxZ - minZ)/(tbl->numIntervalZ);
			float min = minZ + numInterval*step;
			float max = min + step;
		
			ListParameter = QString("This is a (# %1 of %2) item [ %3; %4 ] of the list of \"%5\" [ %6; %7 ]." ).
			arg(numInterval+1).arg(tbl->numIntervalZ).
			arg(min).arg(max).arg(dataPlot->GetTitleZ()).
			arg(minZ).arg(maxZ);
			if (isCommonScale) ListParameter += "Values were normalized by the common maximum of all list items.";
		    else ListParameter += "Values were normalized by the own maximum of this list item.";

		
		}
		
		QString axixX = QString("\"%1\" [ %2; %3 ]. Number of intervals: %4. Step: %5.").arg(dataPlot->GetTitleX()).
		  arg(tbl->minX).arg(tbl->maxX).arg(tbl->numIntervalX).
		  arg((tbl->maxX - tbl->minX)/tbl->numIntervalX);
		QString axixY = QString("\"%1\" [ %2; %3 ]. Number of intervals: %4. Step: %5.").arg(dataPlot->GetTitleY()).
		  arg(tbl->minY).arg(tbl->maxY).arg(tbl->numIntervalY).
		  arg((tbl->maxY - tbl->minY)/tbl->numIntervalY);
		  
		  QString catName = MainWindow::self->GetCurrentCatalogName();
		  if (catName == "") catName = "unnamed";
        QString catalogInfo = QString("Catalog: %1").arg(catName);
		QString catalogDateLimit = QString("Catalog date from: %2 to: %3").
			arg(hcontr->GetDateMin()).
			arg(hcontr->GetDateMax());
			
		if (hcontr->IsSpectrogram() ) {

	if (hcontr->GetTypeHist() == HistogramControl::Weight) {
		tmp = "Weighted sum of ";
	    tmp += hcontr->GetTypeWeight();
		tmp +=  " counts ";
	}
	
	if (hcontr->GetTypeHist() == HistogramControl::Frequency)  tmp = "Frequencies of earthquakes ";
	
          if (tbl->numIntervalZ > 1)
		  tmp = tmp + "2-D histogram by List was created by compicat program  on ";
		  else tmp = tmp + "2-D histogram was created by compicat program  on ";
		  tmp += printOutTime;
		  out << "\"" << tmp << "\"" << endl;
		  
//----------------------------------------------------------------------------------------
if (hcontr->GetTypeHist() == HistogramControl::Weight)  {
	tmp = QString("Type weight:  log10( %1 ) = %2 + %3 * %4  ").arg(hcontr->GetTypeWeight()).
    arg(hcontr->GetCoeffWeightA()).
	arg(hcontr->GetCoeffWeightB()).
	arg(hcontr->GetMagnitudeWeight());	  
		  out << "\"" << tmp << "\"" << endl;
	
	
}
//----------------------------------------------------------------------------------------		  
		  out << "\"" << catalogInfo << "\"" << endl;
		  out << "\"" << catalogDateLimit << "\"" << endl;

		  if (tbl->numIntervalZ > 1) {
		  out << "\"" << ListParameter << "\"";
		  out <<  " " << endl;			  
		  }
		  
		  
		  out << "\"" << "Axis X: " << axixX << "\"";
		  out <<  " " << endl;
		  
		  out << "\"" << "Axis Y: "  << axixY << "\"";
		  out <<  " " << endl;
          out << "\"" << LogScale << "\"";
		  out <<  " " << endl;
		  out <<  " " << endl;
	      int k =  GetSpectrogramPlot()->GetCurrentIntervalZ();
	      for (int i = 0; i < tbl->numIntervalY; ++i) {
		  for (int j = 0; j < tbl->numIntervalX; ++j)
			  out << QString("%1").arg(table[k][i][j]) << ",";
		  out <<  " " << endl;
		 }
		 
		}
		
		if (hcontr->IsHistogramX() || hcontr->IsHistogramY()) {
		  if (tbl->numIntervalZ > 1)
		  tmp = "Frequencies of earthquakes Histogram by List was created by compicat program  on ";
		  else tmp = "Frequencies of earthquakes Histogram was created by compicat program  on ";
		  tmp += printOutTime;
		  out << "\"" << tmp << "\"" << endl;
		  out << "\"" << catalogInfo << "\"" << endl;
		  out << "\"" << catalogDateLimit << "\"" << endl;	
		  
		  if (tbl->numIntervalZ > 1) {
		  out << "\"" << ListParameter << "\"";
		  out <<  " " << endl;			  
		  }
		  int k =  h_plot->GetCurrentIntervalZ();
		  if (hcontr->IsHistogramX()) {
		  
			  out << "\"" << "Axis X: "  << axixX << "\"";
			  out <<  " " << endl;
			  
			  out << "\"" << "Axis Y: " << "Frequencies of earthquakes"  << "\"";		  
			  out <<  " " << endl;
			  
		      
          out << "\"" << LogScale << "\"";
		  out <<  " " << endl;
		  out <<  " " << endl;
			  
			  for (int j = 0; j < tbl->numIntervalX; ++j)
				  out << QString("%1").arg(tableX[k][j]) << ",";
			  out <<  " " << endl;
		  }

		  if (hcontr->IsHistogramY()) {
		  
			  out << "\"" << "Axis X: "  << axixY << "\"";
			  out <<  " " << endl;
			  out << "\"" << "Axis Y: " << "Frequencies of earthquakes"  << "\"";		  
			  out <<  " " << endl;
			  
		      
          out << "\"" << LogScale << "\"";
		  out <<  " " << endl;
		  out <<  " " << endl;
		  
;
			  for (int j = 0; j < tbl->numIntervalY; ++j)
				  out << QString("%1").arg(tableY[k][j]) << ",";
			  out <<  " " << endl;
		  }
		
		  
		}
		
        file.close ();
 

}


void MainWindowPlot::enableZoomMode(bool on)
{
    d_panner->setEnabled(on);

    d_zoomer->setEnabled(on);
    d_zoomer->zoom(0);

    d_picker->setEnabled(!on);

    showInfo();
}

void MainWindowPlot::showInfo(QString text)
{
    if ( text == QString::null )
    {
        if ( d_picker->rubberBand() )
            text = "Cursor Pos: Press left mouse button in plot region";
        else
            text = "Zoom: Press mouse button and drag";
    }
    statusBar()->showMessage(text);
}

void MainWindowPlot::moved(const QPoint &pos)
{
    QString info;
	double x = d_plot->invTransform(QwtPlot::xBottom, pos.x());
	double y = d_plot->invTransform(QwtPlot::yLeft, pos.y());
	double val = 0;
	TableLimits* tbl = datPlot->GetLimits();
	
	if ( x >= tbl->minX && x <= tbl->maxX &&
		 y >= tbl->minY && y <= tbl->maxY ) {
			 
			 if (logscale == Yes)
				 val = d_plot->d_spectrogram->data().value (x, y)-1;
				 //pow(10,d_plot->d_spectrogram->data().value (x, y))  - 1;
			 if (logscale == No)
				 val = d_plot->d_spectrogram->data().value (x, y);
			 info.sprintf("X = %g, Y = %g N = %g",x,y,val);
		 }
		else {
			   info.sprintf("X = %g, Y = %g N = nun",x,y);
		
		}
		
		
    
    showInfo(info);
}

void MainWindowPlot::selected(const QwtPolygon &)
{
    showInfo();
}

///===============================================================================
HistPlot::~HistPlot() {
if (histogram != 0) delete histogram;
}
HistPlot::HistPlot(QWidget * parent,DataPlot* dataPlot,
          HistPlot::TypeHist type,TypeYScale scale):
    QwtPlot(parent),type(type),scale(scale),dataPlot(dataPlot),parent(parent) {
	QFont f = parent->font();
    setFont( parent->font() );

	QwtPlotGrid *grid = new QwtPlotGrid;
    grid->enableXMin(true);
    grid->enableYMin(true);
    grid->setMajPen(QPen(Qt::darkGray, 0, Qt::DotLine));
    grid->setMinPen(QPen(Qt::lightGray, 0 , Qt::DotLine));
    grid->attach(this);

   // setCanvasBackground(QColor(Qt::white));
	histogram = 0;
   	currenIntervalZ = 0;
	isCommonScale = false;
	DrawCurrentHistogram(currenIntervalZ);
	} 
void HistPlot::DrawCurrentHistogram(int currenIntervalZ) {

    if (histogram != 0) delete histogram;
    histogram = new HistogramItem(scale);
	
	MainWindowPlot * wnd = (MainWindowPlot* )parent;
	isCommonScale = false;
	if(wnd->IsCommonScale()) isCommonScale = true;

	TableLimits* tbl = dataPlot->GetLimits();
    bool* isEmpty = dataPlot->GetTableIsEmpty();
	int numInterval = 0;
	int numIntervalSecondAxis = 0;
	double hMax = 0.0;
	double hMin = 0.0;
	float *hist = 0;
	if (type == HistPlot::HistX) {
		numInterval = tbl->numIntervalX; 
		numIntervalSecondAxis = tbl->numIntervalY; 
		if (isCommonScale) {
			hMin = dataPlot->GeMinimumMinimoreX ();
			hMax = dataPlot->GeMaximumMaximoreX ();
		}
		else {
			hMin = dataPlot->GetTableMinX()[currenIntervalZ];
			hMax = dataPlot->GetTableMaxX()[currenIntervalZ];
		}
			
		hist = dataPlot->GetTableX()[currenIntervalZ];
	}
	
	if (type == HistPlot::HistY) {
		numInterval = tbl->numIntervalY;
		numIntervalSecondAxis = tbl->numIntervalX; 
		if (isCommonScale) {
			hMin = dataPlot->GeMinimumMinimoreY ();
			hMax = dataPlot->GeMaximumMaximoreY ();
		}
		else {
			hMin = dataPlot->GetTableMinY()[currenIntervalZ];
			hMax = dataPlot->GetTableMaxY()[currenIntervalZ];
		}
		hist = dataPlot->GetTableY()[currenIntervalZ];
	}

    QwtArray<QwtDoubleInterval> intervals(numInterval);
    QwtArray<double> values(numInterval);

    double pos = 105.0;
	double width = 0.5;
	double xScaleMin = 105.0;
	double xScaleMax = 0.0;
	//double eps = acuracyCompareFloatValue();
	if (type == HistPlot::HistX) {
		xScaleMin = pos = tbl->minX; 
		xScaleMax = tbl->maxX;
		width = (tbl->maxX - tbl->minX)/tbl->numIntervalX; 
	}
	
	if (type == HistPlot::HistY) {
		xScaleMin = pos = tbl->minY; 
		xScaleMax = tbl->maxY;
		width = (tbl->maxY - tbl->minY)/tbl->numIntervalY; 
	}
	

    for ( int i = 0; i < (int)intervals.size(); i++ )
    {

		double min = pos + i*width;
		double max = pos + (i+1)*width;
        
        intervals[i] = QwtDoubleInterval(min,max);
        values[i] = (double)hist[i];
		if (scale == Log && hist[i] < 2 ) values[i] = 1.01;
//QMessageBox::warning(0, "",QString("v %1 max %2").arg((double)hist[i]).arg(hMax));
    }

    histogram->setData(QwtIntervalData(intervals, values));
    histogram->attach(this);
	setAxisScale(QwtPlot::xBottom, xScaleMin, xScaleMax);
   
	if (scale == Log) {
		//QMessageBox::warning(0, "",QString("scale == Log"));
		setAxisScaleEngine(QwtPlot::yLeft, new QwtLog10ScaleEngine);
		setAxisScaleEngine(QwtPlot::yRight, new QwtLog10ScaleEngine);
		setAxisScale(QwtPlot::yLeft, 1.0, hMax);
	    setAxisScale(QwtPlot::yRight, 1.0, hMax);
		
	}
	if (scale == Normal) {
		//QMessageBox::warning(0, "",QString("scale == Normal"));
		setAxisScale(QwtPlot::yLeft, 0.0, hMax);
	    setAxisScale(QwtPlot::yRight, 0.0, hMax);
	}	
	 enableAxis(QwtPlot::yLeft);	
	 enableAxis(QwtPlot::yRight);	
	 enableAxis(QwtPlot::xBottom);


	
	//plot.setAxisMaxMajor(QwtPlot::yLeft, 12);
    //plot.setAxisMaxMinor(QwtPlot::yLeft, 6);
	//-----------------------------------------------
	QFont AxisFont = font ();
    AxisFont.setPointSize(AxisFont.pointSize() + 2 );
	AxisFont.setBold(true);
	QwtText qtext(""); 
	qtext.setFont (AxisFont);
	qtext.setColor (Qt::darkBlue);
	//------------------------------------------------	
	QString tmp = "";

	if (type == HistPlot::HistX) tmp = dataPlot->GetTitleX();
	if (type == HistPlot::HistY) tmp = dataPlot->GetTitleY();

	if(dataPlot->GetLimits()->numIntervalZ > 1) {
		int numIntervalZ = tbl->numIntervalZ;
		float minZ = tbl->minZ;
		float maxZ = tbl->maxZ;
		float step = (maxZ - minZ)/(numIntervalZ);
		float min = minZ + currenIntervalZ*step;
		float max = min + step;
		tmp +=  QString("\nInterval of %1 [ %2 , %3 ]").arg(dataPlot->GetTitleZ()).
		arg(min).arg(max);
	}	
	qtext.setText(tmp);	
	setAxisTitle(QwtPlot::xBottom, qtext);

    MainWindowPlot* wd = (MainWindowPlot*)parent;
	HistogramSettings* hset = (HistogramSettings*)wd->GetWindowParent ();
	HistogramControl* hcntrl = (HistogramControl*)hset->GetHistogramControl(); 	

	tmp = "";
	if (hcntrl->GetTypeHist() == HistogramControl::Weight) {
		tmp = "Weighted sum of ";
		tmp +=  hcntrl->GetTypeWeight();
		tmp +=  " counts.";
	
	}
	if (hcntrl->GetTypeHist() == HistogramControl::Frequency) tmp = "Frequencies of earthquakes";
	if (hcntrl->GetTypeHist() == HistogramControl::Min) {
		tmp = "Minimum of ";
		tmp +=  hcntrl->GetParHistMinMax();
	}
	if (hcntrl->GetTypeHist() == HistogramControl::Max) {
		tmp = "Maximum of ";
		tmp +=  hcntrl->GetParHistMinMax();
	}
	qtext.setText(tmp);	
	setAxisTitle(QwtPlot::yLeft,qtext);
	setAxisTitle(QwtPlot::yRight,qtext);
	
	//---------------------------------------------------
	QString header = "";
	if (hcntrl->GetTypeHist() == HistogramControl::Frequency) 
		header = "Tabulated frequencies";
	if (hcntrl->GetTypeHist() == HistogramControl::Min)  header = "Tabulated minimums";
	if (hcntrl->GetTypeHist() == HistogramControl::Max)  header = "Tabulated maximums";
	if (hcntrl->GetTypeHist() == HistogramControl::Weight) header = "Tabulated weights";
	
	if (isEmpty[currenIntervalZ]) 
		header +=  QString("\n Histogram is empty");
	
	qtext.setColor (Qt::black);
	qtext.setText(header);
	setTitle(qtext);

	
    QPalette pl = MainWindow::self->palette();
    QBrush br = pl.window();

    setCanvasBackground(br.color());
	histogram->setColor(Qt::darkCyan);

	setAxisFont(QwtPlot::xBottom,font ());	
	setAxisFont(QwtPlot::yLeft,font ());	
	setAxisFont(QwtPlot::yRight,font ());
	replot();
}
	
	
///===============================================================================
Plot::~Plot(){
	if (d_spectrogram != 0) delete d_spectrogram;
}

Plot::Plot(QWidget *parent,DataPlot* dataPlot,TypeZScale scale):
    QwtPlot(parent),dataPlot(dataPlot),scale(scale),parent(parent)
{
	QFont f = parent->font();
    setFont( parent->font() );
	QwtPlotGrid *grid = new QwtPlotGrid;
    grid->enableXMin(true);
	grid->enableYMin(true);
    grid->setMajPen(QPen(Qt::white, 0, Qt::DotLine));
    grid->setMinPen(QPen(Qt::gray, 0 , Qt::DotLine));
    grid->attach(this);
	d_spectrogram = 0;
	isCommonScale = false;
	currenIntervalZ = 0;
    DrawCurrentSpectrogram(currenIntervalZ);



   // const QColor c(Qt::darkBlue);
   // zoomer->setRubberBandPen(c);
   // zoomer->setTrackerPen(c);
}
//=============================================================================
void Plot::DrawCurrentSpectrogram(int currenIntervalZ){
MainWindowPlot * wnd = (MainWindowPlot* )parent;
float tableMax = dataPlot->GetTableMax()[currenIntervalZ];
isCommonScale = false;
if(wnd->IsCommonScale()) {
	//int tableMin = dataPlot->GetTableMin()[currenIntervalZ];
	//int tableMax = dataPlot->GetTableMax()[currenIntervalZ];
//	int tableMin = dataPlot->GeMinimumMinimore ();
	tableMax = dataPlot->GeMaximumMaximore ();
	isCommonScale = true;
}
    bool* isEmpty = dataPlot->GetTableIsEmpty();
	if (d_spectrogram != 0) delete d_spectrogram;
    d_spectrogram = new QwtPlotSpectrogram();
	QwtLinearColorMap colorMap(Qt::darkCyan, Qt::red);
   // colorMap.addColorStop(0.1, Qt::cyan);
   // colorMap.addColorStop(0.6, Qt::green);
   // colorMap.addColorStop(0.95, Qt::yellow);

    // LeftButton for the zooming
    // MidButton for the panning
    // RightButton: zoom out by 1
    // Ctrl+RighButton: zoom out to full size
	double d1 = 0.05; //0.1
	double d2 = 0.2;//0.4
	double d3 = 0.35;//0.7
	if (scale == Normal) {
		if (isEmpty[currenIntervalZ]) {
			colorMap.addColorStop(0.95, Qt::cyan);
			colorMap.addColorStop(0.97, Qt::green);
			colorMap.addColorStop(0.98, Qt::yellow);		
		}
		else {
			colorMap.addColorStop(d1, Qt::cyan);
			colorMap.addColorStop(d2, Qt::green);
			colorMap.addColorStop(d3, Qt::yellow);
		}
    }

    MainWindowPlot* wd = (MainWindowPlot*)parent;
	HistogramSettings* hset = (HistogramSettings*)wd->GetWindowParent ();
	HistogramControl* hcntrl = (HistogramControl*)hset->GetHistogramControl(); 
	
	if (scale == Log) {

	if (hcntrl->GetTypeHist() == HistogramControl::Weight)  {
	//d1 = 0.005; 
	//d2 = 0.1;
	//d3 = 0.2;
    double logmax = log(tableMax);
	d1 /= logmax;  //(lstep -  logmin)/logint;
	d2 /= logmax; //(4*lstep -  logmin)/logint;
	d3 /= logmax; //(7*lstep -  logmin)/logint;
		} 
	if (hcntrl->GetTypeHist() == HistogramControl::Frequency) {
    double logmax = log(tableMax+1);
	d1 /= logmax;  //(lstep -  logmin)/logint;
	d2 /= logmax; //(4*lstep -  logmin)/logint;
	d3 /= logmax; //(7*lstep -  logmin)/logint;
		}
	if (hcntrl->GetTypeHist() == HistogramControl::Min) {
    double logmax = log(tableMax+1);
	d1 /= logmax;  //(lstep -  logmin)/logint;
	d2 /= logmax; //(4*lstep -  logmin)/logint;
	d3 /= logmax; //(7*lstep -  logmin)/logint;
	}
	if (hcntrl->GetTypeHist() == HistogramControl::Max) {
    double logmax = log(tableMax+1);
	d1 /= logmax;  //(lstep -  logmin)/logint;
	d2 /= logmax; //(4*lstep -  logmin)/logint;
	d3 /= logmax; //(7*lstep -  logmin)/logint;
	}
		colorMap.addColorStop(d1, Qt::cyan);
		colorMap.addColorStop(d2, Qt::green);
		colorMap.addColorStop(d3, Qt::yellow);	

	}
	
	d_spectrogram->setData(SpectrogramData(currenIntervalZ,dataPlot,scale,isCommonScale)); 
    d_spectrogram->setColorMap(colorMap);

 
    d_spectrogram->attach(this);

	/*
	setAxisMaxMajor(QwtPlot::yLeft, 12);
    setAxisMaxMinor(QwtPlot::yLeft, 2);
	
	setAxisMaxMajor(QwtPlot::xBottom, 10);
    setAxisMaxMinor(QwtPlot::xBottom, 2);
	*/
//----------------------------------------------------	
    QwtScaleWidget *rightAxis = axisWidget(QwtPlot::yRight);

    rightAxis->setColorBarEnabled(true);
    rightAxis->setColorMap(d_spectrogram->data().range(),
        d_spectrogram->colorMap());

	if (scale == Normal) {
		if (isEmpty[currenIntervalZ]) {
		setAxisScale(QwtPlot::yRight,0.0,1.0); 	
		}
		else {
        setAxisScale(QwtPlot::yRight, 
        d_spectrogram->data().range().minValue(),
        d_spectrogram->data().range().maxValue() );
		}
	}
	if (scale == Log) {
		if (isEmpty[currenIntervalZ]) {
			setAxisScale(QwtPlot::yRight,0.0,0.01); 	
		}
		else {
		  if (hcntrl->GetTypeHist() == HistogramControl::Weight){
          setAxisScale(QwtPlot::yRight, 
          tableMax/100000, //d_spectrogram->data().range().minValue()
          d_spectrogram->data().range().maxValue());
		  setAxisScaleEngine(QwtPlot::yRight, new QwtLog10ScaleEngine);
		  } 
		  if (hcntrl->GetTypeHist() == HistogramControl::Frequency){
           setAxisScale(QwtPlot::yRight, 
           d_spectrogram->data().range().minValue(),
           d_spectrogram->data().range().maxValue());
		   setAxisScaleEngine(QwtPlot::yRight, new QwtLog10ScaleEngine);	
		  }
		  if (hcntrl->GetTypeHist() == HistogramControl::Min) {
           setAxisScale(QwtPlot::yRight, 
           d_spectrogram->data().range().minValue(),
           d_spectrogram->data().range().maxValue());
		   setAxisScaleEngine(QwtPlot::yRight, new QwtLog10ScaleEngine);	
		  
		  }
		  if (hcntrl->GetTypeHist() == HistogramControl::Max) {
           setAxisScale(QwtPlot::yRight, 
           d_spectrogram->data().range().minValue(),
           d_spectrogram->data().range().maxValue());
		   setAxisScaleEngine(QwtPlot::yRight, new QwtLog10ScaleEngine);	
		  
		  }
		}
		
	} 
    enableAxis(QwtPlot::yRight);

	QString tmp = "";
	if (hcntrl->GetTypeHist() == HistogramControl::Weight) {
		tmp = "Intensity as weighted sum of ";
		tmp +=  hcntrl->GetTypeWeight();
		tmp +=  " counts.";
	}
	if (hcntrl->GetTypeHist() == HistogramControl::Frequency) 
		tmp = "Intensity as frequencies of earthquakes";
	if (hcntrl->GetTypeHist() == HistogramControl::Min) {
		tmp = "Intensity as minimum of ";
		tmp +=  hcntrl->GetParHistMinMax();
	}
	if (hcntrl->GetTypeHist() == HistogramControl::Max) {
		tmp = "Intensity as maximum of ";
		tmp +=  hcntrl->GetParHistMinMax();
	}
	QString header = tmp;
	if (isEmpty[currenIntervalZ]) 
		header +=  QString("\n 2-D histogram is empty");
	
	//-----------------------------------------------
	QFont AxisFont = font ();
    AxisFont.setPointSize(AxisFont.pointSize() + 2 );
	AxisFont.setBold(true);
	QwtText qtext(header); 
	qtext.setFont (AxisFont);
	qtext.setColor (Qt::black);
    setTitle(qtext);
	
    tmp = dataPlot->GetTitleX();
	if(dataPlot->GetLimits()->numIntervalZ > 1) {
		TableLimits* tbl = dataPlot->GetLimits();
		int numIntervalZ = tbl->numIntervalZ;
		float minZ = tbl->minZ;
		float maxZ = tbl->maxZ;
		float step = (maxZ - minZ)/(numIntervalZ);
		float min = minZ + currenIntervalZ*step;
		float max = min + step;
		tmp +=  QString("\nInterval of %1 [ %2 , %3 ]").arg(dataPlot->GetTitleZ()).
		arg(min).arg(max);
	} 
	
	qtext.setText(tmp);
	qtext.setColor (Qt::darkBlue);
	setAxisTitle(QwtPlot::xBottom,qtext );
	
	qtext.setText(dataPlot->GetTitleY());
	setAxisTitle(QwtPlot::yLeft, qtext );
	qtext.setText("Intensity");
	rightAxis->setTitle(qtext);
	
	setAxisFont(QwtPlot::xBottom,font ());	
	setAxisFont(QwtPlot::yLeft,font ());	
	setAxisFont(QwtPlot::yRight,font ());
	//------------------------------------------------	
    plotLayout()->setAlignCanvasToScales(true);

    replot();

    QwtPlotPanner *panner = new QwtPlotPanner(canvas());
    panner->setAxisEnabled(QwtPlot::yRight, false);
    panner->setMouseButton(Qt::MidButton);

    // Avoid jumping when labels with more/less digits
    // appear/disappear when scrolling vertically

    const QFontMetrics fm(axisWidget(QwtPlot::yLeft)->font());
    QwtScaleDraw *sd = axisScaleDraw(QwtPlot::yLeft);
    sd->setMinimumExtent( fm.width("-100.00") );
 
}
///=========================================================================
SpectrogramData::SpectrogramData(int currentInterval,DataPlot* data,
          Plot::TypeZScale scale,bool isCommonScale):
        QwtRasterData(),
     currentInterval(currentInterval),data(data),scale(scale),isCommonScale(isCommonScale)
	 
    {
	 setBoundingRect (QwtDoubleRect((double)data->GetLimits()->minX, (double)data->GetLimits()->minY,
			          (double)(data->GetLimits()->maxX - data->GetLimits()->minX) , 
					  (double)(data->GetLimits()->maxY - data->GetLimits()->minY) ) )	;
    }
QwtRasterData *SpectrogramData::copy() const
    {
        return new SpectrogramData(currentInterval,data,scale,isCommonScale);
			
    }

QwtDoubleInterval SpectrogramData::range() const
    {
		
		float numMin = data->GetTableMin()[currentInterval];
		float numMax = data->GetTableMax()[currentInterval];
		if (isCommonScale) {
		numMin = data->GeMinimumMinimore ();
		numMax = data->GeMaximumMaximore ();
		}
		bool* isEmpty = data->GetTableIsEmpty();
		if (scale == Plot::Log) {
			if (isEmpty[currentInterval] )
				return QwtDoubleInterval(0.0,0.01);	
			else 
				return QwtDoubleInterval(numMin + 1 ,
		        numMax + 1 );
		}
		if (scale == Plot::Normal) 
			if (isEmpty[currentInterval] )
				return QwtDoubleInterval(0.0,1.0);	
			else return QwtDoubleInterval(numMin,numMax);
		// never reach next line
		return QwtDoubleInterval(0,0);
    }

double SpectrogramData::value(double x, double y) const
    { 
		//double eps = acuracyCompareFloatValue();
		TableLimits* tbl = data->GetLimits();
		
		if (! ( x > tbl->minX && x < tbl->maxX &&
		 y > tbl->minY && y < tbl->maxY )) return 0.0;
		bool* isEmpty = data->GetTableIsEmpty();
		const int v1 = (int)((( (y - tbl->minY))/( tbl->maxY - 
		               tbl->minY))*(tbl->numIntervalY)); //!!
	    const int v2 = (int)((( (x - tbl->minX))/(tbl->maxX - 
		               tbl->minX))*(tbl->numIntervalX)); //!!
/*					   
if (v1 > tbl->numIntervalY -1 || v2 > tbl->numIntervalX -1)
QMessageBox::warning(0, "",QString("v1 %1  (y - tbl->minY) -eps  %2  tbl->maxY - tbl->minY  %3  y  %4 miny %5").
arg(v1).arg((y - tbl->minY) -eps).
arg(tbl->maxY - tbl->minY).arg(y).arg(tbl->minY) );
*/
        if (scale == Plot::Log)
			if (isEmpty[currentInterval])
				return 0.0;	
			else //log10
				return ( (double)(data->GetTable()[currentInterval][v1][v2]+1));
		if (scale == Plot::Normal) 
			if (isEmpty[currentInterval] )
				return 0.0;	
			else 
				return (double)data->GetTable()[currentInterval][v1][v2];
		// never reach next line
		return 1.0;
    }
///========================================================================
TableLimits::TableLimits(int nx, float minx,float maxx,
	                int ny, float miny,float maxy,
					int nz, float minz,float maxz){
	numIntervalX = nx;
	numIntervalY = ny;
	numIntervalZ = nz;
	minX         = minx;
	maxX         = maxx;
	minY         = miny;
	maxY         = maxy;
	minZ         = minz;
	maxZ         = maxz;
	}
///=========================================================================
    DataPlot::~DataPlot() {
	for (int i = 0; i < tbl->numIntervalZ; ++i) {
		for (int j = 0; j < tbl->numIntervalY; ++j)
			delete [] table[i][j];
		delete [] table[i];
		delete [] tableX[i];
		delete [] tableY[i];
	}
	delete [] table;
	delete [] tableX;
	delete [] tableY;
	delete [] numMin;
	delete [] numMax;
	
	delete [] numMinX;
	delete [] numMaxX;
	delete [] numMinY;
	delete [] numMaxY;
	delete [] isEmpty;
	}
DataPlot::DataPlot(HistogramControl* parent,TableLimits* tbl,
	    TypeAxis typeX,TypeAxis typeY,
	    TypeAxis typeZ,TypeAxis typeMinMax):
	   	parent(parent),tbl(tbl),
        typeX(typeX),typeY(typeX),
		typeZ(typeZ),typeMinMax(typeMinMax)
	{  

	in = parent->GetFstream();
	indexCat = parent->GetIndexCat();
	//eps = acuracyCompareFloatValue();
	table = new float**[tbl->numIntervalZ];
	for (int i = 0; i < tbl->numIntervalZ; ++i){
		table[i] = new float*[tbl->numIntervalY];
		for (int j = 0; j < tbl->numIntervalY; ++j){
			table[i][j] = new float [tbl->numIntervalX];
		}
	}
	
	HistogramControl::TypeHist TypeHist = parent->GetTypeHist();
	
	if (TypeHist == HistogramControl::Min) {
//QMessageBox::warning(0, "","HistogramControl::Min");
	for (int k = 0; k < tbl->numIntervalZ; ++k)
	for (int i = 0; i < tbl->numIntervalY; ++i)
		for (int j = 0; j < tbl->numIntervalX; ++j)
	         table[k][i][j] = FLT_MAX;
		
	tableX =  new float*[tbl->numIntervalZ];
	for (int i = 0; i < tbl->numIntervalZ; ++i)
		tableX[i] = new float [tbl->numIntervalX];
		
	for (int k = 0; k < tbl->numIntervalZ; ++k)
	for (int i = 0; i < tbl->numIntervalX; ++i)
		tableX[k][i] = FLT_MAX;
	
	tableY =  new float*[tbl->numIntervalZ];
	for (int i = 0; i < tbl->numIntervalZ; ++i)
		tableY[i] = new float [tbl->numIntervalY];
	
	for (int k = 0; k < tbl->numIntervalZ; ++k)
	for (int i = 0; i < tbl->numIntervalY; ++i)
		tableY[k][i] = FLT_MAX;	
	
	}
	if (TypeHist == HistogramControl::Max) {
//QMessageBox::warning(0, "","HistogramControl::Max");

	for (int k = 0; k < tbl->numIntervalZ; ++k)
	for (int i = 0; i < tbl->numIntervalY; ++i)
		for (int j = 0; j < tbl->numIntervalX; ++j)
	         table[k][i][j] = -FLT_MAX;
		
	tableX =  new float*[tbl->numIntervalZ];
	for (int i = 0; i < tbl->numIntervalZ; ++i)
		tableX[i] = new float [tbl->numIntervalX];
		
	for (int k = 0; k < tbl->numIntervalZ; ++k)
	for (int i = 0; i < tbl->numIntervalX; ++i)
		tableX[k][i] = -FLT_MAX;
	
	tableY =  new float*[tbl->numIntervalZ];
	for (int i = 0; i < tbl->numIntervalZ; ++i)
		tableY[i] = new float [tbl->numIntervalY];
	
	for (int k = 0; k < tbl->numIntervalZ; ++k)
	for (int i = 0; i < tbl->numIntervalY; ++i)
		tableY[k][i] = -FLT_MAX;	
	
	}
	if (TypeHist == HistogramControl::Frequency || TypeHist == HistogramControl::Weight ) {

	for (int k = 0; k < tbl->numIntervalZ; ++k)
	for (int i = 0; i < tbl->numIntervalY; ++i)
		for (int j = 0; j < tbl->numIntervalX; ++j)
	         table[k][i][j] = 0;
		
	tableX =  new float*[tbl->numIntervalZ];
	for (int i = 0; i < tbl->numIntervalZ; ++i)
		tableX[i] = new float [tbl->numIntervalX];
		
	for (int k = 0; k < tbl->numIntervalZ; ++k)
	for (int i = 0; i < tbl->numIntervalX; ++i)
		tableX[k][i] = 0;
	
	tableY =  new float*[tbl->numIntervalZ];
	for (int i = 0; i < tbl->numIntervalZ; ++i)
		tableY[i] = new float [tbl->numIntervalY];
	
	for (int k = 0; k < tbl->numIntervalZ; ++k)
	for (int i = 0; i < tbl->numIntervalY; ++i)
		tableY[k][i] = 0;	
	}

	numMin  = new float [tbl->numIntervalZ];
	numMax  = new float [tbl->numIntervalZ];
	numMinX = new float [tbl->numIntervalZ];
	numMaxX = new float [tbl->numIntervalZ];
	numMinY = new float [tbl->numIntervalZ];
	numMaxY = new float [tbl->numIntervalZ];
	isEmpty = new bool [tbl->numIntervalZ];
	ItrMultiMap pos = indexCat->begin();
	RECORD record;
	Catalog cat;
    in->clear();

    in->seekg(0,ios::beg);
    in->read((char*)&numberRecords,sizeof(int));
	in->seekg(sizeof(RECORD),ios::beg);
	float lat,lon,dep,mb,ms,ml,mp;
	int numRec = 0;
	
//---------------------------------------------------------
	float  coeffa  = parent->GetCoeffWeightA();
	float  coeffb  = parent->GetCoeffWeightB();
	QString magWeight = parent->GetMagnitudeWeight();

    QApplication::setOverrideCursor(Qt::WaitCursor);
	//QMessageBox::warning(0, "",
	//QString("DataPlot::DataPlot  0 minx  %1 maxx %2 miny %3 maxy %4 nx %5 ny %6").
	//arg(tbl->minX).arg(tbl->maxX).arg(tbl->minY).arg(tbl->maxY).
	//arg(tbl->numIntervalX).arg(tbl->numIntervalY)); 
    for(;;) {
      in->read((char*)&record,sizeof(RECORD));
      if(in->eof())  break;
      cat.GetLatLonDepthMagnitude(record,lat,lon,dep,mb,ms,ml,mp);
	  int i = 0;
	  int j = 0;
	  int k = 0;
	  double y = 0;
	  double x = 0;
	  double z = 0;
	  double mw = 0;
	  double qm = 0;
	  float tm =0.0;
	  
		  if(typeMinMax == Depth) qm = dep;
		  if(typeMinMax == M1) qm = mb;
		  if(typeMinMax == M2) qm = ms;
		  if(typeMinMax == M3) qm = ml;
		  if(typeMinMax == M4) qm = mp;
if (qm < 0) qm = 0;
		  if(typeZ == Years) {GetYearIntYearsFloat(pos->first,tm); z = (double)tm;}
		  if(typeZ == Latitude) z = lat;
		  if(typeZ == Longitude) z = lon;
		  if(typeZ == Depth) z = dep;
		  if(typeZ == M1) z = mb;
		  if(typeZ == M2) z = ms;
		  if(typeZ == M3) z = ml;
		  if(typeZ == M4) z = mp;


	if(typeX == Years) { GetYearIntYearsFloat(pos->first,tm); x = (double)tm;}
	if(typeX == Latitude) x = lat;
	if(typeX == Longitude) x = lon;
	if(typeX == Depth) x = dep;
	if(typeX == M1) x = mb;
	if(typeX == M2) x = ms;
	if(typeX == M3) x = ml;
	if(typeX == M4) x = mp;
	
	
	if(typeY == Years) { GetYearIntYearsFloat(pos->first,tm); y = (double)tm;}
	if(typeY == Latitude) y = lat;
	if(typeY == Longitude) y = lon;
	if(typeY == Depth) y = dep;
	if(typeY == M1) y = mb;
	if(typeY == M2) y = ms;
	if(typeY == M3) y = ml;
	if(typeY == M4) y = mp;
 
	if(magWeight == "m1") mw = mb;
	if(magWeight == "m2") mw = ms;
	if(magWeight == "m3") mw = ml;
	if(magWeight == "m4") mw = mp;
	  i = (int)( ( ( (y-tbl->minY))/(tbl->maxY - tbl->minY))*(tbl->numIntervalY)); //!!
	  j = (int)( ( ( (x-tbl->minX))/(tbl->maxX - tbl->minX))*(tbl->numIntervalX)); //!!
	// QMessageBox::warning(0, "",QString("i = %1 j = %2").arg(i).arg(j ));
	if( i >= 0 && j >= 0 && i < tbl->numIntervalY && j <  tbl->numIntervalX ){
	 numRec++;
	  if (tbl->numIntervalZ == 1) {
		  if (TypeHist == HistogramControl::Weight)
			  table[0][i][j] += pow(10.0,coeffa + coeffb * mw); 
		  if (TypeHist == HistogramControl::Frequency) table[0][i][j]++; 
		  if (TypeHist == HistogramControl::Min) 
			  if ( qm < table[0][i][j] ) table[0][i][j] = qm;
		  if (TypeHist == HistogramControl::Max) 
			  if ( qm > table[0][i][j] ) table[0][i][j] = qm;
	  }
      else { 
		  ///    - eps
		  k =  (int)((((z - tbl->minZ) )/(tbl->maxZ - tbl->minZ))*(tbl->numIntervalZ)); //!!
		  if ( k >= 0 && k < tbl->numIntervalZ ) {
		  if (TypeHist == HistogramControl::Weight)
			  table[k][i][j] += pow(10.0,coeffa + coeffb * mw); 
		  if (TypeHist == HistogramControl::Frequency) table[k][i][j]++;
		  if (TypeHist == HistogramControl::Min) 
			  if ( qm < table[k][i][j] ) table[k][i][j] = qm;
		  if (TypeHist == HistogramControl::Max) 
			  if ( qm > table[k][i][j] ) table[k][i][j] = qm;
		  }
		  
	  }
	}
	   pos++;
   }
	in->clear();
	
	for (int k = 0; k < tbl->numIntervalZ; ++k) isEmpty[k] = false;
	QApplication::restoreOverrideCursor();
	
	for (int k = 0; k < tbl->numIntervalZ; ++k) {
		float minT = FLT_MAX;
		float maxT = -FLT_MAX;
		numMinX[k] = FLT_MAX;
        numMaxX[k] = -FLT_MAX;
        numMinY[k] = FLT_MAX;
        numMaxY[k] = -FLT_MAX;
		if (TypeHist == HistogramControl::Min){
			for (int i = 0; i < tbl->numIntervalY; ++i) 
			for (int j = 0; j < tbl->numIntervalX; ++j)
				if (table[k][i][j] > FLT_MAX * 0.9) table[k][i][j] = 0;
			for (int i = 0; i < tbl->numIntervalY; ++i) 
			for (int j = 0; j < tbl->numIntervalX; ++j) {
				if ( table[k][i][j] > maxT  ) maxT = table[k][i][j];
				if ( table[k][i][j] < minT  ) minT = table[k][i][j];
				if (table[k][i][j] > 0){
					if ( table[k][i][j] < tableX[k][j] ) tableX[k][j] = table[k][i][j];
					if ( table[k][i][j] < tableY[k][i] ) tableY[k][i] = table[k][i][j];
				}
			}
			for (int i = 0; i < tbl->numIntervalY; ++i) 
				if (tableY[k][i] > FLT_MAX * 0.9) tableY[k][i] = 0;
			for (int j = 0; j < tbl->numIntervalX; ++j) 
				if (tableX[k][j] > FLT_MAX * 0.9) tableX[k][j] = 0;
		}
		
		if (TypeHist == HistogramControl::Max){
			for (int i = 0; i < tbl->numIntervalY; ++i) 
			for (int j = 0; j < tbl->numIntervalX; ++j)
				if (table[k][i][j] < -FLT_MAX * 0.9) table[k][i][j] = 0;
			for (int i = 0; i < tbl->numIntervalY; ++i) 
			for (int j = 0; j < tbl->numIntervalX; ++j) {
				if ( table[k][i][j] > maxT  ) maxT = table[k][i][j];
				if ( table[k][i][j] < minT  ) minT = table[k][i][j];
				if ( table[k][i][j] > tableX[k][j] ) tableX[k][j] = table[k][i][j];
				if ( table[k][i][j] > tableY[k][i] ) tableY[k][i] = table[k][i][j];
			}
		}
				
		if (TypeHist == HistogramControl::Frequency || TypeHist == HistogramControl::Weight ) {
			for (int i = 0; i < tbl->numIntervalY; ++i) 
			for (int j = 0; j < tbl->numIntervalX; ++j) {
				if ( table[k][i][j] > maxT  ) maxT = table[k][i][j];
				if ( table[k][i][j] < minT  ) minT = table[k][i][j];
				tableX[k][j] += table[k][i][j];
				tableY[k][i] += table[k][i][j];
			}
		}


		for (int i = 0; i < tbl->numIntervalX; ++i) {
		if ( tableX[k][i] > numMaxX[k]  ) numMaxX[k] = tableX[k][i];
		if ( tableX[k][i] < numMinX[k]  ) numMinX[k] = tableX[k][i];
		}
		
		for (int i = 0; i < tbl->numIntervalY; ++i) {
		if ( tableY[k][i] > numMaxY[k]  ) numMaxY[k] = tableY[k][i];
		if ( tableY[k][i] < numMinY[k]  ) numMinY[k] = tableY[k][i];
		}
	numMin[k]       = minT;
	numMax[k]       = maxT;

	if (numMin[k] == 0 &&  numMax[k] == 0) isEmpty[k] = true;
	}
	
	maxMaximore  = -FLT_MAX;
	minMinimore  = FLT_MAX;
	
	minMinimoreX = FLT_MAX;
	maxMaximoreX = -FLT_MAX;
	minMinimoreY = FLT_MAX;
	maxMaximoreY = -FLT_MAX;
	for (int k = 0; k < tbl->numIntervalZ; ++k) {
	        if ( numMax[k] > maxMaximore ) maxMaximore = numMax[k];
	        if ( numMin[k] < minMinimore ) minMinimore = numMin[k];
			
	        if ( numMaxX[k] > maxMaximoreX ) maxMaximoreX = numMaxX[k];
	        if ( numMinX[k] < minMinimoreX ) minMinimoreX = numMinX[k];
			
	        if ( numMaxY[k] > maxMaximoreY ) maxMaximoreY = numMaxY[k];
	        if ( numMinY[k] < minMinimoreY ) minMinimoreY = numMinY[k];
	}

	if(typeX == Years)      titleX = "Years";
	if(typeX == Latitude)   titleX = "Latitude";
	if(typeX == Longitude)  titleX = "Longitude";
	if(typeX == Depth)      titleX = "Depth";
	if(typeX == M1)         titleX = "Magnitude m1";
	if(typeX == M2)         titleX = "Magnitude m2";
	if(typeX == M3)         titleX = "Magnitude m3";
	if(typeX == M4)         titleX = "Magnitude m4";
	
	
	if(typeY == Years)     titleY = "Years";
	if(typeY == Latitude)  titleY = "Latitude";
	if(typeY == Longitude) titleY = "Longitude";
	if(typeY == Depth)     titleY = "Depth";
	if(typeY == M1)        titleY = "Magnitude m1";
	if(typeY == M2)        titleY = "Magnitude m2";
	if(typeY == M3)        titleY = "Magnitude m3";
	if(typeY == M4)        titleY = "Magnitude m4";
	
	if(typeZ == Years)     titleZ = "Years";
	if(typeZ == Latitude)  titleZ = "Latitude";
	if(typeZ == Longitude) titleZ = "Longitude";
	if(typeZ == Depth)     titleZ = "Depth";
	if(typeZ == M1)        titleZ = "Magnitude m1";
	if(typeZ == M2)        titleZ = "Magnitude m2";
	if(typeZ == M3)        titleZ = "Magnitude m3";
	if(typeZ == M4)        titleZ = "Magnitude m4";
	

	if(typeMinMax == Depth)     titleTypeMinMax = "Depth";
	if(typeMinMax == M1)        titleTypeMinMax = "Magnitude m1";
	if(typeMinMax == M2)        titleTypeMinMax = "Magnitude m2";
	if(typeMinMax == M3)        titleTypeMinMax = "Magnitude m3";
	if(typeMinMax == M4)        titleTypeMinMax = "Magnitude m4";
	}
///=========================================================================



