#ifndef _DRAW_HISTOGRAM_DIALOG_H
#define _DRAW_HISTOGRAM_DIALOG_H
#include <QLineEdit>
#include <QList>
#include <QObject>
#include <qmainwindow.h>
#include <qwt_plot.h>
#include <qwt_plot_spectrogram.h>
#include <qwt_plot_zoomer.h>
#include "ui_histogram.h"
#include <fstream>
#include <map>
#include <vector>
using namespace std;
class QwtPlotPicker;
class QwtPlotPanner;
class PropertyValue;
class QWidget;
class QSpinBox;
class QSplitter;
class QGridLayout;
class QHBoxLayout;
class QPushButton;
class QSpacerItem;
class LineDocProperty;
class MainWindowPlot;
class Plot;
class HistogramControl;
class FDocument;
class SettingDocument;
class LineDocProperty;
class PropertyFloatValue;
class HelpWindow;
class DataPlot;
class TableLimits;
class HistogramItem;
class SpectrogramData;
//==========================================================================
class HistogramSettings : public QDialog
{
    Q_OBJECT
public:
   HistogramSettings(FDocument* doc,SettingDocument* setDoc,QWidget *parent);
   ~HistogramSettings();
 MainWindowPlot* GetWindowPlot(){return winPlot;}  
 HistogramControl* GetHistogramControl() { return  control; }
 QSplitter *GetSlitter(){ return splitter;}
protected:
    void closeEvent(QCloseEvent *event);
protected slots:
    void accept();
	void reject();
    void helpButton();
private:
	QSplitter * splitter;
	HistogramControl* control;
	MainWindowPlot* winPlot;
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QPushButton *pushButtonHelp;
    QSpacerItem *spacerItem;
    QPushButton *pushButtonOk;
    QPushButton *pushButtonCancel;
	
	FDocument* doc;
    SettingDocument* setDoc;
    HelpWindow* help;
};
///============================================================================
class HistogramControl : public QDialog, private Ui::HistogramControlBase
{
    Q_OBJECT 
	    typedef multimap<double,int> DoubleIntMultiMap;
		typedef DoubleIntMultiMap::iterator ItrMultiMap;
		typedef vector<PropertyValue* > PropValVector;
		typedef vector<PropertyValue* >::iterator PropValIterator; 
public:
    enum TypeHist
    {
        Frequency,
        Min,
        Max,
        Weight
    };
    HistogramControl(FDocument* doc,SettingDocument* setDoc,QWidget *parent);
    ~HistogramControl();
	fstream* GetFstream(){return in;}
	DoubleIntMultiMap* GetIndexCat() {return  indexCat;}
	TableLimits* GetLimits(){ return tbl;}
	DataPlot* GetDataPlot() { return dataPlot; }
	void GetInitialSpectrogram(){GetSpectrogram();}
	void SaveAcceptValues();
	QToolButton *GetButtonZoom (){return toolButtonZoom;}
	bool IsSpectrogram() { return  isSpectrogramDrawed; }
	bool IsHistogramX()  { return  isHistogramXDrawed;  }
	bool IsHistogramY()  { return  isHistogramYDrawed;  }
	QString &GetDateMin() { return labelYearsMin;}
	QString &GetDateMax() { return labelYearsMax;}
	TypeHist GetTypeHist(){ return type;}
	QString GetTypeWeight(){ return typeWeight;}
	float  GetCoeffWeightA() { return lineEditWeightA->text().toFloat();}
	float  GetCoeffWeightB() { return lineEditWeightB->text().toFloat();}
	QString GetMagnitudeWeight() { return magnitudeWeight;}
	QString GetParHistMinMax() { return parHistMinMax;}

	
protected:
protected slots:
	
private slots:
    void toggledButtonWeightOn(bool);
    void toggledButtonMinOn(bool);
    void toggledButtonMaxOn(bool);
    void enableZoomMode(bool on);
    void print();
    void exportPDF();
    void exportCSV();
    void GetSpectrogram();
    void GetHistogramX();
    void GetHistogramY();
    void EnableByListMode(bool on);
    void EnableCommonScale(bool on);
    void ChangedComboBoxX(const QString & text );
    void ChangedComboBoxY(const QString & text );
    void ChangedComboBoxZ(const QString & text );
    void ChangedComboBoxMinMax(const QString & text );
	bool CheckParametersForDraw();
	bool CheckQLineEdit(QLineEdit* linmin,float min,float max,QString name);
	void SetTextQLineEdit(QLineEdit* lin,bool bold,QString color);
	void DrawPrevImage();
	void DrawNextImage();
    void onWhatsThis();
    void SetWeightCoefficentsToLineEditBoxes(int index);
private:
	TypeHist type;
	QAction *whatsThisAct;
	void SetToolStatusTips();
	void InitDialog();
	
	bool GetParametersLimit();
	HistogramSettings* windowParent;
	FDocument* doc;
    SettingDocument* setDoc;
	QList<LineDocProperty* > propDocLine;
	fstream* in;
	int numberRecords;
	DoubleIntMultiMap *indexCat;

    PropertyFloatValue* 	propYearsMin[3];
    PropertyFloatValue* 	propYearsMax[3];
    PropertyFloatValue* 	propLatitudeMin[3];
    PropertyFloatValue* 	propLatitudeMax[3];
    PropertyFloatValue* 	propLongitudeMin[3];
    PropertyFloatValue* 	propLongitudeMax[3];
    PropertyFloatValue* 	propDepthMin[3];
    PropertyFloatValue* 	propDepthMax[3];
    PropertyFloatValue* 	propM1Min[3];
    PropertyFloatValue* 	propM1Max[3];
    PropertyFloatValue* 	propM2Min[3];
    PropertyFloatValue* 	propM2Max[3];
    PropertyFloatValue* 	propM3Min[3];
    PropertyFloatValue* 	propM3Max[3];
    PropertyFloatValue* 	propM4Min[3];
    PropertyFloatValue* 	propM4Max[3];

	PropertyFloatValue*     propEditWeightA;
	PropertyFloatValue*     propEditWeightB;
	
	
	LineDocProperty* linePropAxisXMinX;
	LineDocProperty* linePropAxisXMaxX;
	LineDocProperty* linePropAxisYMinY;
	LineDocProperty* linePropAxisYMaxY;
	LineDocProperty* linePropAxisZMinZ;
	LineDocProperty* linePropAxisZMaxZ;
	
	LineDocProperty* linePropEditWeightA;
	LineDocProperty* linePropEditWeightB;

	QString labelYearsMin;
	QString labelLatitudeMin;
	QString labelLongitudeMin;
	QString labelDepthMin;
	QString labelM1Min;
	QString labelM2Min;
	QString labelM3Min;
	QString labelM4Min;

	QString labelYearsMax;
	QString labelLatitudeMax;
	QString labelLongitudeMax;
	QString labelDepthMax;
	QString labelM1Max;
	QString labelM2Max;
	QString labelM3Max;
	QString labelM4Max;
//---------------------------------------	
	float YearsMin;
	float LatitudeMin;
	float LongitudeMin;
	float DepthMin;
	float M1Min;
	float M2Min;
	float M3Min;
	float M4Min;

	float YearsMax;
	float LatitudeMax;
	float LongitudeMax;
	float DepthMax;
	float M1Max;
	float M2Max;
	float M3Max;
	float M4Max;
	
	float vldYearsMin;
	float vldLatitudeMin;
	float vldLongitudeMin;
	float vldDepthMin;
	float vldM1Min;
	float vldM2Min;
	float vldM3Min;
	float vldM4Min;

	float vldYearsMax;
	float vldLatitudeMax;
	float vldLongitudeMax;
	float vldDepthMax;
	float vldM1Max;
	float vldM2Max;
	float vldM3Max;
	float vldM4Max;
	
	float valYearsMin;
	float valLatitudeMin;
	float valLongitudeMin;
	float valDepthMin;
	float valM1Min;
	float valM2Min;
	float valM3Min;
	float valM4Min;

	float valYearsMax;
	float valLatitudeMax;
	float valLongitudeMax;
	float valDepthMax;
	float valM1Max;
	float valM2Max;
	float valM3Max;
	float valM4Max;

	QString axisParameterX;
	QString axisParameterY;
	QString axisParameterZ;
	
	PropValVector values;
	TableLimits* tbl;
	DataPlot* dataPlot;
	bool isSpectrogramDrawed;
	bool isHistogramXDrawed;
	bool isHistogramYDrawed;
	
//-----------------------------------------------------------------------
	QString axisX;
	QString axisY;
	QString axisZ;
	QString minX;
	QString minY;
	QString minZ;
	QString maxX;
	QString maxY;
	QString maxZ;
	QString coeffWeightCustomA;
	QString coeffWeightCustomB;
	QString typeWeight;
	QString magnitudeWeight;
	QString parHistMinMax;
	int intervalX;
	int intervalY;
	int intervalZ;
	
	bool isByList;
	float customA;
	float customB;
	int previousTypeWeightIndex;
//-----------------------------------------------------------------------
};

///=========================================================================
class Plot: public QwtPlot
{
    Q_OBJECT

public:
	enum TypeZScale
    {
      Log,
	  Normal
    };
    Plot(QWidget * parent,DataPlot* dataPlot,TypeZScale scale);
   ~Plot();
    void DrawCurrentSpectrogram(int currentNumber);
    QwtPlotSpectrogram *d_spectrogram;
	int &GetCurrentIntervalZ() {return currenIntervalZ;}
public slots:


private:
	DataPlot* dataPlot;
	TypeZScale scale;
	QWidget * parent;
	int currenIntervalZ;
	bool isCommonScale;
};
///=========================================================================
class SpectrogramData: public QwtRasterData
{
public:
    SpectrogramData(int currentInterval,DataPlot* data,Plot::TypeZScale scale,bool isCommonScale);

    virtual QwtRasterData *copy() const;

    virtual QwtDoubleInterval range() const;

    virtual double value(double x, double y) const;
private:
int currentInterval;
DataPlot* data;
Plot::TypeZScale scale;
bool isCommonScale;

};
class HistPlot: public QwtPlot
{
    Q_OBJECT

public:
	enum TypeHist
    {
      HistX,
	  HistY
    };
	enum TypeYScale
    {
      Log,
	  Normal
    };
	~HistPlot();
    HistPlot(QWidget * parent,DataPlot* dataPlot,TypeHist type,TypeYScale scale);
	void DrawCurrentHistogram(int currenIntervalZ);
	int &GetCurrentIntervalZ() {return currenIntervalZ;}
public slots:

private:
	TypeHist type;
	TypeYScale scale;
	DataPlot* dataPlot;
	QWidget * parent;
	int currenIntervalZ;
	HistogramItem *histogram;
	bool isCommonScale;
};
///=========================================================================
class Zoomer: public QwtPlotZoomer
{
public:
    Zoomer(int xAxis, int yAxis, QwtPlotCanvas *canvas):
        QwtPlotZoomer(xAxis, yAxis, canvas)
    {
        setSelectionFlags(QwtPicker::DragSelection | QwtPicker::CornerToCorner);
        setTrackerMode(QwtPicker::AlwaysOff);
        setRubberBand(QwtPicker::NoRubberBand);

        // RightButton: zoom out by 1
        // Ctrl+RightButton: zoom out to full size

        setMousePattern(QwtEventPattern::MouseSelect2,
            Qt::RightButton, Qt::ControlModifier);
        setMousePattern(QwtEventPattern::MouseSelect3,
            Qt::RightButton);
    }
};
///=========================================================================
class MainWindowPlot: public QMainWindow
{

    Q_OBJECT

public:
	enum LogScale
    {
      Yes,
	  No
    };
	~MainWindowPlot();
    MainWindowPlot(QWidget *parent);
    void enableZoomMode(bool);
	void print();
    void exportPDF();
    void exportCSV(DataPlot* dataPlot);
	void GetSpectrogram(DataPlot* dataPlot,LogScale scale);
	void GetHistogramX(DataPlot* dataPlot,LogScale scale);
	void GetHistogramY(DataPlot* dataPlot,LogScale scale);
    QWidget * GetWindowParent () {return parent;}
	LogScale GetLogScale() { return logscale; }
	Plot* GetSpectrogramPlot () { return d_plot; }
	HistPlot* GetHistogramPlot () { return h_plot;}
	void enableCommonScale(bool on) {isCommonScale = on;}
	bool IsCommonScale(){return isCommonScale;}
private slots:
    void moved(const QPoint &);
    void selected(const QwtPolygon &);
private:
    void showInfo(QString text = QString::null);

    Plot *d_plot;
	HistPlot* h_plot;
    QwtPlotZoomer *d_zoomer;
    QwtPlotPicker *d_picker;
    QwtPlotPanner *d_panner;	
	QWidget * parent;
	DataPlot* datPlot;
	LogScale logscale;
	bool isCommonScale;
};
///=========================================================================
class TableLimits
{
public:
	int numIntervalX;
	float minX;
	float maxX;
	int numIntervalY;
	float minY;
	float maxY;
	int numIntervalZ;
	float minZ;
	float maxZ;
	TableLimits(int nx, float minx,float maxx,
	                int ny, float miny,float maxy,
					int nz = 1,float minz = 0,float maxz = 0);

};

///=========================================================================
class DataPlot
{
 	    typedef multimap<double,int> DoubleIntMultiMap;
		typedef DoubleIntMultiMap::iterator ItrMultiMap;
public:
	enum TypeAxis
    {
      Years,
	  Latitude,
	  Longitude,
	  Depth,
	  M1,
	  M2,
	  M3,
	  M4
    };
    DataPlot(HistogramControl* parent,TableLimits* tbl,TypeAxis typeX,TypeAxis typeY,
	TypeAxis typeZ,TypeAxis typeMinMax);
   ~DataPlot();
   TypeAxis GetTypeAxisX () {return typeX;}
   TypeAxis GetTypeAxisY () {return typeY;}
   TypeAxis GetTypeAxisZ () {return typeZ;}
   TypeAxis GetTypeMinMax() {return typeMinMax;}
   int GetIsMinMax       () {return isMinMax;}
   QString GetTitleX     () {return titleX;}
   QString GetTitleY     () {return titleY;}
   QString GetTitleZ     () {return titleZ;}
   QString GetTitleTypeMinMax     () {return titleTypeMinMax;}
   
   
	float*  GetTableMin  () { return  numMin; }
	float*  GetTableMax  () { return  numMax; }
	float GeMinimumMinimore () {return minMinimore;}
	float GeMaximumMaximore () {return maxMaximore;}
	bool*  GetTableIsEmpty() { return  isEmpty; }
    float *** GetTable(){ return table;}
   	float ** GetTableX(){ return tableX;}
	float ** GetTableY(){ return tableY;}

	float * GetTableMinX() { return  numMinX;  }
	float * GetTableMaxX() { return  numMaxX; }
	float * GetTableMinY() { return  numMinY; }
	float * GetTableMaxY() { return  numMaxY;}	
	float GeMinimumMinimoreX () {return minMinimoreX;}
	float GeMaximumMaximoreX () {return maxMaximoreX;}
	float GeMinimumMinimoreY () {return minMinimoreY;}
	float GeMaximumMaximoreY () {return maxMaximoreY;}
   TableLimits* GetLimits(){ return tbl;}
private:
	float ***table;
	float **tableX;
	float **tableY;
	float *numMin;
	float *numMax;
	float *numMinX;
	float *numMaxX;
	float *numMinY;
	float *numMaxY;
	float minMinimore;
	float maxMaximore;
	float minMinimoreX;
	float maxMaximoreX;
	float minMinimoreY;
	float maxMaximoreY;
	bool *isEmpty;
	fstream* in;
	DoubleIntMultiMap *indexCat;
	float eps;
	HistogramControl* parent;
	TableLimits* tbl;
    TypeAxis typeX;
    TypeAxis typeY;
	TypeAxis typeZ;
	TypeAxis typeMinMax;
	int isMinMax; // 0 - none, 1-min,2-max
	QString titleX;
	QString titleY;
	QString titleZ;
	QString titleTypeMinMax;
	int numberRecords;


	
};
///=========================================================================

#endif
