#include <QtGui> 
#include <QStringList>
#include <qapplication.h>
#include "mainwindow.h" 
#include "default.h" 
#include "editline.h"
#include "helpwindow.h"
#include "document.h"
#include "float.h"
#include "viewcatalogimpl.h"
#include "catalog.h"
#include "histogram.h"

HistogramSettings::~HistogramSettings(){
delete control;
delete winPlot;	
if(help !=0) {delete help; help = 0;}
}
HistogramSettings::HistogramSettings(FDocument* doc,SettingDocument* setDoc,QWidget *parent)
      : QDialog(0),doc(doc),setDoc(setDoc)
{

	QFont f = parent->font();
    setFont( parent->font() );
	setWindowIcon(QIcon(":/images/icon.png"));
    gridLayout = new QGridLayout(this);
    gridLayout->setSpacing(6);
    gridLayout->setMargin(9);
    gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
    splitter = new QSplitter(this);
	splitter->setOrientation (Qt::Horizontal);
    control   = new HistogramControl(doc,setDoc,this);
    winPlot  = new MainWindowPlot(this);
    splitter->addWidget(control);
    splitter->addWidget(winPlot);

    control->GetInitialSpectrogram();

    help = 0;
    hboxLayout = new QHBoxLayout();
    hboxLayout->setSpacing(6);
    hboxLayout->setMargin(0);
    hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
    pushButtonHelp = new QPushButton(this);
    pushButtonHelp->setObjectName(QString::fromUtf8("pushButtonHelp"));

    hboxLayout->addWidget(pushButtonHelp);

    spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

    hboxLayout->addItem(spacerItem);

    pushButtonOk = new QPushButton(this);
    pushButtonOk->setObjectName(QString::fromUtf8("pushButtonOk"));

    hboxLayout->addWidget(pushButtonOk);

    pushButtonCancel = new QPushButton(this);
    pushButtonCancel->setObjectName(QString::fromUtf8("pushButtonCancel"));
    hboxLayout->addWidget(pushButtonCancel);

    gridLayout->addWidget(splitter, 0, 0, 1, 1);
    gridLayout->addLayout(hboxLayout, 1, 0, 1, 1);
	QString tmp = MainWindow::self->GetCurrentCatalogName();
	if (tmp.isEmpty()) tmp = "unnamed";
	else tmp = MainWindow::self->strippedName(tmp);
	setWindowTitle(QString("%1 - Draw Histogram Panel").
		arg(tmp));
    pushButtonHelp->setText(QApplication::translate("Dialog", "Help", 0, QApplication::UnicodeUTF8));
    pushButtonOk->setText(QApplication::translate("Dialog", "OK", 0, QApplication::UnicodeUTF8));
    pushButtonCancel->setText(QApplication::translate("Dialog", "Cancel", 0, QApplication::UnicodeUTF8));

    connect( pushButtonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( pushButtonCancel, SIGNAL( clicked() ), this, SLOT( close() ) );
    connect( pushButtonHelp, SIGNAL( clicked() ), this, SLOT( helpButton() ) );

 //    resize(MainWindow::self->projectSettingSize);
 //    move(MainWindow::self->projectSettingPos);

}
//----------------------------------------------------------------------------
void HistogramSettings::helpButton(){
     QString home = searchHelpPath() + "/edcat-tool-histogram.html";
    if(help !=0) {delete help; help =0;}
    help = new HelpWindow(home,this);
    if ( QApplication::desktop()->width() > 400
         && QApplication::desktop()->height() > 500 )
         help->show();
    else
        help->showMaximized();	
}
//----------------------------------------------------------------------------
void HistogramSettings::accept(){

	control->SaveAcceptValues();
	done( Accepted );
}
//----------------------------------------------------------------------------
void HistogramSettings::reject()
{
return;
}
void HistogramSettings::closeEvent(QCloseEvent *event)
{
	event->accept();
}
///=========================================================================
HistogramControl::~HistogramControl(){
if(!propDocLine.empty()) {
foreach (LineDocProperty* prop, propDocLine) {delete prop;prop = 0;}
propDocLine.clear();
}
  for_each(values.begin(),values.end(),delAndNullify);
  values.erase(remove(values.begin(),values.end(),
               static_cast<PropertyValue*>(0)),
               values.end());
  if ( tbl != 0 ) delete tbl;
  if ( dataPlot != 0 ) delete dataPlot;
  
} 
HistogramControl::HistogramControl(FDocument* doc,SettingDocument* setDoc,QWidget *parent)
      : QDialog(0),doc(doc),setDoc(setDoc)

{
    QFont f = parent->font();
    setFont( parent->font() );
    setupUi(this); 
	tbl = 0;
	dataPlot = 0;
    windowParent = (HistogramSettings*)parent;
	
	connect(toolButtonZoom, SIGNAL(toggled(bool)),SLOT(enableZoomMode(bool)));
    connect(toolButtonPrint, SIGNAL(clicked()), SLOT(print()));
    connect(toolButtonPdf, SIGNAL(clicked()), SLOT(exportPDF()));
    connect(toolButtonCsv, SIGNAL(clicked()), SLOT(exportCSV()));
    connect(toolButtonSpectr, SIGNAL(clicked()), SLOT(GetSpectrogram()));
    connect(toolButtonHistX, SIGNAL(clicked()), SLOT(GetHistogramX()));
    connect(toolButtonHistY, SIGNAL(clicked()), SLOT(GetHistogramY()));
    connect(toolButtonHistMulty, SIGNAL(toggled(bool)), SLOT(EnableByListMode(bool)));

    connect(toolButtonScale, SIGNAL(toggled(bool)), SLOT(EnableCommonScale(bool)));

    connect(comboBoxX, SIGNAL(currentIndexChanged ( const QString &  )), 
			SLOT(ChangedComboBoxX(const QString & )));
    connect(comboBoxY, SIGNAL(currentIndexChanged ( const QString &  )), 
			SLOT(ChangedComboBoxY(const QString & )));
    connect(comboBoxZ, SIGNAL(currentIndexChanged ( const QString &  )), 
			SLOT(ChangedComboBoxZ(const QString & )));
    connect(comboBoxMinMax, SIGNAL(currentIndexChanged ( const QString &  )), 
			SLOT(ChangedComboBoxMinMax(const QString & )));
			
    connect(comboBoxWeightType, SIGNAL(currentIndexChanged ( int  )), 
			SLOT(SetWeightCoefficentsToLineEditBoxes(int )));
			

    connect(toolButtonPrev, SIGNAL(clicked()), SLOT(DrawPrevImage()));
    connect(toolButtonNext, SIGNAL(clicked()), SLOT(DrawNextImage()));
    connect(toolButtonWhats, SIGNAL(clicked()), SLOT(onWhatsThis()));
	
	connect(radioButtonWeightOn, SIGNAL(toggled(bool)),SLOT(toggledButtonWeightOn(bool)));
	connect(radioButtonMin, SIGNAL(toggled(bool)),SLOT(toggledButtonMinOn(bool)));
	connect(radioButtonMax, SIGNAL(toggled(bool)),SLOT(toggledButtonMaxOn(bool)));


    SetToolStatusTips();
	if(!GetParametersLimit()){
	
	}
	InitDialog();
}  
void HistogramControl::onWhatsThis()
{
    QWhatsThis::enterWhatsThisMode();
}
//----------------------------------------------------------------------------
void HistogramControl::SetToolStatusTips(){
    whatsThisAct = new QAction(this);
    whatsThisAct->setWhatsThis(tr("Enter What's This? mode."));
    whatsThisAct->setShortcut(Qt::SHIFT + Qt::Key_F1);
	
    toolButtonWhats->addAction (whatsThisAct);
	toolButtonWhats->setToolTip(QString("What's this?"));
	
	
	toolButtonSpectr->setToolTip(QString("Draw 2-D histogram (X, Y, Frequencies of earthquakes) "));
	toolButtonSpectr->setShortcut(tr("Shift+S"));
	const char * toolButtonSpectrText = "<p><img source=\":/images/histXY.png\">  "
				    " Click this button to draw the <em>2-D histogram</em> (X, Y, Frequencies of earthquakes)."
					"A <b>The 2-D histogram </b> displays frequencies as color coded cells"
					"Keyboard shortcut is <b> (Shift+S)</b></p>";
    toolButtonSpectr->setWhatsThis(toolButtonSpectrText);
   
   	toolButtonHistX->setToolTip(QString("Draw 1-D histogram (Axis X, Frequencies of earthquakes)"));
	toolButtonHistX->setShortcut(tr("Shift+X"));
	const char * toolButtonHistXText = "<p><img source=\":/images/histX.png\">  "
				    "Click this button to draw the <em>1-D histogram</em>. (Axis X, Frequencies of earthquakes)"
					"A <b>1-D histogram</b> displays frequencies as bars of different heights."
					"Keyboard shortcut is <b>(Shift+X)</b></p>";
    toolButtonHistX->setWhatsThis(toolButtonHistXText);
   
   	toolButtonHistY->setToolTip(QString("Draw 1-D histogram (Axis Y, Frequencies of earthquakes)"));
	toolButtonHistY->setShortcut(tr("Shift+Y"));
	const char * toolButtonHistYText = "<p><img source=\":/images/histY.png\">  "
				    "Click this button to draw the <em>1-DHistogram</em>. (Axis Y, Frequencies of earthquakes)"
					"A <b>1-D histogram</b> displays frequencies as bars of different heights."
					"Keyboard shortcut is <b>(Shift+Y)</b></p>";
					
    toolButtonHistY->setWhatsThis(toolButtonHistYText);
   
   	toolButtonPrint->setToolTip(QString("Print"));
	toolButtonPrint->setShortcut(tr("Shift+P"));
	const char * toolButtonPrintText = "<p><img source=\":/images/print.png\">  "
				    "Click this button to <em>print the plot</em> to a printer. "   
					"It's used the built-in printer drivers."
					"In addition you can save the plot as a <b>PostScript</b> file "
					"Keyboard shortcut is <b>(Shift+P)</b></p>";
    toolButtonPrint->setWhatsThis(toolButtonPrintText);

   	toolButtonPdf->setToolTip(QString("PDF file"));
	toolButtonPdf->setShortcut(tr("Shift+D"));
	const char * toolButtonPdfText = "<p><img source=\":/images/pdf.png\">  "
				    "Click this button to <em>save the plot</em> as a Portable Document Format (PDF) file. "
					"Keyboard shortcut is <b>(Shift+D)</b></p>";
    toolButtonPdf->setWhatsThis(toolButtonPdfText);
   


   	toolButtonCsv->setToolTip(QString("CSV Exel file"));
	toolButtonCsv->setShortcut(tr("Shift+C"));
	const char * toolButtonCsvText = "<p><img source=\":/images/exel.png\">  "
				    "Click this button to <em>save</em> the plot</em> as "
					"a Comma Separated Values (CSV) Microsoft Exel file. "
					"Keyboard shortcut is <b>(Shift+C)</b></p>";
    toolButtonCsv->setWhatsThis(toolButtonCsvText);
	
	
   	toolButtonZoom->setToolTip(QString("Zoom in a histogram plot."));
	toolButtonZoom->setShortcut(tr("Shift+Z"));
	
	const char * toolButtonZoomText = "<p><img source=\":/images/zoom-in.png\">  "
				    "Toggle this button to <em>zoom</em> in a histogram plot."
					"Zoomer <em>offers</em> rubber band selections on the plot,"
					"translating the selected rectangles into plot coordinates"
					"and adjusting the axes to them."
					"Zooming can <em>operated</em> as often as possible.";
    toolButtonZoom->setWhatsThis(toolButtonZoomText);
	


   	toolButtonLog->setToolTip(QString("Linear/Log10 scale"));
	toolButtonLog->setShortcut(tr("Shift+L"));
	const char * toolButtonLogText = "<p><img source=\":/images/log.png\">  "
				    "Toggle this button to <em>switch</em> a linear/log10 scale."
					"The button is checked  for the decimal logarithm"
					"The button is unchecked  for the linear scale";
    toolButtonLog->setWhatsThis(toolButtonLogText);


   	toolButtonHistMulty->setToolTip(QString("Invoke by List"));
	toolButtonHistMulty->setShortcut(tr("Shift+I"));
	const char * toolButtonHistMultyText = "<p><img source=\":/images/hist.png\">  "
				    "Toggle this button to <em>set<em> the switch for the set of images"
					" by list of selected parameter."
					"The button is checked to set the switch for by list mode. "
					"The button is unchecked to unset the switch for by list mode.";
    toolButtonHistMulty->setWhatsThis(toolButtonHistMultyText);
	
   	toolButtonScale->setToolTip(QString("Switch image scale"));
	toolButtonScale->setShortcut(tr("Shift+W"));
	const char * toolButtonScaleText = "<p><img source=\":/images/scale.png\">  "
				    "Toggle this button to <em>set<em> the switch for an images scale. "
					"The button is checked to set the switch for ????? image scale. "
					"The button is unchecked to unset the switch for ????? mode. ";
    toolButtonScale->setWhatsThis(toolButtonScaleText);

	toolButtonPrev->setToolTip(QString("Back"));
	toolButtonPrev->setShortcut(tr("Shift+B"));
	const char * toolButtonPrevText = "<p><img source=\":/images/top.png\">  "
				    "Click this button to <em>show<em> the previous image from list.";
    toolButtonPrev->setWhatsThis(toolButtonPrevText);

	toolButtonNext->setToolTip(QString("Forward"));
	toolButtonNext->setShortcut(tr("Shift+N"));
	const char * toolButtonNextText = "<p><img source=\":/images/bottom.png\">  "
				    "Click this button to <em>show<em> the next image from list.";
    toolButtonNext->setWhatsThis(toolButtonNextText);

	//toolButtonPrev toolButtonNext

}  
//----------------------------------------------------------------------------
void HistogramControl::SaveAcceptValues(){
	foreach (LineDocProperty *docprop, propDocLine) {
		if (docprop->GetProp()->GetKind()  == PropertyValue::Property_Float)
			docprop->FinishEditLine();
		if (docprop->GetProp()->GetKind()  == PropertyValue::Property_IntSpin)
			docprop->spin->clearFocus();
    }
	int nx = spinBoxIntervalX->value ();
	int ny = spinBoxIntervalY->value ();
	int nz = spinBoxIntervalZ->value ();
	

	setDoc->GetNumberIntervalAxisX()->SetValue(QString("%1").arg(nx));
    setDoc->GetNumberIntervalAxisY()->SetValue(QString("%1").arg(ny));
    setDoc->GetNumberIntervalAxisZ()->SetValue(QString("%1").arg(nz));
	setDoc->GetNumberIntervalAxisX()->SetValueToResetvalue();
    setDoc->GetNumberIntervalAxisY()->SetValueToResetvalue();
    setDoc->GetNumberIntervalAxisZ()->SetValueToResetvalue();
	
}

//----------------------------------------------------------------------------
void HistogramControl::DrawPrevImage(){
int intervalZ = 0;
	int numInterval = 0;
	if (isSpectrogramDrawed) {
		intervalZ = windowParent->GetWindowPlot()->GetSpectrogramPlot()->GetCurrentIntervalZ();
		intervalZ--;
		numInterval = tbl->numIntervalZ;
		if ( intervalZ <  0 ) intervalZ = numInterval - 1;
	    windowParent->GetWindowPlot()->GetSpectrogramPlot()->DrawCurrentSpectrogram(intervalZ);
		windowParent->GetWindowPlot()->GetSpectrogramPlot()->GetCurrentIntervalZ() = intervalZ;
	}
	if (isHistogramXDrawed || isHistogramYDrawed) {
		intervalZ = windowParent->GetWindowPlot()->GetHistogramPlot()->GetCurrentIntervalZ();
		intervalZ--;
		numInterval = tbl->numIntervalZ;
		if ( intervalZ <  0 ) intervalZ = numInterval - 1;
	    windowParent->GetWindowPlot()->GetHistogramPlot()->DrawCurrentHistogram(intervalZ);
		windowParent->GetWindowPlot()->GetHistogramPlot()->GetCurrentIntervalZ() = intervalZ;
	}

}
void HistogramControl::DrawNextImage(){
	int intervalZ = 0;
	int numInterval = 0;
	if (isSpectrogramDrawed) {
		intervalZ = windowParent->GetWindowPlot()->GetSpectrogramPlot()->GetCurrentIntervalZ();
		intervalZ++;
		numInterval = tbl->numIntervalZ;
		if ( intervalZ > numInterval - 1 ) intervalZ = 0;
	    windowParent->GetWindowPlot()->GetSpectrogramPlot()->DrawCurrentSpectrogram(intervalZ);
		windowParent->GetWindowPlot()->GetSpectrogramPlot()->GetCurrentIntervalZ() = intervalZ;
	}
	if (isHistogramXDrawed || isHistogramYDrawed) {
		intervalZ = windowParent->GetWindowPlot()->GetHistogramPlot()->GetCurrentIntervalZ();
		intervalZ++;
		numInterval = tbl->numIntervalZ;
		if ( intervalZ > numInterval - 1 ) intervalZ = 0;
	    windowParent->GetWindowPlot()->GetHistogramPlot()->DrawCurrentHistogram(intervalZ);
		windowParent->GetWindowPlot()->GetHistogramPlot()->GetCurrentIntervalZ() = intervalZ;
	}

}
//-------------------------------------------------------------------------
void HistogramControl::ChangedComboBoxX(const QString & text){
//QMessageBox::warning(0, "",QString("ChangedComboBoxX %1").arg(text) );
	if(text == "Years") {
	linePropAxisXMinX->ReplaceProperty(propYearsMin[0]);
	linePropAxisXMaxX->ReplaceProperty(propYearsMax[0]);
	labelMinimumX->setText(labelYearsMin);
	labelMaximumX->setText(labelYearsMax);
	}
	if(text == "Latitude") {
	linePropAxisXMinX->ReplaceProperty(propLatitudeMin[0]);
	linePropAxisXMaxX->ReplaceProperty(propLatitudeMax[0]);
	labelMinimumX->setText(labelLatitudeMin);
	labelMaximumX->setText(labelLatitudeMax);
	}
	if(text == "Longitude") {
	linePropAxisXMinX->ReplaceProperty(propLongitudeMin[0]);
	linePropAxisXMaxX->ReplaceProperty(propLongitudeMax[0]);
	labelMinimumX->setText(labelLongitudeMin);
	labelMaximumX->setText(labelLongitudeMax);
	}
	if(text == "Depth") {
	linePropAxisXMinX->ReplaceProperty(propDepthMin[0]);
	linePropAxisXMaxX->ReplaceProperty(propDepthMax[0]);
	labelMinimumX->setText(labelDepthMin);
	labelMaximumX->setText(labelDepthMax);
	}
	if(text == "m1") {
	linePropAxisXMinX->ReplaceProperty(propM1Min[0]);
	linePropAxisXMaxX->ReplaceProperty(propM1Max[0]);
	labelMinimumX->setText(labelM1Min);
	labelMaximumX->setText(labelM1Max);
	}
	if(text == "m2") {
	linePropAxisXMinX->ReplaceProperty(propM2Min[0]);
	linePropAxisXMaxX->ReplaceProperty(propM2Max[0]);
	labelMinimumX->setText(labelM2Min);
	labelMaximumX->setText(labelM2Max);
	}
	if(text == "m3") {
	linePropAxisXMinX->ReplaceProperty(propM3Min[0]);
	linePropAxisXMaxX->ReplaceProperty(propM3Max[0]);
	labelMinimumX->setText(labelM3Min);
	labelMaximumX->setText(labelM3Max);
	}
	if(text == "m4") {
	linePropAxisXMinX->ReplaceProperty(propM4Min[0]);
	linePropAxisXMaxX->ReplaceProperty(propM4Max[0]);
	labelMinimumX->setText(labelM4Min);
	labelMaximumX->setText(labelM4Max);
	}
	labelMinimumX->adjustSize();
	labelMaximumX->adjustSize();
}
void HistogramControl::ChangedComboBoxY(const QString & text){
//QMessageBox::warning(0, "",QString("ChangedComboBoxY %1").arg(text) );
	if(text == "Years") {
	linePropAxisYMinY->ReplaceProperty(propYearsMin[1]);
	linePropAxisYMaxY->ReplaceProperty(propYearsMax[1]);
	labelMinimumY->setText(labelYearsMin);
	labelMaximumY->setText(labelYearsMax);
	}
	if(text == "Latitude") {
	linePropAxisYMinY->ReplaceProperty(propLatitudeMin[1]);
	linePropAxisYMaxY->ReplaceProperty(propLatitudeMax[1]);
	labelMinimumY->setText(labelLatitudeMin);
	labelMaximumY->setText(labelLatitudeMax);
	}
	if(text == "Longitude") {
	linePropAxisYMinY->ReplaceProperty(propLongitudeMin[1]);
	linePropAxisYMaxY->ReplaceProperty(propLongitudeMax[1]);
	labelMinimumY->setText(labelLongitudeMin);
	labelMaximumY->setText(labelLongitudeMax);
	}
	if(text == "Depth") {
	linePropAxisYMinY->ReplaceProperty(propDepthMin[1]);
	linePropAxisYMaxY->ReplaceProperty(propDepthMax[1]);
	labelMinimumY->setText(labelDepthMin);
	labelMaximumY->setText(labelDepthMax);
	}
	if(text == "m1") {
	linePropAxisYMinY->ReplaceProperty(propM1Min[1]);
	linePropAxisYMaxY->ReplaceProperty(propM1Max[1]);
	labelMinimumY->setText(labelM1Min);
	labelMaximumY->setText(labelM1Max);
	}
	if(text == "m2") {
	linePropAxisYMinY->ReplaceProperty(propM2Min[1]);
	linePropAxisYMaxY->ReplaceProperty(propM2Max[1]);
	labelMinimumY->setText(labelM2Min);
	labelMaximumY->setText(labelM2Max);
	}
	if(text == "m3") {
	linePropAxisYMinY->ReplaceProperty(propM3Min[1]);
	linePropAxisYMaxY->ReplaceProperty(propM3Max[1]);
	labelMinimumY->setText(labelM3Min);
	labelMaximumY->setText(labelM3Max);
	}
	if(text == "m4") {
	linePropAxisYMinY->ReplaceProperty(propM4Min[1]);
	linePropAxisYMaxY->ReplaceProperty(propM4Max[1]);
	labelMinimumY->setText(labelM4Min);
	labelMaximumY->setText(labelM4Max);
	}
	labelMinimumY->adjustSize();
	labelMaximumY->adjustSize();
}

void HistogramControl::ChangedComboBoxZ(const QString & text){
//QMessageBox::warning(0, "",QString("ChangedComboBoxZ %1").arg(text) );
	if(text == "Years") {
	linePropAxisZMinZ->ReplaceProperty(propYearsMin[2]);
	linePropAxisZMaxZ->ReplaceProperty(propYearsMax[2]);
	labelMinimumZ->setText(labelYearsMin);
	labelMaximumZ->setText(labelYearsMax);
	}
	if(text == "Latitude") {
	linePropAxisZMinZ->ReplaceProperty(propLatitudeMin[2]);
	linePropAxisZMaxZ->ReplaceProperty(propLatitudeMax[2]);
	labelMinimumZ->setText(labelLatitudeMin);
	labelMaximumZ->setText(labelLatitudeMax);
	}
	if(text == "Longitude") {
	linePropAxisZMinZ->ReplaceProperty(propLongitudeMin[2]);
	linePropAxisZMaxZ->ReplaceProperty(propLongitudeMax[2]);
	labelMinimumZ->setText(labelLongitudeMin);
	labelMaximumZ->setText(labelLongitudeMax);
	}
	if(text == "Depth") {
	linePropAxisZMinZ->ReplaceProperty(propDepthMin[2]);
	linePropAxisZMaxZ->ReplaceProperty(propDepthMax[2]);
	labelMinimumZ->setText(labelDepthMin);
	labelMaximumZ->setText(labelDepthMax);
	}
	if(text == "m1") {
	linePropAxisZMinZ->ReplaceProperty(propM1Min[2]);
	linePropAxisZMaxZ->ReplaceProperty(propM1Max[2]);
	labelMinimumZ->setText(labelM1Min);
	labelMaximumZ->setText(labelM1Max);
	}
	if(text == "m2") {
	linePropAxisZMinZ->ReplaceProperty(propM2Min[2]);
	linePropAxisZMaxZ->ReplaceProperty(propM2Max[2]);
	labelMinimumZ->setText(labelM2Min);
	labelMaximumZ->setText(labelM2Max);
	}
	if(text == "m3") {
	linePropAxisZMinZ->ReplaceProperty(propM3Min[2]);
	linePropAxisZMaxZ->ReplaceProperty(propM3Max[2]);
	labelMinimumZ->setText(labelM3Min);
	labelMaximumZ->setText(labelM3Max);
	}
	if(text == "m4") {
	linePropAxisZMinZ->ReplaceProperty(propM4Min[2]);
	linePropAxisZMaxZ->ReplaceProperty(propM4Max[2]);
	labelMinimumZ->setText(labelM4Min);
	labelMaximumZ->setText(labelM4Max);
	}
	labelMinimumZ->adjustSize();
	labelMaximumZ->adjustSize();
}
void HistogramControl::ChangedComboBoxMinMax(const QString & text){
//QMessageBox::warning(0, "",QString("ChangedComboBoxZ %1").arg(text) );
	if(text == "Depth") {
	labelParTypeMin->setText(labelDepthMin);
	labelParTypeMax->setText(labelDepthMax);
	}
	if(text == "m1") {
	labelParTypeMin->setText(labelM1Min);
	labelParTypeMax->setText(labelM1Max);
	}
	if(text == "m2") {
	labelParTypeMin->setText(labelM2Min);
	labelParTypeMax->setText(labelM2Max);
	}
	if(text == "m3") {
	labelParTypeMin->setText(labelM3Min);
	labelParTypeMax->setText(labelM3Max);
	}
	if(text == "m4") {
	labelParTypeMin->setText(labelM4Min);
	labelParTypeMax->setText(labelM4Max);
	}
	labelParTypeMin->adjustSize();
	labelParTypeMax->adjustSize();

	if (radioButtonMin->isChecked ()) groupBoxMinMaxType->setTitle("Minimum of " + text);
	if (radioButtonMax->isChecked ()) groupBoxMinMaxType->setTitle("Minimum of " + text);	
}
bool HistogramControl::GetParametersLimit(){
	Catalog cat;
	char buf[128];
	indexCat =  CatTable::tSelf->GetRecordCatalogIndex();
    in =  CatTable::tSelf->GetFstream();
	
	ItrMultiMap pos = indexCat->begin();
	vldYearsMin = GetYearIntYearsFloat(pos->first,YearsMin);
	valYearsMin = YearsMin;
    labelYearsMin = cat.getQStringTimeInMinuts(pos->first);
	pos = indexCat->end(); pos--;
	vldYearsMax = GetYearIntYearsFloat(pos->first,YearsMax);
    vldYearsMax += 1.0; valYearsMax = YearsMax;
	labelYearsMax = cat.getQStringTimeInMinuts(pos->first);
	
	::sprintf(buf,"%8.3f",YearsMin);
	QString def = QString::fromAscii(buf);
	def =	def.trimmed ();
	QString name = "";
	int decimal = 3;
		labelYearsMin = labelYearsMin + "(" + def + ")";
    propYearsMin[0] =   new PropertyFloatValue(1,def,name,vldYearsMin,vldYearsMax,decimal);
    values.push_back(propYearsMin[0]);
    propYearsMin[1] =   new PropertyFloatValue(2,def,name,vldYearsMin,vldYearsMax,decimal);
    values.push_back(propYearsMin[1]);
    propYearsMin[2] =   new PropertyFloatValue(3,def,name,vldYearsMin,vldYearsMax,decimal);
    values.push_back(propYearsMin[2]);
	
	::sprintf(buf,"%8.3f",YearsMax);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelYearsMax = labelYearsMax + "(" + def + ")";
	name = "";
	decimal = 3;
    propYearsMax[0] =   new PropertyFloatValue(5,def,name,vldYearsMin,vldYearsMax,decimal);
	values.push_back(propYearsMax[0]);
    propYearsMax[1] =   new PropertyFloatValue(6,def,name,vldYearsMin,vldYearsMax,decimal);
	values.push_back(propYearsMax[1]);
    propYearsMax[2] =   new PropertyFloatValue(7,def,name,vldYearsMin,vldYearsMax,decimal);
	values.push_back(propYearsMax[2]);
///------------------------------------------------------------------------------
    RECORD record;
    in->clear();

    in->seekg(0,ios::beg);
    in->read((char*)&numberRecords,sizeof(int));
	in->seekg(sizeof(RECORD),ios::beg);
	float lat,lon,dep,mb,ms,ml,mp;
	LatitudeMin =
	LongitudeMin =
	DepthMin =
	M1Min =
	M2Min =
	M3Min =
	M4Min = FLT_MAX;

	LatitudeMax =
	LongitudeMax =
	DepthMax =
	M1Max =
	M2Max =
	M3Max =
	M4Max = -FLT_MAX;

	int numRec = 1;
	QApplication::setOverrideCursor(Qt::WaitCursor); 
    for(;;){
      in->read((char*)&record,sizeof(RECORD));
      if(in->eof())  break;
	  if(in->fail()){QApplication::restoreOverrideCursor();
	  error(QString("An reading error has occurred (#%1)").arg(numRec),"Curent Catalog");return false;}
      cat.GetLatLonDepthMagnitude(record,lat,lon,dep,mb,ms,ml,mp);
	  if (LatitudeMin  > lat ) LatitudeMin = lat;
	  if (LongitudeMin > lon ) LongitudeMin = lon;
	  if (DepthMin     > dep ) DepthMin = dep;
	  if (M1Min        > mb  ) M1Min = mb;
	  if (M2Min        > ms  ) M2Min = ms;
	  if (M3Min        > ml  ) M3Min = ml;
	  if (M4Min        > mp  ) M4Min = mp;
	  
	  if (LatitudeMax  < lat ) LatitudeMax = lat;
	  if (LongitudeMax < lon ) LongitudeMax = lon;
	  if (DepthMax     < dep ) DepthMax = dep;
	  if (M1Max        < mb  ) M1Max = mb;
	  if (M2Max        < ms  ) M2Max = ms;
	  if (M3Max        < ml  ) M3Max = ml;
	  if (M4Max        < mp  ) M4Max = mp;
	  
	  numRec++;
   }
	in->clear();
	QApplication::restoreOverrideCursor();
	valLatitudeMin = LatitudeMin;
	valLongitudeMin = LongitudeMin;
	valDepthMin = DepthMin;
	valM1Min = M1Min;
	valM2Min = M2Min;
	valM3Min = M3Min;
	valM4Min = M4Min;

	valLatitudeMax = LatitudeMax;
	valLongitudeMax = LongitudeMax;
	valDepthMax = DepthMax;
	valM1Max = M1Max;
	valM2Max = M2Max;
	valM3Max = M3Max;
	valM4Max = M4Max;
	
	vldLatitudeMin = (float)((int)(LatitudeMin - 1));
    if (vldLatitudeMin < -90) vldLatitudeMin =	-90;
	vldLongitudeMin = (float)((int)(LongitudeMin - 1));
    if (vldLongitudeMin < -180) vldLongitudeMin =	-180;
	vldDepthMin = (float)((int)(DepthMin - 1));
	vldM1Min = (float)((int)(M1Min - 1));
	vldM2Min = (float)((int)(M2Min - 1));
	vldM3Min = (float)((int)(M3Min - 1));
	vldM4Min = (float)((int)(M4Min - 1));

	vldLatitudeMax = (float)((int)(LatitudeMax + 1));
    if (vldLatitudeMax > 90) vldLatitudeMax =	90;
	vldLongitudeMax = (float)((int)(LongitudeMax + 1));
    if (vldLongitudeMax > 180) vldLongitudeMax =	180;	
	vldDepthMax = (float)((int)(DepthMax +1));
	vldM1Max = (float)((int)(M1Max +1));
	vldM2Max = (float)((int)(M2Max +1));
	vldM3Max = (float)((int)(M3Max +1));
	vldM4Max = (float)((int)(M4Max +1));
///------------------------------------------------------------------------------
	::sprintf(buf,"%7.3f",LatitudeMin);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelLatitudeMin = "Latitude min: " + def;
	
	name = "";
	decimal = 3;
    propLatitudeMin[0]  =  new PropertyFloatValue(9,def,name,vldLatitudeMin,vldLatitudeMax,decimal);
	values.push_back(propLatitudeMin[0]);
    propLatitudeMin[1]  =  new PropertyFloatValue(10,def,name,vldLatitudeMin,vldLatitudeMax,decimal);
	values.push_back(propLatitudeMin[1]);
    propLatitudeMin[2]  =  new PropertyFloatValue(11,def,name,vldLatitudeMin,vldLatitudeMax,decimal);
	values.push_back(propLatitudeMin[2]);
	
	::sprintf(buf,"%8.3f",LongitudeMin);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelLongitudeMin = "Longitude min: " + def;
	name = "";
	decimal = 3;
    propLongitudeMin[0] =  new PropertyFloatValue(13,def,name,vldLongitudeMin,vldLongitudeMax,decimal);
	values.push_back(propLongitudeMin[0]);
    propLongitudeMin[1] =  new PropertyFloatValue(14,def,name,vldLongitudeMin,vldLongitudeMax,decimal);
	values.push_back(propLongitudeMin[1]);
    propLongitudeMin[2] =  new PropertyFloatValue(15,def,name,vldLongitudeMin,vldLongitudeMax,decimal);
	values.push_back(propLongitudeMin[2]);
	
	::sprintf(buf,"%8.3f",DepthMin);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelDepthMin = "Depth min: " + def;
	name = "";
	decimal = 1;
    propDepthMin[0]     =  new PropertyFloatValue(17,def,name,vldDepthMin,vldDepthMax,decimal);
	values.push_back(propDepthMin[0]);
    propDepthMin[1]     =  new PropertyFloatValue(18,def,name,vldDepthMin,vldDepthMax,decimal);
	values.push_back(propDepthMin[1]);
    propDepthMin[2]     =  new PropertyFloatValue(19,def,name,vldDepthMin,vldDepthMax,decimal);
	values.push_back(propDepthMin[2]);
	
	::sprintf(buf,"%6.2f",M1Min);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelM1Min = "Magnitude m1 min: " + def;
	name = "";
	decimal = 2;
    propM1Min[0]        =  new PropertyFloatValue(21,def,name,vldM1Min,vldM1Max,decimal);
	values.push_back(propM1Min[0]);
    propM1Min[1]        =  new PropertyFloatValue(22,def,name,vldM1Min,vldM1Max,decimal);
	values.push_back(propM1Min[1]);
    propM1Min[2]        =  new PropertyFloatValue(23,def,name,vldM1Min,vldM1Max,decimal);
	values.push_back(propM1Min[2]);
	
	::sprintf(buf,"%6.2f",M2Min);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelM2Min = "Magnitude m2 min: " + def;
    propM2Min[0]        =  new PropertyFloatValue(25,def,name,vldM2Min,vldM2Max,decimal);
	values.push_back(propM2Min[0]);
    propM2Min[1]        =  new PropertyFloatValue(26,def,name,vldM2Min,vldM2Max,decimal);
	values.push_back(propM2Min[1]);
    propM2Min[2]        =  new PropertyFloatValue(27,def,name,vldM2Min,vldM2Max,decimal);
	values.push_back(propM2Min[2]);
	
	::sprintf(buf,"%6.2f",M3Min);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelM3Min = "Magnitude m3 min: " + def;
    propM3Min[0]        =  new PropertyFloatValue(29,def,name,vldM3Min,vldM3Max,decimal);
	values.push_back(propM3Min[0]);
    propM3Min[1]        =  new PropertyFloatValue(30,def,name,vldM3Min,vldM3Max,decimal);
	values.push_back(propM3Min[1]);
    propM3Min[2]        =  new PropertyFloatValue(31,def,name,vldM3Min,vldM3Max,decimal);
	values.push_back(propM3Min[2]);

	::sprintf(buf,"%6.2f",M4Min);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelM4Min = "Magnitude m4 min: " + def;
    propM4Min[0]        =  new PropertyFloatValue(33,def,name,vldM4Min,vldM4Max,decimal);
	values.push_back(propM4Min[0]);
    propM4Min[1]        =  new PropertyFloatValue(34,def,name,vldM4Min,vldM4Max,decimal);
	values.push_back(propM4Min[1]);
    propM4Min[2]        =  new PropertyFloatValue(35,def,name,vldM4Min,vldM4Max,decimal);
	values.push_back(propM4Min[2]);

//-----------------------------------------------------------------------------------
   	::sprintf(buf,"%7.3f",LatitudeMax);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelLatitudeMax = "Latitude max: " + def;
	name = "";
	decimal = 3;
    propLatitudeMax[0]  =  new PropertyFloatValue(37,def,name,vldLatitudeMin,vldLatitudeMax,decimal);
	values.push_back(propLatitudeMax[0]);
    propLatitudeMax[1]  =  new PropertyFloatValue(38,def,name,vldLatitudeMin,vldLatitudeMax,decimal);
	values.push_back(propLatitudeMax[1]);
    propLatitudeMax[2]  =  new PropertyFloatValue(39,def,name,vldLatitudeMin,vldLatitudeMax,decimal);
	values.push_back(propLatitudeMax[2]);
	
	::sprintf(buf,"%8.3f",LongitudeMax);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelLongitudeMax = "Longitude max: " + def;
	name = "";
	decimal = 3;
    propLongitudeMax[0] =  new PropertyFloatValue(41,def,name,vldLongitudeMin,vldLongitudeMax,decimal);
	values.push_back(propLongitudeMax[0]);
    propLongitudeMax[1] =  new PropertyFloatValue(42,def,name,vldLongitudeMin,vldLongitudeMax,decimal);
	values.push_back(propLongitudeMax[1]);
    propLongitudeMax[2] =  new PropertyFloatValue(43,def,name,vldLongitudeMin,vldLongitudeMax,decimal);
	values.push_back(propLongitudeMax[2]);
	
	::sprintf(buf,"%8.3f",DepthMax);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelDepthMax = "Depth max: " + def;
	name = "";
	decimal = 1;
    propDepthMax[0]     =  new PropertyFloatValue(45,def,name,vldDepthMin,vldDepthMax,decimal);
	values.push_back(propDepthMax[0]);
    propDepthMax[1]     =  new PropertyFloatValue(46,def,name,vldDepthMin,vldDepthMax,decimal);
	values.push_back(propDepthMax[1]);
    propDepthMax[2]     =  new PropertyFloatValue(47,def,name,vldDepthMin,vldDepthMax,decimal);
	values.push_back(propDepthMax[2]);
	
	::sprintf(buf,"%6.2f",M1Max);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelM1Max = "Magnitude m1 max: " + def;
	name = "";
	decimal = 2;
    propM1Max[0]        =  new PropertyFloatValue(49,def,name,vldM1Min,vldM1Max,decimal);
	values.push_back(propM1Max[0]);
    propM1Max[1]        =  new PropertyFloatValue(50,def,name,vldM1Min,vldM1Max,decimal);
	values.push_back(propM1Max[1]);
    propM1Max[2]        =  new PropertyFloatValue(51,def,name,vldM1Min,vldM1Max,decimal);
	values.push_back(propM1Max[2]);
	
	::sprintf(buf,"%6.2f",M2Max);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelM2Max = "Magnitude m2 max: " + def;
    propM2Max[0]        =  new PropertyFloatValue(53,def,name,vldM2Min,vldM2Max,decimal);
	values.push_back(propM2Max[0]);
    propM2Max[1]        =  new PropertyFloatValue(54,def,name,vldM2Min,vldM2Max,decimal);
	values.push_back(propM2Max[1]);
    propM2Max[2]        =  new PropertyFloatValue(55,def,name,vldM2Min,vldM2Max,decimal);
	values.push_back(propM2Max[2]);
	
	::sprintf(buf,"%6.2f",M3Max);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelM3Max = "Magnitude m3 max: " + def;
    propM3Max[0]        =  new PropertyFloatValue(57,def,name,vldM3Min,vldM3Max,decimal);
	values.push_back(propM3Max[0]);
    propM3Max[1]        =  new PropertyFloatValue(58,def,name,vldM3Min,vldM3Max,decimal);
	values.push_back(propM3Max[1]);
    propM3Max[2]        =  new PropertyFloatValue(59,def,name,vldM3Min,vldM3Max,decimal);
	values.push_back(propM3Max[2]);

	::sprintf(buf,"%6.2f",M4Max);
	def = QString::fromAscii(buf);
	def =	def.trimmed ();
		labelM4Max = "Magnitude m4 max: " + def;
    propM4Max[0]        =  new PropertyFloatValue(61,def,name,vldM4Min,vldM4Max,decimal);
	values.push_back(propM4Max[0]);
    propM4Max[1]        =  new PropertyFloatValue(62,def,name,vldM4Min,vldM4Max,decimal);
	values.push_back(propM4Max[1]);
    propM4Max[2]        =  new PropertyFloatValue(63,def,name,vldM4Min,vldM4Max,decimal);
	values.push_back(propM4Max[2]);	
//------------------------------------------------------------------------------------
 	return true;
}
void HistogramControl::InitDialog(){
	toolButtonLog->setCheckable(true);
	toolButtonLog->setChecked (false);
    toolButtonZoom->setCheckable(true);
	toolButtonHistMulty->setCheckable(true);
	toolButtonScale->setCheckable(true);
	toolButtonScale->setChecked (false);
	radioButtonFrequencies->setChecked (true);
	groupBoxMinMaxType->setEnabled (false); 
	tabWidget->setTabEnabled (3,false); 
    EnableByListMode(false);
	
	propDocLine.append(new LineDocProperty(spinBoxIntervalX,
                             setDoc->GetNumberIntervalAxisX()));
	propDocLine.append(new LineDocProperty(spinBoxIntervalY,
                             setDoc->GetNumberIntervalAxisY()));
	propDocLine.append(new LineDocProperty(spinBoxIntervalZ,
                             setDoc->GetNumberIntervalAxisZ()));


	linePropAxisXMinX  = new LineDocProperty(lineEditMinX,propYearsMin[0]);
	linePropAxisXMaxX  = new LineDocProperty(lineEditMaxX,propYearsMax[0]);
	
	propDocLine.append(linePropAxisXMinX);
	propDocLine.append(linePropAxisXMaxX);
	
	linePropAxisYMinY = new LineDocProperty(lineEditMinY,propYearsMin[1]);
	linePropAxisYMaxY  = new LineDocProperty(lineEditMaxY,propYearsMax[1]);
	
	propDocLine.append(linePropAxisYMinY);
	propDocLine.append(linePropAxisYMaxY);
	
	
	linePropAxisZMinZ = new LineDocProperty(lineEditMinZ,propYearsMin[2]);
	linePropAxisZMaxZ  = new LineDocProperty(lineEditMaxZ,propYearsMax[2]);
	
	propDocLine.append(linePropAxisZMinZ);
	propDocLine.append(linePropAxisZMaxZ);

	propEditWeightA        =  new PropertyFloatValue(42,defaultHistogramCoefficentA(),
		nameHistogramCoefficentA(),
		minHistogramCoefficentA(),
		maxHistogramCoefficentA(),decimalHistogramCoefficentA());
	linePropEditWeightA    =  new LineDocProperty(lineEditWeightA,propEditWeightA);
	propDocLine.append(linePropEditWeightA);
	
	propEditWeightB        =  new PropertyFloatValue(43,defaultHistogramCoefficentB(),
		nameHistogramCoefficentB(),
		minHistogramCoefficentB(),
		maxHistogramCoefficentB(),decimalHistogramCoefficentB());
	linePropEditWeightB    =  new LineDocProperty(lineEditWeightB,propEditWeightB);
	propDocLine.append(linePropEditWeightB);


	foreach (LineDocProperty *docprop, propDocLine) docprop->SetValueFromDoc();

			customA = QString("%1").arg(customHistogramCoefficentA()).toFloat();
			customB = QString("%1").arg(customHistogramCoefficentB()).toFloat();
			previousTypeWeightIndex = 0;
	SetWeightCoefficentsToLineEditBoxes(0);
	
//---------------------------------------------------------------------------------

	comboBoxX->clear();
	QStringList lst = listHistgramParameters().split ('@');
	comboBoxX->addItems(lst);

	comboBoxY->clear();
	comboBoxY->addItems(lst);
	

	comboBoxZ->clear();
	for (int i = 0; i < lst.size(); i++) {
		comboBoxZ->addItem(lst.at(i));
	}

	lst = listHistgramMinMax().split ('@');	
	comboBoxMinMax->clear();
	comboBoxMinMax->addItems(lst);
	
	lst = enumComboBoxMagnitudeList().split ('@');
	comboBoxWeightMagnitude->clear();
	for (int i = 0; i < lst.size(); i++) {
		comboBoxWeightMagnitude->addItem(lst.at(i));
	}
	 
	lst = enumHistogramTypeWeightList().split ('@');
	comboBoxWeightType->clear();
	for (int i = 0; i < lst.size(); i++) {
		comboBoxWeightType->addItem(lst.at(i));
	}

	comboBoxX->setCurrentIndex (2);
    comboBoxY->setCurrentIndex (1);
    comboBoxZ->setCurrentIndex (3);
	comboBoxMinMax->setCurrentIndex (0);
	
	comboBoxWeightMagnitude->setCurrentIndex (0);

	QString tmp = comboBoxMinMax->currentText ();
	groupBoxMinMaxType->setTitle("Min/max of " + tmp);	
	comboBoxWeightType->setCurrentIndex (0);
	tabWidget->setCurrentIndex (0);
//----------------------------------------------------------------------------------
	axisX = axisY = axisZ =
	minX = minY = minZ =
	maxX = maxY = maxZ  = 
	coeffWeightCustomA = coeffWeightCustomB = typeWeight = magnitudeWeight = parHistMinMax =
	QString();
	intervalX = intervalY = intervalZ = -1;	
	isByList = false;
	type = Frequency;
}

void HistogramControl::SetWeightCoefficentsToLineEditBoxes(int index)	{
	QString ValueA = QString();
	QString ValueB = QString();
	//QMessageBox::warning(0, "",QString("index %1 prev %2").arg(index).arg(previousTypeWeightIndex));
	if (previousTypeWeightIndex == 3) {
		linePropEditWeightA->FinishEditLine();
		linePropEditWeightB->FinishEditLine();
	customA = lineEditWeightA->text().toFloat();
	customB = lineEditWeightB->text().toFloat();
	}
	previousTypeWeightIndex = index;
	if (index == 0) {
	ValueA = QString("%1").arg(energyHistogramCoefficentA());
	ValueB = QString("%1").arg(energyHistogramCoefficentB());
		linePropEditWeightA->editor->setReadOnly(true);
		linePropEditWeightB->editor->setReadOnly(true);
	}
	
	if (index == 1) {
	ValueA = QString("%1").arg(areaHistogramCoefficentA());
	ValueB = QString("%1").arg(areaHistogramCoefficentB());
		linePropEditWeightA->editor->setReadOnly(true);
		linePropEditWeightB->editor->setReadOnly(true);
	}	
	if (index == 2) {
	ValueA = QString("%1").arg(lenghtHistogramCoefficentA());
	ValueB = QString("%1").arg(lenghtHistogramCoefficentB());
		linePropEditWeightA->editor->setReadOnly(true);
		linePropEditWeightB->editor->setReadOnly(true);
	}	
	if (index == 3) {
			ValueA = QString("%1").arg(customA);
			ValueB = QString("%1").arg(customB);
		linePropEditWeightA->editor->setReadOnly(false);
		linePropEditWeightB->editor->setReadOnly(false);
	}	
	propEditWeightA->SetValue(ValueA);
	propEditWeightB->SetValue(ValueB);
	propEditWeightA->SetValueToResetvalue();
	propEditWeightB->SetValueToResetvalue();
	lineEditWeightA->setText(ValueA);
	lineEditWeightB->setText(ValueB);
}
//---------------------------------------------------------------------------------

void HistogramControl::toggledButtonWeightOn(bool on){
	if (on) tabWidget->setTabEnabled (3,true);  
	else tabWidget->setTabEnabled (3,false); 
	
}
void HistogramControl::toggledButtonMinOn(bool on){
	QString tmp = comboBoxMinMax->currentText ();

	if (on) {
		groupBoxMinMaxType->setTitle("Minimum of " + tmp);
		groupBoxMinMaxType->setEnabled (true);  
	}
	else {
		groupBoxMinMaxType->setTitle("Min/Max of " + tmp);
		groupBoxMinMaxType->setEnabled (false); 
	}
	
}
void HistogramControl::toggledButtonMaxOn(bool on){
	QString tmp = comboBoxMinMax->currentText ();
	if (on) {
		groupBoxMinMaxType->setTitle("Maximum of " + tmp);
		groupBoxMinMaxType->setEnabled (true);  
	}
	else {
		groupBoxMinMaxType->setTitle("Min/Max of " + tmp);
		groupBoxMinMaxType->setEnabled (false); 
	}
	
}
bool HistogramControl::CheckParametersForDraw(){
		foreach (LineDocProperty *docprop, propDocLine) {
	if (docprop->GetProp()->GetKind()  == PropertyValue::Property_Float)
		docprop->editor->clearFocus();
	if (docprop->GetProp()->GetKind()  == PropertyValue::Property_IntSpin)
		docprop->spin->clearFocus();
    }
//---------------------------------------------------------	
	TypeHist dialogTypeHist = Frequency;
	if (radioButtonWeightOn->isChecked ()) {
//		QMessageBox::warning(0, "","HistogramControl::Weight");
		dialogTypeHist = Weight;
	}
	if (radioButtonMin->isChecked ())  {
//		QMessageBox::warning(0, "","HistogramControl::Min");
		dialogTypeHist = Min;
	}
	if (radioButtonMax->isChecked ()) {
//		QMessageBox::warning(0, "","HistogramControl::Max");
		dialogTypeHist = Max;
	}
	if (radioButtonFrequencies->isChecked ()) {
//		QMessageBox::warning(0, "","HistogramControl::Frequency");
		dialogTypeHist = Frequency;
	}
	
	if (axisX == comboBoxX->currentText () &&
	axisY == comboBoxY->currentText () &&
	axisZ == comboBoxZ->currentText () &&
	minX == lineEditMinX->text() &&
	minY == lineEditMinY->text() &&
	minZ == lineEditMinZ->text() &&
	maxX == lineEditMaxX->text() &&
	maxY == lineEditMaxY->text() &&
	maxZ == lineEditMaxZ->text() &&
	intervalX == spinBoxIntervalX->value () &&
	intervalY == spinBoxIntervalY->value () &&
	intervalZ == spinBoxIntervalZ->value () &&
	isByList == toolButtonHistMulty->isChecked () &&
	coeffWeightCustomA == lineEditWeightA->text() &&
	coeffWeightCustomB == lineEditWeightB->text() &&
	typeWeight == comboBoxWeightType->currentText () &&
	parHistMinMax == comboBoxMinMax->currentText () &&
	magnitudeWeight == comboBoxWeightMagnitude->currentText () &&
	dialogTypeHist == type
	) 
	{
		//QMessageBox::warning(0, "",QString("Checking NO"));
	return true;
	} 
//-----------------------------------------------------------	
	
	axisParameterX = comboBoxX->currentText () ;
	axisParameterY = comboBoxY->currentText () ;
	axisParameterZ = comboBoxZ->currentText () ;
	if (axisParameterX == axisParameterY) {
	QMessageBox::critical(0, "",QString("Axis X and axis Y have the same names \"%1\" !").arg(axisParameterY) );
	return false;
	}
	if(toolButtonHistMulty->isChecked ()) {
	  if (axisParameterX == axisParameterZ) {
	   QMessageBox::critical(0, "",QString("Axis X and \"by List\" parameters have the same names \"%1\" !").arg(axisParameterZ) );
	  return false;	
	  }
	  if (axisParameterY == axisParameterZ) {
	   QMessageBox::critical(0, "",QString("Axis Y and \"by List\" parameters have the same names \"%1\" !").arg(axisParameterZ) );
	  return false;	
	  }
	}
	


	
	float min = 0;
	float max = 0;
	min = lineEditMinX->text().toFloat();
	max = lineEditMaxX->text().toFloat();
	if(!CheckQLineEdit(lineEditMinX,min,max,axisParameterX + ": Axis X limits: ")){
		tabWidget->setCurrentIndex(0);
		return false;
	}
	
	
	min = lineEditMinY->text().toFloat();
	max = lineEditMaxY->text().toFloat();
	if(!CheckQLineEdit(lineEditMinY,min,max,axisParameterY + ": Axis Y limits: ")){
		tabWidget->setCurrentIndex(1);
		return false;
	}

	if(toolButtonHistMulty->isChecked ()) {
	min = lineEditMinZ->text().toFloat();
	max = lineEditMaxZ->text().toFloat();
	if(!CheckQLineEdit(lineEditMinZ,min,max,axisParameterZ + ": \"by List\" parameter limits: ")){
		tabWidget->setCurrentIndex(0);
		return false;
	}
	
	}
	int nx = spinBoxIntervalX->value ();
	int ny = spinBoxIntervalY->value ();
	int nz = spinBoxIntervalZ->value ();
	float minx = lineEditMinX->text().toFloat();
	float maxx = lineEditMaxX->text().toFloat();
	float miny = lineEditMinY->text().toFloat();
	float maxy = lineEditMaxY->text().toFloat();
	float minz = lineEditMinZ->text().toFloat();
	float maxz = lineEditMaxZ->text().toFloat();
	
	DataPlot::TypeAxis tpX = DataPlot::Years;
	DataPlot::TypeAxis tpY = DataPlot::Years;
	DataPlot::TypeAxis tpZ = DataPlot::Years;
	DataPlot::TypeAxis tpMinMax = DataPlot::Latitude;
	if(comboBoxX->currentIndex () == 0) tpX = DataPlot::Years;
	if(comboBoxX->currentIndex () == 1) tpX = DataPlot::Latitude;
	if(comboBoxX->currentIndex () == 2) tpX = DataPlot::Longitude;
	if(comboBoxX->currentIndex () == 3) tpX = DataPlot::Depth;
	if(comboBoxX->currentIndex () == 4) tpX = DataPlot::M1;
	if(comboBoxX->currentIndex () == 5) tpX = DataPlot::M2;
	if(comboBoxX->currentIndex () == 6) tpX = DataPlot::M3;
	if(comboBoxX->currentIndex () == 7) tpX = DataPlot::M4;
	

	if(comboBoxY->currentIndex () == 0) tpY = DataPlot::Years;
	if(comboBoxY->currentIndex () == 1) tpY = DataPlot::Latitude;
	if(comboBoxY->currentIndex () == 2) tpY = DataPlot::Longitude;
	if(comboBoxY->currentIndex () == 3) tpY = DataPlot::Depth;
	if(comboBoxY->currentIndex () == 4) tpY = DataPlot::M1;
	if(comboBoxY->currentIndex () == 5) tpY = DataPlot::M2;
	if(comboBoxY->currentIndex () == 6) tpY = DataPlot::M3;
	if(comboBoxY->currentIndex () == 7) tpY = DataPlot::M4;

	if(comboBoxZ->currentIndex () == 0) tpZ = DataPlot::Years;
	if(comboBoxZ->currentIndex () == 1) tpZ = DataPlot::Latitude;
	if(comboBoxZ->currentIndex () == 2) tpZ = DataPlot::Longitude;
	if(comboBoxZ->currentIndex () == 3) tpZ = DataPlot::Depth;
	if(comboBoxZ->currentIndex () == 4) tpZ = DataPlot::M1;
	if(comboBoxZ->currentIndex () == 5) tpZ = DataPlot::M2;
	if(comboBoxZ->currentIndex () == 6) tpZ = DataPlot::M3;
	if(comboBoxZ->currentIndex () == 7) tpZ = DataPlot::M4;
	
	
	if(comboBoxMinMax->currentIndex () == 0) tpMinMax = DataPlot::Depth;
	if(comboBoxMinMax->currentIndex () == 1) tpMinMax = DataPlot::M1;
	if(comboBoxMinMax->currentIndex () == 2) tpMinMax = DataPlot::M2;
	if(comboBoxMinMax->currentIndex () == 3) tpMinMax = DataPlot::M3;
	if(comboBoxMinMax->currentIndex () == 4) tpMinMax = DataPlot::M4;
/*
//------------------------------------------------------------------
	if (axisParameterX == "Latitude" && axisParameterY == "Longitude") {
	axisParameterX = "Longitude";
	axisParameterY = "Latitude";
	minx = lineEditMinY->text().toFloat();
	maxx = lineEditMaxY->text().toFloat();
	miny = lineEditMinX->text().toFloat();
	maxy = lineEditMaxX->text().toFloat();	
	tpX = DataPlot::Longitude;
	tpY = DataPlot::Latitude;
	}
		if (axisParameterX == "Depth" && 
		   (axisParameterY == "Latitude" || axisParameterY == "Longitude") ){
	axisParameterX = axisParameterY;
	axisParameterY = "Depth";	
	minx = lineEditMinY->text().toFloat();
	maxx = lineEditMaxY->text().toFloat();
	miny = lineEditMinX->text().toFloat();
	maxy = lineEditMaxX->text().toFloat();
	tpY = DataPlot::Depth; 
	if(axisParameterX == "Longitude") tpX = DataPlot::Longitude;
	if(axisParameterX == "Latitude") tpX = DataPlot::Latitude;
	}
//------------------------------------------------------------------
*/
	axisX = comboBoxX->currentText ();
	axisY = comboBoxY->currentText ();
	axisZ = comboBoxZ->currentText ();
	minX = lineEditMinX->text();
	minY = lineEditMinY->text();
	minZ = lineEditMinZ->text();
	maxX = lineEditMaxX->text();
	maxY = lineEditMaxY->text();
	maxZ = lineEditMaxZ->text();
	intervalX = spinBoxIntervalX->value ();
	intervalY = spinBoxIntervalY->value ();
	intervalZ = spinBoxIntervalZ->value ();
	isByList = toolButtonHistMulty->isChecked ();
	
	coeffWeightCustomA = lineEditWeightA->text();
	coeffWeightCustomB = lineEditWeightB->text();
	typeWeight = comboBoxWeightType->currentText ();
	parHistMinMax = comboBoxMinMax->currentText ();
	magnitudeWeight = comboBoxWeightMagnitude->currentText ();
	type = dialogTypeHist;
	
//------------------------------------------------------------------
		if ( tbl != 0 )      { delete tbl; tbl = 0; }
	if ( dataPlot != 0 ) { delete dataPlot; dataPlot = 0; }
    if(toolButtonHistMulty->isChecked ())
		tbl = new TableLimits(nx,minx,maxx,ny,miny,maxy,nz,minz,maxz);
	else
		tbl = new TableLimits(nx,minx,maxx,ny,miny,maxy);
	
		dataPlot =  new DataPlot(this,tbl,tpX,tpY,tpZ,tpMinMax);
	return true;
}
void HistogramControl::GetHistogramX(){
	if (!CheckParametersForDraw()) return;
	toolButtonZoom->setEnabled (false);
	
	if(toolButtonLog->isChecked ())
	windowParent->GetWindowPlot()->GetHistogramX(dataPlot,MainWindowPlot::Yes);
    else
	windowParent->GetWindowPlot()->GetHistogramX(dataPlot,MainWindowPlot::No);
	
	if (toolButtonHistMulty->isChecked()) {
		toolButtonNext->setEnabled (true);
		toolButtonPrev->setEnabled (true);
	}
	else {
		toolButtonNext->setEnabled (false);
		toolButtonPrev->setEnabled (false);	
	}
	
	windowParent->GetWindowPlot()->statusBar()->showMessage("");
	isSpectrogramDrawed = false;
	isHistogramXDrawed  = true;
	isHistogramYDrawed  = false;
}
void HistogramControl::GetHistogramY(){
	if (!CheckParametersForDraw()) return;
	toolButtonZoom->setEnabled (false);
	
	if(toolButtonLog->isChecked ())
	windowParent->GetWindowPlot()->GetHistogramY(dataPlot,MainWindowPlot::Yes);
	else
	windowParent->GetWindowPlot()->GetHistogramY(dataPlot,MainWindowPlot::No);
	
	if (toolButtonHistMulty->isChecked()) {
		toolButtonNext->setEnabled (true);
		toolButtonPrev->setEnabled (true);
	}
	else {
		toolButtonNext->setEnabled (false);
		toolButtonPrev->setEnabled (false);	
	}
	windowParent->GetWindowPlot()->statusBar()->showMessage("");
	isSpectrogramDrawed = false;
	isHistogramXDrawed  = false;
	isHistogramYDrawed  = true;

}
void HistogramControl::GetSpectrogram(){
	if (!CheckParametersForDraw()) return;
	toolButtonZoom->setEnabled (true);
	if(toolButtonLog->isChecked ()) 
	windowParent->GetWindowPlot()->GetSpectrogram(dataPlot,MainWindowPlot::Yes);
	else
	windowParent->GetWindowPlot()->GetSpectrogram(dataPlot,MainWindowPlot::No);	
	if ( toolButtonZoom ->isChecked ()  ) 
		windowParent->GetWindowPlot()->enableZoomMode(true);
	else 
		windowParent->GetWindowPlot()->enableZoomMode(false);
	

	if (toolButtonHistMulty->isChecked()) {
		toolButtonNext->setEnabled (true);
		toolButtonPrev->setEnabled (true);
	}
	else {
		toolButtonNext->setEnabled (false);
		toolButtonPrev->setEnabled (false);	
	}
	
	isSpectrogramDrawed = true;
	isHistogramXDrawed  = false;
	isHistogramYDrawed  = false;
	

}
void HistogramControl::enableZoomMode(bool on){
windowParent->GetWindowPlot()->enableZoomMode(on);
}
void HistogramControl::EnableCommonScale(bool on){
windowParent->GetWindowPlot()->enableCommonScale(on);
}
void HistogramControl::print(){
windowParent->GetWindowPlot()->print();
}
void HistogramControl::exportPDF(){
windowParent->GetWindowPlot()->exportPDF();
}
void HistogramControl::exportCSV(){
windowParent->GetWindowPlot()->exportCSV(dataPlot);
}
void HistogramControl::EnableByListMode(bool on){
if (!on) {
toolButtonNext->setEnabled (false);
toolButtonPrev->setEnabled (false);
toolButtonScale->setEnabled (false);
tabWidgetList->setEnabled (false);
} else {
	toolButtonScale->setEnabled (true);
    tabWidgetList->setEnabled (true);
}

}
bool HistogramControl::CheckQLineEdit(QLineEdit* linmin,float min,float max,QString name){
  if ( min > max - acuracyCompareFloatValue()) {
   error( name + QString("minimum value (%1) >= maximum value (%2)").
                        arg(min).arg(max),"");
     SetTextQLineEdit(linmin,true,"red");
     return false;
   } else {
     SetTextQLineEdit(linmin,false,"black");
     return true;
   }

}
void HistogramControl::SetTextQLineEdit(QLineEdit* lin,bool bold,QString color){
   QFont fn = font ();
   fn.setBold(bold);
   lin->setFont(fn);
   QPalette pl = lin->palette ();
   pl.setColor (QPalette::Text, QColor(color) );
   lin->setPalette ( pl );
   lin->setText(lin->text());
}


