#ifndef _PROPERTY_LIST_VIEW_H
#define _PROPERTY_LIST_VIEW_H
#include <QItemDelegate>
#include <QModelIndex>
#include <QObject>
#include <QSize>
#include <QPointer>
#include <QStandardItemModel>
#include <QDateTimeEdit>
#include <QLineEdit>
#include <QLabel>
#include <QSpinBox>
#include <QToolButton>
class PropertyValue;
class PropertyValue;
class LineEditor;
class ItemProperty
{
public:
    inline ItemProperty(QModelIndex ind,const QString &initStr,const QString &defStr,PropertyValue* p)
        : invalid(FALSE),
          changed(FALSE),
          initString(initStr),
          defaultString(defStr),
          property(p)
          { index = ind; edited=false;}
inline ItemProperty(QModelIndex ind,const QString &str)
        : invalid(FALSE),
          changed(FALSE),
          initString(str),
          defaultString(str) {index = ind; edited=false; property=0;}
    virtual ~ItemProperty() {}
    QModelIndex index;
    void SetIndex(QModelIndex ind){index = ind;};
    void SetInvalid(bool inv){invalid = inv;}
    bool GetInvalid(){return invalid;}
    void SetChanged(bool ch){changed = ch;}
    bool GetChanged(){return changed;}
    void SetInitStr(const QString &initStr ) {initString = initStr;}
    QString GetInitStr(){return initString;}

    QString GetDefaultStr(){return defaultString;}
    void SetDefaultStr(const QString &defStr ) {defaultString = defStr;}
    bool    edited;
    PropertyValue* GetPropertyValue(){ return property;}

protected:
    bool invalid;
    bool changed;
    QString initString;
    QString defaultString;
    PropertyValue* property;
};
//-----------------------------------------------------------------------------------------------------
class PropertyInt : public ItemProperty
{


public:
    PropertyInt(QModelIndex ind,const QString &initStr,const QString &defStr,PropertyValue* p,int min,int max, bool s );
    ~PropertyInt() {};

    int minimum;
    int maximum;
    bool signedValue;

};
//-----------------------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------------------
class PropertyIntSpin : public ItemProperty
{


public:
    PropertyIntSpin(QModelIndex ind,const QString &initStr,const QString &defStr,PropertyValue* p,int min,int max, int s );
    ~PropertyIntSpin() {};

    int minimum;
    int maximum;
    int step;
};
//-----------------------------------------------------------------------------------------------------
class PropertyNotEdit : public ItemProperty
{
public:
    PropertyNotEdit(QModelIndex ind,const QString &initStr,const QString &defStr,PropertyValue* p);
    ~PropertyNotEdit() {};

};
//-----------------------------------------------------------------------------------------------------
class PropertyString : public ItemProperty
{
public:
    PropertyString(QModelIndex ind,const QString &initStr,const QString &defStr,PropertyValue* p);
    ~PropertyString() {};
};
//-----------------------------------------------------------------------------------------------------
class PropertyTittle : public ItemProperty
{
public:
    PropertyTittle(QModelIndex ind,const QString &title);
    ~PropertyTittle() {};
};
//-----------------------------------------------------------------------------------------------------
class PropertyDouble : public ItemProperty
{


public:
    PropertyDouble(QModelIndex ind,const QString &initStr,const QString &defStr,PropertyValue* p,double min,double max,int dec);
    ~PropertyDouble(){};

    double minimum;
    double maximum;
    int decimals;
};
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
class PropertyDateTime : public ItemProperty
{
public:
    PropertyDateTime(QModelIndex ind,const QString &initStr,const QString &defStr,PropertyValue* p);
    ~PropertyDateTime() {};
};
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
class RoleEditor : public QWidget
{
    Q_OBJECT
public:
    RoleEditor(PropertyNotEdit* ip,const QModelIndex & index,QWidget *parent = 0);
    ~RoleEditor();
    void setLabel(const QString &label);
    void setEdited(bool on);
    bool edited() const;
    QLabel *GetLabel() {return m_label;}
    QToolButton *GetButton() {return button;}
signals:
    void changed(QWidget *widget);
private slots:
    void emitResetProperty();
private:
    QToolButton *button;
    QLabel *m_label;
    bool    m_edited;
    PropertyNotEdit* ip;
    QModelIndex index;
};
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
class ResetSpinBox : public QWidget
{
    Q_OBJECT
public:
    ResetSpinBox(PropertyIntSpin* ip,const QModelIndex & index,QWidget *parent = 0);
   ~ResetSpinBox();
    void setSpin(const QString &label);
    void setEdited(bool on);
    bool edited() const;
    QSpinBox *GetSpin() {return spin;}
    QToolButton *GetButton() {return button;}
signals:
    void changed(QWidget *widget);
private slots:
    void emitResetProperty();
private:
    QToolButton *button;
    QSpinBox *spin;
    bool    m_edited;
    PropertyIntSpin* ip;
    QModelIndex index;
};
//-----------------------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------------------
class ResetLineEdit : public QWidget
{
    Q_OBJECT
public:
    ResetLineEdit(PropertyDouble* ip,const QModelIndex & index,QWidget *parent = 0);
    ~ResetLineEdit();
    void setLine(const QString &label);
    LineEditor  *GetLineEditor() {return lin;}
    QToolButton *GetButton() {return button;}
signals:
    void changed(QWidget *widget);
private slots:
    void emitResetProperty();
private:
    QToolButton *button;
    LineEditor *lin;
    PropertyDouble* ip;
    QModelIndex index;
};
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

class TreePropertyModel : public QStandardItemModel
{
    Q_OBJECT

public:
    TreePropertyModel(int columns, QObject * parent = 0);
    ~TreePropertyModel();
    QList<ItemProperty *> ipList;
    void SetItem(const QString tittle);
    void SetItem(PropertyValue* p);
    void SetItem(QString name,PropertyValue* p,PropertyValue* p1);
    void SetItem(QString name,PropertyValue* p,PropertyValue* p1,PropertyValue* p2);
    void SetChildItem(int col,QModelIndex &index,PropertyValue* p);
    bool CheckValidValue(QModelIndex &index);

    void Default();
    void Reset();
    void Initial();
    void SetPropertyValueToResetValue();
    void GetPropertyValueFromTreeView();

    void DefResIniButton(int ind);
    void SetValuesToPropertyNotEditRows ();
    QString GetItemValue(int row,int child);
    ItemProperty * findProperty(const QModelIndex & index);
private:
    void SetHorizontalHeader();
    int NumRows;
    int NumCols;
};
class TreePropertyDelegate : public QItemDelegate
{
    Q_OBJECT

public:
    TreePropertyDelegate(QObject *parent = 0);

    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option,
                          const QModelIndex &index) const;

    void setEditorData(QWidget *editor, const QModelIndex &index) const;
    void setModelData(QWidget *editor, QAbstractItemModel *model,
                      const QModelIndex &index) const;
    void paint(QPainter *painter, const QStyleOptionViewItem &option,
                       const QModelIndex &index) const;

private slots:

private:
    
};
#endif 
