#include <QtGui>
#include <QWhatsThis>
#include "mainwindow.h"
#include "default.h"
#include "document.h"
#include "newopensave.h"
#include "viewcatalogimpl.h"
#include "bottomwindow.h"
#include "bufsizeimpl.h"
#include "transmagimpl.h"
#include "subcatalog.h"
#include "profile.h"
#include "helpwindow.h"
#include "repairpro.h" 
#include "setting.h"
#include "histogram.h"
#include "append.h"
#include "rangelimit.h" 
#include "projectwizard.h"
#include "exportuser.h"
extern QStringList _OUTPUT_FORMAT_LIST_;
MainWindow *MainWindow::self = 0;
void MainWindow::removeWorkingMaterials(){
 if(tempDirectoryPath.isNull()) return;
 if ( viewcatalogimpl != 0) { delete viewcatalogimpl; viewcatalogimpl=0;}
 homeDirectory.refresh ();
if ( homeDirectory.exists(tempDirectoryPath) ) {
   tempDirectory.refresh ();
   if (homeDirectory.exists (openBinFileName)) homeDirectory.remove(openBinFileName);
   

   QString filename  = openBinFileName;
   filename.replace(openBinaryCatalogfileName(), openTempBinaryCatalogfileName() );
   if (homeDirectory.exists (filename)) homeDirectory.remove(filename);
    

   filename  = openBinFileName;
   filename.replace(openBinaryCatalogfileName(), openSecondBinaryCatalogfileName() );
   if (homeDirectory.exists (filename))  homeDirectory.remove(filename);


   filename  = openBinFileName;
   filename.replace(openBinaryCatalogfileName(), openTempMergeBinaryCatalogfileName() );
   if (homeDirectory.exists (filename)) homeDirectory.remove(filename);

   if (homeDirectory.exists (projectBinCatalogFileName)) homeDirectory.remove(projectBinCatalogFileName);
 
   homeDirectory.rmdir(tempDirectoryPath); 
  }
    tempDirectoryPath = QString ();
}
void  MainWindow::MakeOpenBinaryFileName() {
    QDate date = QDate::currentDate ();
    QTime time = QTime::currentTime ();
    QDateTime datetime (date, time ) ; 
    uint currentTime = datetime.toTime_t ();
    homeDirectoryPath = temporarySystemDirectoryPath();
	//homeDirectoryPath = ecpRootPath();
	homeDirectoryPath =  QDir::toNativeSeparators(homeDirectoryPath);
    QDir d(temporarySystemDirectoryPath());
    homeDirectory = d;
//   if ( !homeDirectory.exists("temp") )
//       d.mkdir ("tmp");
	// tempDirectoryPath = homeDirectoryPath() +
    tempDirectoryPath = temporarySystemDirectoryPath() +
                        QString("/%1%2").arg(temporaryDirectoryPrifix()).arg(currentTime);
	tempDirectoryPath =  QDir::toNativeSeparators(tempDirectoryPath);					
    d.mkdir (tempDirectoryPath);
    QDir dd(tempDirectoryPath);
    tempDirectory = dd;
    openBinFileName = 
                      tempDirectoryPath + QDir::toNativeSeparators("/") +
		      openBinaryCatalogfileName();

    projectBinCatalogFileName =  
                                tempDirectoryPath + QDir::toNativeSeparators("/") + 
		                openProjectBinaryCatalogfileName();

}

MainWindow::~MainWindow (){
   if  (viewcatalogimpl !=0) delete viewcatalogimpl;  
   if  (bottomWindow !=0) delete bottomWindow;
   if  (doc !=0) delete doc;
   if  (setDoc !=0) delete setDoc;
   if  (help !=0) delete help;
   if  (prolist !=0) delete prolist;
    exportAsActs.clear();
    showHideFileActs.clear();
    showHideProfileActs.clear();
    QApplication::clipboard()->clear(QClipboard::Selection);
    QApplication::clipboard()->clear(QClipboard::Clipboard);
}
MainWindow::MainWindow()
{
    undoRedoTimer = startTimer( 500 ); 
    viewcatalogimpl = 0;
    MakeOpenBinaryFileName();
    self = this;
    help = 0;
    doc = 0;
    setDoc = 0;
    setDoc = new SettingDocument();
    doc = new FDocument();
	initUI();
    createActions();
    createWhatsThis();
    createMenus();
    createToolBars();
    createStatusBar();
    undoAct->setEnabled (FALSE);
    redoAct->setEnabled (FALSE);
    readSettings();
    setWindowTitle(mainWindowCaption());
    setWindowIcon(QIcon(":/images/icon.png"));
	fileFilters = tr("Catalog files (*%1);;All Files (*)").arg(fileFilterLocator());
	projectFileFilters = tr("Project files (*%1);;All Files (*)").arg(projectFileFilterLocator());
	protocolFileFilters = tr("Protocol files (*%1);;All Files (*)").arg(protocolFileFilterLocator());
    setCurrentFile("");
    setCurrentProFile("");
    setCurrentProtocol("");
    FileActsShowHide(false);
    ProfileActsShowHide(false);
}
void MainWindow::initUI(){
	splitterVert = new QSplitter(this);
	splitterVert->setOrientation(Qt::Vertical);
    splitter = new QSplitter(splitterVert);
    prolist = new ProfileList(doc,"No Project",this);
    viewcatalogimpl = new ViewCatalogImpl(this,
	                    setDoc->GetWorkSpaceSize()->GetValue().toInt());	
	splitter->addWidget(prolist);
	splitter->addWidget(viewcatalogimpl);
	bottomWindow    = new BottomWindow(this);
  	splitterVert->addWidget(bottomWindow);
	setCentralWidget(splitterVert);
}
void MainWindow::FileActsShowHide(bool b){
  foreach (QAction *action, showHideFileActs) 
  if (b) action->setEnabled ( TRUE ); else action->setEnabled ( FALSE );

}
void MainWindow::ProfileActsShowHide(bool b){
  foreach (QAction *action, showHideProfileActs) 
  if (b) action->setEnabled ( TRUE ); else action->setEnabled ( FALSE );

}
void MainWindow::createActions()
{
   newAct = new QAction(QIcon(":/images/fileimport.png"), tr( "&Import"), this);
   newAct->setShortcut(tr("Ctrl+M"));
   newAct->setStatusTip(tr("Import a  catalog file"));
   connect(newAct, SIGNAL(triggered()), this, SLOT(newFile()));

    openAct = new QAction(QIcon(":/images/open.png"), tr("&Open"),this);
    openAct->setShortcut(tr("Ctrl+O"));
    openAct->setStatusTip(tr("Open an existing catalog file"));
    connect(openAct, SIGNAL(triggered()), this, SLOT(open()));


    closeAct = new QAction(QIcon(":/images/fileclose.png"), tr("C&lose"),this);
    closeAct->setShortcut(tr("Ctrl+L"));
    closeAct->setStatusTip(tr("Close an catalog file"));
    connect(closeAct, SIGNAL(triggered()), this, SLOT(closeFile()));
    //showHideFileActs.append(closeAct);
    
    saveAct = new QAction(QIcon(":/images/save.png"), tr("&Save"),this);
    saveAct->setShortcut(tr("Ctrl+S"));
    saveAct->setStatusTip(tr("Save the catalog to disk"));
    connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

    showHideFileActs.append(saveAct);

    saveAsAct =new QAction(QIcon(":/images/saveas.png"), tr( "Save &As..."), this);
    saveAsAct->setStatusTip(tr("Save the catalog under a new "
                               "name"));
    connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

    showHideFileActs.append(saveAsAct);

        for (int i = 0; i < MaxRecentFiles; ++i) {
            recentFileActs[i] = new QAction(this);
            recentFileActs[i]->setVisible(false);
            connect(recentFileActs[i], SIGNAL(triggered()),
                    this, SLOT(openRecentFile()));
        }

       for (int i = 0; i < MaxRecentFiles; ++i) {
            recentProjectFileActs[i] = new QAction(this);
            recentProjectFileActs[i]->setVisible(false);
            connect(recentProjectFileActs[i], SIGNAL(triggered()),
                    this, SLOT(openRecentProjectFile()));
        }
    newProfileAct =  new QAction(QIcon(":/images/project_new.png"), tr("&New project"),this);
    newProfileAct->setShortcut(tr("Shift+N"));
    newProfileAct->setStatusTip(tr("Generate a new project"));
    connect(newProfileAct, SIGNAL(triggered()), this, SLOT(newProfile()));


    openProfileAct =  new QAction(QIcon(":/images/project_open.png"), tr("&Open project"),this);
    openProfileAct->setShortcut(tr("Shift+O"));
    openProfileAct->setStatusTip(tr("Open an existing project"));
    connect(openProfileAct, SIGNAL(triggered()), this, SLOT(openProfile()));

    editProfileAct =  new QAction(QIcon(":/images/project_edit.png"), tr("&Edit project"),this);
    editProfileAct->setShortcut(tr("Shift+E"));
    editProfileAct->setStatusTip(tr("Edit an opened project"));
    connect(editProfileAct, SIGNAL(triggered()), this, SLOT(editProfile()));

    showHideProfileActs.append(editProfileAct);
    execAct =  new QAction(QIcon(":/images/exec.png"), tr("Execute &project"),this);
    execAct->setShortcut(tr("Shift+P"));
    execAct->setStatusTip(tr("Execute project task list"));
    connect(execAct, SIGNAL(triggered()), this, SLOT(buildProfile()));
    showHideProfileActs.append(execAct);


    closeProfileAct=  new QAction(QIcon(":/images/project_close.png"), tr("C&lose project"),this);
    closeProfileAct->setShortcut(tr("Shift+L"));
    closeProfileAct->setStatusTip(tr("Close an project"));
    connect(closeProfileAct, SIGNAL(triggered()), this, SLOT(closeProfile()));
    showHideProfileActs.append(closeProfileAct);
    

    saveProfileAct = new QAction(QIcon(":/images/project_save.png"), tr("&Save project"),this);
    saveProfileAct->setShortcut(tr("Shift+S"));
    saveProfileAct->setStatusTip(tr("Save the project to disk"));
    connect(saveProfileAct, SIGNAL(triggered()), this, SLOT(saveProfile()));    
    showHideProfileActs.append(saveProfileAct);


    saveAsProfileAct =  new QAction(QIcon(":/images/project_saveas.png"), tr("Save project &As..."),this);
    saveAsProfileAct->setStatusTip(tr("Save the project under a new "
                               "name"));
    connect(saveAsProfileAct, SIGNAL(triggered()), this, SLOT(saveAsProfile()));
    showHideProfileActs.append(saveAsProfileAct);
 //-------------------------------------------------------------------------------
        QString text;
		QAction *action;
        for(int i = 0; i < 3; i++){
        QString format =  _OUTPUT_FORMAT_LIST_.at(i);
        text = tr("%1").arg(QString(format).toUpper());
        action = new QAction(text, this);
        action->setData(format);
        connect(action, SIGNAL(triggered()), this, SLOT(exportSlot()));
        exportAsActs.append(action);
        showHideFileActs.append(action);
     }
        action = new QAction(exportUserFormat(), this);
        action->setData(exportUserFormat());
        connect(action, SIGNAL(triggered()), this, SLOT(exportSlot()));
        exportAsActs.append(action);
        showHideFileActs.append(action);

//----------------------------------------------------------------------------
    exportAct = new QAction(QIcon(":/images/fileexport.png"), tr("Export ..."),this);
    exportAct->setStatusTip(tr("Export a  catalog file "));
   // connect(exportAct, SIGNAL(triggered()), this, SLOT(exportSlot()));    
    showHideFileActs.append(exportAct);
   
    saveAllAct = new QAction(QIcon(":/images/saveall.png"), tr("Save All..."),this);  
    saveAllAct->setStatusTip(tr("Save catalog project and protocol files to disk "));
    connect(saveAllAct, SIGNAL(triggered()), this, SLOT(saveAll()));    
    showHideFileActs.append(saveAllAct);


    exitAct = new QAction(QIcon(":/images/exit.png"), tr("E&xit"),this);  
    exitAct->setShortcut(tr("Ctrl+Q"));
    exitAct->setStatusTip(tr("Exit the application"));
    connect(exitAct, SIGNAL(triggered()), qApp, SLOT(closeAllWindows()));

    contentAct = new QAction(QIcon(":/images/helpt.png"), tr("&Content"),this);  
    contentAct->setShortcut(tr("F1"));
    contentAct->setStatusTip(tr("Show the content of manual"));
    connect(contentAct, SIGNAL(triggered()), this, SLOT(mainHelpSlot()));

    histAct = new QAction(QIcon(":/images/hist.png"), tr("Draw histo&grams"),this);  
    histAct->setShortcut(tr("Alt+G"));
    histAct->setStatusTip(tr("Draw histograms"));
    connect(histAct, SIGNAL(triggered()), this, SLOT(drawHist()));
	
	

    cutAct = new QAction(QIcon(":/images/cut.png"), tr("Cu&t"),this);
    cutAct->setShortcut(tr("Ctrl+X"));
    cutAct->setStatusTip(tr("Cut the current selection's contents "
                            "to the clipboard"));
    connect(cutAct, SIGNAL(triggered()), this, SLOT(cut()));

    copyAct = new QAction(QIcon(":/images/copy.png"), tr("&Copy"),this);
    copyAct->setShortcut(tr("Ctrl+C"));
    copyAct->setStatusTip(tr("Copy the current selection's contents "
                             "to the clipboard"));
    connect(copyAct, SIGNAL(triggered()), this, SLOT(copy()));


    pasteAct = new QAction(QIcon(":/images/paste.png"), tr("&Paste"),this);
    pasteAct->setShortcut(tr("Ctrl+V"));
    pasteAct->setStatusTip(tr("Paste the clipboard's contents into "
                              "the current selection"));
    connect(pasteAct, SIGNAL(triggered()),
            this, SLOT(paste()));


    undoAct = new QAction(QIcon(":/images/undo.png"), tr("&Undo"),this);
    undoAct->setShortcut(tr("Ctrl+Z"));
    undoAct->setStatusTip(tr("Undo edit action"));
    connect(undoAct, SIGNAL(triggered()),this, SLOT(undo()));


    redoAct = new QAction(QIcon(":/images/redo.png"), tr("&Redo"),this);
    redoAct->setShortcut(tr("Ctrl+Y"));
    redoAct->setStatusTip(tr("Redo edit action"));
    connect(redoAct, SIGNAL(triggered()),this, SLOT(redo()));
   
 

    zoominAct = new QAction(QIcon(":/images/viewmag+.png"), tr("Zoom &In"),this);
    zoominAct->setShortcut(tr("Ctrl++"));
    zoominAct->setStatusTip(tr("Zoom in"));
    connect(zoominAct, SIGNAL(triggered()),this, SLOT(zoomin()));



    zoomoutAct = new QAction(QIcon(":/images/viewmag-.png"), tr("Zoom &Out"),this);
    zoomoutAct->setShortcut(tr("Ctrl+-"));
    zoomoutAct->setStatusTip(tr("Zoom out"));
    connect(zoomoutAct, SIGNAL(triggered()),this, SLOT(zoomout()));

    deleteAct =  new QAction(QIcon(":/images/delete.png"), tr("&Delete"),this);
    deleteAct->setShortcut(QKeySequence::Delete);
    deleteAct->setStatusTip(tr("Delete the current selection's "
                               "contents"));
    connect(deleteAct, SIGNAL(triggered()),this, SLOT(del()));


    
    insertRowAct = new QAction(QIcon(":/images/insert_table_row.png"), tr("&Insert row"),this);
    insertRowAct->setShortcut(tr("Ctrl+I"));
    insertRowAct->setStatusTip(tr("Insert row before the selected "
                               "row"));
    connect(insertRowAct, SIGNAL(triggered()),this, SLOT(insert()));

    selectAllAct = new QAction(tr("Select &All"),this);
    selectAllAct->setStatusTip(tr("Select all the cells in the "
                                  "View Buffer"));
    connect(selectAllAct, SIGNAL(triggered()),this, SLOT(selectAll()));

//  View
	viewFindAct = new QAction(QIcon(":/images/view-segment-find.png"), tr("&Find record"),this);
    viewFindAct->setShortcut(tr("Shift+F"));
    viewFindAct->setStatusTip(tr("Find catalog record by date or by number"));
    connect(viewFindAct, SIGNAL(triggered()),viewcatalogimpl->cattable, 
	        SLOT(viewfind()));

	viewInfoAct = new QAction(QIcon(":/images/view-segment-info.png"), tr("&Info on catalog"),this);
    viewInfoAct->setShortcut(tr("Shift+I")); 
    viewInfoAct->setStatusTip(tr("The information on the catalog and the view segment"));
    connect(viewInfoAct, SIGNAL(triggered()),this, SLOT(viewinfo()));

	viewSaveAct = new QAction(QIcon(":/images/view-segment-save.png"), tr("Save &view segment changes"),this);
    viewSaveAct->setShortcut(tr("Shift+V"));
    viewSaveAct->setStatusTip(tr("Save view catalog segment changes"));
    connect(viewSaveAct, SIGNAL(triggered()),viewcatalogimpl->cattable, 
	        SLOT(buttonSaveArea()));

	viewCancelAct = new QAction(QIcon(":/images/view-segment-rem.png"), tr("&Cancel view segment changes"),this);
    viewCancelAct->setShortcut(tr("Shift+C"));
    viewCancelAct->setStatusTip(tr("Cancel view catalog segment changes"));
    connect(viewCancelAct, SIGNAL(triggered()),viewcatalogimpl->cattable, 
	        SLOT(buttonCancelArea()));
 

//  Tools

    settingsAct = new QAction(QIcon(":/images/settings.png"), tr("Se&ttings..."),this);
    settingsAct->setShortcut(tr("Shift+T"));
    settingsAct->setStatusTip(tr("Configuration of  parameters "));
    connect(settingsAct, SIGNAL(triggered()), this, SLOT(settingsParameters()));
 
    checkerrAct = new QAction(QIcon(":/images/error.png"), tr("&Check ranges"),this);
    checkerrAct->setShortcut(tr("Alt+C"));
    checkerrAct->setStatusTip(tr("Check specified ranges "));
    connect(checkerrAct, SIGNAL(triggered()), this, SLOT(checkerror()));
    
    duplicatelAct = new QAction(QIcon(":/images/duplicate.png"), tr("Remove &duplicates"),this);
    duplicatelAct->setShortcut(tr("Alt+D"));
    duplicatelAct->setStatusTip(tr("Check and remove duplicate records "));
    connect(duplicatelAct, SIGNAL(triggered()), this, SLOT(duplicate()));
   

    magnitudeAct= new QAction(QIcon(":/images/magnitude.png"), tr("Co&nvert magnitudes"),this);
    magnitudeAct->setShortcut(tr("Alt+N"));
    magnitudeAct->setStatusTip(tr("Convert magnitudes to common type"));
    connect(magnitudeAct, SIGNAL(triggered()), this, SLOT(magnitude()));
    
    sampleAct= new QAction(QIcon(":/images/sample.png"), tr("&Select"),this);
    sampleAct->setShortcut(tr("Alt+S"));
    sampleAct->setStatusTip(tr("Select subcatalog"));
    connect(sampleAct, SIGNAL(triggered()), this, SLOT(sample()));  
    
    mergeAct= new QAction(QIcon(":/images/mergecell.png"), tr("&Merge"),this);
    mergeAct->setShortcut(tr("Alt+M"));
    mergeAct->setStatusTip(tr("Merge catalogs"));
    connect(mergeAct, SIGNAL(triggered()), this, SLOT(merge()));

  
    addAct= new QAction(QIcon(":/images/append.png"), tr("S&ubstitute"),this);
    addAct->setShortcut(tr("Alt+U"));
    addAct->setStatusTip(tr("Substitute parts of the initial catalog"));
    connect(addAct, SIGNAL(triggered()), this, SLOT(add()));

    compAct= new QAction(QIcon(":/images/comp.png"), tr("Compare cata&logs"),this);
    compAct->setShortcut(tr("Alt+L"));
    compAct->setStatusTip(tr("Compare two catalogs"));
    connect(compAct, SIGNAL(triggered()), this, SLOT(comp()));
        
    checkSortAct= new QAction(QIcon(":/images/checksort.png"), tr("Chec&k order"),this);
    checkSortAct->setShortcut(tr("Alt+K"));
    checkSortAct->setStatusTip(tr("Check time order"));
    connect(checkSortAct, SIGNAL(triggered()), this, SLOT(checksort()));
   
    sortAct= new QAction(QIcon(":/images/sort.png"), tr("Sort by t&ime"),this);
    sortAct->setShortcut(tr("Alt+I"));
    sortAct->setStatusTip(tr("Sort catalog records by ascending time"));
    connect(sortAct, SIGNAL(triggered()), this, SLOT(sort()));
    
    optviewbufAct = new QAction(nameViewBufferCatalog(),this);
    optviewbufAct->setStatusTip(nameViewBufferCatalog());
    connect(optviewbufAct, SIGNAL(triggered()), this, SLOT(viewbuf()));

    opterrbufAct = new QAction(nameViewBufferError(),this);
    opterrbufAct->setStatusTip(nameViewBufferError());
    connect(opterrbufAct, SIGNAL(triggered()), this, SLOT(errbuf()));

    optcorrectlineAct = new QAction(nameMaxLinesErrorsCorrectonPage(),this);
    optcorrectlineAct->setStatusTip(nameMaxLinesErrorsCorrectonPage());
    connect(optcorrectlineAct, SIGNAL(triggered()), this, SLOT(correctline()));

    aboutAct = new QAction(tr("&About"),this);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));
    
	
    aboutQwtAct = new QAction(tr("&About Q&wt"), this);
    aboutQwtAct->setStatusTip(tr("Show the Qwt library's About box"));
    connect(aboutQwtAct, SIGNAL(triggered()), this, SLOT(aboutQwt()));

    aboutQtAct = new QAction(tr("About &Qt"),this);
    aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
//----------------------------------------------------------------------------
    saveCommentAct = new QAction(QIcon(":/images/savecom.png"), tr("Save pro&tocol"),this);
    saveCommentAct->setShortcut(tr("Shift+T"));
    saveCommentAct->setStatusTip(tr("Save the protocol to disk"));
    connect(saveCommentAct, SIGNAL(triggered()), this, SLOT(saveComment()));

//    showHideFileActs.append(saveCommentAct);

    saveAsCommentAct =new QAction(QIcon(":/images/saveascom.png"), tr( "Save protocol As ..."), this);
    saveAsCommentAct->setStatusTip(tr("Save the protocol under a new name"));
    connect(saveAsCommentAct, SIGNAL(triggered()), this, SLOT(saveAsComment()));
//    showHideFileActs.append(saveAsCommentAct);

    printPreviewAct = new QAction(QIcon(":/images/preview.png"), tr("P&review protocol"),this);
    printPreviewAct->setShortcut(tr("Shift+R"));
    printPreviewAct->setStatusTip(tr("Preview protocol"));
    connect(printPreviewAct, SIGNAL(triggered()), this, SLOT(previewProtocol()));

//    showHideFileActs.append(printPreviewAct);
//----------------------------------------------------------------------------
}
void MainWindow::createWhatsThis(){
/// file
   const char * FileOpenText = "<p><img source=\":/images/open.png\"> "
				    "Click this button to open a <em> existing standard format catalog file</em>."
				    "You can also select the <b>Open</b> command "
				    "from the <b>File</b> menu. Keyboard shortcut is <b>(Ctrl+O)</b></p>";
    openAct->setWhatsThis( FileOpenText );

 const char * ImportText = "<p><img source=\":/images/fileimport.png\"> "
				    "Click this button to import a <em> catalog file</em>."
				    "You can also select the <b>Import</b> command "
				    "from the <b>File</b> menu. Keyboard shortcut is <b>(Ctrl+M)</b></p>";

    newAct->setWhatsThis( ImportText );

  const char * FileSaveText = "<p><img source=\":/images/save.png\"> "
				    "Click this button to save a <em>catalog to disk</em>."
				    "You can also select the <b>Save </b> command "
				    "from the <b>File</b> menu. Keyboard shortcut is <b>(Ctrl+S)</b></p>";
  
    saveAct->setWhatsThis( FileSaveText );
    
   const char * FileSaveAsText = "<p><img source=\":/images/saveas.png\"> "
				    "Click this button to save a <em>currently catalog to disk under a new name</em>."
				    "You can also select the <b>Save as ... </b> command "
				    "from the <b>File</b> menu. </p>";

    saveAsAct->setWhatsThis( FileSaveAsText );

const char * saveallText = "<p><img source=\":/images/saveall.png\"> "
				    "Click this button to save <em>catalog, project and protocol files to disk</em>."
				    "You can also select the <b>Save All ...</b> command "
				    "from the <b>File</b> menu.</p>";

   saveAllAct->setWhatsThis(saveallText);

const char * exportFileText = "<p><img source=\":/images/fileexport.png\"> "
				    "Click this button to export a <em>catalog file to disk</em>."
				    "You can also select the <b>Export...</b> command "
				    "from the <b>File</b> menu.</p>";

   exportAct->setWhatsThis(exportFileText);



/// project

const char * newProfileText = "<p><img source=\":/images/project_new.png\"> "
				    "Click this button to generate a <em>new project</em>."
				    "You can also select the <b>New project</b> command "
				    "from the <b>Project</b> menu. Keyboard shortcut is <b>(Shift+N)</b></p>";

   newProfileAct->setWhatsThis( newProfileText);

const char * openProfileText = "<p><img source=\":/images/project_open.png\"> "
				    "Click this button to open an <em>existing project</em>."
				    "You can also select the <b>Open project</b> command "
				    "from the <b>Project</b> menu. Keyboard shortcut is <b>(Shift+O)</b></p>";

   openProfileAct->setWhatsThis(openProfileText);

const char * editProfileText = "<p><img source=\":/images/project_edit.png\"> "
				    "Click this button to edit an <em>opened project</em>."
				    "You can also select the <b>Edit project</b> command "
				    "from the <b>Project</b> menu. Keyboard shortcut is <b>(Shift+E)</b></p>";

   editProfileAct->setWhatsThis(editProfileText);

const char * execProfileText = "<p><img source=\":/images/exec.png\"> "
				    "Click this button to execute an <em>opened project task list</em>."
				    "You can also select the <b>Execute project</b> command "
				    "from the <b>Project</b> menu. Keyboard shortcut is <b>(Shift+P)</b></p>";

   execAct->setWhatsThis(execProfileText);

const char * saveProfileText = "<p><img source=\":/images/project_save.png\"> "
				    "Click this button to save a <em>project to disk</em>."
				    "You can also select the <b>Save project</b> command "
				    "from the <b>Project</b> menu. Keyboard shortcut is <b>(Shift+S)</b></p>";

   saveProfileAct->setWhatsThis(saveProfileText);

const char * saveasProfileText = "<p><img source=\":/images/project_saveas.png\"> "
				    "Click this button to save a <em>project to disk under a new name</em>."
				    "You can also select the <b>Save project As ...</b> command "
				    "from the <b>Project</b> menu.</p>";

   saveAsProfileAct->setWhatsThis(saveasProfileText);


const char * exitText = "<p><img source=\":/images/exit.png\"> "
				    "Click this button to <em>exit the application</em>."
				    "You can also select the <b>Exit</b> command "
				    "from the <b>File</b> menu. Keyboard shortcut is <b>(Ctrl+Q)</b></p>";

   exitAct->setWhatsThis(exitText);

const char * zoominText = "<p><img source=\":/images/viewmag+.png\"> "
				    "Click this button to increase the <em>display font size</em>."
				    "You can also select the <b>Zoom In</b> command "
				    "from the <b>Edit</b> menu. Keyboard shortcut is <b>(Ctrl++)</b></p>";

   zoominAct->setWhatsThis(zoominText);

const char * zoomoutText = "<p><img source=\":/images/viewmag-.png\"> "
				    "Click this button to decrease the <em>display font size</em>."
				    "You can also select the <b>Zoom Out</b> command "
				    "from the <b>Edit</b> menu. Keyboard shortcut is <b>(Ctrl+-)</b></p>";

   zoomoutAct->setWhatsThis(zoomoutText);

const char * undoText = "<p><img source=\":/images/undo.png\"> "
				    "Click this button to revert the <em>most recent editing actions</em>."
				    "You can also select the <b>Undo</b> command "
				    "from the <b>Edit</b> menu. Keyboard shortcut is <b>(Ctrl+Z)</b></p>";

   undoAct->setWhatsThis(undoText);

const char * redoText = "<p><img source=\":/images/redo.png\"> "
				    "Click this button to revert the <em>most recent undo operation</em>."
				    "You can also select the <b>Redo</b> command "
				    "from the <b>Edit</b> menu. Keyboard shortcut is <b>(Ctrl+Y)</b></p>";

   redoAct->setWhatsThis(redoText);

const char * cutText = "<p><img source=\":/images/cut.png\"> "
				    "Click this button to cut the <em>current selection's contents to the clipboard</em>."
				    "You can also select the <b>Cut</b> command "
				    "from the <b>Edit</b> menu. Keyboard shortcut is <b>(Ctrl+X)</b></p>";


   cutAct->setWhatsThis(cutText);

const char * copyText = "<p><img source=\":/images/copy.png\"> "
				    "Click this button to copy the <em>current selection's contents to the clipboard</em>."
				    "You can also select the <b>Copy</b> command "
				    "from the <b>Edit</b> menu. Keyboard shortcut is <b>(Ctrl+C)</b></p>";

   copyAct->setWhatsThis(copyText);

const char * pasteText = "<p><img source=\":/images/paste.png\"> "
				    "Click this button to paste the <em>clipboard's contents into the current selection</em>."
				    "You can also select the <b>Paste</b> command "
				    "from the <b>Edit</b> menu. Keyboard shortcut is <b>(Ctrl+Y)</b></p>";

   pasteAct->setWhatsThis(pasteText);

const char * deleteText = "<p><img source=\":/images/delete.png\"> "
				    "Click this button to delete the <em>current selections</em>."
				    "You can also select the <b>Delete</b> command "
				    "from the <b>Edit</b> menu. Keyboard shortcut is <b>(Del)</b></p>";

   deleteAct->setWhatsThis(deleteText);

const char * insertText = "<p><img source=\":/images/insert_table_row.png\"> "
				    "Click this button to insert the <em>row before the selected row</em>."
				    "You can also select the <b>Insert</b> command "
				    "from the <b>Edit</b> menu. Keyboard shortcut is <b>(Ctrl+I)</b></p>";

   insertRowAct->setWhatsThis(insertText);

const char * checkSortText = "<p><img source=\":/images/checksort.png\"> "
				    "Click this button to check the <em>time oder</em>."
				    "You can also select the <b>Check order</b> command "
				    "from the <b>Tool</b> menu. Keyboard shortcut is <b>(Alt+I)</b>F";

   checkSortAct->setWhatsThis(checkSortText);

const char * sortSortText = "<p><img source=\":/images/sort.png\"> "
				    "Click this button to <em>sort by time</em>."
				    "You can also select the <b>Sort by time</b> command "
				    "from the <b>Tool</b> menu. Keyboard shortcut is <b>(Alt+T)</b></p>";

   sortAct->setWhatsThis(sortSortText);

const char * magnitudeText = "<p><img source=\":/images/magnitude.png\"> "
				    "Click this button to comvert the <em>magnitudes to common type</em>."
				    "You can also select the <b>Convert magnitudes</b> command "
				    "from the <b>Tool</b> menu. Keyboard shortcut is <b>(Alt+N)</b></p>";

   magnitudeAct->setWhatsThis(magnitudeText);

const char * duplicatelText = "<p><img source=\":/images/duplicate.png\"> "
				    "Click this button to check and remove the <em>duplicate record</em>."
				    "You can also select the <b>Remove duplicates</b> command "
				    "from the <b>Tool</b> menu. Keyboard shortcut is <b>(Alt+D)</b></p>";

   duplicatelAct->setWhatsThis(duplicatelText);

const char * sampleText = "<p><img source=\":/images/sample.png\"> "
				    "Click this button to select the <em>subcatalog</em>."
				    "You can also select the <b>Select</b> command "
				    "from the <b>Tool</b> menu. Keyboard shortcut is <b>(Alt+S)</b></p>";

   sampleAct->setWhatsThis(sampleText);

const char * mergeText = "<p><img source=\":/images/mergecell.png\"> "
				    "Click this button to merge the <em>catalogs</em>."
				    "You can also select the <b>Merge</b> command "
				    "from the <b>Tool</b> menu. Keyboard shortcut is <b>(Alt+M)</b></p>";

   mergeAct->setWhatsThis(mergeText);

const char * addText = "<p><img source=\":/images/append.png\"> "
				    "Click this button to substitute the <em>parts of the initial catalog</em>."
				    "You can also select the <b>Substitute</b> command "
				    "from the <b>Tool</b> menu. Keyboard shortcut is <b>(Alt+U)</b></p>";

   addAct->setWhatsThis(addText);

const char * compText = "<p><img source=\":/images/comp.png\"> "
				    "Click this button to compare <em>two catalogs</em>."
				    "You can also select the <b>Compare</b> command "
				    "from the <b>Tool</b> menu. Keyboard shortcut is <b>(Alt+L)</b></p>";

   compAct->setWhatsThis(compText);
  

const char * checkerrText = "<p><img source=\":/images/error.png\"> "
				    "Click this button to check the <em>specified ranges</em>."
				    "You can also select the <b>Check range</b> command "
				    "from the <b>Tool</b> menu. Keyboard shortcut is <b>(Alt+C)</b></p>";

   checkerrAct->setWhatsThis(checkerrText);

const char * saveCommentText = "<p><img source=\":/images/savecom.png\"> "
				    "Click this button to save the <em>protocol to disk</em>."
				    "You can also select the <b>Save Protocol</b> command "
				    "from the <b>Protocol</b> menu. Keyboard shortcut is <b>(Shift+T)</b></p>";

   saveCommentAct->setWhatsThis(saveCommentText);

const char * saveAsCommentText = "<p><img source=\":/images/saveascom.png\"> "
				    "Click this button to save the <em>protocol under a new name</em>."
				    "You can also select the <b>Save protocol As...</b> command "
				    "from the <b>Protocol</b> menu.</p>";

   saveAsCommentAct->setWhatsThis(saveAsCommentText);

const char * printPreviewText = "<p><img source=\":/images/preview.png\"> "
				    "Click this button to preview the <em>protocol</em>."
				    "You can also select the <b>Preview protocol</b> command "
				    "from the <b>Protocol</b> menu. Keyboard shortcut is <b>(Shift+R)</b></p>";

   printPreviewAct->setWhatsThis(printPreviewText);

 const char * histActText = "<p><img source=\":/images/hist.png\"> "
				    "Click this button to draw the <em>histograms</em>."
				    "You can also select the <b>Draw histograms</b> command "
				    "from the <b>Tools</b> menu. Keyboard shortcut is <b>(Alt+G)</b></p>";

   histAct->setWhatsThis(histActText);  
   
//==============================================================================================
//  View

const char * viewFindActText = "<p><img source=\":/images/view-segment-find.png\"> "
				    "Click this button to find the catalog record by <em>date or by number</em>."
				    "You can also select the <b>Find record</b> command "
				    "from the <b>View</b> menu. Keyboard shortcut is <b>(Shift+F)</b></p>";

   viewFindAct->setWhatsThis(viewFindActText);

const char * viewInfoActText = "<p><img source=\":/images/view-segment-info.png\"> "
				    "Click this button to get the information on the <em>catalog and the view segment</em>."
				    " Keyboard shortcut is <b>(Shift+I)</b></p>";
   viewInfoAct->setWhatsThis(viewInfoActText);

const char * viewSaveActText = "<p><img source=\":/images/view-segment-save.png\"> "
				    "Click this button to save the <em>view segment changes</em>."
				    "You can also select the <b>Save view segment changes</b> command "
				    "from the <b>View</b> menu. Keyboard shortcut is <b>(Shift+V)</b></p>";

   viewSaveAct->setWhatsThis(viewSaveActText);

const char * viewCancelActText = "<p><img source=\":/images/view-segment-rem.png\"> "
				    "Click this button to cancel the <em>view segment changes</em>."
				    "You can also select the <b>Cancel view segment changes</b> command "
				    "from the <b>View</b> menu. Keyboard shortcut is <b>(Shift+C)</b></p>";

   viewCancelAct->setWhatsThis(viewCancelActText);

//=================================================================================================
}
void MainWindow::createMenus()
{
   
    exportAsMenu = new QMenu(tr("Export ..."), this);
    foreach (QAction *action, exportAsActs)
        exportAsMenu->addAction(action);
    exportAsMenu->setIcon(QIcon(":/images/fileexport.png"));
    exportAct->setMenu(exportAsMenu);
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(openAct);
    fileMenu->addAction(newAct);
    fileMenu->addAction(saveAct);
    fileMenu->addAction(saveAsAct);
    fileMenu->addAction(saveAllAct);
    fileMenu->addMenu(exportAsMenu);
    separatorAct = fileMenu->addSeparator();
        for (int i = 0; i < MaxRecentFiles; ++i)
            fileMenu->addAction(recentFileActs[i]);
    fileMenu->addSeparator();
    fileMenu->addAction(closeAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);
    updateRecentFileActions();

	editMenu = menuBar()->addMenu(tr("&Edit")); 
	editMenu->addAction(undoAct);
    editMenu->addAction(redoAct);	
    editMenu->addAction(cutAct);
    editMenu->addAction(copyAct);
    editMenu->addAction(pasteAct);
    editMenu->addAction(deleteAct);
    editMenu->addAction(insertRowAct);
    editMenu->addSeparator();
    editMenu->addAction(selectAllAct);

    viewMenu = menuBar()->addMenu(tr("&View")); 
     
    viewMenu->addAction(viewFindAct);   
    viewMenu->addAction(viewSaveAct);    
    viewMenu->addAction(viewCancelAct);
	viewMenu->addSeparator();
	viewMenu->addAction(zoominAct);
    viewMenu->addAction(zoomoutAct);

 
	
    projectMenu  = menuBar()->addMenu(tr("&Project"));
    projectMenu->addAction(newProfileAct);
    projectMenu->addAction(openProfileAct);
    projectMenu->addAction(editProfileAct);
    projectMenu->addAction(execAct);
    projectMenu->addAction(saveProfileAct);
    projectMenu->addAction(saveAsProfileAct);
    separatorProjectAct = projectMenu->addSeparator();
        for (int i = 0; i < MaxRecentFiles; ++i)
            projectMenu->addAction(recentProjectFileActs[i]);
    projectMenu->addSeparator();
    updateRecentProjectFileActions();
    projectMenu->addSeparator();
    projectMenu->addAction(closeProfileAct);	

	protocolMenu = menuBar()->addMenu(tr("P&rotocol"));
    protocolMenu->addAction(printPreviewAct);
    protocolMenu->addAction(saveCommentAct);
    protocolMenu->addAction(saveAsCommentAct);
//    protocolMenu->addSeparator();

	
    toolsMenu = menuBar()->addMenu(tr("&Tools"));
    toolsMenu->addSeparator();
    toolsMenu->addAction(checkSortAct);
    toolsMenu->addAction(sortAct);
    toolsMenu->addSeparator();
    toolsMenu->addAction(magnitudeAct);
    toolsMenu->addAction(duplicatelAct);
    toolsMenu->addAction(sampleAct);
    toolsMenu->addAction(mergeAct);
    toolsMenu->addAction(addAct);
    toolsMenu->addSeparator();
    toolsMenu->addAction(checkerrAct);
    toolsMenu->addAction(compAct);
    toolsMenu->addSeparator();
    toolsMenu->addAction(histAct);


    optionsMenu = menuBar()->addMenu(tr("&Options"));
	optionsMenu->addAction(optviewbufAct);
	optionsMenu->addAction(opterrbufAct);
    optionsMenu->addAction(optcorrectlineAct);

    //optionsMenu->addAction(settingsAct);

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(contentAct);

    whatsThisAct = helpMenu->addAction(QIcon(":/images/help.png"), tr("&What's This?"),
                               this, SLOT(onWhatsThis()));
    whatsThisAct->setWhatsThis(tr("Enter What's This? mode."));

    whatsThisAct->setShortcut(Qt::SHIFT + Qt::Key_F1);

    helpMenu->addAction(aboutAct);
    helpMenu->addAction(aboutQwtAct);
    helpMenu->addAction(aboutQtAct);



}


void MainWindow::createToolBars()
{
 

    fileToolBar = addToolBar(tr("File"));
    fileToolBar->addAction(openAct);
    fileToolBar->addAction(newAct);
    fileToolBar->addAction(saveAct);
    fileToolBar->addAction(saveAsAct);
    fileToolBar->addAction(saveAllAct);
    fileToolBar->addAction(exportAct);
    fileToolBar->addSeparator();
//    fileToolBar->addSeparator();   
    fileToolBar->addAction(newProfileAct);
    fileToolBar->addAction(openProfileAct);
    fileToolBar->addAction(editProfileAct);
    fileToolBar->addAction(execAct);
    fileToolBar->addAction(saveProfileAct);
    fileToolBar->addAction(saveAsProfileAct);
//    fileToolBar->addSeparator();
    fileToolBar->setOrientation(Qt::Horizontal);
    fileToolBar->setAllowedAreas(Qt::TopToolBarArea);
    fileToolBar->setIconSize(QSize(22,22));

    editToolBar = addToolBar(tr("Edit"));
    editToolBar->addAction(undoAct);
    editToolBar->addAction(redoAct);
//    editToolBar->addSeparator();    
    editToolBar->addAction(cutAct);                
    editToolBar->addAction(copyAct);
    editToolBar->addAction(pasteAct);
    editToolBar->addAction(deleteAct);
    editToolBar->addAction(insertRowAct);  
//    editToolBar->addSeparator();    
    editToolBar->addAction(zoominAct);
    editToolBar->addAction(zoomoutAct);

    editToolBar->addAction(whatsThisAct);

    editToolBar->addSeparator();
    editToolBar->addAction(exitAct);   
    editToolBar->setOrientation(Qt::Horizontal);
    editToolBar->setAllowedAreas(Qt::TopToolBarArea);
    editToolBar->setIconSize(QSize(22,22));

    viewToolBar =   new QToolBar (tr("View"),this );
    viewToolBar->addAction(viewInfoAct);    
    viewToolBar->addAction(viewFindAct);   
    viewToolBar->addAction(viewSaveAct);    
    viewToolBar->addAction(viewCancelAct);
    viewToolBar->setOrientation(Qt::Vertical);
    addToolBar ( Qt::RightToolBarArea,viewToolBar);

    viewToolBar->setIconSize(QSize(22,22));
	
    toolToolBar  =   new QToolBar (tr("Tool"),this );
 //   toolToolBar->addAction(settingsAct);
    toolToolBar->addAction(checkerrAct);
    toolToolBar->addAction(checkSortAct);
    toolToolBar->addAction(sortAct);
    toolToolBar->addSeparator();
    toolToolBar->addAction(magnitudeAct);

    toolToolBar->addAction(duplicatelAct);
    toolToolBar->addAction(sampleAct);
    toolToolBar->addAction(mergeAct);
    toolToolBar->addAction(addAct);
    toolToolBar->addSeparator();
    toolToolBar->addAction(checkerrAct);
    toolToolBar->addAction(compAct); 
    toolToolBar->addSeparator();
    toolToolBar->addAction(histAct);
	
    toolToolBar->setOrientation(Qt::Vertical);
    addToolBar ( Qt::LeftToolBarArea,toolToolBar);
//    fileToolBar->setOrientation(Qt::Horizontal);
//    addToolBar ( Qt::TopToolBarArea,toolToolBar);

    toolToolBar->setIconSize(QSize(22,22));

    commentBar  =   new QToolBar (tr("Protocol"),this );
    commentBar->addAction(printPreviewAct);
    commentBar->addAction(saveCommentAct);
    commentBar->addAction(saveAsCommentAct);
    commentBar->addSeparator();
    protocolLineEdit  = new QLabel ("Protocol: unnamed",commentBar) ;
    commentBar->addWidget(protocolLineEdit);
    addToolBar ( Qt::BottomToolBarArea,commentBar);
    commentBar->setIconSize(QSize(22,22));

}

void MainWindow::newFile()
{

  if (maybeSave()) if (maybeProtocolSave()) {
  ImportFormatSetDialogs dlg(this);
  dlg.resize(MyOpenFileDialogSize);
  dlg.move(MyOpenFileDialogPos);
  dlg.GetSlitter()->restoreState(splitterImportFile);
  importUserDirectory = QDir::toNativeSeparators(importUserDirectory);
  if(  dlg.exec() == QDialog::Accepted) { 
    setCurrentFile("");

    if(doc !=0) {delete doc; doc =0;}
    doc = new FDocument();
    prolist->closeProfile();
    initCentralWidget(MainWindow::check);
    ProfileActsShowHide(false);
    modLabel->setText(" Tools ");
    actionLabel->setText(" Ready ");
   // messageLabel->setText("");
	
    }
   MyOpenFileDialogPos   =  dlg.pos();
   MyOpenFileDialogSize  =  dlg.size();
 } 
}
void MainWindow::onWhatsThis()
{
    QWhatsThis::enterWhatsThisMode();
}
QString MainWindow::AbsDir(const QString fileName) const{

   if (!fileName.isEmpty()) return QFileInfo (fileName).absoluteDir ().absolutePath ();
   else return QDir::homePath ();

}
void MainWindow::open()
{
    if (maybeSave()) if (maybeProtocolSave()){
		QString fileName =
        QFileDialog::getOpenFileName(this,"Open an existing catalog file",openUserFileDirectory, fileFilters);
		if (!fileName.isEmpty()) {
                        openInitial();
                        prolist->closeProfile();
			if(! loadFile(fileName) ) return;
		        openUserFileDirectory = QFileInfo(fileName).absolutePath ();
        }
	}
}
void MainWindow::openInitial(){
			if(doc !=0) {delete doc; doc =0;}
			doc = new FDocument();
			bottomWindow->OpenProblemsProtocolPage();
			bottomWindow->messageTab->clear();
			bottomWindow->correctionTab->clear();
			ProfileActsShowHide(false);
			modLabel->setText(" Tools ");
			actionLabel->setText(" Ready ");
			messageLabel->setText("");
			
}
void MainWindow::loadFileAuto(const QString &fileName){
    if (!fileName.isEmpty()) {
                openInitial();
		if(! loadFile(fileName) ) return;
			
	
	}
}
bool MainWindow::loadFile(const QString &fileName)
{
    QApplication::setOverrideCursor(Qt::WaitCursor);
    Catalog catalog;
    if(catalog.loadBinaryFile(fileName,MainWindow::First)) {
    setCurrentFile(fileName);
	
    initCentralWidget(MainWindow::ignore);
    } else return false;

    QApplication::restoreOverrideCursor();
    return true;

}

bool MainWindow::save()
{


    if (curFile.isEmpty()) {
        return saveAs();
    } else {
        saveFile(curFile);
        return true;
    }

}

void MainWindow::saveFile(const QString &fileName)
{
if (viewcatalogimpl->isCatalogLoaded())
if( viewcatalogimpl->cattable->maybeSave()) {
    QApplication::setOverrideCursor(Qt::WaitCursor);
    if (viewcatalogimpl->cattable->writeFile(fileName)) {

	   setCurrentFile(fileName);

    statusBar()->showMessage(tr("File saved"), 2000);

    } else {
        statusBar()->showMessage(tr("Saving canceled"), 2000);
    }

    QApplication::restoreOverrideCursor();
}
}

bool MainWindow::saveAs()
{ 
    QString fileName =
            QFileDialog::getSaveFileName(this,"Save the catalog under a new name",saveUserFileDirectory, fileFilters);
    if (fileName.isEmpty())
        return false;
    int ind = fileName.length () - fileFilterLocator().length ()-1; if (ind < 0) ind =0;
    if ( ( fileName.indexOf(fileFilterLocator(),ind)) == -1 && ( fileName.indexOf(fileFilterLocator().toUpper(),ind)) == -1) {
        fileName.append(fileFilterLocator());
        if (QFile::exists(fileName)) {
        int ret = QMessageBox::warning(this, mainWindowCaption(),
                     QString("File %1 already exists.\n Do you want to overwrite it?")
                     .arg(QDir::toNativeSeparators(fileName)),
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No | QMessageBox::Escape);
        if (ret == QMessageBox::No)
            return true;
       }
    }

    if (!fileName.isEmpty())
        saveFile(fileName);
		saveUserFileDirectory = QFileInfo(fileName).absolutePath ();
    return true;

}

void MainWindow::closeEvent(QCloseEvent *event)
{
 if (viewcatalogimpl->isCatalogLoaded())
 if( !viewcatalogimpl->cattable->maybeSave()) {event->ignore(); return;}
 if (!maybeSave()) {event->ignore(); return;} 
 if (!maybeProtocolSave()) {event->ignore(); return;} 
 if (!maybeProjectSave()) {event->ignore(); return;}
       writeSettings();
       removeWorkingMaterials();
       killTimer (undoRedoTimer);
       event->accept();

}
bool MainWindow::maybeProjectSave()
{

    if (modifiedProject) {
        int ret = QMessageBox::warning(this, mainWindowCaption(),
                     "The project has been modified.\n"
                        "Do you want to save your changes?",
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No,
                     QMessageBox::Cancel | QMessageBox::Escape);
        if (ret == QMessageBox::Yes)
            return saveProfile();
        else if (ret == QMessageBox::Cancel)
            return false;
    }
    return true;

}
bool MainWindow::maybeSave()
{

    if (modifiedCatalog) {
        int ret = QMessageBox::warning(this, mainWindowCaption(),
                     "The catalog has been modified.\n"
                        "Do you want to save your changes?",
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No,
                     QMessageBox::Cancel | QMessageBox::Escape);
        if (ret == QMessageBox::Yes)
            return save();
        else if (ret == QMessageBox::Cancel)
            return false;
    }
    return true;

}

void MainWindow::setCurrentFile(const QString &fileName)
{

    curFile = fileName;
    modifiedCatalog = false;
    viewBufferModified =  false;
   // modifiedProtocol = false;
	//modifiedProject  = false;
	//bottomWindow->CleanAllEntries();
    if (curFile.isEmpty()) { setWindowTitle(QString("%1 - %2").arg("unnamed ").arg(mainWindowCaption()) ); return;}

       setWindowTitle(QString("%1 - %2").arg(strippedName(curFile)).arg(mainWindowCaption()) );

        QSettings settings("mitp.ru", "EditCatalogPanel");
  
        QStringList files = settings.value("recentFileList").toStringList();

        files.removeAll(fileName);
        files.prepend(fileName);
        while (files.size() > MaxRecentFiles)
            files.removeLast();
      
        settings.setValue("recentFileList", files);
     
        foreach (QWidget *widget, QApplication::topLevelWidgets()) {
            MainWindow *mainWin = qobject_cast<MainWindow *>(widget);
            if (mainWin)
                mainWin->updateRecentFileActions();
        }

}

QString MainWindow::strippedName(const QString &fullFileName)
{
    return QFileInfo(fullFileName).fileName();
}
void MainWindow::updateRecentFileActions()
    {
        QSettings settings("mitp.ru", "EditCatalogPanel");
        
        QStringList files = settings.value("/recentFileList").toStringList();
        
        int numRecentFiles = qMin(files.size(), (int)MaxRecentFiles);

        for (int i = 0; i < numRecentFiles; ++i) {
            QString text = tr("&%1 %2").arg(i + 1).arg(strippedName(files[i]));
            recentFileActs[i]->setText(text);
            recentFileActs[i]->setData(files[i]);
            recentFileActs[i]->setVisible(true);
        }
        for (int j = numRecentFiles; j < MaxRecentFiles; ++j)
            recentFileActs[j]->setVisible(false);

        separatorAct->setVisible(numRecentFiles > 0);
}



void MainWindow::openRecentFile() {
    if (maybeSave()) if (maybeProtocolSave()) {
        QAction *action = qobject_cast<QAction *>(sender());
        if (action) {
             QString fileName = action->data().toString();
             if (QFile::exists(fileName))  {
                if(doc !=0) {delete doc; doc =0;}
                doc = new FDocument();
				prolist->closeProfile();

                if(! loadFile(fileName) ) return;
    bottomWindow->OpenProblemsProtocolPage();
    bottomWindow->messageTab->clear();
    bottomWindow->correctionTab->clear();
                ProfileActsShowHide(false);
modLabel->setText(" Tools ");
actionLabel->setText(" Ready ");
messageLabel->setText("");
            } else error(QString("File '%1' does not exist").arg(fileName),"");
         }
} 
}


void MainWindow::createStatusBar()
{
//statusBar()->showMessage(tr("Progress"));
modLabel = new QLabel("Project",statusBar());
actionLabel = new QLabel("Ready",statusBar());
messageLabel = new QLabel("Operation",statusBar());

statusBar()->addPermanentWidget(modLabel);
statusBar()->addWidget(actionLabel);
statusBar()->addWidget(messageLabel,200);

modLabel->setText(" Start ");
actionLabel->setText(" Action: ");
messageLabel->setText(" Open catalog or project; Create new project; Import catalog");
}

void MainWindow::SaveStatusbarMessage(){
saveModLabel = modLabel->text();
saveActionLabel = actionLabel->text();
saveMessageLabel = messageLabel->text();
}
void MainWindow::RestoreStatusbarMessage(){
modLabel->setText(saveModLabel);
actionLabel->setText(saveActionLabel);
messageLabel->setText(saveMessageLabel);

}

void MainWindow::about()
{
	
	QString text =  tr("<h2>CompiCat version 1.2.4</h2>"
     "<p>Copyright &copy; 2005-2007 MITPAN."
     "<p><b>CompiCat</b> is an application that combines editing, search "
     "and analysiing of earthquake catalogs."
     "<p><b>CompiCat</b> provides single-source portability across "
     "<b>Linux</b>, <b>Unix</b>, <b>Mac OS X</b> and <b>Windows</b> "
     "<p>See http://www.mitp.ru/soft/ecp/doc/mainindex.html "
     );
	  QMessageBox msgb(QMessageBox::NoIcon, "About CompiCat program",text);
	  msgb.setIconPixmap (QIcon(":/images/iconLogo.png").pixmap (QSize(90,150)));
	  msgb.setWindowIcon(QIcon(":/images/icon.png"));
	  msgb.exec();

}
void MainWindow::aboutQwt()
{
	QString text = tr("<h2>About Qwt</h2>"
	 "<p>Qwt version is 5.0.2 "
     "<p><b>Qwt</b> is <b>Qt</b> windows for technical applications."
     "<p><b>Qwt library</b>contains GUI Components and utility classes, "
     "which are primarily useful for programs with a technical background "
	 "(2D plots, histograms and etc.)."
     "<p><b>Qwt</b> provides single-source portability across "
     "<b>Linux</b> , <b>Unix</b> , <b>Windows</b>  and <b>Macintosh</b>."
    "<p>See http://qwt.sourceforge.net "
     );
	 QMessageBox msgb(QMessageBox::NoIcon, "About Qwt",text);
	  msgb.setIconPixmap (QIcon(":/images/qwtwidget.png").pixmap (QSize(60,60)));
	  msgb.setWindowIcon(QIcon(":/images/icon.png"));
	  msgb.exec();
}
void MainWindow::writeSettings()
{
    QSettings settings("mitp.ru", "EditCatalogPanel");
    settings.beginGroup("/EditCatalog");
    settings.setValue("/geometry/pos", pos());
    settings.setValue("/geometry/size", size());
    settings.setValue("/font/PointSize", font().pointSize());
    settings.setValue("/geometry/splitter",splitter->saveState());
    settings.setValue("/geometry/splitterVert",splitterVert->saveState());
    settings.endGroup();

    settings.beginGroup("/DoubleIdentificationPanel");
    settings.setValue("/geometry/pos",doubleImplPos);
    settings.setValue("/geometry/size",doubleImplSize );
    settings.endGroup();

    settings.beginGroup("/TwoCatalogActionPanel");
    settings.setValue("/geometry/pos",catalImplPos);
    settings.setValue("/geometry/size",catalImplSize );
    settings.endGroup();
   
    settings.beginGroup("/TransfMagnitudePanel");
    settings.setValue("/geometry/pos",trmagImplPos);
    settings.setValue("/geometry/size",trmagImplSize);
    settings.endGroup();

    settings.beginGroup("/SubCatalogPanel");
    settings.setValue("/geometry/pos",subCatalogPos);
    settings.setValue("/geometry/size",subCatalogSize);
    settings.endGroup();

    settings.beginGroup("/ProjectWizardPanel");
    settings.setValue("/geometry/pos",projectWizardPos);
    settings.setValue("/geometry/size",projectWizardSize);
    settings.endGroup();

    settings.beginGroup("/MyOpenFileDialog");
    settings.setValue("/geometry/pos",MyOpenFileDialogPos);
    settings.setValue("/geometry/size",MyOpenFileDialogSize);
    settings.setValue("/geometry/splitterImportFile",splitterImportFile);
	
    settings.setValue("importUserDirectory",importUserDirectory);
    settings.setValue("exportUserDirectory",exportUserDirectory);
	
    settings.setValue("openUserFileDirectory",openUserFileDirectory);
    settings.setValue("saveUserFileDirectory",saveUserFileDirectory);
    settings.setValue("exportUserFileDirectory",exportUserFileDirectory);
    settings.setValue("poligonOpenFileDirectory",poligonOpenFileDirectory);
	
    settings.setValue("openUserProjectDirectory",openUserProjectDirectory);
    settings.setValue("saveUserProjectDirectory",saveUserProjectDirectory);
    settings.setValue("saveUserProtocolDirectory",saveUserProtocolDirectory);
	
    settings.endGroup();

    settings.beginGroup("/ProtocolViewPanel");
    settings.setValue("/geometry/pos",protocolPos);
    settings.setValue("/geometry/size",protocolSize);
    settings.endGroup();
	
    settings.beginGroup("/VeiwSegmentCatalog");
    settings.setValue("/geometry/pos",viewSegmentPos);
    settings.setValue("/geometry/size",viewSegmentSize);
    settings.endGroup();

    settings.beginGroup("/RepairProject");
    settings.setValue("/geometry/pos",repairProjectPos);
    settings.setValue("/geometry/size",repairProjectSize);
    settings.endGroup();

    settings.beginGroup("/DrawHistogram");
    settings.setValue("/geometry/pos",histPos);
    settings.setValue("/geometry/size",histSize);
	settings.setValue("/geometry/splitterPlot",splitterPlot);
    settings.endGroup();
	
	settings.beginGroup("/SubstituteCatalogPanel");
    settings.setValue("/geometry/pos",addPos);
    settings.setValue("/geometry/size",addSize );
    settings.endGroup();
	
	settings.beginGroup("/CheckRangeCatalogPanel");
    settings.setValue("/geometry/pos",rangePos);
    settings.setValue("/geometry/size",rangeSize );
    settings.endGroup();

	settings.beginGroup("/ExportUserPanel");
    settings.setValue("/geometry/pos",exportUserPos);
    settings.setValue("/geometry/size",exportUserSize);
    settings.endGroup();

	
	setDoc->write();
}

void MainWindow::readSettings()
{
    QSettings settings("mitp.ru", "EditCatalogPanel");
    settings.beginGroup("/EditCatalog");
        QPoint pos = settings.value("/geometry/pos", QPoint(200, 200)).toPoint();
        QSize size = settings.value("/geometry/size", QSize(800, 800)).toSize();
        resize(size);
        move(pos);
    int fontPointSize = settings.value("/font/PointSize", 10).toInt();
    QFont f(font());
    f.setPointSize(fontPointSize );
    setFont( f );	
	viewcatalogimpl->zoom();
    QByteArray splitterData = settings.value("/geometry/splitter").toByteArray();
	splitter->restoreState(splitterData);
    splitterData = settings.value("/geometry/splitterVert").toByteArray();
	splitterVert->restoreState(splitterData);	
    settings.endGroup();

    settings.beginGroup("/DoubleIdentificationPanel");
    doubleImplPos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    doubleImplSize = settings.value("/geometry/size",QSize(400, 200)).toSize();
    settings.endGroup();   

    settings.beginGroup("/TwoCatalogActionPanel");
    catalImplPos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    catalImplSize = settings.value("/geometry/size",QSize(400, 200)).toSize();
    settings.endGroup();

    settings.beginGroup("/TransfMagnitudePanel");
    trmagImplPos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    trmagImplSize = settings.value("/geometry/size",QSize(500, 400)).toSize();
    settings.endGroup();

    settings.beginGroup("/SubCatalogPanel");
    subCatalogPos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    subCatalogSize = settings.value("/geometry/size",QSize(500, 400)).toSize();
    settings.endGroup();

    settings.beginGroup("/ProjectWizardPanel");
    projectWizardPos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    projectWizardSize = settings.value("/geometry/size",QSize(500, 400)).toSize();
    settings.endGroup();

    settings.beginGroup("/MyOpenFileDialog");
    MyOpenFileDialogPos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    MyOpenFileDialogSize = settings.value("/geometry/size",QSize(500, 400)).toSize();
    splitterImportFile   = settings.value("/geometry/splitterImportFile").toByteArray();
	
    importUserDirectory   = settings.value("importUserDirectory",userHomePath()).toString();
    exportUserDirectory   = settings.value("exportUserDirectory",userHomePath()).toString();
    openUserFileDirectory   = settings.value("openUserFileDirectory",userHomePath()).toString();
    saveUserFileDirectory   = settings.value("saveUserFileDirectory",userHomePath()).toString();
    exportUserFileDirectory   = settings.value("exportUserFileDirectory",userHomePath()).toString();
    poligonOpenFileDirectory   = settings.value("poligonOpenFileDirectory",userHomePath()).toString();
	
	
    openUserProjectDirectory   = settings.value("openUserProjectDirectory",userHomePath()).toString();
    saveUserProjectDirectory   = settings.value("saveUserProjectDirectory",userHomePath()).toString();
    saveUserProtocolDirectory   = settings.value("saveUserProtocolDirectory",userHomePath()).toString();
	
	settings.endGroup();
	
	
	
  openUserFileDirectory = QDir::toNativeSeparators(openUserFileDirectory);
  saveUserFileDirectory = QDir::toNativeSeparators(saveUserFileDirectory);
  exportUserFileDirectory = QDir::toNativeSeparators(exportUserFileDirectory);
  poligonOpenFileDirectory = QDir::toNativeSeparators(poligonOpenFileDirectory);
  openUserProjectDirectory = QDir::toNativeSeparators(openUserProjectDirectory);
  saveUserProjectDirectory = QDir::toNativeSeparators(saveUserProjectDirectory);
  saveUserProtocolDirectory = QDir::toNativeSeparators(saveUserProtocolDirectory);

    settings.beginGroup("/ProtocolViewPanel");
    protocolPos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    protocolSize = settings.value("/geometry/size",QSize(500, 400)).toSize();
    settings.endGroup();

	settings.beginGroup("/VeiwSegmentCatalog");
	viewSegmentPos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    viewSegmentSize = settings.value("/geometry/size",QSize(200, 300)).toSize();
    settings.endGroup();

    settings.beginGroup("/RepairProject");
    repairProjectPos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    repairProjectSize = settings.value("/geometry/size",QSize(300, 400)).toSize();
    settings.endGroup();
	
	settings.beginGroup("/DrawHistogram");
    histPos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    histSize = settings.value("/geometry/size",QSize(600, 500)).toSize();
	splitterPlot  = settings.value("/geometry/splitterPlot").toByteArray();
    settings.endGroup();
	
	settings.beginGroup("/SubstituteCatalogPanel");
    addPos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    addSize = settings.value("/geometry/size",QSize(400, 200)).toSize();
    settings.endGroup();   
	
	settings.beginGroup("/CheckRangeCatalogPanel");
    rangePos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    rangeSize = settings.value("/geometry/size",QSize(400, 200)).toSize();
    settings.endGroup(); 

	settings.beginGroup("/ExportUserPanel");
    exportUserPos  = settings.value("/geometry/pos",QPoint(100, 100)).toPoint();
    exportUserSize = settings.value("/geometry/size",QSize(400, 200)).toSize();
    settings.endGroup(); 
	
	setDoc->read();
}

void MainWindow::initCentralWidget(AutoCheckErrors act){
	viewcatalogimpl->cattable->SetCatalog(doc,setDoc,act);
   // bottomWindow->messageTab->clear();
    bottomWindow->correctionTab->clear();
}
void MainWindow::settingsParameters(){
    SettingImpl  dialog(setDoc,this);
    SaveStatusbarMessage();
    modLabel->setText(" Tools ");
    actionLabel->setText(" Action: ");
    messageLabel->setText("Settings of parameters");
    if (dialog.exec()) {
    if (dialog.modifed)
      if (viewcatalogimpl->isCatalogLoaded()) {
            viewcatalogimpl->cattable->SetViewBufSize(setDoc->GetWorkSpaceSize()->GetValue().toInt());
        }
    }
    RestoreStatusbarMessage();
    QSettings settings("mitp.ru", "EditCatalogSettings");
    settings.beginGroup("/Geometry");
    settings.setValue("/geometry/pos", dialog.pos());
    settings.setValue("/geometry/size",dialog.size());
    settings.endGroup();


}
void MainWindow::viewbuf(){
    BufSizeBaseImpl  dialog(this,idViewBufferCatalog(),setDoc->GetWorkSpaceSize()->GetValue().toInt());
    if (dialog.exec()) {
       QString temp = QString("%1").arg(dialog.GetSpinBoxNumber()->value ());
	   
       if (temp != setDoc->GetWorkSpaceSize()->GetValue()) {
              setDoc->GetWorkSpaceSize()->SetValue(temp);
			  setDoc->GetWorkSpaceSize()->SetValueToResetvalue();
               if (viewcatalogimpl->isCatalogLoaded()) {
                viewcatalogimpl->cattable->SetViewBufSize(setDoc->GetWorkSpaceSize()->GetValue().toInt());
               } 
       }
    }
}
void MainWindow::errbuf(){

    BufSizeBaseImpl  dialog(this,idViewBufferError(),setDoc->GetMaxErrorLines()->GetValue().toInt());
    if (dialog.exec()) {
    QString temp = QString("%1").arg(dialog.GetSpinBoxNumber()->value ());
      if (temp != setDoc->GetMaxErrorLines()->GetValue())
          setDoc->GetMaxErrorLines()->SetValue(temp);setDoc->GetMaxErrorLines()->SetValueToResetvalue();
   }
}

void MainWindow::correctline(){

    BufSizeBaseImpl  dialog(this,idMaxLinesErrorsCorrectonPage(),setDoc->GetMaxCorrectionLines()->GetValue().toInt());
    if (dialog.exec()) {
    QString temp = QString("%1").arg(dialog.GetSpinBoxNumber()->value ());
      if (temp != setDoc->GetMaxCorrectionLines()->GetValue())
          setDoc->GetMaxCorrectionLines()->SetValue(temp);setDoc->GetMaxCorrectionLines()->SetValueToResetvalue();
    }
}
void MainWindow::zoomin(){
   QFont f( font() );
    f.setPointSize( f.pointSize() + 1 );  
    setFont( f );
    viewcatalogimpl->zoom();    
}
void MainWindow::zoomout(){
    QFont f(font() );
    f.setPointSize( f.pointSize() - 1 );  
    setFont( f );
    viewcatalogimpl->zoom();    
}


void MainWindow::cut(){

if (viewcatalogimpl->isCatalogLoaded())viewcatalogimpl->cattable->cut();
}
void MainWindow::copy(){

if (viewcatalogimpl->isCatalogLoaded())viewcatalogimpl->cattable->copy();
}
void MainWindow::paste(){

if (viewcatalogimpl->isCatalogLoaded())viewcatalogimpl->cattable->paste();
}
void MainWindow::del(){
if (viewcatalogimpl->isCatalogLoaded())viewcatalogimpl->cattable->del();    
}
void MainWindow::insert(){
if (viewcatalogimpl->isCatalogLoaded())viewcatalogimpl->cattable->insert();     
}
void MainWindow::timerEvent(QTimerEvent *e)
{

  if ( e->timerId() == undoRedoTimer) {
   if (viewcatalogimpl->cattable->curUndoIndex < 0) undoAct->setEnabled (FALSE);
    else  undoAct->setEnabled (TRUE);
    if (viewcatalogimpl->cattable->curUndoIndex  + 1 == viewcatalogimpl->cattable->undoList.size())
       redoAct->setEnabled (FALSE);
    else  redoAct->setEnabled (TRUE);
  }

 //QObject::timerEvent(e);

}
//------------------------------------------------------------
void MainWindow::magnitude(){
    if (!viewcatalogimpl->isCatalogLoaded()) return;
    SaveStatusbarMessage();
    modLabel->setText(" Tools ");
    actionLabel->setText(" Action: ");
    messageLabel->setText(projectActionList().split ('@').at(0));
    viewcatalogimpl->cattable->magnitude();


}
void MainWindow::duplicate(){
    if (!viewcatalogimpl->isCatalogLoaded()) return;
    SaveStatusbarMessage();
    modLabel->setText(" Tools ");
    actionLabel->setText(" Action: ");
    messageLabel->setText(projectActionList().split ('@').at(1));
    viewcatalogimpl->cattable->CheckDuplicate();

}
void MainWindow::sample(){
    if (!viewcatalogimpl->isCatalogLoaded()) return;
    SaveStatusbarMessage();
    modLabel->setText(" Tools ");
    actionLabel->setText(" Action: ");
    messageLabel->setText(projectActionList().split ('@').at(2));
    viewcatalogimpl->cattable->sample();

}
void MainWindow::merge(){
    if (!viewcatalogimpl->isCatalogLoaded()) return;
    SaveStatusbarMessage();
    modLabel->setText(" Tools ");
    actionLabel->setText(" Action: ");
    messageLabel->setText(projectActionList().split ('@').at(3));
    viewcatalogimpl->cattable->merge();

}
void MainWindow::add(){
    if (!viewcatalogimpl->isCatalogLoaded()) return;
    SaveStatusbarMessage();
    modLabel->setText(" Tools ");
    actionLabel->setText(" Action: ");
    messageLabel->setText(projectActionList().split ('@').at(4));
    viewcatalogimpl->cattable->add();

}

void MainWindow::comp(){
    if (!viewcatalogimpl->isCatalogLoaded()) return;
    SaveStatusbarMessage();
    modLabel->setText(" Tools ");
    actionLabel->setText(" Action: ");
    messageLabel->setText(projectCompareTitle());
    viewcatalogimpl->cattable->comp();

}
void MainWindow::checkerror(){
    if (!viewcatalogimpl->isCatalogLoaded()) return;
    SaveStatusbarMessage();
    modLabel->setText(" Tools ");
    actionLabel->setText(" Action: ");
    messageLabel->setText(projectRangeTitle());
    viewcatalogimpl->cattable->range();

}
//=======================================================
void MainWindow::viewfind(){

}
void MainWindow::viewinfo(){
if(!viewcatalogimpl->isCatalogLoaded()) return;
QString tmp = viewcatalogimpl->GetCatalogInfo();
    if (tmp != QString()) {
 //  QMessageBox::information(this,mainWindowCaption(),tmp)
	QMessageBox mb(mainWindowCaption(),tmp,
	QMessageBox::Information,
	QMessageBox::Ok | QMessageBox::Default,
	QMessageBox::NoButton,
	QMessageBox::NoButton,this);
	mb.setFont(font());
	mb.exec();
	}
}
void MainWindow::viewsave(){}
void MainWindow::viewcancel(){}
//=======================================================	
void MainWindow::undo(){
if (viewcatalogimpl->isCatalogLoaded()) viewcatalogimpl->cattable->undo();
}
void MainWindow::redo(){
if (viewcatalogimpl->isCatalogLoaded()) viewcatalogimpl->cattable->redo();
}

void MainWindow::checksort(){
if (viewcatalogimpl->isCatalogLoaded()) viewcatalogimpl->cattable->CheckSortButton();  
}
void MainWindow::sort(){
if (viewcatalogimpl->isCatalogLoaded()) viewcatalogimpl->cattable->SortCatalog();
}

bool MainWindow::exportSlot(){
if (!viewcatalogimpl->isCatalogLoaded()) return false;
    QAction *action = qobject_cast<QAction *>(sender());
    QByteArray fileFormat = action->data().toByteArray();
	if( fileFormat.data() == exportUserFormat() ) {
		userExportFormat();
	return true;
	}
    exportFile(fileFormat);
return true;
}
void MainWindow::userExportFormat() {
 
 
     ExportUser exportUser(this);
     exportUser.resize(exportUserSize);
     exportUser.move(exportUserPos);
	 
     if (exportUser.exec()) {
		 Catalog cat;
		 cat.exportUserFormatFile(exportUser.exportFileName, exportUser.fieldList);
	 }
	 exportUserPos   =  exportUser.pos();
	 exportUserSize  =  exportUser.size();
}

bool MainWindow::exportFile(const QByteArray &fileFormat)
{
    QString format = QString(fileFormat.toUpper());
    QString filter = format.left(format.indexOf( '-'));
    QString title = fileFormat.right(fileFormat.length() - (fileFormat.lastIndexOf('-') + 1));
    filter.remove(QChar(' '));
	if (!filter.contains("68")) filter = "dat"; else filter = "68bytes";
    Catalog cat;
	exportUserDirectory = QDir::toNativeSeparators(exportUserDirectory);
    QString initialPath = exportUserDirectory + "/untitled." + filter.toLower();
	initialPath = QDir::toNativeSeparators(initialPath);
    QString fileName = QFileDialog::getSaveFileName(this, tr("Export a catalog file"),
                               initialPath,
                               tr("%1 Files (*.%2);;All Files (*)")
                               .arg(title)
                               .arg(filter));
    if (fileName.isEmpty()) return false;
//========================================================================================
		int ind = 0;
if( ( ind = fileName.lastIndexOf('.')) != -1 )
    if ( ( fileName.indexOf(filter,ind)) == -1 && ( fileName.indexOf(filter.toUpper(),ind)) == -1) {
        fileName.append(filter);
	
        if (QFile::exists(fileName)) {
        int ret = QMessageBox::warning(this, mainWindowCaption(),
                     QString("File %1 already exists.\n Do you want to overwrite it?")
                     .arg(QDir::toNativeSeparators(fileName)),
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No | QMessageBox::Escape);
        if (ret == QMessageBox::No)
            return true;
       }
}
//========================================================================================		
		
		
		exportUserDirectory = QFileInfo(fileName).absolutePath ();
        return cat.expotFile(fileName, format);
}
void MainWindow::selectAll(){
if (viewcatalogimpl->isCatalogLoaded()) viewcatalogimpl->cattable->selectAll();  
}

void MainWindow::newProfile() {
if (maybeProjectSave()) {
	FDocument* defDoc = new FDocument();
//----------------------------------------------------------------------------
defDoc->GetProjectCatalogName()->SetValue(curFile);
defDoc->GetProjectCatalogName()->SetValueToResetvalue();
defDoc->GetProjectCatalogName()->SetValueToDefaultvalue();

doc->GetProjectCatalogName()->SetValue(curFile);
doc->GetProjectCatalogName()->SetValueToResetvalue();
doc->GetProjectCatalogName()->SetValueToDefaultvalue();
//-----------------------------------------------------------------------------	
	ProWizard  dialog(this,defDoc,doc,ProWizard::ProjectNew);
	SaveStatusbarMessage();
	modLabel->setText(" Project ");
	actionLabel->setText(" Action: ");
	messageLabel->setText("Create new project");
	dialog.resize(projectWizardSize);
	dialog.move(projectWizardPos);
	if (dialog.exec()) {
		ProfileImpl* projectActionPage = 
		   ((ProjectActionPage * )dialog.page (ProWizard::Project_Action))->proPage;
		if(projectActionPage->GetRadioButtonDefault()->isChecked ())
			{delete doc; doc = defDoc;}
		else delete defDoc;
		if( !loadFile(doc->GetProjectCatalogName()->GetValue()) ) { RestoreStatusbarMessage(); return;}
		Catalog cat;
		if ( !cat.CopyFile(openBinFileName,projectBinCatalogFileName) ) {  RestoreStatusbarMessage(); return;}
        if ( doc->isModified() ) modifiedProject = true;
		prolist->SetRowList(doc,QString("unnamed"));
		setCurrentProFile("");
        ProfileActsShowHide(true);
        if ( doc->CheckModified() ) modifiedProject = true; else modifiedProject = false;
		bottomWindow->messageTab->ensureCursorVisible ();
		modLabel->setText(" Project ");
		actionLabel->setText(" Ready ");
		messageLabel->setText("");		
	} else  {
		delete defDoc;
		RestoreStatusbarMessage();
		bottomWindow->messageTab->clear();
	}
	projectWizardPos   =  dialog.pos();
	projectWizardSize  =  dialog.size();
}

}
void MainWindow::editProfile(){
	FDocument* defDoc = new FDocument();
//-----------------------------------------------------------------------------	
	defDoc->GetProjectCatalogName()->SetValue(curFile);
	defDoc->GetProjectCatalogName()->SetValueToResetvalue();
	defDoc->GetProjectCatalogName()->SetValueToDefaultvalue();
	doc->GetProjectCatalogName()->SetValueToDefaultvalue();
//-----------------------------------------------------------------------------	
	ProWizard  dialog(this,defDoc,doc,ProWizard::ProjectEdit);
    dialog.resize(projectWizardSize);
    dialog.move(projectWizardPos);
           SaveStatusbarMessage();
           modLabel->setText(" Project ");
           actionLabel->setText(" Action: ");
           messageLabel->setText("Edit project");
	if (dialog.exec()) {
		ProfileImpl* projectActionPage = 
		((ProjectActionPage * )dialog.page (ProWizard::Project_Action))->proPage;
		if(projectActionPage->GetRadioButtonDefault()->isChecked ())
			{delete doc; doc = defDoc;}
		else delete defDoc;
		if( !loadFile(doc->GetProjectCatalogName()->GetValue()) ) { RestoreStatusbarMessage(); return;}
		Catalog cat;
		if ( !cat.CopyFile(openBinFileName,projectBinCatalogFileName) ) {  RestoreStatusbarMessage(); return;}
		prolist->SetRowList(doc,QString("unnamed"));
		setCurrentProFile("");if ( doc->isModified() ) modifiedProject = true;
        ProfileActsShowHide(true);
        if ( doc->CheckModified() ) modifiedProject = true; else modifiedProject = false;
		bottomWindow->messageTab->ensureCursorVisible ();
		modLabel->setText(" Project ");
		actionLabel->setText(" Ready ");
		messageLabel->setText("");		
	} else  {
		delete defDoc;
		RestoreStatusbarMessage();
		bottomWindow->messageTab->clear();
	}		   
}
bool MainWindow::IsProjectFilesExist() {
repairFileName.clear ();
QString projectCatalog = doc->GetProjectCatalogName()->GetValue();
QFileInfo info(projectCatalog);
if(!info.exists()) repairFileName.append(projectCatalog);
QStringList listAllAction = projectActionList().split ('@');
QStringList  listAction = doc->GetProjectActionList()->GetStringList();
for (int i = 0; i <  listAction.size(); i++){
 QString str = listAction.at(i);

if (str == listAllAction.at(3)) { // merge
      QStringList listMergeCatalog = doc->GetListMergeCatalog()->GetStringList();
      for (int j = 0; j <  listMergeCatalog.size(); j++) {
      QString file = listMergeCatalog.at(j);
      QFileInfo info(file);
      if(!info.exists()) repairFileName.append(file);
  }

 }
 if (str == listAllAction.at(4)) { //append
      QStringList listAddCatalog = doc->GetListAddCatalog()->GetStringList();
      for (int j = 0; j <  listAddCatalog.size(); j++) {
      QString file = listAddCatalog.at(j);
      QFileInfo info(file);
      if(!info.exists()) repairFileName.append(file);

     }
 }
 if (str == projectCompareTitle()) { //compare
      QString file = doc->GetSecondCatalogName()->GetValue();
      QFileInfo info(file);
      if(!info.exists()) repairFileName.append(file);
  }

}
if(!repairFileName.empty ()) return false;
return true;
}
void MainWindow::loadProjectFile(const QString &fileName){
      FDocument* tmpDoc = doc;
      doc = new FDocument(fileName);
      if (!IsProjectFilesExist()) {
          int ret = QMessageBox::warning(this, mainWindowCaption(),
          QString("Project '%1' : Some input files do not exist. Try fixing the problem?")
                  .arg(QDir::toNativeSeparators(strippedName(fileName))),
                  QMessageBox::Yes | QMessageBox::Default,
                  QMessageBox::No | QMessageBox::Escape);
          if (ret == QMessageBox::No) {
                   delete doc;doc = tmpDoc;return;
          }
          RepairPro  dialog(this,doc,fileName);
          if (dialog.exec()) {
                 int ret = QMessageBox::warning(this, mainWindowCaption(),
                 QString("Project file %1 was repaired.\n Do you want to save it?")
                  .arg(QDir::toNativeSeparators(strippedName(fileName))),
                  QMessageBox::Yes | QMessageBox::Default,
                  QMessageBox::No | QMessageBox::Escape);
                 if (ret == QMessageBox::Yes) {
                   saveAsProfile();
                 } else {
                    curProFile = "";modifiedProject = true;
                   }
          } else {
                delete doc;doc = tmpDoc;return;
          }
     } else setCurrentProFile(fileName);
//QMessageBox::critical(0,"",doc->GetProjectCatalogName()->GetValue());
          
          if(!doc->GetProjectFileGood()){
                   delete doc;doc = tmpDoc;return;
          }
          if (!loadFile(doc->GetProjectCatalogName()->GetValue())){
                   delete doc;doc = tmpDoc;return;
          }
          Catalog cat;
          if ( !cat.CopyFile(openBinFileName,projectBinCatalogFileName) ){
                   delete doc;doc = tmpDoc;return;
          }
          delete tmpDoc;
          if (curProFile == "")
          prolist->SetRowList(doc,QString("unnamed"));
          else 
          prolist->SetRowList(doc,QString(strippedName(curProFile)));

          bottomWindow->OpenProblemsProtocolPage();

          ProfileActsShowHide(true);

}
void MainWindow::openProfile(){

  if (maybeSave()) if (maybeProtocolSave()) {
        QString fileName =
        QFileDialog::getOpenFileName(this,"Open an existing project file",openUserProjectDirectory, projectFileFilters);
          if (!fileName.isEmpty()) {
		   bottomWindow->CleanAllEntries();			  
           loadProjectFile(fileName);
		  
           modLabel->setText(" Project ");
           actionLabel->setText(" Ready ");
           messageLabel->setText("");
		   openUserProjectDirectory = QFileInfo(fileName).absolutePath ();
         }
    }
}

void MainWindow::buildProfile() {
if (!viewcatalogimpl->isCatalogLoaded()) return;
MainWindow::self->statusBar()->clearMessage ();
           modLabel->setText(" Project ");
statusBar()->showMessage ("Execute project");
viewcatalogimpl->cattable->buildProfile();
statusBar()->showMessage ("Execute project done",1000);
           modLabel->setText(" Project ");
           actionLabel->setText(" Ready ");
           messageLabel->setText("");
}

bool MainWindow::closeFile(){
    maybeProjectSave();
  if (viewcatalogimpl->isCatalogLoaded()) {
	viewcatalogimpl->cattable->maybeSave();
	maybeSave(); 
	maybeProtocolSave();
	bottomWindow->CleanAllEntries();
	viewcatalogimpl->cattable->ClearCatalogTable();
	modLabel->setText(" Start ");
	actionLabel->setText(" Action: ");
	messageLabel->setText(" Open catalog or project; Create new project; Import catalog");
    setCurrentFile("");
    setCurrentProFile("");
    setCurrentProtocol("");
    FileActsShowHide(false);
  }
prolist->closeProfile();
ProfileActsShowHide(false);
return true;
}
bool MainWindow::saveAll(){  
 if (viewcatalogimpl->isCatalogLoaded())
 if( !viewcatalogimpl->cattable->maybeSave()) return true;
 if (!maybeSave()) return true;
 if (!maybeProtocolSave())return true;
 if (!maybeProjectSave()) return true;


return true;
}

bool MainWindow::closeProfile(){
closeFile();
return true;
}

//--------------------------------------------------------------------------------------------
bool MainWindow::saveProfile(){

    if (curProFile.isEmpty()) {
        return saveAsProfile();
    } else {
        saveProjectFile(curProFile);
        return true;
    }

}
void MainWindow::saveProjectFile(const QString &fileName)
{

    QApplication::setOverrideCursor(Qt::WaitCursor);
    if (doc->saveBinary(fileName)) {

	   setCurrentProFile(fileName);

    statusBar()->showMessage(tr("Project file saved"), 2000);

    } else {
        statusBar()->showMessage(tr("Saving Project file canceled"), 2000);
    }

    QApplication::restoreOverrideCursor();
  
}
bool MainWindow::saveAsProfile(){

    QString fileName =
            QFileDialog::getSaveFileName(this,"Save the Project file under a new name",saveUserProjectDirectory,projectFileFilters);
    if (fileName.isEmpty())
        return false;
    int ind = fileName.length () - projectFileFilterLocator().length ()-1; if (ind < 0) ind =0;
    if ( ( fileName.indexOf(projectFileFilterLocator(),ind)) == -1 && ( fileName.indexOf(projectFileFilterLocator().toUpper(),ind)) == -1) {
        fileName.append(projectFileFilterLocator());
        if (QFile::exists(fileName)) {
        int ret = QMessageBox::warning(this, mainWindowCaption(),
                     QString("File %1 already exists.\n Do you want to overwrite it?")
                     .arg(QDir::toNativeSeparators(fileName)),
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No | QMessageBox::Escape);
        if (ret == QMessageBox::No)
            return true;
       }
    }
    if (!fileName.isEmpty())
        saveProjectFile(fileName);
	    saveUserProjectDirectory = QFileInfo(fileName).absolutePath ();
    return true;
}
void MainWindow::setCurrentProFile(const QString &fileName){
    curProFile = fileName;
    modifiedProject = false;
    if (curProFile.isEmpty()) return;
    QString tmp = strippedName(curProFile);
    prolist->SetProjectName(tmp);

        QSettings settings("mitp.ru", "EditCatalogPanel");
  
        QStringList files = settings.value("recentProjectFileList").toStringList();

        files.removeAll(fileName);
        files.prepend(fileName);
        while (files.size() > MaxRecentFiles)
            files.removeLast();
      
        settings.setValue("recentProjectFileList", files);
     
        foreach (QWidget *widget, QApplication::topLevelWidgets()) {
            MainWindow *mainWin = qobject_cast<MainWindow *>(widget);
            if (mainWin)
                mainWin->updateRecentProjectFileActions();
        }

}
//--------------------------------------------------------------------------------------------

void MainWindow::openRecentProjectFile(){
   if (maybeProjectSave()) {
        QAction *action = qobject_cast<QAction *>(sender());
        if (action) {
            QString fileName = action->data().toString();
            if (QFile::exists(fileName)) {
			   bottomWindow->CleanAllEntries();
               loadProjectFile(fileName);
               modLabel->setText(" Project ");
               actionLabel->setText(" Ready ");
               messageLabel->setText("");
           }
            else error(QString("File '%1' does not exist").arg(fileName),""); 
     }
} 
}
//--------------------------------------------------------------------------------------------

void MainWindow::updateRecentProjectFileActions(){
      QSettings settings("mitp.ru", "EditCatalogPanel");

        QStringList files = settings.value("/recentProjectFileList").toStringList();

        int numRecentFiles = qMin(files.size(), (int)MaxRecentFiles);

        for (int i = 0; i < numRecentFiles; ++i) {
            QString text = tr("&%1 %2").arg(i + 1).arg(strippedName(files[i]));
            recentProjectFileActs[i]->setText(text);
            recentProjectFileActs[i]->setData(files[i]);
            recentProjectFileActs[i]->setVisible(true);
        }
        for (int j = numRecentFiles; j < MaxRecentFiles; ++j)
            recentProjectFileActs[j]->setVisible(false);

        separatorProjectAct->setVisible(numRecentFiles > 0);
}
//--------------------------------------------------------------------------------------------
void MainWindow::setCurrentProtocol(const QString &fileName)
{
curProtocolFile = fileName;
modifiedProtocol = false;
if (curProtocolFile.isEmpty()) { protocolLineEdit->setText("Protocol: unnamed"); return;}
protocolLineEdit->setText("Protocol: " + strippedName(fileName));


}
bool MainWindow::saveComment(){

    if (curProtocolFile.isEmpty()) {
        return saveAsComment();
    } else {
        saveProtocolFile(curProtocolFile);
        return true;
    }

}
bool MainWindow::saveAsComment(){

    QString fileName =
            QFileDialog::getSaveFileName(this,"Save the protocol file under a new name",saveUserProtocolDirectory,
            MainWindow::self->GetProtocolFilters());
    if (fileName.isEmpty()) return false;

    int ind = fileName.length () - protocolFileFilterLocator().length ()-1; if (ind < 0) ind =0;
    if ( ( fileName.indexOf(protocolFileFilterLocator(),ind)) == -1 && 
         ( fileName.indexOf(protocolFileFilterLocator().toUpper(),ind)) == -1) {
        fileName.append(protocolFileFilterLocator());
        if (QFile::exists(fileName)) {
        int ret = QMessageBox::warning(this, mainWindowCaption(),
                     QString("File %1 already exists.\n Do you want to overwrite it?")
                     .arg(QDir::toNativeSeparators(fileName)),
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No | QMessageBox::Escape);
        if (ret == QMessageBox::No)
            return true;
       }
    }

    if (!fileName.isEmpty())
		saveUserProtocolDirectory = QFileInfo(fileName).absolutePath ();
        saveProtocolFile(fileName);
    return true;
}

void MainWindow::previewProtocol(){
   PreviewImpl dialog(this,doc);
   dialog.resize(protocolSize);
   dialog.move(protocolPos);
   dialog.exec();
   if(!dialog.protocolFile.isEmpty()) setCurrentProtocol(dialog.protocolFile);
   protocolPos   =  dialog.pos();
   protocolSize  =  dialog.size();

}
void MainWindow::saveProtocolFile(const QString &fileName){
   PreviewImpl dialog(this,doc);
   dialog.saveProtocol(fileName);
   setCurrentProtocol(fileName);

}
bool MainWindow::maybeProtocolSave(){

    if (modifiedProtocol) {
        int ret = QMessageBox::warning(this, mainWindowCaption(),
                     "The protocol has been modified.\n"
                        "Do you want to save your changes?",
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No,
                     QMessageBox::Cancel | QMessageBox::Escape);
        if (ret == QMessageBox::Yes)
            return saveComment();
        else if (ret == QMessageBox::Cancel)
            return false;
    }
    return true;

}
void MainWindow::mainHelpSlot(){

    QString home = searchHelpPath() + "/edcat-contents.html";
    if(help !=0) {delete help; help = 0;}
    help = new HelpWindow(home, this);
    if ( QApplication::desktop()->width() > 400
       && QApplication::desktop()->height() > 500 )
       help->show();
    else
	help->showMaximized();
}
	
	
	
void MainWindow::drawHist(){
	if(!viewcatalogimpl->isCatalogLoaded()) return;
//QMessageBox::critical(0,"","MainWindow::drawHist");
    HistogramSettings  dialog(doc,setDoc,this);
    dialog.resize(histSize);
	dialog.move(histPos);
	dialog.GetSlitter()->restoreState(splitterPlot);
    if (dialog.exec()) {
    }
    histPos   =  dialog.pos();
    histSize  =  dialog.size();
	splitterPlot = dialog.GetSlitter()->saveState();
}


