#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QDir>
#include <QStringList>

const char _FILE_FORMAT_SEPARATOR_[] = "--------------------------";
class QAction;
class QMenu;
class QLabel;
class QSplitter;
class QTextEdit;
class QTimerEvent;
class Catalog;
class BottomWindow;
class ViewCatalogImpl;
class FDocument;
class SettingDocument;
class Qdir;
class ProfileList;
class ProjectWizard;
class HelpWindow;
class MainWindow : public QMainWindow
{
    Q_OBJECT
public:
    enum TempFileType {
        First,
        Second
    };
    enum AutoCheckErrors {
         check,
         ignore
    };

    MainWindow();
     ~MainWindow();
     FDocument* &GetDocument(){return doc;}
     SettingDocument*  &GetSetDocument() {return setDoc;}
     QStringList& GetRepairFileName() {return  repairFileName;}
     ViewCatalogImpl * viewcatalogimpl;
     BottomWindow*        bottomWindow; 
     ProfileList*         prolist;
     HelpWindow* help;
     QDir homeDirectory;
     QDir tempDirectory;
     QString    tempDirectoryPath;
    static MainWindow *self;
    bool loadFile(const QString &fileName);
    void loadFileAuto(const QString &fileName);
    void openInitial();
    void loadProjectFile(const QString &fileName);
    bool IsProjectFilesExist();
    bool exportFile(const QByteArray &fileFormat);
    void initCentralWidget(AutoCheckErrors act);
    void MakeOpenBinaryFileName();
    void removeWorkingMaterials();
    QString&    GetOpenBinaryFileName() {return openBinFileName;}
    QString&    GetProjectBinaryCatalogFileName() {return projectBinCatalogFileName;}
    QString&    GetCurrentSecondCatalogName() {  return currentSecondCatalogName;}
    QString&    GetCurrentCatalogName(){  return curFile;}
    QString&    GetCurrentProjectName(){  return curProFile;}
    QString&    GetCurrentProtocolName(){  return curProtocolFile;}
    void setCurrentFile(const QString &fileName);
    QString AbsDir(const QString fileName) const;
    QString strippedName(const QString &fullFileName);
    void FileActsShowHide(bool b);
    void ProfileActsShowHide(bool b);
    QString  GetFileFilters(){return fileFilters;}
    QString  GetProtocolFilters(){return protocolFileFilters;}
    void SaveStatusbarMessage();
    void RestoreStatusbarMessage();
    QAction *cutAct;
    QAction *copyAct;
    QAction *pasteAct;
    QAction *deleteAct;
    QAction *insertRowAct;
    QAction *duplicatelAct;
    QAction *undoAct;
    QAction *redoAct;
    QAction *execAct;

	
    QAction *viewSaveAct;
    QAction *viewCancelAct;
    QAction *viewFindAct;
    QAction *viewInfoAct;

	
    QLabel* modLabel;
    QLabel* actionLabel;
    QLabel* messageLabel;

// modifed
    bool modifiedCatalog;
    bool modifiedProject;
    bool viewBufferModified;
    bool modifiedProtocol;
    QPoint doubleImplPos,catalImplPos,trmagImplPos,subCatalogPos,projectWizardPos,MyOpenFileDialogPos;
    QSize  doubleImplSize,catalImplSize,trmagImplSize,subCatalogSize,projectWizardSize,MyOpenFileDialogSize;
    QPoint protocolPos,viewSegmentPos,repairProjectPos,histPos,addPos,rangePos,exportUserPos;
    QSize  protocolSize,viewSegmentSize,repairProjectSize,histSize,addSize,rangeSize,exportUserSize;
	QString importUserDirectory,exportUserDirectory;
	QString openUserFileDirectory,saveUserFileDirectory, exportUserFileDirectory,poligonOpenFileDirectory,
	        openUserProjectDirectory,saveUserProjectDirectory,saveUserProtocolDirectory;
			
protected:
    void closeEvent(QCloseEvent *event);
    void timerEvent(QTimerEvent *e);

public slots:    
    void cut();
    void copy();
    void paste();
    void del();
    void selectAll();    
    void insert();
    void undo();
    void redo();    
    void zoomin();
    void zoomout();
    void merge();
    void buildProfile();
    void viewfind();
    void viewinfo();
    void viewsave();
    void viewcancel();	
private slots:
    void newFile();
    void open();
    bool closeFile();
    bool save();
    bool saveAs();
    bool exportSlot();
    void onWhatsThis();
    void mainHelpSlot();
    bool saveAll();
    void newProfile();
    void openProfile();
    void editProfile();
    bool closeProfile();
    bool saveProfile();
    bool saveAsProfile();

    bool saveComment();
    bool saveAsComment();
    void previewProtocol();
    void settingsParameters();
    void checkerror();
    void duplicate();
    void magnitude();
    void sample();

    void add();
    void comp();
    void checksort();
    void sort();    
    void viewbuf();
    void errbuf();
    void correctline();
    void about(); 
	void aboutQwt(); 
//    void updateCellIndicators();
//    void spreadsheetModified();
    void openRecentFile();
    void openRecentProjectFile();
    void 	drawHist();
private:
    int undoRedoTimer;
    FDocument* doc;
    SettingDocument* setDoc;
    void createActions();
    void createWhatsThis();
    void createMenus();
    void createToolBars();
    void createStatusBar();
    void readSettings();
    void writeSettings();
    bool maybeSave();
    bool maybeProjectSave();
    bool maybeProtocolSave();
    void saveFile(const QString &fileName);
    void updateRecentFileActions();
    void userExportFormat();
    QString curFile;
    QString fileFilters;

//-----------------------------------------
    QString curProFile;
    void saveProjectFile(const QString &fileName);
    void setCurrentProFile(const QString &fileName);
    void setCurrentProtocol(const QString &fileName);
    void updateRecentProjectFileActions();
	void initUI();
    QString projectFileFilters;

    QString curProtocolFile;
    QString protocolFileFilters;
    void saveProtocolFile(const QString &fileName);
//------------------------------------------
    enum { MaxRecentFiles = 5 };
    QAction *recentFileActs[MaxRecentFiles];
    QAction *recentProjectFileActs[MaxRecentFiles];
    QMenu *exportAsMenu;
    QMenu *fileMenu;
    QMenu *projectMenu;
    QMenu *protocolMenu;
    QMenu *editMenu;
    QMenu *viewMenu;
    QMenu *selectSubMenu;
    QMenu *toolsMenu;
    QMenu *optionsMenu;
    QMenu *helpMenu;
    QToolBar *fileToolBar;
    QToolBar *editToolBar;
    QToolBar *viewToolBar;
    QToolBar *toolToolBar;
    QToolBar *commentBar;
    QList<QAction *> exportAsActs;
    QList<QAction *> showHideFileActs;
    QList<QAction *> showHideProfileActs;
    QAction *newAct;
    QAction *openAct;
    QAction *closeAct;        
    QAction *saveAct;
    QAction *saveAsAct;
    
    QAction *newProfileAct;
    QAction *openProfileAct;
    QAction *editProfileAct;
    QAction *closeProfileAct;
    QAction *saveProfileAct;
    QAction *saveAsProfileAct;
    
    QAction *saveCommentAct;
    QAction *saveAsCommentAct;
    QAction *printPreviewAct;

    QLabel* protocolLineEdit;
    QAction *exportAct;    
    QAction *saveAllAct;
    QAction *exitAct;
    
    QAction *zoominAct; 
    QAction *zoomoutAct;     
   
    
//   QAction *selectRowAct;
//   QAction *selectColumnAct;
    QAction *selectAllAct;
    
    QAction *settingsAct;   
    QAction *checkerrAct;
    
    QAction *magnitudeAct;
    QAction *sampleAct;
    QAction *mergeAct;
    QAction *addAct;
    QAction *compAct;
    QAction *checkSortAct;    
    QAction *sortAct;	     
    QAction *optviewbufAct;	
    QAction *opterrbufAct;
    QAction *optcorrectlineAct;
    QAction *whatsThisAct;
    QAction *contentAct;
    QAction *aboutAct;
    QAction *aboutQtAct;
    QAction *aboutQwtAct;
    QAction *separatorAct;
    QAction *separatorProjectAct;
	
    QAction *histAct;
	
    QString  homeDirectoryPath;


    QString    openBinFileName;
    QString    currentSecondCatalogName;
    QString    projectBinCatalogFileName;

    QString saveModLabel;
    QString saveActionLabel;
    QString saveMessageLabel;
	QSplitter * splitter;
	QSplitter * splitterVert;
    QStringList repairFileName;
	QByteArray splitterPlot;

	QByteArray splitterImportFile;

};

#endif
