#include <QtGui>
#include "mainwindow.h"
#include "newopensave.h"
#include "default.h"
extern QStringList _INPUT_FORMAT_LIST_;
extern QStringList _OUTPUT_FORMAT_LIST_;
extern QStringList _IMPORT_FORMAT_FILTER_LIST_;
///================================================================
ImportFormatSetDialogs::~ImportFormatSetDialogs(){
	delete openDialog;
	delete imFormat;	
} 
ImportFormatSetDialogs::ImportFormatSetDialogs(QWidget* parent)
	: QDialog(parent){
	setFont( MainWindow::self->font() );
	setWindowTitle("Choose a catalog and it's format");	
	QGridLayout * gridLayout = new QGridLayout(this);
	splitter = new QSplitter(this);
	splitter->setOrientation (Qt::Vertical );
	openDialog   = new MyOpenFileDialog(this);
	imFormat     = new ImportFormat(this);	
	splitter->addWidget(openDialog);
	splitter->addWidget(imFormat);
	gridLayout->addWidget(splitter, 0, 0, 1, 1);

	setSizeGripEnabled (true);
    
}
void ImportFormatSetDialogs::accept (){

	QDialog::accept();
}	
void ImportFormatSetDialogs::reject (){

	QDialog::reject ();

}
///================================================================
ImportFormat::ImportFormat(QWidget* parent)
	: QDialog(0),parent(parent)
{
	QFont f = parent->font();
	setFont( parent->font() );
	setupUi(this);
	comboBox->insertItems(0,_INPUT_FORMAT_LIST_);
	comboBox->setCurrentIndex(1);
	textEdit->setReadOnly (TRUE);
	connect( comboBox,   SIGNAL( activated ( const QString &  ) ), this,
		 SLOT( textChangeComboBox( const QString & ) ) );
} 
ImportFormat::~ImportFormat (){

}
void ImportFormat::textChangeComboBox( const QString &  ){
	QStringList strlist = ((ImportFormatSetDialogs* )parent)->GetOpenDialog()->selectedFiles ();
	if (strlist.isEmpty ()) {
		QMessageBox::warning(0, "Choose a catalog and it's format",
				     QString("Select some file in the list of files of the selected directory") );
		return;
	}
	QString tmp1 =  strlist[0] ;
	QFileInfo info(tmp1);
	if (!info.isFile ()) return;
	if(!info.exists()) return;
	CheckCatalogFormat(tmp1,comboBox->currentIndex());
	
}
bool ImportFormat::AutoCheckCatalogFormat(QString name){
	bool ret_code = false;
	QString  catLine;
	for (int i = 0; i <  comboBox->count (); ++i){
		if (i > 0)  ret_code = catalog.tryAsciiFormat(name,i,catLine);
		else  ret_code = catalog.tryBin20Format(name,catLine);
		if(ret_code) {comboBox->setCurrentIndex(i);  break;}
	}
	if (ret_code == false) {
		textEdit->clear ();
		textEdit->setTextColor ("red");
		textEdit->setPlainText(" ----------------------  Error format  -----------------------");
		return false;
	}
	
	textEdit->setTextColor ("blue");
	catLine.remove(0,1);
	textEdit->clear ();
	textEdit->setPlainText(catLine);
	return true;
}
bool ImportFormat::CheckCatalogFormat(QString name,int numFormat){
	bool ret_code = false;
	QString  catLine;
	if (numFormat > 0)  ret_code = catalog.tryAsciiFormat(name,numFormat,catLine);   
	else  ret_code = catalog.tryBin20Format(name,catLine);
	if (ret_code == false) {
		textEdit->clear ();
		textEdit->setTextColor ("red");
		textEdit->setPlainText(" ----------------------  Error format  -----------------------");
		return false;
	}
	textEdit->setTextColor ("blue");
	catLine.remove(0,1);
	textEdit->clear ();
	textEdit->setPlainText(catLine);
	return true;
}
///================================================================
MyOpenFileDialog::MyOpenFileDialog(QWidget* parent):
		QFileDialog(parent,"Choose a catalog and it's format",
		MainWindow::self->importUserDirectory),parent(parent)
{
	setFont( parent->font() );
	setSizeGripEnabled (false );
	/*
	QStringList types;
	types     << "Any files (*)"
			<< "Standard Format - Binary (*.20byte)"
			<< "Standard Format - Text (*.41byte)"
			<< "Standard Wide Format - Text (*.68byte)"
			<< "Advanced National Seismic System (*.anss)"
			<< "Southern California Earthquake Data Center (*.scedc)"
			<< "Southern California Seismographic Network (*.scsn)"
			<< "NEIC Preliminary Determinations of Epicenters (*.pde)"
			<< "Summary hypo71 format year 2000 (*.hypo71)"
			<< "NEIC Hypocenter Database System (*.hds)";
	*/
	setFilters( _IMPORT_FORMAT_FILTER_LIST_  );

	setFileMode (QFileDialog::ExistingFile);
	setViewMode ( QFileDialog::Detail );
	
	QList<QTreeView    *> viewList = findChildren<QTreeView    *>();
	QTreeView* wl = viewList.at(0);
	QItemSelectionModel* selmod = wl->selectionModel ();
	connect( selmod,SIGNAL(selectionChanged ( const QItemSelection & , const QItemSelection &  ) ), this,
		 SLOT( changedSelection ( const QItemSelection & , const QItemSelection &  )) );

}
void MyOpenFileDialog::changedSelection ( const QItemSelection & , const QItemSelection & ){
	QStringList strlist = selectedFiles();
	QString tmp1 =  strlist[0] ;
	QFileInfo info(tmp1);
	if (!info.isFile ()) {
		((ImportFormatSetDialogs* )parent)->GetImportformatWindow()->GetTextEdit()->clear ();
		return;
	}
	if(!info.exists()) return;
	if (!((ImportFormatSetDialogs* )parent)->GetImportformatWindow()->AutoCheckCatalogFormat(tmp1)) {
	}
}


void MyOpenFileDialog::reject(){
	((ImportFormatSetDialogs* )parent)->reject();
}
void MyOpenFileDialog::accept (){
	QStringList strlist = selectedFiles();
	QString tmp1 =  strlist[0] ;
	QFileInfo info(tmp1);
	if (!info.isFile ()) return;
	if(!info.exists()) return;
	QComboBox* combobox = ((ImportFormatSetDialogs* )parent)->GetImportformatWindow()->GetBomboBox();
	
	if (((ImportFormatSetDialogs* )parent)->GetImportformatWindow()->CheckCatalogFormat(tmp1,combobox->currentIndex()) ) 
	{
		QApplication::setOverrideCursor(Qt::WaitCursor);
	
		if(!catalog.makeBinaryFile(tmp1,combobox->currentIndex (),1)) {
			QApplication::restoreOverrideCursor();
			return; }
			QApplication::restoreOverrideCursor();
			MainWindow::self->importUserDirectory = MainWindow::self->AbsDir(tmp1);
			QDialog::accept();
			((ImportFormatSetDialogs* )parent)->accept();
	}
	return;
}

MyOpenFileDialog::~MyOpenFileDialog(){
}

//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------
