#include <QtGui>
#include "tableproperty.h"
#include "subcatalog.h"
#include "polygon.h"
#include "default.h"
#include "document.h"
//-------------------------------------------
PolygonImpl::PolygonImpl(QWidget *parent,int rows,FDocument* dc)
      : QDialog(parent),doc(dc)

{
	setFont( parent->font() );
    setWindowIcon(QIcon(":/images/icon.png"));
    setupUi(this);
    gridLayout->removeWidget(tableWidget);
    parentWindow = (SubCatalogImpl*) parent;
    tableWidget->hide();
    setModal (true);
    table = new TableMinMax(this,rows,doc);

    gridLayout->addWidget(table,  0, 0, 1, 1);


    connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
    connect(fileButton, SIGNAL(clicked()), this, SLOT(verticesFromFile()));
    connect(helpButton, SIGNAL(clicked()), this, SLOT(help()));
}
void PolygonImpl::verticesFromFile() {
	
	QString fileFilters = tr("txt files (*%1);;All Files (*)").arg(poligonFilterLocator());
    QString fileName =
                    QFileDialog::getOpenFileName(this,"Open an existing catalog file",
					MainWindow::self->poligonOpenFileDirectory, fileFilters);
    if (fileName.isEmpty()) return;
	
	//QMessageBox::critical(0,"",QString("verticesFromFile %1 ").arg(fileName));
	MainWindow::self->poligonOpenFileDirectory = QFileInfo(fileName).absolutePath ();

    QFile file(fileName);
    if (!file.open(QFile::ReadOnly | QFile::Text)) {
		QApplication::restoreOverrideCursor();
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("Cannot read file %1:\n%2.")
                             .arg(fileName)
                             .arg(file.errorString()));
        return;
    }
    QTextStream in(&file);
	
	QString line;
	int numLine = 0;
	QVector<float> xVector;
	QVector<float> yVector;
	line = in.readLine();

  for (;;){
	  if(line.isNull()) break;
	  if(line[0] == '#') {
		  numLine++;
		  line = in.readLine();
		  continue;
	  }
	  if(line.count(' ') == line.length ()) {
		  QMessageBox::warning(0,"",QString("i %1 count %2").arg(numLine+1).arg(line.count(' ')));
		  numLine++;
		  line = in.readLine();
		  continue;
	  }
	  QStringList list = line.split(" ", QString::SkipEmptyParts);
	  
	 if (list.size() !=2) {
		QApplication::restoreOverrideCursor();
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("Bad format file. Line (%1) does not contain two numbers.\nfile %2")
                             .arg(numLine+1)
                             .arg(fileName));
        return;
	 }
	 bool ok;
	 QString first =  list.at(0);
	 double x = first.toDouble(&ok);
     if(!ok) {
		QApplication::restoreOverrideCursor();
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("Bad format file: Line (%1). First number is not float.\nfile %2")
                             .arg(numLine+1)
                             .arg(fileName));
        return;
	 }
	 
	 QString second =  list.at(1); 
	 double y = second.toDouble(&ok);
     if(!ok) {
		QApplication::restoreOverrideCursor();
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("Bad format file: Line (%1). Second number is not float.\nfile %2")
                             .arg(numLine+1)
                             .arg(fileName));
        return;
	 }
	 xVector.append(x);
	 yVector.append(y);
	 numLine++;
     line = in.readLine();
 } 
 
 
	file.close();
	table->model->removeRows(0,table->model->rowCount());
	table->model->insertRows (0,xVector.size());
	if(!table->model->ipList.empty())
	{ 
		qDeleteAll(table->model->ipList);
		table->model->ipList.clear();
	}
	for (int i = 0; i < xVector.size(); ++i) {
		QString col1 = QString("%1").arg(xVector.at(i));
		QString col2 = QString("%1").arg(yVector.at(i));
		table->model->SetPolygonRow(i,col1,col2); 
	}
	table->ResizeHorizontalHeader(table->horizontalHeader ());
    table->ResizeRows();
	
}
void PolygonImpl::reject()
{


    done( Rejected );
}
void PolygonImpl::accept()
{
  QPointer<TableLineEditor> lin = table->delegate->GetLineEdit();
  if (lin != 0) {
  IProperty* prop = lin->GetProperty();
  QModelIndex index = prop->index;
  QString tmp = lin->text();
  table->model->setData(index, tmp);
 }

  table->SetPolygonValue();
  parentWindow->GetSpinBoxPolygon()->setValue(doc->GetSampleHipoPolygonVertice()->GetValue().toInt());
  if (checkBox180->isChecked()) doc->GetSampleHipoPolygonMinus180()->SetValue("1");
  else doc->GetSampleHipoPolygonMinus180()->SetValue("0");
  done( Accepted );
//qWarning("accept() 1");
}
void PolygonImpl::help()
{

}
//--------------------------------------------------------------------------------
//---------------------------------------------------------------------------------
void TableMinMax::SetPolygonValue(){
  QStringList lat;
  QStringList lon;

  for (int i = 0; i < model->rowCount(); i++) {
  lat.append(model->data(model->index(i, 0, QModelIndex())).toString ());
  lon.append(model->data(model->index(i, 1, QModelIndex())).toString ());
  }
  doc->GetSampleListLatitude()->SetListToValue(lat);
  doc->GetSampleListLongitude()->SetListToValue(lon);
  QString tmp = QString("%1").arg((int)model->rowCount());
  doc->GetSampleHipoPolygonVertice()->SetValue(tmp);
}
TableMinMax::TableMinMax(QWidget *parent,int rows,FDocument* dc)
  : QTableView(),doc(dc)
{
    QFont f = parent->font();
    setFont( f );
    model = new PropertyModel(this,rows, 3);

    setAlternatingRowColors (TRUE);
    setModel(model);
    delegate = new PropertyDelegate;

    setItemDelegate(delegate);
    selectModel = selectionModel();

     QStringList lat = doc->GetSampleListLatitude()->GetStringList();
     QStringList lon = doc->GetSampleListLongitude()->GetStringList();
     QPolygonF polygon;
     for (int i = 0; i < lat.size(); ++i) {
     polygon << QPointF(lat.at(i).toFloat(),lon.at(i).toFloat());
    }


   for (int i = 0; i < polygon.size(); ++i) {
   QString col1 = QString("%1").arg(polygon.at(i).x());
   QString col2 = QString("%1").arg(polygon.at(i).y());
   model->SetPolygonRow(i,col1,col2);
   }
   if (rows > polygon.size())
      for (int i = polygon.size(); i < rows; ++i) 
         model->SetPolygonRow(i,"0","0");

    PolygonImpl* parentWindow = (PolygonImpl*)parent;

    if ( doc->GetSampleHipoPolygonMinus180()->GetValue() == "0")
    parentWindow->GetCheckBox180()->setChecked (FALSE);
    else  parentWindow->GetCheckBox180()->setChecked (TRUE);
 /*
    for(int i = 0; i < rows; ++i)  {

       model->SetPolygonRow(i,QString("%1").arg(i),QString("%1").arg(i + 5.0));

    }   	
*/
	
    ResizeHorizontalHeader(horizontalHeader ());
    ResizeRows();
    createActions();

    verticalHeader ()->setContextMenuPolicy (Qt::CustomContextMenu);
    connect( verticalHeader (), SIGNAL(  customContextMenuRequested ( const QPoint & )  ),
	     this, SLOT( verticalContextMenu ( const QPoint & ) ) );
}
void TableMinMax::ResizeHorizontalHeader(QHeaderView *hd)
{
QFont fnOld = font();
QFont fn = fnOld;
fn.setBold(true);

hd->resizeSection(0,fontMetrics().width(fontmetricCatalogLatitude()));
hd->resizeSection(1,fontMetrics().width(fontmetricCatalogLongitude()));

hd->setStretchLastSection(TRUE);
setFont(fnOld);
}
void TableMinMax::ResizeRows()
{
   for (int row = 0; row < model->rowCount(); ++row)  resizeRowToContents(row);
}
void TableMinMax::createActions()
{
    deleteAct =  new QAction(QIcon(":/images/delete.png"), tr("&Delete"),this);
    deleteAct->setShortcut(QKeySequence::Delete);
    deleteAct->setStatusTip(tr("Delete the current selection's "
                               "contents"));
    connect(deleteAct, SIGNAL(triggered()),this, SLOT(del()));

 //   deleteAct->setShortcutContext ( Qt::ApplicationShortcut);


    insertRowAct = new QAction(QIcon(":/images/insert_table_row.png"), tr("&Insert row"),this);
    insertRowAct->setShortcut(tr("Ctrl+I"));
    insertRowAct->setStatusTip(tr("Insert row before the selected "
                               "row"));
    connect(insertRowAct, SIGNAL(triggered()),this, SLOT(insert()));

}
void TableMinMax::verticalContextMenu ( const QPoint &  )
{
    SetContextMenu (cursor ().pos());
}
void TableMinMax::SetContextMenu ( const QPoint & pos )
{
	QItemSelection    isel =  selectModel->selection ();
    if(isel.empty()) return;
    QMenu menu(this);
    menu.addAction(deleteAct);
    menu.addAction(insertRowAct);
    menu.exec(pos);
}
void TableMinMax::contextMenuEvent(QContextMenuEvent *event)
{
    SetContextMenu (event->globalPos());
}
 void TableMinMax::keyPressEvent(QKeyEvent *event)
 {
     switch (event->key()) {
     case Qt::Key_Delete:
         del();
         break;
     case Qt::Key_I:
		 if (Qt::ControlModifier == event->modifiers ())
         insert();
         break;
	
     default:
         QFrame::keyPressEvent(event);
} 

}
void TableMinMax::del(){
	
  QItemSelection    isel =  selectModel->selection ();
  if(isel.empty()) return;
   QItemSelectionRange range;
   QModelIndex index;
  for (int i = 0; i < isel.size(); ++i) {

range = isel.at(i);
//int selLeftCol   =   range.left();
int selTopRow    =   range.top();
//int selRightCol  =   range.right();
int selNumRows   =   range.height();
int selNumCols   =   range.width();
//QMessageBox::critical(0,"",QString("TableMinMax::del cols %1 rows 2").arg(selNumCols).arg(selNumRows));

//int selBottomRow =   range.bottom();
if (selNumCols == 3) {
       model->removeRows(selTopRow,selNumRows);
       model->RemovePolygonRowsProperty(selTopRow,selNumRows);
       model->RepairPolygonRowsProperty(selTopRow);
       ResizeRows();
} 
}
 
}
void TableMinMax::insert(){
QItemSelection    isel =  selectModel->selection ();
//QMessageBox::critical(0,"",QString("TableMinMax::insert %1").arg(isel.size()));

QItemSelectionRange range = isel.first () ;

//int selLeftCol   =   range.left();
int selTopRow    =   range.top();
//int selRightCol  =   range.right();
//int selNumRows   =   range.height();
int selNumCols   =   range.width();
//int selBottomRow =   range.bottom();
if (selNumCols == 3) {
model->insertRows (selTopRow,1);
model->InsertEmptyPolygonValuePropertyRows (selTopRow,1);
model->RepairPolygonRowsProperty(selTopRow);
ResizeRows();
}
}

