#ifndef _PROFILE_DIALOG_H
#define _PROFILE_DIALOG_H
#include <QDockWidget>
#include <QTreeView>
#include <QItemDelegate>
#include <QModelIndex>
#include <QObject>
#include <QStandardItemModel>
#include <QDialog>
#include <QList> 
#include <QMap>
#include <QVector>
#include "ui_profile.h"
#include "ui_project.h"
#include "ui_preview.h"
#include "catalimpl.h"
#include "projectwizard.h"
#include <vector>
using namespace std;
class QWidget;
class QDialog;
class QTextEdit;
class FDocument;
class ProfileTreeView;
class TreeProfileModel;
class TransfMagnImpl;
class DoubleBaseImpl;
class SubCatalogImpl;
class CatalBaseImpl;
class AppendImpl;
class RangeLimitImpl;
class HelpWindow;
class TreeProfileDelegate;
class ProfileImpl : public QDialog, private Ui::ProfileBase
{
    Q_OBJECT
public:
    ProfileImpl(QWidget *parent,FDocument* docCurrent,FDocument* docDefault,
		ProWizard::DocumentUsing documentUsing);
    ~ProfileImpl();
QListWidget* GetOutList(){return listWidgetOut;}
QRadioButton * GetRadioButtonDefault(){return radioButtonDefault;} 
int nextIdPage(int id);
int projectActSize;
QMap<int,int> nextIdMap;
bool NextOk();
void FinishOk();
bool GetIsDocDfault(){return isDocDfault;}
public slots:
	
protected slots:
void accept();
void reject();
void add();
void remove();
void up();
void down();
void browse();
void rbList();
void rbCompare();
void rbRange();
void reset();
void project();
void defaults();
void rbCurrentDoc();
void rbDefaultDoc();

private:
void GetDialogFromDocument();
void GetDocumentFromDialog();
FDocument* doc;
FDocument* docDefault;
FDocument* docCurrent;

QStringList listProjectAction;
QList<QToolButton *> toolButtonList;
QList<PropertyValue* > values;
QString openUserToolFileDirectory;
bool isDocDfault;
ProWizard::DocumentUsing documentUsing;
};
//------------------------------------------------------------------------------------
class ProfileList : public QDialog
{
    Q_OBJECT
public:
    enum TypeProfile
    {
        Normal,
        Repair
    };
          TypeProfile type;
          ProfileList (FDocument* doc, QString tittle,QWidget * parent,
          TypeProfile type = Normal);
         ~ProfileList       (){};
void SetRowList(FDocument* dc,QString name);
void SetProjectName(QString &name);
ProfileTreeView* GetListWidget(){return listWidget; }
void closeOpenedEditor();
void closeProfile();
private:
ProfileTreeView*   listWidget;
FDocument* doc;
};
//----------------------------------------------------------------------------------
class ProfileTreeView : public QTreeView
{ 
    Q_OBJECT 
public:
    ProfileTreeView(int numCol,QWidget* parent, QString name,ProfileList::TypeProfile type);
    ~ProfileTreeView(){};
void SetRowList(QString name,QString &file,QStringList &listAction,QStringList &listMergeCatalog,QStringList &listAddCatalog,
          QString &compCatalogName,CatalBaseImpl::ModeAction compModAct);
void SetProjectName(QString &name);
TreeProfileModel* GetModel(){ return model;}
TreeProfileDelegate* GetDelegate(){ return delegate;}
ProfileList::TypeProfile type;
public slots:

protected:
void drawBranches(QPainter *painter, const QRect &rect, const QModelIndex &index) const;
protected slots:

private:

TreeProfileModel* model;
TreeProfileDelegate *delegate;
QModelIndex edIndex;
};
//------------------------------------------------------------------------------------
class TreeProfileModel : public QStandardItemModel
{
    Q_OBJECT

public:
TreeProfileModel(int columns, QString tittle, QObject * parent = 0);
~TreeProfileModel(){};  
void SetColumns(int rowFirst);
void SetColumns(int rowFirst,int rowSecond);
void SetItem(int rowFirst,QString value);
void SetChildItem(int rowFirst,int rowSecond,const QString value);
void SetItem(int rowFirst,int rowSecond,int rowThird,const QString value,QIcon * icon = 0);

void SetChildItem(int rowFirst,QStringList &listAction,
                  QStringList &listMergeCatalog,QStringList &listAddCatalog,
                  QString &compFile,CatalBaseImpl::ModeAction compModAct);
int &GetNumRows(){return NumRows;}
ProfileTreeView* parentWindow;
void SetActiveAction(const QString &act);
QString GetItemValue(int first);
QString GetItemValue(int first,int second);
QString GetItemValue(int first,int second,int third);
void closeProfile();
int numberActiveRow;
void SetHorizontalHeader();
ProfileList::TypeProfile type;
QList<QModelIndex> rlist;
QString curOpenFile;
private:
int NumRows;
int NumCols;

};
//------------------------------------------------------------------------------------
class TreeProfileDelegate : public QItemDelegate
{
    Q_OBJECT

public:
   TreeProfileDelegate(QObject *parent = 0);
    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option,
                          const QModelIndex &index) const;

    void setEditorData(QWidget *editor, const QModelIndex &index) const;
    void setModelData(QWidget *editor, QAbstractItemModel *model,
                      const QModelIndex &index) const;
    void paint(QPainter *painter, const QStyleOptionViewItem &option,
                       const QModelIndex &index) const;
bool& GetExecuteAction() { return executeAction; }
private slots:

private:
bool executeAction;
};
//===============================================================================
class FindFileEditor : public QWidget
{
    Q_OBJECT
public:
    FindFileEditor(const QModelIndex & index,QWidget *parent = 0);
    ~FindFileEditor();
    void setLine(const QString &label);
    QLineEdit  *GetLineEditor() {return lin;}
    QPushButton *GetButton() {return button;}
private slots:
    void clickButton();
    void editFinish();
private:
    QPushButton *button;
    QLineEdit *lin;
    QModelIndex index;
    QString openUserToolFileDirectory;
};
//===============================================================================
class PreviewImpl : public QDialog, private Ui::PreviewProtocol
{
    Q_OBJECT

public:
   ~PreviewImpl();
   PreviewImpl(QWidget *parent,FDocument* dc);
   QString protocolFile;
   void saveProtocol(const QString &fileName);
private slots:
    void accept();
    void save();
    void btnHelp();
private:
   FDocument* doc;
   HelpWindow* help;
};

#endif 
