#include <QtGui>
#include <QMessageBox> 
#include <QMap>
#include "profile.h"
#include "default.h"
#include "document.h"
#include "transmagimpl.h"
#include "doubleimpl.h"
#include "subcatalog.h"
#include "helpwindow.h"
#include "append.h"
#include "rangelimit.h"
#include "mainwindow.h"
#include "bottomwindow.h"
#include "projectwizard.h"
int currentIdPage = 0; 
static bool firstWizardVisit  = true;
ProWizard::~ProWizard(){
	    if(help !=0) {delete help; help =0;}
}	

ProWizard::ProWizard(QWidget *parent,FDocument* docDefault,FDocument* docCurrent,
		DocumentUsing documentUsing) : QWizard(parent),
		docDefault(docDefault),docCurrent(docCurrent),documentUsing(documentUsing)
{

	setFont( parent->font() );
	pageProjectAction = new ProjectActionPage(this,docDefault,docCurrent,documentUsing);

	pageCurrentConvertMagnitudes   = new ConvertMagnitudesPage(this,docCurrent);
	pageDefaultConvertMagnitudes   = new ConvertMagnitudesPage(this,docDefault);
	
	pageCurrentRemoveDuplicate     = new RemoveDuplicatePage(this,docCurrent);
	pageDefaultRemoveDuplicate     = new RemoveDuplicatePage(this,docDefault);

	pageCurrentSelectSubcatalog    = new SelectSubcatalogPage(this,docCurrent);
	pageDefaultSelectSubcatalog    = new SelectSubcatalogPage(this,docDefault);
	
	pageCurrentMergeCatalogs       = new MergeCatalogsPage(this,docCurrent);
	pageDefaultMergeCatalogs       = new MergeCatalogsPage(this,docDefault);
	
	pageCurrentSubstituteCatalogs  = new SubstituteCatalogsPage(this,docCurrent);
	pageDefaultSubstituteCatalogs  = new SubstituteCatalogsPage(this,docDefault);

	pageCurrentCompareCatalogs     = new ComparePage(this,docCurrent);
	pageDefaultCompareCatalogs     =  new ComparePage(this,docDefault);	
		
	pageCurrentCheckRanges         = new CheckRangePage(this,docCurrent);
	pageDefaultCheckRanges         = new CheckRangePage(this,docDefault);

	setPage(Project_Action,pageProjectAction);
	
	setPage(Current_ConvertMagnitudes , pageCurrentConvertMagnitudes);
	setPage(Current_RemoveDuplicate   , pageCurrentRemoveDuplicate);
    setPage(Current_SelectSubcatalog  , pageCurrentSelectSubcatalog);
    setPage(Current_MergeCatalogs     , pageCurrentMergeCatalogs);
    setPage(Current_SubstituteCatalogs, pageCurrentSubstituteCatalogs);
    setPage(Current_CompareCatalogs   , pageCurrentCompareCatalogs);
    setPage(Current_CheckRanges       , pageCurrentCheckRanges);

	
	setPage(Default_ConvertMagnitudes , pageDefaultConvertMagnitudes);
    setPage(Default_RemoveDuplicate   , pageDefaultRemoveDuplicate);
    setPage(Default_SelectSubcatalog  , pageDefaultSelectSubcatalog);
    setPage(Default_MergeCatalogs     , pageDefaultMergeCatalogs);
    setPage(Default_SubstituteCatalogs, pageDefaultSubstituteCatalogs);
    setPage(Default_CompareCatalogs   , pageDefaultCompareCatalogs);
    setPage(Default_CheckRanges       , pageDefaultCheckRanges);
    setPage(Last_Page                 , new QWizardPage());
	
	setStartId(Project_Action);
	setModal (true);
#ifndef Q_WS_MAC
    setWizardStyle(ModernStyle);
#endif
    setOption(HaveHelpButton, true);

    connect(this, SIGNAL(helpRequested()), this, SLOT(showHelp()));
	if (documentUsing == ProWizard::ProjectNew)
		setWindowTitle(tr("Project Wizard - New Project"));
	if (documentUsing == ProWizard::ProjectEdit)
		setWindowTitle(QString("Project Wizard - Edit Project: %1").
			arg(MainWindow::self->strippedName(docCurrent->GetProjectCatalogName()->GetValue())));
	connect( button ( QWizard::NextButton ), SIGNAL( clicked() ), this, SLOT( nextButton() ) ); 
	connect( button ( QWizard::BackButton ), SIGNAL( clicked() ), this, SLOT( backButton() ) );
	currentIdPage = 0;
	firstWizardVisit  = true;
	help =0;
}
void ProWizard::showHelp()
{
	QString home = searchHelpPath();
	 switch (currentId()) {
    case Project_Action:
        home += "/edcat-project-wizard.html";
        break;
    case Current_ConvertMagnitudes:
    case Default_ConvertMagnitudes:
        home += "/edcat-tool-transform.html";
        break;
    case Current_RemoveDuplicate:
    case Default_RemoveDuplicate:
        home += "/edcat-tool-duplicates.html";
        break;
    case Current_SelectSubcatalog:
    case Default_SelectSubcatalog:
        home += "/edcat-tool-subcatalog.html";
        break;
    case Current_MergeCatalogs:
    case Default_MergeCatalogs:
        home += "/edcat-tool-merge.html";
        break;
    case Current_CompareCatalogs:
    case Default_CompareCatalogs:
        home += "/edcat-tool-compare.html";
        break;
    case Current_SubstituteCatalogs:
    case Default_SubstituteCatalogs:
        home += "/edcat-tool-substitute.html";
        break;
    case Current_CheckRanges:
    case Default_CheckRanges:
        home += "/edcat-tool-range.html";
        break;
    default:
		return;
    }
  
	if(help !=0) {delete help; help =0;}
	help = new HelpWindow(home,this);
    if ( QApplication::desktop()->width() > 400
         && QApplication::desktop()->height() > 500 )
         help->show();
    else
        help->showMaximized();
}
void ProWizard::backButton() {
	ProfileImpl* projectActionPage = pageProjectAction->proPage;
	int size = projectActionPage->projectActSize;
	if(currentIdPage == size-1 || currentIdPage == 0) setButtonText (QWizard::NextButton, "Next>" ); 
	currentIdPage--;


}
void ProWizard::nextButton(){
	ProfileImpl* projectActionPage = pageProjectAction->proPage;
	int size = projectActionPage->projectActSize;
	currentIdPage++;
	if( currentIdPage > 0 && currentIdPage == size-1) 
		setButtonText (QWizard::NextButton, "Finish" ); 
	if (currentIdPage == size) 
		Finish();

}
void ProWizard::Finish(){ 
	QList<int> listPages = visitedPages ();
	for (int i = 1; i < listPages.size(); ++i) {
		QWizardPage * pg  = page(listPages.at(i));
		if (WizardPage* wd = qobject_cast<WizardPage *>(pg))
			if(!wd->NextOk()) {currentIdPage--;return;}
	} 
	MainWindow::self->bottomWindow->CleanAllEntries();
	MainWindow::self->bottomWindow->OpenMessagesProtocolPage();
	
	ProfileImpl* projectActionPage = pageProjectAction->proPage;
	projectActionPage->FinishOk();
	for (int i = 1; i < listPages.size(); ++i) {
		QWizardPage * pg  = page(listPages.at(i));
		if (WizardPage* wd = qobject_cast<WizardPage *>(pg))
			wd->FinishOk();
		
	}
	
	firstWizardVisit  = true;
	
	return accept ();
}
int ProWizard::nextId() const{
	int id = currentId ();
	if (firstWizardVisit) {firstWizardVisit = false; return id;}
	ProfileImpl* projectActionPage = pageProjectAction->proPage;
    switch (id) {
		case Project_Action:
		{
			bool ret = projectActionPage->NextOk();
				if (!ret) {
					return Project_Action;
				} else {
					currentIdPage = 0;
					return projectActionPage->nextIdPage(Project_Action);
				}
			currentIdPage = 0;
			return projectActionPage->nextIdPage(Project_Action);
		}
//----------------------------------------------------------------------------
		case Default_ConvertMagnitudes:
		{	 
			return projectActionPage->nextIdPage(Default_ConvertMagnitudes);	
		}
		case Current_ConvertMagnitudes:
		{
			return projectActionPage->nextIdPage(Current_ConvertMagnitudes);
		}
//----------------------------------------------------------------------------
		case Default_RemoveDuplicate:
		{
			return projectActionPage->nextIdPage(Default_RemoveDuplicate);
		}
		case Current_RemoveDuplicate:
		{
			return projectActionPage->nextIdPage(Current_RemoveDuplicate);
		}
//----------------------------------------------------------------------------
		case Default_SelectSubcatalog:
		{
			return projectActionPage->nextIdPage(Default_SelectSubcatalog);
		}
		case Current_SelectSubcatalog:
		{
			return projectActionPage->nextIdPage(Current_SelectSubcatalog);
		}
//----------------------------------------------------------------------------
		case Default_MergeCatalogs:
		{
			return projectActionPage->nextIdPage(Default_MergeCatalogs);
		}
		case Current_MergeCatalogs:
		{
			return projectActionPage->nextIdPage(Current_MergeCatalogs);
		}
//----------------------------------------------------------------------------
		case Default_SubstituteCatalogs:
		{
			return projectActionPage->nextIdPage(Default_SubstituteCatalogs);
		}
		case Current_SubstituteCatalogs:
		{
			return projectActionPage->nextIdPage(Current_SubstituteCatalogs);
		}
//----------------------------------------------------------------------------
		case Default_CompareCatalogs:
		{
			return projectActionPage->nextIdPage(Default_CompareCatalogs);
		}
		case Current_CompareCatalogs:
		{
			return projectActionPage->nextIdPage(Current_CompareCatalogs);
		}
//----------------------------------------------------------------------------
		case Default_CheckRanges:
		{
			return projectActionPage->nextIdPage(Default_CheckRanges);
		}
		case Current_CheckRanges:
		{
			return projectActionPage->nextIdPage(Current_CheckRanges);
		}
//----------------------------------------------------------------------------
		default:
			return -1;
     }


}
/*
//--------------------------------------------------------------------------

	QList<int> keys = nextIdMap.keys ();
	QList<int> values = nextIdMap.values ();
	QString tmp = QString("Task List Current_RemoveDuplicate: size %1\n").arg(nextIdMap.size());
	for (int i = 0; i <  nextIdMap.size(); i++)
	  tmp += QString("n page  %1  id %2\n").arg(keys[i]).arg(values[i]);
	QMessageBox::critical(0,"",tmp);

//-------------------------------------------------------------------------	
*/
//=========================================================================
ProjectActionPage::ProjectActionPage(QWidget *parent,FDocument* docDefault,FDocument* docCurrent,
	ProWizard::DocumentUsing documentUsing)
    : QWizardPage(parent)
{

    setTitle(tr("Project Info"));
    setSubTitle(tr("Define a sequence of operations on catalog(s).")); 
	
	ProWizard::DocumentUsing type = ((ProWizard *) parent)->GetDocumentUsing();
	if (type == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (type == ProWizard::ProjectEdit) 
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png")); 
		
	proPage = new ProfileImpl(this,docDefault,docCurrent,documentUsing);

    proPage->setWindowFlags(Qt::SubWindow);
    proPage->setSizeGripEnabled ( FALSE );
    proPage->setModal (false);
	
    QGridLayout *layout = new QGridLayout;
    layout->addWidget(proPage);
    setLayout(layout);
	
}

//=========================================================================
ConvertMagnitudesPage::ConvertMagnitudesPage(QWidget *parent,FDocument* doc)
    : WizardPage(parent)
{
	QStringList listAction = projectActionList().split ('@');
	
    setTitle(listAction.at(0));
    setSubTitle(tr("Define output magnitude(s) conversion(s)"));
	
	ProWizard::DocumentUsing type = ((ProWizard *) parent)->GetDocumentUsing();
	if (type == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (type == ProWizard::ProjectEdit) 
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png")); 
	
	magnitudePage  = new TransfMagnImpl(this,doc,TransfMagnImpl::Wizard);

    QList<QPushButton *> allPButtons = magnitudePage->findChildren<QPushButton *>();
    foreach (QPushButton *button, allPButtons)
    button->hide();
    QProgressBar* bar = magnitudePage->findChild<QProgressBar *>();
    if(bar !=0)bar->hide();
    magnitudePage->setSizeGripEnabled ( FALSE );
    magnitudePage->setWindowFlags(Qt::SubWindow);
    magnitudePage->setModal (false);
	
	QGridLayout *layout = new QGridLayout;
    layout->addWidget(magnitudePage);
    setLayout(layout);
	
}
bool ConvertMagnitudesPage::NextOk(){return magnitudePage->NextOk();}
void ConvertMagnitudesPage::FinishOk(){return magnitudePage->FinishOk();}
//=========================================================================
 
RemoveDuplicatePage::RemoveDuplicatePage(QWidget *parent,FDocument* doc)
    : WizardPage(parent)
{
	QStringList listAction = projectActionList().split ('@');
	
    setTitle(listAction.at(1));
    setSubTitle(tr("Define Proximity limits and Criterion for autoremoval of duplicates"));
	
	ProWizard::DocumentUsing type = ((ProWizard *) parent)->GetDocumentUsing();
	if (type == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (type == ProWizard::ProjectEdit) 
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png")); 
	
    doublePage     = new DoubleBaseImpl(this,doc,DoubleBaseImpl::Wizard); 

    QList<QPushButton *> allPButtons = doublePage->findChildren<QPushButton *>();
    foreach (QPushButton *button, allPButtons)
    button->hide();
    QProgressBar* bar = doublePage->findChild<QProgressBar *>();
    if(bar !=0)bar->hide();
    doublePage->GetTable()->hide();
    doublePage->GetToolButtonSearch()->hide();
    doublePage->GetRadioDialog()->hide();
    doublePage->setSizeGripEnabled ( FALSE );
    doublePage->setWindowFlags(Qt::SubWindow);
    doublePage->setModal (false);

	QGridLayout *layout = new QGridLayout;
    layout->addWidget(doublePage);
    setLayout(layout);
	
}
bool RemoveDuplicatePage::NextOk(){return doublePage->NextOk();}
void RemoveDuplicatePage::FinishOk(){return doublePage->FinishOk();}
//=========================================================================

SelectSubcatalogPage::SelectSubcatalogPage(QWidget *parent,FDocument* doc)
    : WizardPage(parent)
{
	
	QStringList listAction = projectActionList().split ('@');
	
    setTitle(listAction.at(2));
    setSubTitle(tr("Define search parameters"));
	
	ProWizard::DocumentUsing type = ((ProWizard *) parent)->GetDocumentUsing();
	if (type == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (type == ProWizard::ProjectEdit) 
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png")); 
	
	subcatalogPage = new SubCatalogImpl(this,doc,SubCatalogImpl::Wizard);

    QList<QPushButton *> allPButtons = subcatalogPage->findChildren<QPushButton *>();
    foreach (QPushButton *button, allPButtons)
    button->hide();
    QProgressBar* bar = subcatalogPage->findChild<QProgressBar *>();
    if(bar !=0)bar->hide();
    subcatalogPage->setSizeGripEnabled ( FALSE );
    subcatalogPage->setWindowFlags(Qt::SubWindow);
    subcatalogPage->setModal (false);
	
	QGridLayout *layout = new QGridLayout;
    layout->addWidget(subcatalogPage);
    setLayout(layout);
	
}
bool SelectSubcatalogPage::NextOk(){return subcatalogPage->NextOk();}
void SelectSubcatalogPage::FinishOk(){return subcatalogPage->FinishOk();}
//=========================================================================

MergeCatalogsPage::MergeCatalogsPage(QWidget *parent,FDocument* doc)
    : WizardPage(parent)
{
	QStringList listAction = projectActionList().split ('@');
	
    setTitle(listAction.at(3));
    setSubTitle(tr("Define the list of Catalog add-ins and Proximity limits for autoremoval of duplicates"));
	
	ProWizard::DocumentUsing type = ((ProWizard *) parent)->GetDocumentUsing();
	if (type == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (type == ProWizard::ProjectEdit) 
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png")); 

    mergePage      = new CatalBaseImpl(this,doc,CatalBaseImpl::MergeCatalog,CatalBaseImpl::Wizard);


    QList<QPushButton *> allPButtons = mergePage->findChildren<QPushButton *>();
    foreach (QPushButton *button, allPButtons)
    button->hide();
    QProgressBar* bar = mergePage->findChild<QProgressBar *>();
    if(bar !=0)bar->hide();
    mergePage->setSizeGripEnabled ( FALSE );
    mergePage->setWindowFlags(Qt::SubWindow);
    mergePage->setModal (false);
	
	QGridLayout *layout = new QGridLayout;
    layout->addWidget(mergePage);
    setLayout(layout);
}

bool MergeCatalogsPage::NextOk(){return mergePage->NextOk();}
void MergeCatalogsPage::FinishOk(){return mergePage->FinishOk();}
//=========================================================================

SubstituteCatalogsPage::SubstituteCatalogsPage(QWidget *parent,FDocument* doc)
    : WizardPage(parent)
{

	QStringList listAction = projectActionList().split ('@');
	
    setTitle(listAction.at(4));
    setSubTitle(tr("Define the list of Catalog add-ins"));
	
	ProWizard::DocumentUsing type = ((ProWizard *) parent)->GetDocumentUsing();
	if (type == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (type == ProWizard::ProjectEdit) 
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png")); 

	addPage        = new AppendImpl(this,doc,AppendImpl::Wizard);

    QList<QPushButton *> allPButtons = addPage->findChildren<QPushButton *>();
    foreach (QPushButton *button, allPButtons)
    button->hide();
    addPage->setSizeGripEnabled ( FALSE );
    addPage->setWindowFlags(Qt::SubWindow);
    addPage->setModal (false);
	
	QGridLayout *layout = new QGridLayout;
    layout->addWidget(addPage);
    setLayout(layout);
}

bool SubstituteCatalogsPage::NextOk(){return addPage->NextOk();}
void SubstituteCatalogsPage::FinishOk(){return addPage->FinishOk();}
//=========================================================================

ComparePage::ComparePage(QWidget *parent,FDocument* doc)
    : WizardPage(parent)
{

    setTitle(projectCompareTitle());
    setSubTitle(tr("Define Reference catalog along with the method of comparison and Proximity limits for autoremoval of duplicates"));
	
	ProWizard::DocumentUsing type = ((ProWizard *) parent)->GetDocumentUsing();
	if (type == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (type == ProWizard::ProjectEdit) 
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png")); 
    compPage      = new CatalBaseImpl(this,doc,CatalBaseImpl::CompareCatalog,CatalBaseImpl::Wizard);

    QList<QPushButton *> allPButtons = compPage->findChildren<QPushButton *>();
    foreach (QPushButton *button, allPButtons)
    button->hide();
    QProgressBar* bar = compPage->findChild<QProgressBar *>();
    if(bar !=0)bar->hide();
    compPage->setSizeGripEnabled ( FALSE );
    compPage->setWindowFlags(Qt::SubWindow);
    compPage->setModal (false);
	
	QGridLayout *layout = new QGridLayout;
    layout->addWidget(compPage);
    setLayout(layout);
}

bool ComparePage::NextOk(){return compPage->NextOk();}
void ComparePage::FinishOk(){return compPage->FinishOk();}
//=========================================================================

CheckRangePage::CheckRangePage(QWidget *parent,FDocument* doc)
    : WizardPage(parent)
{

    setTitle(projectRangeTitle());
    setSubTitle(tr("Define parameters to search the records outside Range limits"));
	
	ProWizard::DocumentUsing type = ((ProWizard *) parent)->GetDocumentUsing();
	if (type == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (type == ProWizard::ProjectEdit) 
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png")); 
	rangePage      = new RangeLimitImpl(this,doc,RangeLimitImpl::Wizard);

    QList<QPushButton *> allPButtons = rangePage->findChildren<QPushButton *>();
    foreach (QPushButton *button, allPButtons)
    button->hide();
    QProgressBar* bar = rangePage->findChild<QProgressBar *>();
    if(bar !=0)bar->hide();
    rangePage->setSizeGripEnabled ( FALSE );
    rangePage->setWindowFlags(Qt::SubWindow);
    rangePage->setModal (false);
	
	QGridLayout *layout = new QGridLayout;
    layout->addWidget(rangePage);
    setLayout(layout);
}

bool CheckRangePage::NextOk(){return rangePage->NextOk();}
void CheckRangePage::FinishOk(){return rangePage->FinishOk();}

