#ifndef PROJECT_WIZARD_H
#define PROJECT_WIZARD_H
#include <QWizard>
class ProfileImpl;
class QCheckBox;
class QLabel;
class QLineEdit;
class QRadioButton;
class FDocument;
class ProfileImpl;
class TransfMagnImpl;
class DoubleBaseImpl;
class SubCatalogImpl;
class CatalBaseImpl;
class AppendImpl;
class RangeLimitImpl;
class ProjectActionPage;
class ConvertMagnitudesPage;
class RemoveDuplicatePage;
class SelectSubcatalogPage;
class MergeCatalogsPage;
class SubstituteCatalogsPage;
class ComparePage;
class CheckRangePage;
class WizardPage;
class HelpWindow;
class ProWizard : public QWizard
{
    Q_OBJECT

public:
    enum DocumentUsing
    {
        ProjectNew,
        ProjectEdit
    };
    enum Pages
	{ 
		Project_Action,
		Current_ConvertMagnitudes,
		Current_RemoveDuplicate,
		Current_SelectSubcatalog,
		Current_MergeCatalogs,
        Current_SubstituteCatalogs,
		Current_CompareCatalogs,
		Current_CheckRanges, 
		Default_ConvertMagnitudes,
		Default_RemoveDuplicate,
		Default_SelectSubcatalog,
		Default_MergeCatalogs,
        Default_SubstituteCatalogs,
		Default_CompareCatalogs,
		Default_CheckRanges,
		Last_Page
	};
 
    ProWizard(QWidget *parent,FDocument* docDefault,FDocument* docCurrent,
		DocumentUsing documentUsing);
	DocumentUsing GetDocumentUsing(){return documentUsing;}	
		~ProWizard();
    int nextId () const;
private slots:
	void nextButton();
	void backButton();
    void showHelp();
private:
	 void Finish();
	ProjectActionPage*     pageProjectAction;
	
	ConvertMagnitudesPage* pageCurrentConvertMagnitudes;
	ConvertMagnitudesPage* pageDefaultConvertMagnitudes;
	
	RemoveDuplicatePage*    pageCurrentRemoveDuplicate;
	RemoveDuplicatePage*    pageDefaultRemoveDuplicate;
	
	SelectSubcatalogPage*   pageCurrentSelectSubcatalog;
	SelectSubcatalogPage*   pageDefaultSelectSubcatalog;
	
	MergeCatalogsPage*      pageCurrentMergeCatalogs;
	MergeCatalogsPage*      pageDefaultMergeCatalogs;

	SubstituteCatalogsPage* pageCurrentSubstituteCatalogs;
	SubstituteCatalogsPage* pageDefaultSubstituteCatalogs;

	ComparePage*            pageCurrentCompareCatalogs;
	ComparePage*            pageDefaultCompareCatalogs;
	
	CheckRangePage*         pageCurrentCheckRanges;
	CheckRangePage*         pageDefaultCheckRanges;

    FDocument* docDefault;
    FDocument* docCurrent;
    DocumentUsing documentUsing;
	HelpWindow* help;
	
};
class WizardPage : public QWizardPage
{
	
    Q_OBJECT

 public:
    WizardPage(QWidget * parent = 0){}
virtual     ~WizardPage(){}
virtual bool NextOk()  = 0;
virtual void FinishOk() = 0;

private:
};

class ProjectActionPage : public QWizardPage
{
    Q_OBJECT

public:
    ProjectActionPage(QWidget *parent,FDocument* docDefault,FDocument* docCurrent,
		ProWizard::DocumentUsing documentUsing);
	ProfileImpl* proPage;
//    int nextId() const;
private:
};

class ConvertMagnitudesPage : public WizardPage
{
    Q_OBJECT

public:
    ConvertMagnitudesPage(QWidget *parent,FDocument* docCurrent);

	TransfMagnImpl* magnitudePage;
	bool NextOk(); 
	void FinishOk();

private:
};

class RemoveDuplicatePage : public WizardPage
{
    Q_OBJECT

public:
    RemoveDuplicatePage(QWidget *parent,FDocument* docCurrent);

	DoubleBaseImpl* doublePage;
	bool NextOk();
	void FinishOk();
	
//    int nextId() const;

private:
};

class SelectSubcatalogPage : public WizardPage
{
    Q_OBJECT

public:
    SelectSubcatalogPage(QWidget *parent,FDocument* docCurrent);
	SubCatalogImpl* subcatalogPage;
	bool NextOk();
	void FinishOk();

//    int nextId() const;

private:
};

class MergeCatalogsPage : public WizardPage
{
    Q_OBJECT

public:
    MergeCatalogsPage(QWidget *parent,FDocument* docCurrent);
	CatalBaseImpl* mergePage;
	bool NextOk();
	void FinishOk();

//    int nextId() const;

private:
};

class SubstituteCatalogsPage : public WizardPage
{
    Q_OBJECT

public:
    SubstituteCatalogsPage(QWidget *parent,FDocument* docCurrent);
	AppendImpl* addPage;
	bool NextOk();
	void FinishOk();
private slots:

private:
};

class ComparePage : public WizardPage
{
    Q_OBJECT

public:
    ComparePage(QWidget *parent,FDocument* docCurrent);
	CatalBaseImpl* compPage;
	bool NextOk();
	void FinishOk();
private slots:

private:
};

class CheckRangePage : public WizardPage
{
    Q_OBJECT

public:
    CheckRangePage(QWidget *parent,FDocument* docCurrent);
	RangeLimitImpl* rangePage;
	bool NextOk();
	void FinishOk();
//    int nextId() const;
private slots:

private:
};
#endif
