#include <QtGui>
#include "setting.h"
#include "listproperty.h"
#include "document.h"
#include "helpwindow.h"
#include "default.h"
SettingImpl::~SettingImpl(){
    if(help !=0) {delete help; help =0;}
    delete list;
}
SettingImpl::SettingImpl(SettingDocument* doc,QWidget* parent)
    : QDialog(parent),doc(doc)
{
    setupUi(this);
    setFont( parent->font() );
    help =0;
    gridLayout->removeWidget(listWidget);
    listWidget->hide();
    list = new SettingProperty(doc,2,this, "Setting" );
    gridLayout->addWidget(list, 0, 0, 1, 1);
    modifed = false;
    connect(buttonOk, SIGNAL(clicked()), this, SLOT(accept()));
    connect(buttonCancel, SIGNAL(clicked()), this, SLOT(reject()));
    connect(buttonHelp, SIGNAL(clicked()), this, SLOT(Help()));

    QSettings settings("mitp.ru", "EditCatalogSettings");
    settings.beginGroup("/Geometry");
    QPoint Pos  = settings.value("/geometry/pos", QPoint(200, 200)).toPoint();
    QSize  Size = settings.value("/geometry/size", QSize(400, 300)).toSize();
    settings.endGroup();

        resize(Size);
        move(Pos);
}
void SettingImpl::accept() {
list->setCurrentIndex(QModelIndex());
QModelIndex index;
if (!list->CheckValidValue(index)) {
  QMessageBox::critical(0,"Setting parameters","Numbers must be sorted in ascending order",
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No | QMessageBox::Escape);
  list->OpenEditor(index);
return;
}
//list->PrintValues();
list->GetModel()->GetPropertyValueFromTreeView();
if (doc->isModified()) modifed = true; else modifed = false;
list->GetModel()->SetPropertyValueToResetValue();
doc->write();
done( Accepted );

}
void SettingImpl::RejectCloseImpl(){
//doc->write();
}
void SettingImpl::reject()
{
RejectCloseImpl();
done( Rejected );
}
void SettingImpl::closeEvent(QCloseEvent *event)
{
// like as reject
RejectCloseImpl();
 event->accept();
}
void SettingImpl::Help(){
    QString home = searchHelpPath() + "/edcat-settings.html";
    if(help !=0) {delete help; help =0;}
    help = new HelpWindow(home,this);
    if ( QApplication::desktop()->width() > 400
         && QApplication::desktop()->height() > 500 )
         help->show();
    else
        help->showMaximized();

}
//-----------------------------------------------------------------------------------------
SettingProperty::~SettingProperty(){
ipList.clear();

}
SettingProperty::SettingProperty(SettingDocument* doc,int numCol,QWidget* parent, const char *)
          : doc(doc) {
    QFont f = parent->font();
    setFont( f );
    parentWindow = (SettingImpl*)parent;
    setAlternatingRowColors (TRUE);
    setEditTriggers (QTreeView::SelectedClicked );
     // setEditTriggers (QTreeView::DoubleClicked );                      
    edIndex =  QModelIndex();
    indexOpenEditor = QModelIndex();
    model = new TreePropertyModel(numCol,this);
    setModel(model);

    delegate = new TreePropertyDelegate;
    setItemDelegate(delegate);
//-------------------------------------------------------------------------------------------
    PropertyValue *p  = 0;  QString title = QString();
   //PropertyValue *p1  = 0;
    title = "Limits of protocol number lines:   ";
    model->SetItem(title);

    p  = doc->GetWorkSpaceSize(); ipList.append(p); 
    model->SetItem(p);
    p  = doc->GetMaxErrorLines();ipList.append(p); 
    model->SetItem(p);
    p  = doc->GetMaxCorrectionLines();ipList.append(p);
    model->SetItem(p);
/*
    title = "Limits of range hipocenter:   ";
    model->SetItem(title);

    p  = doc->GetMinLatitude(); ipList.append(p); 
    p1 = doc->GetMaxLatitude(); ipList.append(p1);
    model->SetItem("Latitude",p,p1);

    p  = doc->GetMinLongitude(); ipList.append(p); 
    p1 = doc->GetMaxLongitude(); ipList.append(p1);
    model->SetItem("Longitude",p,p1);

    p  = doc->GetMinDepth(); ipList.append(p); 
    p1 = doc->GetMaxDepth(); ipList.append(p1);
    model->SetItem("Depth",p,p1);

    title = "Limits of range magnitudes:   ";
    model->SetItem(title);

    p  = doc->GetMinMb(); ipList.append(p); 
    p1 = doc->GetMaxMb(); ipList.append(p1);
    model->SetItem("Mb",p,p1);

    p  = doc->GetMinMs(); ipList.append(p); 
    p1 = doc->GetMaxMs(); ipList.append(p1);
    model->SetItem("Ms",p,p1);

    p  = doc->GetMinMl(); ipList.append(p); 
    p1 = doc->GetMaxMl(); ipList.append(p1);
    model->SetItem("Ml",p,p1);

    p  = doc->GetMinMp(); ipList.append(p); 
    p1 = doc->GetMaxMp(); ipList.append(p1);
    model->SetItem("Mp",p,p1);
*/
//---------------------------------------------------------------------------------------------
/*
// Checking mindata <= max data does'not work; It's need to do it 
    // Qt::ISODate :>   YYYY-MM-DDTHH:MM:SS  yyyy:MM:dd hh:mm:ss
//    p  = new   PropertyDateTimeValue(0,"2001/01/01 00:00:00","min");
//    p1 = new   PropertyDateTimeValue(1,"2005/01/01 00:00:00","max");
//    model->SetItem("DateTime",p,p1);

    p  = new   PropertyFloatValue(1,"45","Latitude",-90,90,2);ipList.append(p);
    p1 = new   PropertyFloatValue(1,"90","Longitude",-180,180,2);ipList.append(p1);
    PropertyValue *p2 = new   PropertyFloatValue(1,"30","Radius",-90,90,2);ipList.append(p2);
    model->SetItem("Circle",p,p1,p2);

//---------------------------------------------------------------------------------------
*/
    resizeColumnToContents(0);resizeColumnToContents(1);

    connect( this, SIGNAL( clicked(QModelIndex) ),
	     this, SLOT( clickedRowCol(QModelIndex ) ) );
    connect(parentWindow->GetButtonDefault(), SIGNAL(clicked()), this, SLOT(Default()));
    connect(parentWindow->GetButtonReset(), SIGNAL(clicked()), this, SLOT(Reset()));
    connect(parentWindow->GetButtonInitial(), SIGNAL(clicked()), this, SLOT(Initial()));
}

void  SettingProperty::OpenEditor(QModelIndex &index){
   QModelIndex parent = index.parent();
   if (indexOpenEditor != QModelIndex() && index != indexOpenEditor) {
       clearSelection ();
       QModelIndex temp = model->index( index.row(),1, QModelIndex());
       if (indexOpenEditor == temp) return;
       closePersistentEditor(indexOpenEditor);
      QModelIndex sel = model->index(index.row(), 0, QModelIndex());
       setCurrentIndex (sel);
       indexOpenEditor = temp;
       openPersistentEditor(indexOpenEditor);
   }
}
void SettingProperty::clickedRowCol(QModelIndex index){
   QModelIndex parent = index.parent();
if (index.column() == 1) {
  if(parent ==  QModelIndex()) {

    QModelIndex temp = model->index(index.row(), 1, QModelIndex());
    if (indexOpenEditor == temp) return;
    if (indexOpenEditor != QModelIndex()) closePersistentEditor(indexOpenEditor);
     indexOpenEditor = temp;
     openPersistentEditor(indexOpenEditor);
   }
   else {

    QModelIndex temp = model->index( index.row(),1,parent);
    if (indexOpenEditor == temp) return;
    if (indexOpenEditor != QModelIndex()) closePersistentEditor(indexOpenEditor);
    indexOpenEditor = temp;
    openPersistentEditor(indexOpenEditor);
   }
}
if (index.column() == 0){
 if(parent ==  QModelIndex()) {

   QModelIndex temp = model->index(index.row(), 1, QModelIndex());
//   qWarning("no parent open editor'%s' close editor '%s' ",model->data(temp).toString().toAscii().data(),model->data(indexOpenEditor).toString().toAscii().data());
   if (indexOpenEditor == temp) return;
 
   if (indexOpenEditor != QModelIndex()) {
          QString closeData = model->data(indexOpenEditor).toString();
          
          closePersistentEditor(indexOpenEditor);
         QModelIndex tmp = model->index(indexOpenEditor.row(), 1, QModelIndex());
//qWarning("col 0 indopened '%s'",model->data(indexOpenEditor).toString().toAscii().data());
   model->setData(indexOpenEditor,closeData);

    }
   indexOpenEditor = temp;
   openPersistentEditor(indexOpenEditor);
update ();
   }

   else {

    QModelIndex temp = model->index( index.row(),1,parent);
//qWarning(" yes parent '%s'",model->data(temp).toString().toAscii().data());
    if (indexOpenEditor == temp) return;
    if (indexOpenEditor != QModelIndex()) closePersistentEditor(indexOpenEditor);
    indexOpenEditor = temp;
    openPersistentEditor(indexOpenEditor);
   }

}
} 
void SettingProperty::Default(){
model->Default();

}
void SettingProperty::Reset(){
model->Reset();

}
void SettingProperty::Initial(){
model->Initial();

}
bool SettingProperty::CheckValidValue(QModelIndex &index){
TreePropertyModel* model = GetModel();
return model->CheckValidValue(index);

}
void SettingProperty::drawBranches(QPainter *painter, const QRect &rect, const QModelIndex &index) const
{
    QStyleOptionViewItem opt = viewOptions();
    QStyleOptionViewItem option = opt;

      if (selectionModel()->isSelected(index))
        painter->fillRect(option.rect, option.palette.brush(QPalette::Highlight));

    if (model->hasChildren(index)) {
        static const int size = 9;
        opt.state |= QStyle::State_Children;
        opt.rect.setRect(rect.width() - (indentation() + size) / 2,
                         rect.y() + (rect.height() - size) / 2, size, size);
        if (isExpanded(index))
            opt.state |= QStyle::State_Open;
        painter->fillRect(opt.rect, Qt::white);
        style()->drawPrimitive(QStyle::PE_IndicatorBranch, &opt, painter, this);
    }
    painter->drawLine(rect.x(), rect.bottom(), rect.right(), rect.bottom());
}
void  SettingProperty::GetDataFromTreeView(){
qWarning("------- %d",model->rowCount());
QString value0 = model->GetItemValue(0,0); //row 0 child 0
QString value1 = model->GetItemValue(0,1); //row 0 child 1
qWarning("val1 = '%s'  val2 '%s",value0.toAscii().data(),value1.toAscii().data());

value0 = model->GetItemValue(1,0);//row 1 child 0
value1 = model->GetItemValue(1,1);//row 1 child 1
qWarning("val1 = '%s'  val2 '%s",value0.toAscii().data(),value1.toAscii().data());

value0 = model->GetItemValue(2,0);
value1 = model->GetItemValue(2,1);
qWarning("val1 = '%s'  val2 '%s",value0.toAscii().data(),value1.toAscii().data());

}

void  SettingProperty::PrintValues(){
foreach(PropertyValue * p,ipList) {
qWarning("%s :> %s",p->GetName().toAscii().data(),p->GetValue().toAscii().data());
}

}
