#ifndef _CREATE_SUBCATALOG_H
#define _CREATE_SUBCATALOG_H
#include "ui_selection.h"
#include <fstream>
#include <map>
#include <vector>
#include "catalog.h"
#include "document.h"
double distance(double la1, double la2, double lo1, double lo2);
using namespace std;
class QWidget;
class QTextEdit;
class QCheckBox;
class FDocument;
class QTimerEvent;
class CatTable;
class PolygonImpl;
class RangeLimitImpl;
class LineDocProperty;
class HelpWindow;
class SampleFunc
{
public:
  SampleFunc();
  ~SampleFunc();

  void  SetRectangle (float latmin,float latmax,float lonmin,float lonmax);
  bool  &GetLon180()       {return Lon180;       }
  double &GetCircleLat()    {return CircleLat;    }
  double &GetCircleLon()    {return CircleLon;    }
  double &GetCircleRadius() {return  CircleRadius;   }
  int   &GetVertices()     {return  Vertices;    }
  QPolygonF &GetPolygon()  {return polygon;}
  void  SetPolygon(QPolygonF &polygon);
  bool  &GetPolygonLon180(){return PolygonLon180;}
  void  SetDepth(float valmin,float valmax);
  void  SetMb(float valmin,float valmax); 
  void  SetMs(float valmin,float valmax);
  void  SetMl(float valmin,float valmax);
  void  SetMp(float valmin,float valmax);
  void  SetMfunct(float valmin,float valmax);
  void  SetIntensity(int valmin,int valmax);
  void  SetRecord(RECORD &record);
  void  SetValFunctPriority(vector<int> &priority,vector<QString> &valFunc);
  float Value() {return value;}
  void  Rectangle();
  void  Circle();
  void  Polygon();
  void  Depth();
  void  Mb();
  void  Ms();
  void  Ml();
  void  Mp();
  void  Intensity();
  void  MagnitudeValue();
  void  MagnitudeMaximum();
  void  MagnitudeMinimum();
  void  MagnitudeAverage();
  bool Inside(QPointF irov, const QPolygonF &foliplot);
private:
  float RectLatMin,RectLatMax,RectLonMin,RectLonMax;
  bool Lon180;
  bool PolygonLon180;
  double CircleLat,CircleLon,CircleRadius;
  int Vertices;
  vector<int> priority;
  vector<float> a,b,c,d;
  vector<float> magnitudeRecord;
  float mFunct(int i,float m){if (m < d[i] + eps) return c[i];  else return  m*a[i] + b[i];}
  float DepthMin,DepthMax;
  float MbMin,MbMax,MsMin,MsMax,MlMin,MlMax,MpMin,MpMax;
  float MfunctMin,MfunctMax;
  short IntensityMin,IntensityMax;
  bool value;
  float lat,lon,eps;
  float dep,mb,ms,ml,mp;
  RECORD record;
  QPolygonF polygon;
  
};
class SubCatalogImpl : public QDialog, private Ui::SubCatalogBase
{
    typedef vector<PropertyValue* > PropValVector;
    typedef vector<PropertyValue* >::iterator PropValIterator;

    typedef multimap<double,int> DoubleIntMultiMap;
    typedef DoubleIntMultiMap::iterator ItrMultiMap;
    typedef void (SampleFunc::*PDF)();
    Q_OBJECT
public:
    enum TypeUsing
    {
        Tool,
        Wizard,
        Execute
    };

    TypeUsing GetTypeUsing() {return type;}
    SubCatalogImpl(QWidget *parent);
    SubCatalogImpl(QWidget *parent,FDocument* dc,TypeUsing type); 
    ~SubCatalogImpl();
    DoubleIntMultiMap* indexCat;
    void DuplicateProgress(int progress);
    bool GetSampleFunc();
    QSpinBox* GetSpinBoxPolygon(){return spinBoxPolygon;}
    bool NextOk();
    void FinishOk();
    bool execute();
    bool applay();
protected:
    void closeEvent(QCloseEvent *event); 
    void timerEvent(QTimerEvent *e);
public slots:
protected slots:
    void accept(); 
    void reject();
    void buttonDefault();
    void buttonHelp();
    void start();
    void vertices();
    void reset();
    void project();
    void defaults();
    void ProgressBarChanged(int value);
private:
    void InitLineDocProperty();
    void GetDialogFromDocument();
    void GetDocumentFromDialog();
    void GetDateTimeForWizard();
    void GetDateTimeForTool();
    void SetDateTimeNumberRecordsToDialog();
    bool CheckQLineEdit(QLineEdit* linmin,float min,float max,QString name);
    bool CheckQLineEditWithRect180(QLineEdit* linmin,float min,float max,QString name);
    void SetTextQLineEdit(QLineEdit* lin,bool bold,QString color);
    void SetTextQSpinBox(QSpinBox* lin,bool bold,QString color);
    bool CheckQSpinBox(QSpinBox* linmin,int min,int max,QString name);
    void SubCatalogProccessing();
    PolygonImpl* polygon;
    CatTable* parentWindow;
    fstream* in;
    FDocument* doc;
    ItrMultiMap position;
    int timer;
    QList<QCheckBox *> checkBoxes;
    QList<LineDocProperty* > propDocLine;
    QString  startTime;
    QString  endTime;
    int catSize;
    int SpinBoxNumberMin;
    int SpinBoxNumberMax;
    SampleFunc funct;
    // void  (FindDupFunc::*ptrFindFunction) ();
    vector <PDF> ptrFunct;
    float eps;
    TypeUsing type;
    QList<QComboBox *> comboBoxes;
PropertyTransformFunction* first;
PropertyTransformFunction* second;
PropertyTransformFunction* third;
PropertyTransformFunction* fourth;
QList<PropertyTransformFunction *> transFunc;
PropValVector EditLineValues;
QList<PropertyValue* > values;
bool firstStart;
    QTextEdit* texted;
    QString message;

    QString messageDateTime;
    QString messageNumber;
    QString messageRectangle;
    QString messageCircle;
    QString messagePoligon;
    QString messageDepth;
    QString messageInterval;
    QString messageFunction;
    QString messageIntensity;
    HelpWindow* help;
friend class RangeLimitImpl;
};

#endif
