#include <QtGui>
#include "mainwindow.h"
#include "tableproperty.h"
#include "viewcatalogimpl.h"
#include "undo.h"
#include "default.h"
#include "editline.h"

PropertyInt::PropertyInt(QModelIndex ind,const QString &initStr,int min,int max, bool s )
     : IProperty(ind,initStr),
       minimum(min),maximum(max),signedValue(s)  {}
//-------------------------------------------------------------------
PropertyDouble::PropertyDouble(QModelIndex ind,const QString &initStr,double min,double max,int dec)
     : IProperty(ind,initStr),
       minimum(min),maximum(max),decimals(dec)  {}
//-------------------------------------------------------------------
PropertyString::PropertyString(QModelIndex ind,const QString &initStr)
     : IProperty(ind,initStr) {}
//-------------------------------------------------------------------
PropertyNotEdit::PropertyNotEdit(QModelIndex ind,const QString &initStr)
     : IProperty(ind,initStr) {}
//-------------------------------------------------------------------
 TableLineEditor::~TableLineEditor(){

}
 TableLineEditor::TableLineEditor(IProperty* prop,QString val,Kind kind,QWidget *parent)
    : QLineEdit(val,parent),prop(prop),editedValue(val),kind(kind){

if (kind == Property_Double) {
 PropertyDouble * pp =  (PropertyDouble *)prop;
 setValidator(new QDoubleValidator (pp->minimum,pp->maximum,pp->decimals, this));
}
if (kind == Property_Int) {
  PropertyInt * pp = (PropertyInt *)prop;
  setValidator(new QIntValidator(pp->minimum,pp->maximum, this));
 }

       installEventFilter(this);
       connect( this, SIGNAL( textChanged( const QString & ) ),
		      this, SLOT( setValue(const QString &) ) );
       connect( this, SIGNAL( editingFinished () ),
                      this, SLOT( editFinish() ) );
       connect( this, SIGNAL( returnPressed () ),
                      this, SLOT( returnPressed() ) );

}
void TableLineEditor::escAction(){
setText(editedValue);
CatTable::tSelf->CloseOpenedTableLineEditor();
}
void TableLineEditor::returnPressed(){
editFinish();
CatTable::tSelf->CloseOpenedTableLineEditor();
}
void TableLineEditor::editFinish() {
  int pos = 0; QString temp = text();
  QString edVal = QString();
  if ( validator()->validate(temp,pos)  != QValidator::Acceptable)
       {setText(editedValue); edVal = editedValue;}
  else {
     if (kind == Property_Double){
       bool ok;
       QString tmp = QString("%1").arg(temp.toFloat(&ok));
        if(ok){ setText(tmp); edVal = tmp;}else{ setText(editedValue); edVal = editedValue;}
     }
     if (kind == Property_Int){
       bool ok;
       QString tmp = QString("%1").arg(temp.toInt(&ok));
        if(ok) { setText(tmp); edVal = tmp;} else {setText(editedValue);edVal = editedValue;}

     }
   }
  editedValue = edVal;
  if ( validator()->validate(editedValue,pos)  != QValidator::Acceptable)
   prop->SetInvalid(true); else prop->SetInvalid(false);
}

void TableLineEditor::setValue( const QString  &v )
{
        QString temp = v;
        blockSignals( TRUE );
        int oldCursorPos;
        oldCursorPos = cursorPosition();
        int j = 0;

    if (kind  == Property_Double){
       const QValidator * vld = validator ();
       if (!(temp[0] == '-' && temp.length() == 1)){
          if ( vld->validate( temp , j )  == QValidator::Intermediate) {
             double min =    ((QDoubleValidator*) vld)->bottom ();
             double max =    ((QDoubleValidator*) vld)->top ();
             if (min > 0 ) {
               if(temp.toFloat() >= ((QDoubleValidator*) vld)->bottom () || temp.toFloat() <= 0)
                if ( vld->validate( temp , j )  != QValidator::Acceptable) temp.remove( temp.length()-1, 1);
             }
             if (max < 0) {
                if(temp.toFloat() <= ((QDoubleValidator*) vld)->top () || temp.toFloat() >=0)
                  if ( vld->validate( temp , j )  != QValidator::Acceptable) temp.remove( temp.length()-1, 1);
     
            }
            if (min <= 0 && max >= 0) if ( vld->validate( temp , j )  != QValidator::Acceptable) 
              temp.remove( temp.length()-1, 1);
      }
    }
  }

   if (kind  == Property_Int){
      const QValidator * vld = validator ();
      if (!(temp[0] == '-' && temp.length() == 1)){
          if ( vld->validate( temp , j )  == QValidator::Intermediate) {
             int min =    ((QIntValidator*) vld)->bottom ();
             int max =    ((QIntValidator*) vld)->top ();
             if (min > 0 ) {
               if(temp.toFloat() >= ((QIntValidator*) vld)->bottom () || temp.toFloat() <= 0)
                if ( vld->validate( temp , j )  != QValidator::Acceptable) temp.remove( temp.length()-1, 1);
             }
             if (max < 0) {
                if(temp.toFloat() <= ((QIntValidator*) vld)->top () || temp.toFloat() >=0)
                  if ( vld->validate( temp , j )  != QValidator::Acceptable) temp.remove( temp.length()-1, 1);
     
            }
            if (min <= 0 && max >= 0) if ( vld->validate( temp , j )  != QValidator::Acceptable) 
              temp.remove( temp.length()-1, 1);
      }
    }
  }

        setText(temp);
        if ( oldCursorPos < (int)text().length() )
            setCursorPosition( oldCursorPos );
        blockSignals( FALSE );
}


//-------------------------------------------------------------------
PropertyModel::~PropertyModel()  {

if(!ipList.empty()) qDeleteAll(ipList);  ipList.clear();
}
PropertyModel::PropertyModel( int rows, int columns,QObject * parent )
    : QStandardItemModel(rows, columns,parent),NumRows(rows),NumCols(columns)
{
parentTable =   (QTableView*) parent;
SetHorizontalHeader();     
}
PropertyModel::PropertyModel(QObject * parent, int rows, int columns )
    : QStandardItemModel(rows, columns,parent),NumRows(rows),NumCols(columns)
{
parentTable =   (QTableView*) parent;
setHeaderData(0, Qt::Horizontal, headerCatalogLatitude());
setHeaderData(1, Qt::Horizontal, headerCatalogLongitude());
setHeaderData(2, Qt::Horizontal, headerCatalogComment());  
}
void PropertyModel::SetHorizontalHeader(){

//setHeaderData(numberCatalogRecord()    , Qt::Horizontal, headerCatalogRecord());
setHeaderData(numberCatalogYear()      , Qt::Horizontal, headerCatalogYear());
setHeaderData(numberCatalogMonth()     , Qt::Horizontal, headerCatalogMonth());
setHeaderData(numberCatalogDay()       , Qt::Horizontal, headerCatalogDay() );  
setHeaderData(numberCatalogHour()      , Qt::Horizontal, headerCatalogHour());
setHeaderData(numberCatalogMinute()    , Qt::Horizontal, headerCatalogMinute() );
setHeaderData(numberCatalogSecond()    , Qt::Horizontal, headerCatalogSecond());
setHeaderData(numberCatalogLatitude()  , Qt::Horizontal, headerCatalogLatitude() );
setHeaderData(numberCatalogLongitude() , Qt::Horizontal, headerCatalogLongitude());
setHeaderData(numberCatalogDepth()     , Qt::Horizontal, headerCatalogDepth());
setHeaderData(numberCatalogMb()        , Qt::Horizontal, headerCatalogMb());
setHeaderData(numberCatalogMs()        , Qt::Horizontal, headerCatalogMs());
setHeaderData(numberCatalogMl()        , Qt::Horizontal, headerCatalogMl() );
setHeaderData(numberCatalogMp()        , Qt::Horizontal, headerCatalogMp() );
setHeaderData(numberCatalogIntensity() , Qt::Horizontal, headerCatalogIntensity() );
setHeaderData(numberCatalogComment()   , Qt::Horizontal, headerCatalogComment() );

}

void PropertyModel::setText(QModelIndex index, QString text){
setData(index, QVariant(text),Qt::DisplayRole  );
//qWarning("%d %d",index.row(),index.column ());
}
void PropertyModel::SetValuePropertyRow(int i,int shift,QList<UndoEdit *>& list)
{
//  setHeaderData(i,Qt::Vertical, QString("ss"));
  QModelIndex ind;
  int k = i*(NumCols+1);
  int m = shift*(NumCols+1);
  UndoEdit* e = 0;
  IProperty * p = 0;

  ind = index(i, 0, QModelIndex());
  e = list.at(m);
  setHeaderData(i,Qt::Vertical, QString(e->GetRedo()));
	p = new PropertyNotEdit(QModelIndex(),e->GetInitString());
  ipList.insert(k,p); k++; m++;

//----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogYear(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyInt(ind,e->GetInitString(),minCatalogYear(),maxCatalogYear(),signedCatalogYear());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
  ipList.insert(k,p); k++;   m++;

//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogMonth(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyInt(ind,e->GetInitString(),minCatalogMonth(),maxCatalogMonth(),signedCatalogMonth());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
	ipList.insert(k,p); k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogDay(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyInt(ind,e->GetInitString(),minCatalogDay(),maxCatalogDay(),signedCatalogDay());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
  ipList.insert(k,p);  k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogHour(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyInt(ind,e->GetInitString(),minCatalogHour(),maxCatalogHour(),signedCatalogHour());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
  ipList.insert(k,p); k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogMinute(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyInt(ind,e->GetInitString(),minCatalogMinute(),maxCatalogMinute(),signedCatalogMinute());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
  ipList.insert(k,p); k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogSecond(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyDouble(ind,e->GetInitString() , minCatalogSecond(), maxCatalogSecond(),decimalCatalogSecond());
   p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
  ipList.insert(k,p); k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogLatitude(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyDouble(ind,e->GetInitString() , minCatalogLatitude(), maxCatalogLatitude(),decimalCatalogLatitude());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
  ipList.insert(k,p);  k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogLongitude(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyDouble(ind,e->GetInitString() , minCatalogLongitude(), maxCatalogLongitude(),decimalCatalogLongitude());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
  ipList.insert(k,p);   k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogDepth(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyDouble(ind,e->GetInitString() , minCatalogDepth(), maxCatalogDepth(),decimalCatalogDepth());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
	ipList.insert(k,p);  k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogMb(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyDouble(ind,e->GetInitString(), minCatalogMb(), maxCatalogMb(),decimalCatalogMb());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
  ipList.insert(k,p);   k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogMs(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyDouble(ind,e->GetInitString() , minCatalogMs(), maxCatalogMs(),decimalCatalogMs());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
  ipList.insert(k,p);  k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogMl(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyDouble(ind,e->GetInitString() , minCatalogMl(), maxCatalogMl(),decimalCatalogMl());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
  ipList.insert(k,p);  k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogMp(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyDouble(ind,e->GetInitString(), minCatalogMp(), maxCatalogMp(),decimalCatalogMp());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
  ipList.insert(k,p); k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogIntensity(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyInt(ind,e->GetInitString(),minCatalogIntensity(),maxCatalogIntensity(),signedCatalogIntensity());
  p->SetInvalid(e->GetInvalid());
  p->SetChanged(e->GetChanged());
  ipList.insert(k,p); k++;  m++;
//-----------------------------------------------------------------------------------------
  ind = index(i, numberCatalogComment(), QModelIndex());
  e = list.at(m);
  setData(ind, QVariant(e->GetRedo()) );
  p = new   PropertyString(ind,e->GetInitString());
  ipList.insert(k,p);

}
void PropertyModel::SetEmptyValuePropertyRow(int i){
//  setHeaderData(i,Qt::Vertical, QString());
  QModelIndex ind = QModelIndex ();
  QModelIndex ind1 = QModelIndex ();
  int k = i*(NumCols+1);
  IProperty * p = 0;
 //-------------------------------------------------------
  ind = index(i, 0, QModelIndex());
  setHeaderData(i,Qt::Vertical, QString());
	p = new PropertyNotEdit(QModelIndex (),"");
  ipList.insert(k,p); k++; 

//--------------------------------------------------
/*
  ind = index(i, 0, QModelIndex());
  ind1 = index(i+1, 0, QModelIndex()); 
  setData(ind, QVariant(""),Qt::DisplayRole  );
	p = new   PropertyInt(ind,"",minCatalogRecord(),maxCatalogRecord(),signedCatalogRecord());
  ipList.insert(k,p); k++;
*/

  ind = index(i, numberCatalogYear(), QModelIndex());
  ind1 = index(i+1, numberCatalogYear(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyInt(ind,"-1",minCatalogYear(),maxCatalogYear(),signedCatalogYear());
  IProperty *p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
  ipList.insert(k,p); k++; 

  ind = index(i, numberCatalogMonth(), QModelIndex());
  ind1 = index(i+1, numberCatalogMonth(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyInt(ind,"-1",minCatalogMonth(),maxCatalogMonth(),signedCatalogMonth());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
	ipList.insert(k,p); k++;

  ind = index(i, numberCatalogDay(), QModelIndex());
  ind1 = index(i+1, numberCatalogDay(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyInt(ind,"-1",minCatalogDay(),maxCatalogDay(),signedCatalogDay());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
  ipList.insert(k,p);  k++;

  ind = index(i, numberCatalogHour(), QModelIndex());
  ind1 = index(i+1, numberCatalogHour(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyInt(ind,"-1",minCatalogHour(),maxCatalogHour(),signedCatalogHour());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
  ipList.insert(k,p); k++;
                      
  ind = index(i, numberCatalogMinute(), QModelIndex());
  ind1 = index(i+1, numberCatalogMinute(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyInt(ind,"-1",minCatalogMinute(),maxCatalogMinute(),signedCatalogMinute());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
  ipList.insert(k,p); k++;

  ind = index(i, numberCatalogSecond(), QModelIndex());
  ind1 = index(i+1, numberCatalogSecond(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyDouble(ind,"-1" , minCatalogSecond(), maxCatalogSecond(),decimalCatalogSecond());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
  ipList.insert(k,p); k++;

  ind = index(i, numberCatalogLatitude(), QModelIndex());
  ind1 = index(i+1,numberCatalogLatitude(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyDouble(ind,"100" , minCatalogLatitude(), maxCatalogLatitude(),decimalCatalogLatitude());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
  ipList.insert(k,p);  k++;

  ind = index(i, numberCatalogLongitude(), QModelIndex());
  ind1 = index(i+1, numberCatalogLongitude(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyDouble(ind,"190" , minCatalogLongitude(), maxCatalogLongitude(),decimalCatalogLongitude());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
  ipList.insert(k,p);   k++;

  ind = index(i, numberCatalogDepth(), QModelIndex());
  ind1 = index(i+1, numberCatalogDepth(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyDouble(ind,"-1" , minCatalogDepth(), maxCatalogDepth(),decimalCatalogDepth());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
	ipList.insert(k,p);  k++;

  ind = index(i, numberCatalogMb(), QModelIndex());
  ind1 = index(i+1, numberCatalogMb(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyDouble(ind,"-1" , minCatalogMb(), maxCatalogMb(),decimalCatalogMb());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
  ipList.insert(k,p);   k++;


  ind = index(i, numberCatalogMs(), QModelIndex());
  ind1 = index(i+1, numberCatalogMs(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyDouble(ind,"-1" , minCatalogMs(), maxCatalogMs(),decimalCatalogMs());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
  ipList.insert(k,p);  k++;

  ind = index(i, numberCatalogMl(), QModelIndex());
  ind1 = index(i+1, numberCatalogMl(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyDouble(ind,"-1" , minCatalogMl(), maxCatalogMl(),decimalCatalogMl());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
  ipList.insert(k,p);  k++;


  ind = index(i, numberCatalogMp(), QModelIndex());
  ind1 = index(i+1, numberCatalogMp(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyDouble(ind,"-1" , minCatalogMp(), maxCatalogMp(),decimalCatalogMp());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
  ipList.insert(k,p); k++;


  ind = index(i, numberCatalogIntensity(), QModelIndex());
  ind1 = index(i+1, numberCatalogIntensity(), QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyInt(ind,"-1",minCatalogIntensity(),maxCatalogIntensity(),signedCatalogIntensity());
  p1 =  findProperty(ind) ;
  p->SetInvalid(p1->GetInvalid());
  p->SetChanged(true);
  ipList.insert(k,p); k++;

  ind = index(i, numberCatalogComment(), QModelIndex());
  setData(ind, QVariant("") );
  p = new   PropertyString(ind,"");
  ipList.insert(k,p);

}
void PropertyModel::SetNotEditRow(int i,int inpos,QString& temp){
  setHeaderData(i,Qt::Vertical, QString("%1").arg(inpos));
  QModelIndex ind  = QModelIndex ();
  IProperty * p = 0;
/*
  QString apos = QString("%1").arg(inpos);
  ind = index(i, 0, QModelIndex());
  setData(ind, QVariant(inpos),Qt::DisplayRole  );
  IProperty * p = new   PropertyNotEdit(ind,"");
	ipList.append(p);
*/
	QString yr = temp.section(' ',numberCatalogYear(),numberCatalogYear());
  ind = index(i, numberCatalogYear(), QModelIndex());
  setData(ind,yr,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);
 
	QString mo = temp.section(' ',numberCatalogMonth(),numberCatalogMonth());
  ind = index(i, numberCatalogMonth(), QModelIndex());
  setData(ind,mo,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

	QString da = temp.section(' ',numberCatalogDay(),numberCatalogDay());
  ind = index(i, numberCatalogDay(), QModelIndex());
  setData(ind,da,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

	QString ho = temp.section(' ',numberCatalogHour(),numberCatalogHour());
  ind = index(i, numberCatalogHour(), QModelIndex());
  setData(ind,ho,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

	QString mi = temp.section(' ',numberCatalogMinute(),numberCatalogMinute());
  ind = index(i, numberCatalogMinute(), QModelIndex());
  setData(ind,mi,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

	QString se = temp.section(' ',numberCatalogSecond(),numberCatalogSecond());
  ind = index(i, numberCatalogSecond(), QModelIndex());
  setData(ind, se,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

	QString fi = temp.section(' ',numberCatalogLatitude(),numberCatalogLatitude());
  ind = index(i, numberCatalogLatitude(), QModelIndex());
  setData(ind,fi,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

	QString la = temp.section(' ',numberCatalogLongitude(),numberCatalogLongitude());
  ind = index(i, numberCatalogLongitude(), QModelIndex());
  setData(ind,la,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

	QString h = temp.section(' ',numberCatalogDepth(),numberCatalogDepth());
  ind = index(i, numberCatalogDepth(), QModelIndex());
  setData(ind,h,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

	QString m1 = temp.section(' ',numberCatalogMb(),numberCatalogMb());
  ind = index(i, numberCatalogMb(), QModelIndex());
  setData(ind, m1,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

	QString m2 = temp.section(' ',numberCatalogMs(),numberCatalogMs());
  ind = index(i, numberCatalogMs(), QModelIndex());
  setData(ind,m2,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

	QString m3 = temp.section(' ',numberCatalogMl(),numberCatalogMl());
  ind = index(i, numberCatalogMl(), QModelIndex());
  setData(ind,m3,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

	QString m4 = temp.section(' ',numberCatalogMp(),numberCatalogMp());
  ind = index(i, numberCatalogMp(), QModelIndex());
  setData(ind,m4,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

	QString inten = temp.section(' ',numberCatalogIntensity(),numberCatalogIntensity());
  ind = index(i, numberCatalogIntensity(), QModelIndex());
  setData(ind, inten,Qt::DisplayRole );
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

  ind = index(i,numberCatalogComment(), QModelIndex());
  setData(ind, "");
  p = new   PropertyNotEdit(ind,"");
	ipList.append(p);

}
void PropertyModel::SetRow(int i,int inpos,QString& temp){

 // setHeaderData(i,Qt::Vertical, QString("%1").arg(inpos));
  QModelIndex ind =   QModelIndex();
  IProperty * p = 0;
  QString apos = QString("%1").arg(inpos);
 // ind = index(i, 0, QModelIndex());
  setHeaderData(i,Qt::Vertical, QVariant(inpos));
	p = new   PropertyNotEdit(QModelIndex(),apos);
	ipList.append(p); 

	QString yr = temp.section(' ',numberCatalogYear(),numberCatalogYear());
  ind = index(i, numberCatalogYear(), QModelIndex());
  setData(ind, QVariant(yr));
  p = new   PropertyInt(ind,yr,minCatalogYear(),maxCatalogYear(),signedCatalogYear());
	ipList.append(p);

	QString mo = temp.section(' ',numberCatalogMonth(),numberCatalogMonth());
  ind = index(i, numberCatalogMonth(), QModelIndex());
  setData(ind, QVariant(mo) );
  p = new   PropertyInt(ind,mo,minCatalogMonth(),maxCatalogMonth(),signedCatalogMonth());
	ipList.append(p);

	
	QString da = temp.section(' ',numberCatalogDay(),numberCatalogDay());
  ind = index(i, numberCatalogDay(), QModelIndex());
  setData(ind, QVariant(da) );
  p = new   PropertyInt(ind,da,minCatalogDay(),maxCatalogDay(),signedCatalogDay());
	ipList.append(p);
	
	QString ho = temp.section(' ',numberCatalogHour(),numberCatalogHour());
  ind = index(i, numberCatalogHour(), QModelIndex());
  setData(ind, QVariant(ho) );
  p = new   PropertyInt(ind,ho,minCatalogHour(),maxCatalogHour(),signedCatalogHour());
	ipList.append(p);
	
	QString mi = temp.section(' ',numberCatalogMinute(),numberCatalogMinute());
  ind = index(i, numberCatalogMinute(), QModelIndex());
  setData(ind, QVariant(mi) );
  p = new   PropertyInt(ind,mi,minCatalogMinute(),maxCatalogMinute(),signedCatalogMinute());
	ipList.append(p);

	QString se = temp.section(' ',numberCatalogSecond(),numberCatalogSecond());
  ind = index(i, numberCatalogSecond(), QModelIndex());
  setData(ind, QVariant(se) );
  p = new   PropertyDouble(ind,se , minCatalogSecond(), maxCatalogSecond(),decimalCatalogSecond());
	ipList.append(p);
	
	QString fi = temp.section(' ',numberCatalogLatitude(),numberCatalogLatitude());
  ind = index(i, numberCatalogLatitude(), QModelIndex());
  setData(ind, QVariant(fi) );
  p = new   PropertyDouble(ind,fi , minCatalogLatitude(), maxCatalogLatitude(),decimalCatalogLatitude());
	ipList.append(p);

	QString la = temp.section(' ',numberCatalogLongitude(),numberCatalogLongitude());
  ind = index(i, numberCatalogLongitude(), QModelIndex());
  setData(ind, QVariant(la) );
  p = new   PropertyDouble(ind,la , minCatalogLongitude(), maxCatalogLongitude(),decimalCatalogLongitude());
	ipList.append(p);

	QString h = temp.section(' ',numberCatalogDepth(),numberCatalogDepth());
  ind = index(i, numberCatalogDepth(), QModelIndex());
  setData(ind, QVariant(h) );
  p = new   PropertyDouble(ind,h , minCatalogDepth(), maxCatalogDepth(),decimalCatalogDepth());
	ipList.append(p);

	QString m1 = temp.section(' ',numberCatalogMb(),numberCatalogMb());
  ind = index(i, numberCatalogMb(), QModelIndex());
  setData(ind, QVariant(m1) );
  p = new   PropertyDouble(ind,m1 , minCatalogMb(), maxCatalogMb(),decimalCatalogMb());
	ipList.append(p);

	
	QString m2 = temp.section(' ',numberCatalogMs(),numberCatalogMs());
  ind = index(i, numberCatalogMs(), QModelIndex());
  setData(ind, QVariant(m2) );
  p = new   PropertyDouble(ind,m2 , minCatalogMs(), maxCatalogMs(),decimalCatalogMs());
	ipList.append(p);


	QString m3 = temp.section(' ',numberCatalogMl(),numberCatalogMl());
  ind = index(i, numberCatalogMl(), QModelIndex());
  setData(ind, QVariant(m3) );
  p = new   PropertyDouble(ind,m3 , minCatalogMl(), maxCatalogMl(),decimalCatalogMl());
	ipList.append(p);

	QString m4 = temp.section(' ',numberCatalogMp(),numberCatalogMp());
  ind = index(i, numberCatalogMp(), QModelIndex());
  setData(ind, QVariant(m4) );
  p = new   PropertyDouble(ind,m4 , minCatalogMp(), maxCatalogMp(),decimalCatalogMp());
	ipList.append(p);

	QString inten = temp.section(' ',numberCatalogIntensity(),numberCatalogIntensity());
  ind = index(i, numberCatalogIntensity(), QModelIndex());
  setData(ind, QVariant(inten) );
  p = new   PropertyInt(ind,inten,minCatalogIntensity(),maxCatalogIntensity(),signedCatalogIntensity());
	ipList.append(p);

  ind = index(i, numberCatalogComment(), QModelIndex());
  setData(ind, QVariant("") );
  p = new   PropertyString(ind,"");
	ipList.append(p);
}
int PropertyModel::GetHeaderRowRecordNumber(int row)
{
bool ok;
int  num =  headerData(row,Qt::Vertical).toInt(&ok);
if(ok) return num;else return -1;
}

QStringList PropertyModel::GetTableData(int i){
 QStringList lst;
 for(int j = 0; j < numberCatalogColumns(); j++)  lst << data(index(i, j, QModelIndex())).toString ();
 return     lst;

}
/*
IProperty * PropertyModel::GetProperty(const QModelIndex & index)
{
  return
        ipList.at(index.row()*(NumCols + 1)  + index.column ());
}
*/
int PropertyModel::findPropListIndex(const QModelIndex & index)
{
    for (int i = 0; i < ipList.size(); ++i) {
        if (ipList.at(i)->index == index) {return i;}
     }
return -1;
}
IProperty * PropertyModel::findProperty(const QModelIndex & index)
{
    for (int i = 0; i < ipList.size(); ++i) {
        if (ipList.at(i)->index == index) {return ipList.at(i);}
     }
return 0;
}
//-----------------------------------------------------------
QString PropertyModel::GetRowInitialValues(int Row){
QModelIndex ind;
QString str;
        for (int j = 0; j < columnCount(); j++)
        {
         ind = index(Row, j, QModelIndex());
         IProperty * p = ipList.at(Row*(columnCount() + 1) + j + 1);
         str += p->GetInitStr() + " ";
        }
str.remove(str.length()-2, 2);
return str;
}
//-----------------------------------------------------------
void PropertyModel::RepairRowsProperty(int Row){
QModelIndex ind;
  for (int i = Row; i < rowCount(); i++)
        for (int j = 0; j < columnCount(); j++)
        {
         ind = index(i, j, QModelIndex());
         IProperty * p = ipList.at(i*(columnCount() + 1) + j + 1);
         p->SetIndex(ind);
  }
}
void PropertyModel::InsertValuePropertyRows (int Row,int Count ,QList<UndoEdit *>& list)
{
   int k = 0;
   for (int i = Row  ; i < Row + Count; i++)
   {
  
        SetValuePropertyRow(i,k,list); k++;

   }

}
//-----------------------------------------------------------
void PropertyModel::InsertEmptyValuePropertyRows(int Row,int Count)
{


   for (int i = Row; i < Row + Count; i++)
   {
        int k = i;
     
        SetEmptyValuePropertyRow(k);
        
   }

}

void PropertyModel::RemoveRowsProperty(int Row,int Count)
{
int m = Row*(columnCount() + 1);
        for (int i = Row; i < Row + Count; i++)
        for (int j = 0; j < columnCount()  + 1; j++)        
        {
          delete ipList.at(m);
          ipList.removeAt (m);
        }
}

QString PropertyModel::GetPropertyInit(int row,int col){
 QModelIndex ind = index(row, col, QModelIndex());
 IProperty * p = findProperty(ind); //GetProperty (ind);

 return QString("%1").arg(p->GetInitStr());
}
//================================================================
void PropertyModel::SetPolygonRow(int i,QString col1,QString col2){
  QModelIndex ind =   QModelIndex();
  IProperty * p = 0;
	QString tmp = col1;
  ind = index(i,0, QModelIndex());
  setData(ind, QVariant(tmp) );
  p = new   PropertyDouble(ind,tmp, -90,90,3);
	ipList.append(p);

	tmp = col2;
  ind = index(i, 1, QModelIndex());
  setData(ind, QVariant(tmp) );
  p = new   PropertyDouble(ind,tmp , -180, 180,3);
	ipList.append(p);

  ind = index(i, 2, QModelIndex());
  setData(ind, QVariant("") );
  p = new   PropertyString(ind,"");
	ipList.append(p);
}
void PropertyModel::RemovePolygonRowsProperty(int Row,int Count)
{
int m = Row*(columnCount() );
        for (int i = Row; i < Row + Count; i++)
        for (int j = 0; j < columnCount(); j++)
        {
          delete ipList.at(m);
          ipList.removeAt (m);
        }
}
void PropertyModel::RepairPolygonRowsProperty(int Row){
QModelIndex ind;
  for (int i = Row; i < rowCount(); i++)
        for (int j = 0; j < columnCount(); j++)
        {
         ind = index(i, j, QModelIndex());
         IProperty * p = ipList.at(i*(columnCount() ) + j);
         p->SetIndex(ind);
  }
}
void PropertyModel::InsertEmptyPolygonValuePropertyRows(int Row,int Count)
{


   for (int i = Row; i < Row + Count; i++)
   {
        int k = i;

        SetEmptyPolygonValuePropertyRow(k);

   }

}
void PropertyModel::SetEmptyPolygonValuePropertyRow(int i){
//  setHeaderData(i,Qt::Vertical, QString());
  QModelIndex ind = QModelIndex ();
  QModelIndex ind1 = QModelIndex ();
  int k = i*(columnCount());
  IProperty * p = 0;

  ind = index(i, 0, QModelIndex());
  ind1 = index(i+1, 0, QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyDouble(ind,"100", -90,90,3);
  IProperty *p1 =  findProperty(ind) ;
  p->SetChanged(true);
  ipList.insert(k,p); k++;

  ind = index(i, 1, QModelIndex());
  ind1 = index(i+1, 1, QModelIndex());
  setData(ind, QVariant(data(ind1) ));
  p = new   PropertyDouble(ind,"190" , -180, 180,3);
  p1 =  findProperty(ind) ;
  p->SetChanged(true);
	ipList.insert(k,p); k++;


  ind = index(i, 2, QModelIndex());
  setData(ind, QVariant("") );
  p = new   PropertyString(ind,"");
  ipList.insert(k,p);

}
//================================================================

//-------------------------------------------------------------------
//-------------------------------------------------------------------
PropertyDelegate::PropertyDelegate(QObject *parent)
    : QItemDelegate(parent)
{

lin = 0;
}

QWidget *PropertyDelegate::createEditor(QWidget *parent,
    const QStyleOptionViewItem &,
    const QModelIndex & index) const
{

    PropertyModel *mod = (PropertyModel *)index.model();
    
	  IProperty * p = mod->findProperty(index);

    if(p != 0) {
      if (dynamic_cast<IProperty *>(p) == dynamic_cast<PropertyNotEdit *>(p)) return 0;

      if (dynamic_cast<IProperty *>(p) == dynamic_cast<PropertyInt *>(p)) {
       QString val = index.model()->data(index, Qt::DisplayRole).toString();
       ( (PropertyDelegate*)this )->lin = new TableLineEditor(p,val,TableLineEditor::Property_Int,parent);
       return(lin);
       }
       if (dynamic_cast<IProperty *>(p) == dynamic_cast<PropertyDouble *>(p)) {
       QString val = index.model()->data(index, Qt::DisplayRole).toString();
       ( (PropertyDelegate*)this )->lin = new TableLineEditor(p,val,TableLineEditor::Property_Double,parent);
       return(lin );
       }

       if (dynamic_cast<IProperty *>(p) == dynamic_cast<PropertyString *>(p)) {                                                      //"[_a-zA-Z][_a-zA-Z0-9]*"   
       QLineEdit *editor = new QLineEdit(parent);
       editor->setValidator(new QRegExpValidator(QRegExp("[!@#$%^&*()_+-=<>?,./:\"\\|;'a-zA-Z0-9]*"), editor));
       editor->installEventFilter(const_cast<PropertyDelegate*>(this));
       QObject::connect( editor, SIGNAL( textChanged( const QString & ) ),
		      this, SLOT( setValue(const QString &) ) );
        return editor;
       }
    } else return 0;


    return 0;
}

void PropertyDelegate::setEditorData(QWidget *editor,
                                    const QModelIndex &index) const
{


    QString value = index.model()->data(index, Qt::DisplayRole).toString();
    PropertyModel *mod = (PropertyModel *)index.model();
    
	  IProperty * p = mod->findProperty(index);
if(p != 0) {
      if (dynamic_cast<IProperty *>(p) == dynamic_cast<PropertyInt *>(p)) {
       TableLineEditor *lineEdit = static_cast<TableLineEditor*>(editor);
       lineEdit->setText(value);
       }
       if (dynamic_cast<IProperty *>(p) == dynamic_cast<PropertyDouble *>(p)) {
       TableLineEditor *lineEdit = static_cast<TableLineEditor*>(editor);
       lineEdit->setText(value);
       }
       if (dynamic_cast<IProperty *>(p) == dynamic_cast<PropertyString *>(p)) {
        QLineEdit *lineEdit = static_cast<QLineEdit*>(editor);
        lineEdit->setText(value);
      }

}
}

void PropertyDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
                                   const QModelIndex &index) const
{
    QString prev = index.model()->data(index, Qt::DisplayRole).toString();
    PropertyModel *mod = (PropertyModel *)index.model();
    IProperty * p = mod->findProperty(index);
    QString value = "";
if(p != 0) {
      if (dynamic_cast<IProperty *>(p) == dynamic_cast<PropertyInt *>(p)) {
       TableLineEditor *lineEdit = static_cast<TableLineEditor*>(editor);
        lineEdit->editFinish();
        value =lineEdit->text();( (PropertyDelegate*)this )->lin = 0;
       }
       if (dynamic_cast<IProperty *>(p) == dynamic_cast<PropertyDouble *>(p)) {
       TableLineEditor *lineEdit = static_cast<TableLineEditor*>(editor);
        lineEdit->editFinish();
        value =lineEdit->text();( (PropertyDelegate*)this )->lin = 0;
       }
       if (dynamic_cast<IProperty *>(p) == dynamic_cast<PropertyString *>(p)) {
        QLineEdit *lineEdit = static_cast<QLineEdit*>(editor);
        value = lineEdit->text();
      }

}
    model->setData(index, value);
    if (p->GetInitStr() == value) p->SetChanged(false);
    else {
  if(mod->columnCount() == numberCatalogColumns()) {   
    UndoChange* u = new  UndoChange(p,prev,value,index.row(),index.column());

    if (CatTable::tSelf->curUndoIndex + 1 == CatTable::tSelf->undoList.size())
        CatTable::tSelf->undoList.append(u);
    else
        CatTable::tSelf->undoList.insert(CatTable::tSelf->curUndoIndex+1 ,u);
        CatTable::tSelf->curUndoIndex++;
        p->SetChanged(true);
        MainWindow::self->viewBufferModified =  true;
  }   else p->SetChanged(true);
    }
}

void PropertyDelegate::setValue( const QString  &v ) 
{

    QString temp = v;
    if ( lin ) {
	lin->blockSignals( TRUE );
	int oldCursorPos;
	oldCursorPos = lin->cursorPosition();
	int j = 0;
	QValidator * vld = (QValidator *)lin->validator ();
        vld->fixup(temp );
	if ( vld->validate( temp, j )  != QValidator::Acceptable){	     
	   temp.remove( temp.length()-1, 1); 
	}
	lin->setText(temp);
	if ( oldCursorPos < (int)lin->text().length() )
	    lin->setCursorPosition( oldCursorPos );
	    lin->blockSignals( FALSE );
    }

}
void PropertyDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt, const QModelIndex &index) const
{

    QStyleOptionViewItem option = opt;
    option.displayAlignment =  Qt::AlignRight|Qt::AlignVCenter;
    //option.displayAlignment =  Qt::AlignHCenter;
    PropertyModel *mod = (PropertyModel *)index.model();
    IProperty * p = mod->findProperty(index);

    if(p != 0) {
    if (p->GetChanged()) { option.font.setBold(true); option.palette.setColor(QPalette::Text,QColor("blue"));     }
	  if (p->GetInvalid())  option.palette.setColor(QPalette::Text,QColor("red"));    
    }

    option.palette.setColor(QPalette::Highlight,QColor(100,160,180)); //QColor(80,132,166)    
    QItemDelegate::paint(painter, option, index);
//    painter->drawLine(option.rect.x(), option.rect.bottom(),
//            option.rect.right(), option.rect.bottom());
//    painter->drawLine(option.rect.x(), option.rect.top(),
//            option.rect.right(), option.rect.top());
}

//---------------------------------------------------

