
#ifndef TABLE_PROPERTY__H
#define TABLE_PROPERTY__H

#include <QItemDelegate>
#include <QModelIndex>
#include <QObject>
#include <QSize>
#include <QPointer>
#include <QStandardItemModel>
#include <QList>
#include <QLineEdit>
class QTableView;
class UndoEdit;
class SettingDocument;
class IProperty
{
public:
    inline IProperty(QModelIndex ind,const QString &initStr)
        : invalid(FALSE),
          changed(FALSE),
          initString(initStr) {
	        index = ind;
	  }

    virtual ~IProperty() {}
    QModelIndex index;
    void SetIndex(QModelIndex ind){index = ind;};
    void SetInvalid(bool inv){invalid = inv;}
    bool GetInvalid(){return invalid;}
    void SetChanged(bool ch){changed = ch;}
    bool GetChanged(){return changed;}
    void SetInitStr(const QString &initStr ) {initString = initStr;}
    QString GetInitStr(){return initString;}
protected:
    bool invalid;
    bool changed;
    QString initString;
};
//-----------------------------------------------------------------------------------------------------
class PropertyInt : public IProperty
{


public:
    PropertyInt(QModelIndex ind,const QString &initStr,int min,int max, bool s );
    ~PropertyInt() {};

    int minimum;
    int maximum;
    bool signedValue;
};
//-----------------------------------------------------------------------------------------------------
class PropertyNotEdit : public IProperty
{
public:
    PropertyNotEdit(QModelIndex ind,const QString &initStr);
    ~PropertyNotEdit() {};
};
//-----------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------
class PropertyString : public IProperty
{
public:
    PropertyString(QModelIndex ind,const QString &initStr);
    ~PropertyString() {};
};
//-----------------------------------------------------------------------------------------------------
class PropertyDouble : public IProperty
{
        

public:
    PropertyDouble(QModelIndex ind,const QString &initStr,double min,double max,int dec);
    ~PropertyDouble(){};

    double minimum;
    double maximum;
    int decimals;
};
//-----------------------------------------------------------------------------------------------------
class TableLineEditor : public QLineEdit
{
    Q_OBJECT
public:
   enum Kind
    {
        Property_Int,
        Property_Double
    };
     TableLineEditor(IProperty* prop,QString val,Kind kind,QWidget *parent = 0);
    ~TableLineEditor();
IProperty* GetProperty(){return prop;}
TableLineEditor::Kind GetKind() {return kind;}
QString GetEditedValue(){return editedValue;}
public slots:
    void setValue( const QString  &v );
    void editFinish();
    void returnPressed();
private slots:
    void escAction();
private:
IProperty* prop;
QString editedValue;
TableLineEditor::Kind kind;
QAction *escAct;
};
//-----------------------------------------------------------------------------------------------------
class PropertyModel : public QStandardItemModel
{    
    Q_OBJECT

public:
PropertyModel(int rows, int columns,QObject * parent = 0);
PropertyModel( QObject * parent,int rows, int columns);
~PropertyModel() ;
IProperty * findProperty(const QModelIndex & index);
int findPropListIndex(const QModelIndex & index);
//IProperty * GetProperty(const QModelIndex & index);
QList<IProperty *> ipList;
void SetRow(int i,int inpos,QString& temp);
void SetNotEditRow(int i,int inpos,QString& temp);
QStringList GetTableData(int row);
void setText(QModelIndex index, QString text);
void RemoveRowsProperty(int Row,int Count);
void RepairRowsProperty(int Row);
QString GetRowInitialValues(int Row);
void InsertEmptyValuePropertyRows (int Row,int Count);
void SetEmptyValuePropertyRow(int i);
void InsertValuePropertyRows (int Row,int Count ,QList<UndoEdit *>& list);
void SetValuePropertyRow(int i,int shift,QList<UndoEdit *>& list);
QString GetPropertyInit(int row,int col);
void SetHorizontalHeader();
int GetHeaderRowRecordNumber (int row);

void SetPolygonRow(int i,QString col1,QString col2);
void RemovePolygonRowsProperty(int Row,int Count);
void RepairPolygonRowsProperty(int Row);
void InsertEmptyPolygonValuePropertyRows (int Row,int Count);
void SetEmptyPolygonValuePropertyRow(int i);
private:

QTableView* parentTable;
int NumRows;
int NumCols;
};
class PropertyDelegate : public QItemDelegate
{
    Q_OBJECT

public:
    PropertyDelegate(QObject *parent = 0);

    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option,
                          const QModelIndex &index) const;

    void setEditorData(QWidget *editor, const QModelIndex &index) const;
    void setModelData(QWidget *editor, QAbstractItemModel *model,
                      const QModelIndex &index) const;
    void paint(QPainter *painter, const QStyleOptionViewItem &option,
                       const QModelIndex &index) const;
   QPointer<TableLineEditor> GetLineEdit(){return lin;}
private slots:
   void setValue( const QString  &v );
private:
    QPointer<TableLineEditor> lin;
};
#endif
