#include <QtGui>
#include "mainwindow.h"
#include "bottomwindow.h"
#include "transmagimpl.h"
#include "viewcatalogimpl.h"
#include "subcatalog.h"
#include "document.h"
#include "default.h"
#include "editline.h"
#include "helpwindow.h"
TransformFunc::TransformFunc(vector<int> &priority,vector<QString> &valFunc,int index):
                         indexMagnitude(index),priority(priority){
  eps = acuracyCompareFloatValue();
  a.resize(4);b.resize(4);c.resize(4);d.resize(4);
  for(int i = 0; i< 4;i++) {
   QStringList param = valFunc[i].split( '@');
   d[i] = param[0].toFloat(); a[i]  = param[1].toFloat(); b[i] = param[2].toFloat(); c[i] = param[3].toFloat();
  }
}
//----------------------------------------------------------------------------------
void  TransformFunc::SetRecord(RECORD rec){
  record = rec;
  vector<float> &mag = magnitudeRecord;
  float magnification = magnitudeMagnification();
  mag.resize(4); mag[0] = record.m[0]/magnification; mag[1] = record.m[1]/magnification;
               mag[2] = record.m[2]/magnification; mag[3] = record.m[3]/magnification;
}
//----------------------------------------------------------------------------------
float TransformFunc::mFunct(int i,float m){
  if ( m > d[i] || fabs(m - d[i]) < eps) return  m*a[i] + b[i];  else  return c[i];
}
//----------------------------------------------------------------------------------
void  TransformFunc::Value(){
  if(!priority.empty()) {
    vector<float> &mag = magnitudeRecord;
    bool findNotZeroMagnitude = false;
    int indexNotZeroMagnitude = 0;
	if ((int)priority.size() == 1) findNotZeroMagnitude = true;
	else
		for(int i = 0; i < (int)priority.size();i++)   {
			if( !( fabs(mag[priority[i]]) < eps ))
			{findNotZeroMagnitude = true;indexNotZeroMagnitude = i;break;}
		}
	
  if (findNotZeroMagnitude) {
    float flt  = mFunct(priority[indexNotZeroMagnitude],mag[priority[indexNotZeroMagnitude]]);
    magnitude =  (short)((flt + 0.005)*magnitudeMagnification());
   } else  magnitude = 0;
 } 
}
//----------------------------------------------------------------------------------
void  TransformFunc::Maximum(){
  if(!priority.empty()) {
  vector<float> &mag = magnitudeRecord;
  float fm = mFunct(priority[0],mag[priority[0]]);
  for(int i = 1; i < (int)priority.size();i++)   {
    float   mm = mFunct(priority[i],mag[priority[i]]);
    if( mm > fm ) fm = mm;
  }
  if(magnitude < 0)
	  magnitude = (short)((fm - 0.005)*magnitudeMagnification());
  else
	  magnitude = (short)((fm + 0.005)*magnitudeMagnification());
 }
}
//----------------------------------------------------------------------------------
void  TransformFunc::Minimum(){
  if(!priority.empty()) {
  vector<float> &mag = magnitudeRecord;
  float fm = mFunct(priority[0],mag[priority[0]]);
  //QMessageBox::information(0,"",QString("%1").arg(fm));
  for(int i = 1; i < (int)priority.size();i++)   {
   float   mm = mFunct(priority[i],mag[priority[i]]);
   if( mm < fm ) fm = mm;
   }
  if(magnitude < 0)
	  magnitude = (short)((fm - 0.005)*magnitudeMagnification());
  else
	  magnitude = (short)((fm + 0.005)*magnitudeMagnification());
 } 
}
//----------------------------------------------------------------------------------
void  TransformFunc::Average(){
  if(!priority.empty()) {
  vector<float> &mag = magnitudeRecord;
  float fm = mFunct(priority[0],mag[priority[0]]);
  for(int i = 1; i < (int)priority.size();i++)
   fm += mFunct(priority[i],mag[priority[i]]);
  fm /= fm / priority.size();
  if(magnitude < 0)
	  magnitude = (short)((fm - 0.005)*magnitudeMagnification());
  else
	  magnitude = (short)((fm + 0.005)*magnitudeMagnification());
 }
}
//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------
TransfMagnImpl::~TransfMagnImpl(){
  comboBoxes.clear();
  foreach (LineDocProperty* prop, propDocLine) {delete prop;prop = 0;}
  propDocLine.clear();
  for_each(values.begin(),values.end(),delAndNullify);
  values.erase(remove(values.begin(),values.end(),
  static_cast<PropertyValue*>(0)),values.end());
  if(!funct.isEmpty ())  foreach (TransformFunc* fun, funct) {delete fun;fun = 0;}
  if(!ptrFunct.empty()) ptrFunct.clear();
  if(help !=0) {delete help; help =0;}
}
//----------------------------------------------------------------------------------
TransfMagnImpl::TransfMagnImpl(QWidget *parent,FDocument* dc,TypeUsing tp)
    : QDialog(0),doc(dc),type(tp)
{
     
     setupUi(this);
	 setModal (true);
     setFont( parent->font() );
     setWindowIcon(QIcon(":/images/icon.png"));
     setWindowTitle(projectActionList().split ('@').at(0));
     comboBoxMagFunction->clear();
     QStringList lst = doc->GetDeltaMagnitudeFunction()->GetStringList();
     for (int i = 0; i < lst.size(); i++) {
     comboBoxMagFunction->addItem(lst.at(i));
     }
     lst = enumComboBoxMagnitudeList().split ('@');
     comboBoxes.append(comboBoxMagnitude);
     comboBoxes.append(comboBoxM1); comboBoxes.append(comboBoxM2);
     comboBoxes.append(comboBoxM3); comboBoxes.append(comboBoxM4);

     foreach (QComboBox * box, comboBoxes) box->clear();
     for (int i = 0; i < lst.size(); i++) foreach (QComboBox * box, comboBoxes) box->addItem(lst.at(i));
     texted = MainWindow::self->bottomWindow->messageTab;
     if (type == Tool){
     toolButtonProject->hide();
      MainWindow::self->bottomWindow->OpenMessagesProtocolPage();
      QString tmp = projectActionList().split ('@').at(0);
      texted->insertPlainText(prefixMessage() + tmp + " ***\n\n");
      }



    connect(buttonOk, SIGNAL(clicked()), this, SLOT(accept()));
    connect(buttonCancel, SIGNAL(clicked()), this, SLOT(close()));
    connect(toolButtonDefault, SIGNAL(clicked()), this, SLOT(buttonDefault()));
    connect(toolButtonReset, SIGNAL(clicked()), this, SLOT(buttonReset()));
    connect(toolButtonProject, SIGNAL(clicked()), this, SLOT(buttonProject()));
    connect(buttonHelp, SIGNAL(clicked()), this, SLOT(helpButton()));
    connect(comboBoxMagnitude, SIGNAL( activated( int ) ),
     this, SLOT(comboBoxMagnitudeActivate( int ) ) );
    connect( progressBar, SIGNAL( valueChanged ( int) ), this, SLOT(ProgressBarChanged(int) ) );
    comboBoxMagnitudeIndex = 0;
    progressBar->setValue(0);
    textLabel->setText("");
	 if (type == Wizard)  textLabel->hide();
    InitLineDocProperty();
    GetDialogFromDocument(comboBoxMagnitudeIndex);
    help =0;
	 comboBoxM1->adjustSize();
	 comboBoxM2->adjustSize();
	 comboBoxM3->adjustSize(); 
	 comboBoxM4->adjustSize();
}
//----------------------------------------------------------------------------------
void TransfMagnImpl::comboBoxMagnitudeActivate(int index ){
  if (comboBoxMagnitudeIndex == index) return;
  GetDocumentFromDialog(comboBoxMagnitudeIndex);
  GetDialogFromDocument(index);
  comboBoxMagnitudeIndex = index;
}
//----------------------------------------------------------------------------------
bool TransfMagnImpl::NextOk(){
 message = "";
 DefineTransformFunction();
 if(funct.isEmpty()) {
    error(QString("Nothing to do! Even one transform function for magnitude must be define."),"");
    return false;}
   QString tmp = QString("Do you accept your choice for %1 action?\n\n").arg(projectActionList().split ('@').at(0));
   int ret = QMessageBox::warning(this,windowTitle (),tmp + message,
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No | QMessageBox::Escape);
   if (ret == QMessageBox::No) return false;

return true;
}
void TransfMagnImpl::FinishOk(){
      message = "";
      QString tmp = projectActionList().split ('@').at(0);
      texted->insertPlainText(prefixMessage() + tmp + " ***\n\n");
  DefineTransformFunction();
  
  foreach( PropertyEnumValue* prop,doc->GetTransformMagnitudeFunction())    prop->SetValueToResetvalue();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMb()) prop->SetValueToResetvalue();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMs()) prop->SetValueToResetvalue();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMl()) prop->SetValueToResetvalue();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMp()) prop->SetValueToResetvalue();
  texted->insertPlainText(message);
  tmp = projectActionList().split ('@').at(0);
      texted->insertPlainText(postfixActionMessage() + tmp + " ***\n\n");
}
bool TransfMagnImpl::execute(){
bool ret = applay();
done( Accepted );
return ret;
}
bool TransfMagnImpl::applay(){
  message = "";
  DefineTransformFunction();

  if (type == Tool || type == Wizard) {
  QString tmp = QString("Do you want to execute %1?\n\n").arg(projectActionList().split ('@').at(0));
       int ret = QMessageBox::warning(0,windowTitle (),tmp + message,
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No | QMessageBox::Escape);
        if (ret == QMessageBox::No) return false;
  }
  repaint();
  if(!TransformMagnitude()) return false;

  texted->insertPlainText(message);
  QString tmp = projectActionList().split ('@').at(0);
  texted->insertPlainText(postfixActionMessage()  + tmp + " ***\n\n");
  CatTable::tSelf->RestoreIndex();
  return true;
}
void TransfMagnImpl::accept()
{
  if (!applay()) return;
  MainWindow::self->modifiedCatalog = true;
  MainWindow::self->modifiedProtocol = true;
  foreach( PropertyEnumValue* prop,doc->GetTransformMagnitudeFunction())    prop->SetValueToResetvalue();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMb()) prop->SetValueToResetvalue();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMs()) prop->SetValueToResetvalue();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMl()) prop->SetValueToResetvalue();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMp()) prop->SetValueToResetvalue();
  if (type == Tool) {
      texted->ensureCursorVisible ();
  }
 MainWindow::self->RestoreStatusbarMessage();
  done( Accepted );

}
//----------------------------------------------------------------------------------
void TransfMagnImpl::reject()
{
return;
}
void TransfMagnImpl::closeEvent(QCloseEvent *event)
{
  texted->insertPlainText(cancelMessage() + "\n\n");
  buttonReset();
  MainWindow::self->RestoreStatusbarMessage();
  event->accept();
}
//----------------------------------------------------------------------------------
void TransfMagnImpl::buttonDefault(){
  foreach( PropertyEnumValue* prop,doc->GetTransformMagnitudeFunction())    prop->SetDefault();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMb()) prop->SetDefault();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMs()) prop->SetDefault();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMl()) prop->SetDefault();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMp()) prop->SetDefault();
  GetDialogFromDocument(comboBoxMagnitudeIndex);   
}
//----------------------------------------------------------------------------------
void TransfMagnImpl::buttonReset(){
  foreach( PropertyEnumValue* prop,doc->GetTransformMagnitudeFunction())    prop->SetResetvalueToValue();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMb()) prop->SetResetvalueToValue();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMs()) prop->SetResetvalueToValue();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMl()) prop->SetResetvalueToValue();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMp()) prop->SetResetvalueToValue();
  GetDialogFromDocument(comboBoxMagnitudeIndex);
}
//----------------------------------------------------------------------------------
void TransfMagnImpl::buttonProject(){
  foreach( PropertyEnumValue* prop,doc->GetTransformMagnitudeFunction())    prop->SetInitial();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMb()) prop->SetInitial();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMs()) prop->SetInitial();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMl()) prop->SetInitial();
  foreach( PropertyTransformFunction * prop,doc->GetTransformMagnitudeMp()) prop->SetInitial();
  GetDialogFromDocument(comboBoxMagnitudeIndex);
}
//----------------------------------------------------------------------------------
void TransfMagnImpl::helpButton(){
    QString home = searchHelpPath() + "/edcat-tool-transform.html";
    if(help !=0) {delete help; help =0;}
    help = new HelpWindow(home,this);
    if ( QApplication::desktop()->width() > 400
         && QApplication::desktop()->height() > 500 )
         help->show();
    else
        help->showMaximized();
}
//----------------------------------------------------------------------------------
void TransfMagnImpl::InitLineDocProperty() {
 PropertyFloatValue* M1_D = new PropertyFloatValue(101,"","",
 minTransformMagnitudeD(),maxTransformMagnitudeD(),decimalTransformMagnitudeD());
 propDocLine.append(new LineDocProperty(lineEditM1_D,M1_D));
 values.push_back(M1_D);

 PropertyFloatValue* M1_A = new PropertyFloatValue(102,"","",
 minTransformMagnitudeA(),maxTransformMagnitudeA(),decimalTransformMagnitudeA());
 propDocLine.append(new LineDocProperty(lineEditM1_A,M1_A));
 values.push_back(M1_A);

 PropertyFloatValue* M1_B = new PropertyFloatValue(103,"","",
 minTransformMagnitudeB(),maxTransformMagnitudeB(),decimalTransformMagnitudeB());
 propDocLine.append(new LineDocProperty(lineEditM1_B,M1_B));
 values.push_back(M1_B);

 PropertyFloatValue* M1_C = new PropertyFloatValue(104,"","",
 minTransformMagnitudeC(),maxTransformMagnitudeC(),decimalTransformMagnitudeC());
 propDocLine.append(new LineDocProperty(lineEditM1_C,M1_C));
 values.push_back(M1_C);


 PropertyFloatValue* M2_D = new PropertyFloatValue(105,"","",
 minTransformMagnitudeD(),maxTransformMagnitudeD(),decimalTransformMagnitudeD());

 propDocLine.append(new LineDocProperty(lineEditM2_D,M2_D));
 values.push_back(M2_D);

 PropertyFloatValue* M2_A = new PropertyFloatValue(106,"","",
 minTransformMagnitudeA(),maxTransformMagnitudeA(),decimalTransformMagnitudeA());
 propDocLine.append(new LineDocProperty(lineEditM2_A,M2_A));
 values.push_back(M2_A);

 PropertyFloatValue* M2_B = new PropertyFloatValue(107,"","",
 minTransformMagnitudeB(),maxTransformMagnitudeB(),decimalTransformMagnitudeB());
 propDocLine.append(new LineDocProperty(lineEditM2_B,M2_B));
 values.push_back(M2_B);

 PropertyFloatValue* M2_C = new PropertyFloatValue(108,"","",
 minTransformMagnitudeC(),maxTransformMagnitudeC(),decimalTransformMagnitudeC());
 propDocLine.append(new LineDocProperty(lineEditM2_C,M2_C));
 values.push_back(M2_C);

 PropertyFloatValue* M3_D = new PropertyFloatValue(109,"","",
 minTransformMagnitudeD(),maxTransformMagnitudeD(),decimalTransformMagnitudeD());

 propDocLine.append(new LineDocProperty(lineEditM3_D,M3_D));
 values.push_back(M3_D);

 PropertyFloatValue* M3_A = new PropertyFloatValue(110,"","",
 minTransformMagnitudeA(),maxTransformMagnitudeA(),decimalTransformMagnitudeA());
 propDocLine.append(new LineDocProperty(lineEditM3_A,M3_A));
 values.push_back(M3_A);

 PropertyFloatValue* M3_B = new PropertyFloatValue(111,"","",
 minTransformMagnitudeB(),maxTransformMagnitudeB(),decimalTransformMagnitudeB());
 propDocLine.append(new LineDocProperty(lineEditM3_B,M3_B));
 values.push_back(M3_B);

 PropertyFloatValue* M3_C = new PropertyFloatValue(112,"","",
 minTransformMagnitudeC(),maxTransformMagnitudeC(),decimalTransformMagnitudeC());
 propDocLine.append(new LineDocProperty(lineEditM3_C,M3_C));
 values.push_back(M3_C);
 
 PropertyFloatValue* M4_D = new PropertyFloatValue(113,"","",
 minTransformMagnitudeD(),maxTransformMagnitudeD(),decimalTransformMagnitudeD());

 propDocLine.append(new LineDocProperty(lineEditM4_D,M4_D));
 values.push_back(M4_D);

 PropertyFloatValue* M4_A = new PropertyFloatValue(114,"","",
 minTransformMagnitudeA(),maxTransformMagnitudeA(),decimalTransformMagnitudeA());
 propDocLine.append(new LineDocProperty(lineEditM4_A,M4_A));
 values.push_back(M4_A);

 PropertyFloatValue* M4_B = new PropertyFloatValue(115,"","",
 minTransformMagnitudeB(),maxTransformMagnitudeB(),decimalTransformMagnitudeB());
 propDocLine.append(new LineDocProperty(lineEditM4_B,M4_B));
 values.push_back(M4_B);

 PropertyFloatValue* M4_C = new PropertyFloatValue(116,"","",
 minTransformMagnitudeC(),maxTransformMagnitudeC(),decimalTransformMagnitudeC());
 propDocLine.append(new LineDocProperty(lineEditM4_C,M4_C));
 values.push_back(M4_C);

 QFont fnOld = font();
 QFont fn = fnOld;
 fn.setBold(true);
// int width =  fontMetrics().width(fontmetricLineEdit());
 foreach (LineDocProperty *docprop, propDocLine) {
 docprop->SetValueFromDoc();
/*
  if (docprop->GetProp()->GetKind()  == PropertyValue::Property_Float) {
   QLineEdit * lin = (QLineEdit *) docprop->GetEditor();
    lin->setFixedWidth (width);
  }
*/
 }

} 
//----------------------------------------------------------------------------------
void TransfMagnImpl::GetDialogFromDocument(int indexMagnitude){

 PropertyEnumValue* TransformMagnitudeFunction =
                   doc->GetTransformMagnitudeFunction().at(indexMagnitude);
 comboBoxMagFunction->setCurrentIndex(TransformMagnitudeFunction->GetValue().toInt());

 PropertyTransformFunction * MagnitudeM1 = 0;
 PropertyTransformFunction * MagnitudeM2 = 0;            
 PropertyTransformFunction * MagnitudeM3 = 0;            
 PropertyTransformFunction * MagnitudeM4 = 0;
            
 if (indexMagnitude == 0) {
  MagnitudeM1 =  doc->GetTransformMagnitudeMb().at(0);
  MagnitudeM2 =  doc->GetTransformMagnitudeMb().at(1);
  MagnitudeM3 =  doc->GetTransformMagnitudeMb().at(2);
  MagnitudeM4 =  doc->GetTransformMagnitudeMb().at(3);
 }
 if (indexMagnitude == 1) {
  MagnitudeM1 =  doc->GetTransformMagnitudeMs().at(0);
  MagnitudeM2 =  doc->GetTransformMagnitudeMs().at(1);
  MagnitudeM3 =  doc->GetTransformMagnitudeMs().at(2);
  MagnitudeM4 =  doc->GetTransformMagnitudeMs().at(3);
 }
 if (indexMagnitude == 2) {
  MagnitudeM1 =  doc->GetTransformMagnitudeMl().at(0);
  MagnitudeM2 =  doc->GetTransformMagnitudeMl().at(1);
  MagnitudeM3 =  doc->GetTransformMagnitudeMl().at(2);
  MagnitudeM4 =  doc->GetTransformMagnitudeMl().at(3);
 }
 if (indexMagnitude == 3) {
  MagnitudeM1 =  doc->GetTransformMagnitudeMp().at(0);
  MagnitudeM2 =  doc->GetTransformMagnitudeMp().at(1);
  MagnitudeM3 =  doc->GetTransformMagnitudeMp().at(2);
  MagnitudeM4 =  doc->GetTransformMagnitudeMp().at(3);
 }

 if (MagnitudeM1->valIsOn) checkBoxM1->setChecked (TRUE);
 else   checkBoxM1->setChecked (FALSE);
 comboBoxM1->setCurrentIndex(MagnitudeM1->valIndexMagnitude);

 if (MagnitudeM2->valIsOn) checkBoxM2->setChecked (TRUE);
 else   checkBoxM2->setChecked (FALSE);
 comboBoxM2->setCurrentIndex(MagnitudeM2->valIndexMagnitude);

 if (MagnitudeM3->valIsOn) checkBoxM3->setChecked (TRUE);
 else   checkBoxM3->setChecked (FALSE);
 comboBoxM3->setCurrentIndex(MagnitudeM3->valIndexMagnitude);

 if (MagnitudeM4->valIsOn) checkBoxM4->setChecked (TRUE);
 else   checkBoxM4->setChecked (FALSE);
 comboBoxM4->setCurrentIndex(MagnitudeM4->valIndexMagnitude);

 PropValIterator it = values.begin();
 for(it = values.begin(); it != values.end(); it++)   {
   if ((*it)->GetId() == 101) (*it)->SetValue(MagnitudeM1->valD);
   if ((*it)->GetId() == 102) (*it)->SetValue(MagnitudeM1->valA);
   if ((*it)->GetId() == 103) (*it)->SetValue(MagnitudeM1->valB);
   if ((*it)->GetId() == 104) (*it)->SetValue(MagnitudeM1->valC);

   if ((*it)->GetId() == 105) (*it)->SetValue(MagnitudeM2->valD);
   if ((*it)->GetId() == 106) (*it)->SetValue(MagnitudeM2->valA);
   if ((*it)->GetId() == 107) (*it)->SetValue(MagnitudeM2->valB);
   if ((*it)->GetId() == 108) (*it)->SetValue(MagnitudeM2->valC);

   if ((*it)->GetId() == 109) (*it)->SetValue(MagnitudeM3->valD);
   if ((*it)->GetId() == 110) (*it)->SetValue(MagnitudeM3->valA);
   if ((*it)->GetId() == 111) (*it)->SetValue(MagnitudeM3->valB);
   if ((*it)->GetId() == 112) (*it)->SetValue(MagnitudeM3->valC);

   if ((*it)->GetId() == 113) (*it)->SetValue(MagnitudeM4->valD);
   if ((*it)->GetId() == 114) (*it)->SetValue(MagnitudeM4->valA);
   if ((*it)->GetId() == 115) (*it)->SetValue(MagnitudeM4->valB);
   if ((*it)->GetId() == 116) (*it)->SetValue(MagnitudeM4->valC);

 }
foreach (LineDocProperty *docprop, propDocLine) docprop->SetValueFromDoc();
    
}
//----------------------------------------------------------------------------------
void TransfMagnImpl::GetDocumentFromDialog(int indexMagnitude){
     QString tmp = QString("%1").arg(comboBoxMagFunction->currentIndex());
     doc->GetTransformMagnitudeFunction().at(indexMagnitude)->SetValue(tmp);

     vector<QString> valFunc;valFunc.resize(4);

     if(checkBoxM1->isChecked())
           tmp = QString("1@%1@").arg(comboBoxM1->currentIndex());
     else  tmp = QString("0@%1@").arg(comboBoxM1->currentIndex());
     valFunc[0] = tmp + QString("%1@%2@%3@%4").arg(lineEditM1_D->text())
                                              .arg(lineEditM1_A->text())
                                              .arg(lineEditM1_B->text())
                                              .arg(lineEditM1_C->text());
     if(checkBoxM2->isChecked())
           tmp =  QString("1@%1@").arg(comboBoxM2->currentIndex());
     else  tmp =  QString("0@%1@").arg(comboBoxM2->currentIndex());
     valFunc[1] = tmp + QString("%1@%2@%3@%4").arg(lineEditM2_D->text())
                                              .arg(lineEditM2_A->text())
                                              .arg(lineEditM2_B->text())
                                              .arg(lineEditM2_C->text());

     if(checkBoxM3->isChecked ())
           tmp = QString("1@%1@").arg(comboBoxM3->currentIndex());
     else  tmp = QString("0@%1@").arg(comboBoxM3->currentIndex());
     valFunc[2] = tmp + QString("%1@%2@%3@%4").arg(lineEditM3_D->text())
                                              .arg(lineEditM3_A->text())
                                              .arg(lineEditM3_B->text())
                                              .arg(lineEditM3_C->text());

     if(checkBoxM4->isChecked())
           tmp = QString("1@%1@").arg(comboBoxM4->currentIndex());
     else  tmp = QString("0@%1@").arg(comboBoxM4->currentIndex());
     valFunc[3] = tmp + QString("%1@%2@%3@%4").arg(lineEditM4_D->text())
                                              .arg(lineEditM4_A->text())
                                              .arg(lineEditM4_B->text())
                                              .arg(lineEditM4_C->text());

     if (indexMagnitude == 0)
     for(int i = 0;i<4;i++) doc->GetTransformMagnitudeMb().at(i)->SetValue(valFunc[i]);
     if (indexMagnitude == 1)
     for(int i = 0;i<4;i++) doc->GetTransformMagnitudeMs().at(i)->SetValue(valFunc[i]);
     if (indexMagnitude == 2)
     for(int i = 0;i<4;i++) doc->GetTransformMagnitudeMl().at(i)->SetValue(valFunc[i]);
     if (indexMagnitude == 3)
     for(int i = 0;i<4;i++) doc->GetTransformMagnitudeMp().at(i)->SetValue(valFunc[i]);
}
//----------------------------------------------------------------------------------
void TransfMagnImpl::DefineTransformFunction(){
     GetDocumentFromDialog(comboBoxMagnitudeIndex); 
     GetTransformFunctionFromDocument(0);
     GetTransformFunctionFromDocument(1);
     GetTransformFunctionFromDocument(2);
     GetTransformFunctionFromDocument(3);
}
//----------------------------------------------------------------------------------
void TransfMagnImpl::GetTransformFunctionFromDocument(int indexMagnitude){    
     PropertyEnumValue* TransformMagnitudeFunction =
                    doc->GetTransformMagnitudeFunction().at(indexMagnitude);
     int indexFunct = TransformMagnitudeFunction->GetValue().toInt();

     PropertyTransformFunction * MagnitudeM1 = 0;
     PropertyTransformFunction * MagnitudeM2 = 0;
     PropertyTransformFunction * MagnitudeM3 = 0;
     PropertyTransformFunction * MagnitudeM4 = 0;

     QString magn = "";

     if (indexMagnitude == 0) {
     magn = "M1";
     MagnitudeM1 =  doc->GetTransformMagnitudeMb().at(0);
     MagnitudeM2 =  doc->GetTransformMagnitudeMb().at(1);
     MagnitudeM3 =  doc->GetTransformMagnitudeMb().at(2);
     MagnitudeM4 =  doc->GetTransformMagnitudeMb().at(3);
     }
     if (indexMagnitude == 1) {
     magn = "M2";
     MagnitudeM1 =  doc->GetTransformMagnitudeMs().at(0);
     MagnitudeM2 =  doc->GetTransformMagnitudeMs().at(1);
     MagnitudeM3 =  doc->GetTransformMagnitudeMs().at(2);
     MagnitudeM4 =  doc->GetTransformMagnitudeMs().at(3);
     }
     if (indexMagnitude == 2) {
     magn = "M3";
     MagnitudeM1 =  doc->GetTransformMagnitudeMl().at(0);
     MagnitudeM2 =  doc->GetTransformMagnitudeMl().at(1);
     MagnitudeM3 =  doc->GetTransformMagnitudeMl().at(2);
     MagnitudeM4 =  doc->GetTransformMagnitudeMl().at(3);
     }
     if (indexMagnitude == 3) {
     magn = "M4";
     MagnitudeM1 =  doc->GetTransformMagnitudeMp().at(0);
     MagnitudeM2 =  doc->GetTransformMagnitudeMp().at(1);
     MagnitudeM3 =  doc->GetTransformMagnitudeMp().at(2);
     MagnitudeM4 =  doc->GetTransformMagnitudeMp().at(3);
     }
     vector<int> priorityFind;
     vector<QString> valFunc;valFunc.resize(4);
     if (MagnitudeM1->valIsOn) priorityFind.push_back(MagnitudeM1->valIndexMagnitude);
     valFunc[0] = QString("%1@%2@%3@%4").arg(MagnitudeM1->valD)
                                        .arg(MagnitudeM1->valA)
                                        .arg(MagnitudeM1->valB)
                                        .arg(MagnitudeM1->valC);

     if (MagnitudeM2->valIsOn) priorityFind.push_back(MagnitudeM2->valIndexMagnitude);
     valFunc[1] = QString("%1@%2@%3@%4").arg(MagnitudeM2->valD)
                                        .arg(MagnitudeM2->valA)
                                        .arg(MagnitudeM2->valB)
                                        .arg(MagnitudeM2->valC);

     if (MagnitudeM3->valIsOn) priorityFind.push_back(MagnitudeM3->valIndexMagnitude);
     valFunc[2] = QString("%1@%2@%3@%4").arg(MagnitudeM3->valD)
                                        .arg(MagnitudeM3->valA)
                                        .arg(MagnitudeM3->valB)
                                        .arg(MagnitudeM3->valC);

     if (MagnitudeM4->valIsOn) priorityFind.push_back(MagnitudeM4->valIndexMagnitude);
     valFunc[3] = QString("%1@%2@%3@%4").arg(MagnitudeM4->valD)
                                        .arg(MagnitudeM4->valA)
                                        .arg(MagnitudeM4->valB)
                                        .arg(MagnitudeM4->valC);
    QString strmes = "";
    if (indexMagnitude == 0) {
       Catalog cat;
        if (type == Tool) strmes += cat.GetCatalogInfoMessage(MainWindow::self->GetCurrentCatalogName());
        if (type == Execute) {
		  strmes += "Current catalog ";
			strmes += cat.GetCatalogShortInfoMessage(MainWindow::self->GetOpenBinaryFileName());
		}
          }

    int size = (int) priorityFind.size();

    if(!priorityFind.empty()){
     strmes += "Magnitude " + magn + " equals to";
     TransformFunc *f = new TransformFunc(priorityFind,valFunc,indexMagnitude);
     if (indexFunct == 0 ) {
        if (size > 1) strmes += " Priority Value";
        ptrFunct.push_back( &TransformFunc::Value); 
        }
     if (indexFunct == 1 )  {
        if (size > 1) strmes += " Maximum";
        ptrFunct.push_back( &TransformFunc::Maximum); 
        }
     if (indexFunct == 2 ) {
        if (size > 1) strmes += " Minimum";
        ptrFunct.push_back( &TransformFunc::Minimum); 
        }
     if (indexFunct == 3 ) {
        if (size > 1) strmes += " Average";
        ptrFunct.push_back( &TransformFunc::Average);
        }
     funct.push_back(f);

     QString mag = "";
     for(int i = 0; i < size;i++) {
         if(priorityFind[i] == 0)mag += " f(m1),";
         if(priorityFind[i] == 1)mag += " f(m2),";
         if(priorityFind[i] == 2)mag += " f(m3),";
         if(priorityFind[i] == 3)mag += " f(m4),";
     }
     mag.replace(mag.lastIndexOf(','),1," ");
     if (size > 1) strmes += " { " + mag + " }\n";
     else strmes += " " + mag + " \n";
//  prepare message for message page
     for(int i = 0; i < size;i++) {
     QStringList p = valFunc[priorityFind[i]].split ('@');
     QString mag = ""; 
     if(priorityFind[i] == 0)mag = "m1";
     if(priorityFind[i] == 1)mag = "m2";
     if(priorityFind[i] == 2)mag = "m3";
     if(priorityFind[i] == 3)mag = "m4";
     QString fun = "f(" + mag + ")" + " = " ;
     strmes += fun + p.at(1) + " * " + mag + " + " + p.at(2) +
               " if " + mag + " greater or equal to " + p.at(0);
     strmes += " else " + fun + p.at(3) + "\n";
    }
    message += strmes + "\n";

}
}
//----------------------------------------------------------------------------------
bool TransfMagnImpl::TransformMagnitude() {
  if(funct.isEmpty()) {
    error(QString("Nothing to do! Even one transform function for magnitude must be define."),"");
    return false;}
    QApplication::setOverrideCursor(Qt::WaitCursor);

    DoubleIntMultiMap *indexCat =  CatTable::tSelf->GetRecordCatalogIndex();
    fstream *in =  CatTable::tSelf->GetFstream();
    RECORD record;
    in->clear();
    ItrMultiMap position = indexCat->begin();
    progressBar->setRange(1,indexCat->size());
    while (position != indexCat->end()) {
    int inpos = position->second +1;
    in->seekg(inpos*sizeof(RECORD),ios::beg);
    in->read((char*)&record,sizeof(RECORD ));
    vector <PDF>::iterator it = ptrFunct.begin();
    foreach (TransformFunc* fun, funct) fun->SetRecord(record);
    foreach (TransformFunc* fun, funct) {
      PDF ff =  *it;
      (fun->*ff)();
      it++;

    }

    foreach (TransformFunc* fun, funct) record.m[fun->indexMagnitude] = fun->magnitude;

    //in->seekg(-sizeof(RECORD),ios::cur);
    in->seekg(inpos*sizeof(RECORD),ios::beg);
    in->write((char*)&record,sizeof(RECORD));
    position++;
    
    if ((inpos/100)*100 == inpos ) progressBar->setValue(inpos);
  }
 
  progressBar->setValue(CatTable::tSelf->GetCatSize());
  QApplication::restoreOverrideCursor();
  return true;
}


//----------------------------------------------------------------------------------
void TransfMagnImpl::ProgressBarChanged(int value){
if (value > 1) {
 textLabel->setText(tr("Record: %1").arg(value));
 textLabel->repaint ();
}
}
//----------------------------------------------------------------------------------
