#include <QtGui>
#include "mainwindow.h"
#include "viewcatalogimpl.h"
#include "document.h"
#include "default.h"
#include "undo.h"
#include "convert.h"
#include "tableproperty.h"
#include "transmagimpl.h"
#include "bottomwindow.h"
#include "subcatalog.h"
#include "append.h"
#include "rangelimit.h"
#include "profile.h"
#include "findrecord.h"

//void eraseMinusPos(pair<const double, int> &x) {if(x.second <= -1) x.second = 0;}
CatTable *CatTable::tSelf = 0;
const int WRITE_CATALOG_BUFFER_SIZE = 1000;
struct recSortNumberComp : public binary_function < QString  const & , QString  const &, bool> {
        bool operator()(QString  const & x, QString  const & y) {
        QStringList xlist = x.split( "@");
        QStringList ylist = y.split( "@");
	      if(xlist[1].toInt() > ylist[1].toInt())
	        return false; else return true; }
};

ViewCatalogImpl::~ViewCatalogImpl(){
    delete cattable;
}
ViewCatalogImpl::ViewCatalogImpl(QWidget *parent,int numRows)
 : QDialog(0)
{
    QFont f = parent->font();
    setFont( f );
    setupUi(this);
    gridLayout->removeWidget(table);
    table->hide();
    cattable = new CatTable(numberCatalogColumns(),numRows,this);
    gridLayout->addWidget(cattable, 0, 0, 1, 1);
    escAct =  new QAction(this);
    escAct->setShortcut(Qt::Key_Escape);
    addAction (escAct);
    connect(escAct, SIGNAL(triggered()), this, SLOT(escAction()));	
    isCatLoaded = false;


}

void ViewCatalogImpl::escAction(){
cattable->EscTableLineEditor();
}
void ViewCatalogImpl::zoom(){
   QFont f = MainWindow::self->font();
   setFont( f );
   cattable->zoom();
}
//---------------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------
void CatTable::RemoveOwnPoiners(){
    if (in != 0) in->close();delete in; in = 0;
    if (inSecond != 0) { inSecond->close();delete inSecond;inSecond = 0; }
    if(!undoList.empty()) qDeleteAll(undoList);
    if (selCopy != 0) {delete selCopy; selCopy = 0;}
    if (!index.empty()) index.clear();
    if(doubleDialog != 0) {delete doubleDialog; doubleDialog = 0;}
    if(catalDialog != 0) {delete catalDialog; catalDialog = 0;}
    if(appendDialog != 0) {delete appendDialog; appendDialog = 0;}
    if(rangeDialog != 0) {delete rangeDialog; rangeDialog = 0;}
    if(findDialog != 0) {delete findDialog; findDialog = 0;}
    if(subCatalogDialog != 0) {delete subCatalogDialog; subCatalogDialog = 0;}
    if (!errorvec.empty()) errorvec.clear();
    if (!dupvec.empty()) dupvec.clear(); 
    if (progress != 0 ) {delete progress; progress = 0;}
}
CatTable::~CatTable() {
RemoveOwnPoiners();
}
//=====================================================================
CatTable::CatTable(int numCols,int numRows,QWidget* parent)
    : QTableView(),NumCols( numCols)
{
    QFont f = parent->font();
    setFont( f );
    tSelf = this;
	parentWindow = (ViewCatalogImpl*)parent;
    curUndoIndex = -1;
    errorSortRange = true;
    selCopy = 0;
    progress = 0;
    doubleDialog = 0;
    catalDialog = 0;
    appendDialog = 0;
    subCatalogDialog = 0;
    rangeDialog = 0;
    findDialog = 0;
    in = 0;
    inSecond=0;

    firstCorrection = true;
    checkRangeLimits = false;
	setSelectionMode(QAbstractItemView::NoSelection);
   // SelectionMode mod = selectionMode ();
   // QMessageBox::warning(this, "",QString ("mode %1").arg(mod));	
    model = new PropertyModel(numRows, numCols,this);

    setAlternatingRowColors (TRUE);
    setModel(model);
    delegate = new PropertyDelegate;

    setItemDelegate(delegate);
    selectModel = selectionModel();  

    parentWindow = (ViewCatalogImpl*)parent;
 
    ResizeHorizontalHeader(horizontalHeader ());
 //   QMessageBox::warning(this, "",QString ("numRows %1").arg(numRows));	
    ResizeRows();
   verticalHeader ()->setContextMenuPolicy (Qt::CustomContextMenu);
  
   connect( this, SIGNAL( clicked(QModelIndex) ),
	     this, SLOT( clickedRowCol(QModelIndex ) ) );
   connect( verticalHeader (), SIGNAL(  sectionClicked(int)  ),
	     this, SLOT( clickedVerticalHeader(int ) ) );

   connect( verticalHeader (), SIGNAL(  customContextMenuRequested ( const QPoint & )  ),
	     this, SLOT( verticalContextMenu ( const QPoint & ) ) );

   connect(parentWindow->GetToolButtonHome(), SIGNAL(clicked()), this, SLOT(ButtonHome()));
   connect(parentWindow->GetToolButtonEnd() , SIGNAL(clicked()), this, SLOT(ButtonEnd()));
   connect(parentWindow->GetToolButtonDown(), SIGNAL(clicked()), this, SLOT(ButtonDown()));
   connect(parentWindow->GetToolButtonUp()  , SIGNAL(clicked()), this, SLOT(ButtonUp()));
   

    

}
//=====================================================================
void CatTable::SetCatalog(FDocument*dc,SettingDocument*setDc,
	                      MainWindow::AutoCheckErrors act){
    doc = dc; setDoc = setDc;
    ViewBufSize = setDoc->GetWorkSpaceSize()->GetValue().toInt();
    setFont(parentWindow->font());
    tSelf = this;
	setSelectionMode(QAbstractItemView::ExtendedSelection);
	setEditTriggers (QAbstractItemView::DoubleClicked);
    curUndoIndex = -1;
    if (act == MainWindow::check ) errorRange = true;
    if (act == MainWindow::ignore) errorRange = false;
    errorSortRange = true;
	//-----------------------------------------------------------
	RemoveOwnPoiners(); 	
	//-----------------------------------------------------------
    firstCorrection = true;
    checkRangeLimits = false;
    model->removeRows(0,model->rowCount());
    scrollPosStart = 0;
    CheckErrorCatalog();
    InitBufferTable();
    selectRow (0);
	ResizeHorizontalHeader(horizontalHeader ());
	parentWindow->isCatalogLoaded() = true;
}
void CatTable::ClearCatalogTable(){
	setSelectionMode(QAbstractItemView::NoSelection);
	setEditTriggers (QAbstractItemView::NoEditTriggers);
	model->removeRows(0,model->rowCount());	
	ViewBufSize = setDoc->GetWorkSpaceSize()->GetValue().toInt();
	model->insertRows(0,ViewBufSize);
	ResizeRows();
	ResizeHorizontalHeader(horizontalHeader ());
    parentWindow->isCatalogLoaded() = false;
}
//=====================================================================
void CatTable::SetViewBufSize(int num) {
if (num == ViewBufSize ) return;
ViewBufSize = num;
RestoreWorkSpaceAfterChangeIndex();
}
void CatTable::ResizeHorizontalHeader(QHeaderView *hd)
{
QFont fnOld = font();
QFont fn = fnOld;
fn.setBold(true); 

hd->resizeSection(numberCatalogYear(),fontMetrics().width(fontmetricCatalogYear()) );
hd->resizeSection(numberCatalogMonth(),fontMetrics().width(fontmetricCatalogMonth()));
hd->resizeSection(numberCatalogDay(),fontMetrics().width(fontmetricCatalogDay()));
hd->resizeSection(numberCatalogHour(),fontMetrics().width(fontmetricCatalogHour()));
hd->resizeSection(numberCatalogMinute(),fontMetrics().width(fontmetricCatalogMinute()));
hd->resizeSection(numberCatalogSecond(),fontMetrics().width(fontmetricCatalogSecond()));
hd->resizeSection(numberCatalogLatitude(),fontMetrics().width(fontmetricCatalogLatitude()));
hd->resizeSection(numberCatalogLongitude(),fontMetrics().width(fontmetricCatalogLongitude()));
hd->resizeSection(numberCatalogDepth(),fontMetrics().width(fontmetricCatalogDepth()));
hd->resizeSection(numberCatalogMb(),fontMetrics().width(fontmetricCatalogMb()));
hd->resizeSection(numberCatalogMs(),fontMetrics().width(fontmetricCatalogMs()));
hd->resizeSection(numberCatalogMl(),fontMetrics().width(fontmetricCatalogMl()));
hd->resizeSection(numberCatalogMp(),fontMetrics().width(fontmetricCatalogMp()));
hd->resizeSection(numberCatalogIntensity(),fontMetrics().width(fontmetricCatalogIntensity()));

hd->setStretchLastSection(TRUE);
setFont(fnOld);  
}
void CatTable::zoom(){
   QFont f = MainWindow::self->font();
   setFont( f );
   ResizeHorizontalHeader(horizontalHeader ());
   ResizeRows();
}
void CatTable::ResizeRows()
{
   for (int row = 0; row < model->rowCount(); ++row)  resizeRowToContents(row);
}

void CatTable::SetOutOfRangeParameters(){

}

QStringList CatTable::CheckErrorRecord(RECORD record,int numrec){
    QStringList err;
    //Note that QDate should not be used for date calculations for dates prior to the introduction of the Gregorian calendar.
    //This calendar was adopted by England from the 14th September 1752

    bool errordate = false;
    QString tmp;
    if ( record.yr < minCatalogYear() || record.yr > maxCatalogYear()) {
	tmp = QString(errorRangePattern()).arg(nameErrorYear()).arg(headerCatalogYear());
	tmp += QString(": value %1 , range [ %2 , %3 ]").arg(record.yr).
                      arg(minCatalogYear()).arg(maxCatalogYear());
	err.append ( tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogYear())  );
	errordate = true;
    }
    if (record.mo < minCatalogMonth() || record.mo > maxCatalogMonth()) {
	tmp = QString(errorRangePattern()).arg(nameErrorMonth()).arg(headerCatalogMonth());
	tmp += QString(": value %1 , range [ %2 , %3 ]").arg(record.mo).
                      arg(minCatalogMonth()).arg(maxCatalogMonth());
	err.append (  tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogMonth()) );
	errordate = true;
    }
    if (record.da < minCatalogDay() || record.da > maxCatalogDay()) {
	tmp = QString(errorRangePattern()).arg(nameErrorDay()).arg(headerCatalogDay());
	tmp += QString(": value %1 , range [ %2 , %3 ]").arg(record.da).
                      arg(minCatalogDay()).arg(maxCatalogDay());
	err.append ( tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogDay())  );
	errordate = true;
    }
    if (record.yr >1752 && !errordate) {
	if(!QDate::isValid( record.yr,record.mo,record.da)) {
	    tmp = QString(errorValidPattern()).arg(nameErrorDate());
            tmp += QString(": value %1/%2/%3").arg( record.yr).arg(record.mo).arg(record.da);
	    err.append ( tmp + QString("@%1@%2@%3@%4").arg(numrec).arg(numberCatalogYear()).arg(numberCatalogMonth()).arg(numberCatalogDay())   );
	}
    }
	//===============================================================================
    if (record.hr < minCatalogHour() || record.hr > maxCatalogHour()) {
	tmp = QString(errorRangePattern()).arg(nameErrorHour()).arg(headerCatalogHour());
	tmp += QString(": value %1 , range [ %2 , %3 ]").arg(record.hr).
                      arg(minCatalogHour()).arg(maxCatalogHour());
	err.append ( tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogHour())  );
	errordate = true;
    }	
	
    if (record.mn < minCatalogMinute() || record.mn > maxCatalogMinute()) {
	tmp = QString(errorRangePattern()).arg(nameErrorMinute()).arg(headerCatalogMinute());
	tmp += QString(": value %1 , range [ %2 , %3 ]").arg(record.mn).
                      arg(minCatalogMinute()).arg(maxCatalogMinute());
	err.append ( tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogMinute())  );
	errordate = true;
    }		
	float sec = record.sc/secondMagnification();
    if (sec < minCatalogSecond() || sec > maxCatalogSecond()) {
	tmp = QString(errorRangePattern()).arg(nameErrorSecond()).arg(headerCatalogSecond());
	tmp += QString(": value %1 , range [ %2 , %3 ]").arg(sec).
                      arg(minCatalogSecond()).arg(maxCatalogSecond());
	err.append ( tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogSecond())  );
	errordate = true;
    }		
	//===============================================================================
    Catalog cat;
    float lat,lon,dep,mb,ms,ml,mp;
    cat.GetLatLonDepthMagnitude(record,lat,lon,dep,mb,ms,ml,mp);
    float min = 0; float max = 0;
    if(checkRangeLimits) {
        min = doc->GetRangeLatitudeMin()->GetValue().toFloat();
        max = doc->GetRangeLatitudeMax()->GetValue().toFloat();
    }   else {
        min = minCatalogLatitude();
        max = maxCatalogLatitude();
    }
    if (lat < min || lat > max ) {
    tmp = QString(errorRangePattern()).arg(nameErrorLatitude()).arg(headerCatalogLatitude());
    tmp += QString(": value %1 , range [ %2 , %3 ]").arg(lat).
                      arg(min).arg(max);
    err.append ( tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogLatitude())  );
    }

    if(checkRangeLimits) {
        min = doc->GetRangeLongitudeMin()->GetValue().toFloat();
        max = doc->GetRangeLongitudeMax()->GetValue().toFloat();
    }   else {
        min = minCatalogLongitude();
        max = maxCatalogLongitude();
    }
 
    if (lon < min || lon > max ) {
    tmp = QString(errorRangePattern()).arg(nameErrorLongitude()).arg(headerCatalogLongitude());
    tmp += QString(": value %1 , range [ %2 , %3 ]").arg(lon).
                      arg(min).arg(max);
    err.append ( tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogLongitude())  );
    }
    if(checkRangeLimits) {
        min = doc->GetRangeDepthMin()->GetValue().toFloat();
        max = doc->GetRangeDepthMax()->GetValue().toFloat();
    }   else {
        min = minCatalogDepth();
        max = maxCatalogDepth();
    }

    if (dep < min || dep > max ) {
    tmp = QString(errorRangePattern()).arg(nameErrorDepth()).arg(headerCatalogDepth());
    tmp += QString(": value %1 , range [ %2 , %3 ]").arg(dep).
                      arg(min).arg(max);
    err.append ( tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogDepth())  );
    }
    if(checkRangeLimits) {
        min = doc->GetRangeMbMin()->GetValue().toFloat();
        max = doc->GetRangeMbMax()->GetValue().toFloat();
    }   else {
        min = minCatalogMb();
        max = maxCatalogMb();
    }
 
    if (mb < min || mb > max ) {
    tmp = QString(errorRangePattern()).arg(nameErrorMb()).arg(headerCatalogMb());
    tmp += QString(": value %1 , range [ %2 , %3 ]").arg(mb).
                      arg(min).arg(max);
    err.append ( tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogMb())  );
    }

    if(checkRangeLimits) {
        min = doc->GetRangeMsMin()->GetValue().toFloat();
        max = doc->GetRangeMsMax()->GetValue().toFloat();
    }   else {
        min = minCatalogMs();
        max = maxCatalogMs();
    }
    if (ms < min || ms > max ) {
    tmp = QString(errorRangePattern()).arg(nameErrorMs()).arg(headerCatalogMs());
    tmp += QString(": value %1 , range [ %2 , %3 ]").arg(ms).
                      arg(min).arg(max);
    err.append ( tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogMs())  );
    }

    if(checkRangeLimits) {
        min = doc->GetRangeMlMin()->GetValue().toFloat();
        max = doc->GetRangeMlMax()->GetValue().toFloat();
    }   else {
        min = minCatalogMl();
        max = maxCatalogMl();
    }
    if (ml < min || ml > max ) {
    tmp = QString(errorRangePattern()).arg(nameErrorMl()).arg(headerCatalogMl());
    tmp += QString(": value %1 , range [ %2 , %3 ]").arg(ml).
                      arg(min).arg(max);
    err.append ( tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogMl())  );
    }

    if(checkRangeLimits) {
        min = doc->GetRangeMpMin()->GetValue().toFloat();
        max = doc->GetRangeMpMax()->GetValue().toFloat();
    }   else {
        min = minCatalogMp();
        max = maxCatalogMp();
    }
   if (mp < min || mp > max ) {
    tmp = QString(errorRangePattern()).arg(nameErrorMp()).arg(headerCatalogMp());
    tmp += QString(": value %1 , range [ %2 , %3 ]").arg(mp).
                      arg(min).arg(max);
    err.append ( tmp + QString("@%1@%2").arg(numrec).arg(numberCatalogMp())  );
    }

    return err;
}

bool CatTable::CheckErrorCatalog(){
 if (errorRange) MainWindow::self->bottomWindow->OpenProblemsProtocolPage();
//if(errorRange) qWarning("CatTable::CheckErrorCatalog errorRange TRUE"); else qWarning("CatTable::CheckErrorCatalog errorRange FALSE");

  QApplication::setOverrideCursor(Qt::WaitCursor); 
  QString    name = MainWindow::self->GetOpenBinaryFileName();
  RECORD record;

  if (in != 0) { in->close();  delete in;}

  in = new fstream (name.toAscii(),ios::binary | ios::in | ios::out);
in->clear();
  if (!(*in)) {QApplication::restoreOverrideCursor();error(QString("cannot open input file"),name);return false;}
    in->seekg(0,ios::beg);
    in->read((char*)&catSize,sizeof(int));
  in->seekg(sizeof(RECORD),ios::beg);
  double datetime = 0;
//QMessageBox::information(this,"",QString("3:> catSize %1 ViewBufSize %2")
//	.arg(catSize).arg(ViewBufSize));

  if (!index.empty()) index.clear();
  if (!errorvec.empty()) errorvec.clear();
  int numErr = 0;
  int numRec = 0;
  bool stopFind = false;

    while(in->read((char*)&record,sizeof(RECORD ))) {
	    if(in->fail()){QApplication::restoreOverrideCursor();error(QString("An reading error has occurred"),name);return false;}
	    datetime =  DateTime(record);
	    index.insert(make_pair(datetime,numRec));
            if(errorRange) {
	        QStringList err = CheckErrorRecord(record,numRec+1);
	        if (!err. isEmpty ()) {
	          for ( QStringList::Iterator it = err.begin(); it != err.end(); ++it )
		  {
                    if ( numErr > setDoc->GetMaxErrorLines()->GetValue().toInt() ) {stopFind = true; break;}
                    errorvec.push_back(*it); numErr++;             
                  }
                }
            if (stopFind) break;
            }
//qWarning("Point 3 %d",numRec);
            numRec++;
    }
//QMessageBox::information(this,"",QString("4:> catSize %1 ViewBufSize %2 sizeind %3")
//	.arg(catSize).arg(ViewBufSize).arg((int)index.size()));	
   if (!errorvec.empty()) {
      if (stopFind)  MainWindow::self->bottomWindow->comtable->InitCommentTable(&errorvec,ComTable::ParamFirst);
      else
      MainWindow::self->bottomWindow->comtable->InitCommentTable(&errorvec,ComTable::ParamAll);
   }
   else errorRange = false;

  if (!errorRange) CheckSortCatalog();

  QApplication::restoreOverrideCursor();
  if(!errorRange) MainWindow::self->FileActsShowHide(true);
   else MainWindow::self->FileActsShowHide(false);


  return true;
}
QString CatTable::GetCatalogInfo(){
	QString empty = QString();
if (model->rowCount() >0)  {
	QString infoCatalog = "";
	QString infoViewSegment = "";
    RECORD record;
    Catalog cat;
    in->clear();
    in->seekg(sizeof(RECORD),ios::beg);
    in->read((char*)&record,sizeof(RECORD ));
    QString time = cat.getQStringTime(record);
    QString time1 = GetRowRecordTime (0);

	infoCatalog += 
	QString("Current catalog:\n\nFirst record:\ndate: %1\n").arg(time);
	infoViewSegment += 
	QString("View segment:\n\nFirst record:\ndate: %1  number: %2\n").arg(time1).arg(GetRowRecordNumber (0));
	
//	parentWindow->GetTextLabelBegin()->setText( QString("%1").arg(time));
//    parentWindow->GetTextLabelStart()->setText( QString("%1").arg(1));
//    parentWindow->GetLabelWorkSpaceUp()->setText( QString("%1\n%2").arg(GetRowRecordNumber (0)).arg(time1));

    time = GetRowRecordTime (model->rowCount()-1);
    in->seekg(-((int)sizeof(RECORD)),ios::end);
    in->read((char*)&record,sizeof(RECORD ));
    time1 = cat.getQStringTime(record);

//    parentWindow->GetTextLabelEnd()->setText( QString("%1\nlast: %2").arg(time).arg(time1));
//    parentWindow->GetTextLabelFinish()->setText( QString("%1\nlast: %2").arg(model->rowCount()-1).arg(catSize));

	infoCatalog += 
	QString("Last record:\ndate: %1\nCatalog size: %2\n").arg(time1).arg(catSize);
	infoViewSegment += 
	QString("Last record:\ndate: %1  number: %2\n").arg(time).arg(GetRowRecordNumber (model->rowCount()-1));
    infoViewSegment += QString("Segment size: %1").arg(ViewBufSize);
//   parentWindow->GetTextLabelEnd()->setText( QString("%1").arg(time1));
//    parentWindow->GetTextLabelFinish()->setText( QString("%1").arg(catSize));
//    parentWindow->GetLabelWorkSpaceDown()->setText(QString("%1\n%2").arg(time).arg(GetRowRecordNumber (model->rowCount()-1)) );
   
    return infoCatalog + "\n\n" + infoViewSegment;
} else return QString();

}

void CatTable::RemoveRowsDelLists(){
    if (model->rowCount() > 0  )   model->removeRows(0,model->rowCount());
    if(!model->ipList.empty() ){ qDeleteAll(model->ipList); model->ipList.clear();}
    if(!undoList.empty() ){ qDeleteAll(undoList); undoList.clear(); curUndoIndex = -1;}
    MainWindow::self->redoAct->setEnabled (FALSE);  MainWindow::self->undoAct->setEnabled (FALSE);
    MainWindow::self->viewBufferModified  = false;
}
bool CatTable::InitBufferTable(){
//qWarning("CatTable::InitBufferTable()"); 
    RECORD record;
    Catalog cat;
    QString    name = MainWindow::self->GetOpenBinaryFileName();
    in->clear();
    int i = 0;
    in->seekg(0,ios::beg);
    in->read((char*)&catSize,sizeof(int));

                         RemoveRowsDelLists();
                         if (ViewBufSize > catSize) ViewBufSize = catSize;
                         model->insertRows(0,ViewBufSize); 

    in->seekg(0,ios::beg);
    in->seekg((scrollPosStart + 1)*sizeof(RECORD),ios::beg);
    while( i < ViewBufSize) {
	     in->read((char*)&record,sizeof(RECORD ));
       if(in->eof()) break;
       if(in->fail()){QApplication::restoreOverrideCursor();error(QString("An reading error has occurred (rec %1)").arg(scrollPosStart + i + 1),name);return false;}
	     QString temp = cat.getFieldsFromRecord(record);
       model->SetRow(i,scrollPosStart + i + 1,temp);
       i++;
    }
    ResizeRows();
	if (findDialog != 0 ) initFindDialog(false);
    SetEnableArrowKeys();
	
MainWindow::self->viewInfoAct->setFont(font());
MainWindow::self->viewInfoAct->setText (GetCatalogInfo());

    return true;
}
void CatTable::SetEnableArrowKeys(){
     parentWindow->GetToolButtonHome()->setEnabled (TRUE);
     parentWindow->GetToolButtonUp()->setEnabled (TRUE);
     parentWindow->GetToolButtonEnd()->setEnabled (TRUE);
     parentWindow->GetToolButtonDown()->setEnabled (TRUE);
if (scrollPosStart  == 0 && scrollPosStart + ViewBufSize == catSize) {
     parentWindow->GetToolButtonUp()->setEnabled (FALSE);
     parentWindow->GetToolButtonDown()->setEnabled (FALSE);
return;
}
if (scrollPosStart  == 0) {
     parentWindow->GetToolButtonHome()->setEnabled (FALSE);
     parentWindow->GetToolButtonUp()->setEnabled (FALSE);
}

if (scrollPosStart + ViewBufSize == catSize) {
     parentWindow->GetToolButtonEnd()->setEnabled (FALSE);
     parentWindow->GetToolButtonDown()->setEnabled (FALSE);
}


}
bool CatTable::InsertRowTable(){

              RemoveRowsDelLists();
              model->insertRows(0,ViewBufSize);
    in->clear();
    int i = 0;
    RECORD record;
    Catalog cat;
    in->seekg(0,ios::beg);
    in->seekg((scrollPosStart + 1)*sizeof(RECORD),ios::beg); 
    for (i = 0; i < ViewBufSize ;i++) {     
	   in->read((char*)&record,sizeof(RECORD ));
	   if(in->eof()) break;
	   if(in->fail()){ QApplication::restoreOverrideCursor();
     error(QString("An reading error has occurred (rec %1)").arg(scrollPosStart + i + 1),"");return false;}
     QString temp = cat.getFieldsFromRecord(record);
     model->SetRow(i,scrollPosStart + i + 1,temp);
    }
    ResizeRows();   
    SetEnableArrowKeys();   
    if (findDialog != 0 ) initFindDialog(false);
MainWindow::self->viewInfoAct->setFont(font());
MainWindow::self->viewInfoAct->setText (GetCatalogInfo());	
    return true;
}

QString CatTable::GetRowRecordTime (int row) {
QStringList tmp = model->GetTableData(row);
return
	    QString("%1/%2/%3 %4:%5:%6").arg(tmp.at(numberCatalogYear())).arg(tmp.at(numberCatalogMonth()))
                                  .arg(tmp.at(numberCatalogDay())).arg(tmp.at(numberCatalogHour()))
                                  .arg(tmp.at(numberCatalogMinute())).arg(tmp.at(numberCatalogSecond()));
}
int CatTable::GetRowRecordNumber (int row){
return model->GetHeaderRowRecordNumber(row);
}

void CatTable::clickedRowCol(QModelIndex ){
//qWarning("CatTable::clickedRowCol %d  %d",currentIndex ().row (),index.row());

}
void CatTable::clickedVerticalHeader(int ){


}
void CatTable::SetContextMenu ( const QPoint & pos )
{
  QItemSelection    isel =  selectModel->selection ();
  if(isel.empty()) return;
    QMenu menu(this);
    menu.addAction(MainWindow::self->cutAct);
    menu.addAction(MainWindow::self->copyAct);
    menu.addAction(MainWindow::self->pasteAct);
    menu.addSeparator();
    menu.addAction(MainWindow::self->deleteAct);
    menu.addAction(MainWindow::self->insertRowAct);
    menu.exec(pos);
}
void CatTable::verticalContextMenu ( const QPoint &  )
{
    SetContextMenu (cursor ().pos());
}
void CatTable::contextMenuEvent(QContextMenuEvent *event)
{
    SetContextMenu (event->globalPos());
}

void CatTable::ButtonHome(){
   if (maybeSave()) { 
    scrollPosStart = 0;
    if (ViewBufSize < catSize) InitBufferTable();
    selectRow (0);

}
}
void CatTable::ButtonEnd(){
   if (maybeSave()) {
    if (ViewBufSize < catSize  ) {
    scrollPosStart = catSize  - ViewBufSize;
    InitBufferTable();
    } 
    selectRow (ViewBufSize -1);

   
}
}

void CatTable::ButtonUp(){
   int  cellpos = 0;
   if (maybeSave()) {
   if ( scrollPosStart  <= 0 ) { scrollPosStart  =0; return;}

	if (scrollPosStart - (ViewBufSize -1) <= 0) scrollPosStart =0;
	else scrollPosStart = scrollPosStart - (ViewBufSize -1);
	cellpos = ViewBufSize -1;
    InitBufferTable();
    selectRow (cellpos);
}
}

void CatTable::ButtonDown(){
  int  cellpos = 0;
  if (maybeSave()) {
	  if ( scrollPosStart + ViewBufSize - 1 >= catSize-1) return;
     if ( scrollPosStart + 2*ViewBufSize - 1 > catSize-1) scrollPosStart = catSize - ViewBufSize;
      else
	    scrollPosStart =  scrollPosStart + ViewBufSize -1;
	    cellpos = 0;  
  InitBufferTable();
  selectRow (cellpos);
}

}
bool CatTable::maybeSave()
{
    CloseOpenedTableLineEditor();
    if (MainWindow::self->viewBufferModified) {

    if(ViewBufSize == catSize && model->rowCount() == 0 )  {
    error(QString("After saving catalog will not contain records.Save canceled."),"");
    MainWindow::self->viewBufferModified = false;
    RestoreWorkSpaceAfterChangeIndex();
    return true;
    }


	int ret = QMessageBox::warning(this, mainWindowCaption(),
				       QString("The catalog view segment has been modified.\n Do you want to save your changes?"),
				       QMessageBox::Yes | QMessageBox::Default,
				       QMessageBox::No,
				       QMessageBox::Cancel | QMessageBox::Escape);
	if (ret == QMessageBox::Yes) return save();	
  if (ret == QMessageBox::Cancel) return false;
  if (ret == QMessageBox::No) {  RestoreWorkSpaceAfterChangeIndex(); return true;  }

    }
    return true;
}
RECORD CatTable::GetRecordFromFileds(int row ){
    QStringList tmp = model->GetTableData(row);
    RECORD record;
    record.yr = tmp.at(numberCatalogYear() ).toShort();
    record.mo = tmp.at(numberCatalogMonth()).toShort();
    record.da = tmp.at(numberCatalogDay()).toShort();
    record.hr = tmp.at(numberCatalogHour()).toShort();
    record.mn = tmp.at(numberCatalogMinute()).toShort();

    float flt = tmp.at(numberCatalogSecond()).toFloat();
    if (flt < 0.0) record.sc = (short)((flt - 0.005)*secondMagnification());
    else   record.sc = (short)((flt + 0.005)*secondMagnification());

    flt = tmp.at(numberCatalogLatitude()).toFloat();
    if (flt < 0.0) record.fi = (int)((flt - 0.0005)*latitudeMagnification());
    else record.fi = (int)((flt + 0.0005)*latitudeMagnification());

    flt = tmp.at(numberCatalogLongitude()).toFloat();
    if (flt < 0.0) record.la = (int)((flt - 0.0005)*longitudeMagnification());
    else record.la = (int)((flt + 0.0005)*longitudeMagnification());

    flt = tmp.at(numberCatalogDepth()).toFloat();
    if (flt < 0.0) record.h = (short)((flt - 0.05)*depthMagnification());
    else  record.h = (short)((flt + 0.05)*depthMagnification());

    flt = tmp.at(numberCatalogMb()).toFloat();
    if (flt < 0.0)  record.m[0] = (short)((flt - 0.005)*magnitudeMagnification());
    else record.m[0] = (short)((flt + 0.005)*magnitudeMagnification());

    flt = tmp.at(numberCatalogMs()).toFloat();
    if (flt < 0.0) record.m[1] = (short)((flt - 0.005)*magnitudeMagnification());
    else record.m[1] = (short)((flt + 0.005)*magnitudeMagnification());

    flt = tmp.at(numberCatalogMl()).toFloat();
    if (flt < 0.0) record.m[2] = (short)((flt - 0.005)*magnitudeMagnification());
    else record.m[2] = (short)((flt + 0.005)*magnitudeMagnification());

    flt = tmp.at(numberCatalogMp()).toFloat();
    if (flt < 0.0) record.m[3] = (short)((flt - 0.005)*magnitudeMagnification());
    else record.m[3] = (short)((flt + 0.005)*magnitudeMagnification());

    record.inten = (tmp.at(numberCatalogIntensity()).toShort());
    return record;
} 
bool CatTable::save() {


    RECORD record;
    RECORD inirecord;
    QString filename  = MainWindow::self->GetOpenBinaryFileName();
    filename.replace( openBinaryCatalogfileName(), openTempBinaryCatalogfileName() );
 
    char buf[WRITE_CATALOG_BUFFER_SIZE*sizeof(RECORD)]="";

    int numBigBuffer =  scrollPosStart/WRITE_CATALOG_BUFFER_SIZE;
    int numRest = scrollPosStart - numBigBuffer*WRITE_CATALOG_BUFFER_SIZE;

    QApplication::setOverrideCursor(Qt::WaitCursor);
    ofstream outfile(filename.toAscii(), ios::binary | ios::trunc);
    if (!(outfile) ){ QApplication::restoreOverrideCursor();error(QString("cannot open temporary output file"),filename);return false;}
    in->clear();
    int numberNewRecord = catSize - ViewBufSize + model->rowCount();
	catSize = numberNewRecord;
//	QMessageBox::information(this,"",QString("1:> catSize %1 ViewBufSize %2 find %3  model->rowCount() %4 numberNewRecord %5")
//	.arg(catSize).arg(ViewBufSize).arg(scrollPosStart + ViewBufSize + 1).arg( model->rowCount()).arg(numberNewRecord));


	
//	QMessageBox::information(this,"",QString("2:> catSize %1 ViewBufSize %2 find %3  model->rowCount() %4 numberNewRecord %5")
//	.arg(catSize).arg(ViewBufSize).arg(scrollPosStart + ViewBufSize + 1).arg( model->rowCount()).arg(numberNewRecord));
	
	
    in->seekg(sizeof(int),ios::beg);
    quint32 magic;
    in->read((char*)&magic,sizeof(quint32));

    if(in->fail()){QApplication::restoreOverrideCursor();error(QString("An reading error has occurred (rec %1)").arg(1),"");return false;}

    outfile.write(buf,sizeof(RECORD));
    if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing error has occurred (rec %1)").arg(1),"");return false;}
    outfile.seekp(0,ios::beg);
    outfile.write((char*)&numberNewRecord,sizeof(int));
    if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing error has occurred (catalog size)"),"");return false;}
    outfile.write((char*)&magic,sizeof(quint32));
    if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing error has occurred (format indicator)"),"");return false;}
    outfile.seekp(sizeof(RECORD),ios::beg);
    in->seekg(sizeof(RECORD),ios::beg);

    for(int i = 0; i < numBigBuffer; i++) {
    in->read(buf,WRITE_CATALOG_BUFFER_SIZE*sizeof(RECORD));
    if(in->fail()){QApplication::restoreOverrideCursor();error(QString("An reading buffer error has occurred (buf #%1 * %2)").arg(i+1).arg(WRITE_CATALOG_BUFFER_SIZE),"");return false;}
    outfile.write(buf,WRITE_CATALOG_BUFFER_SIZE*sizeof(RECORD));
    if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing error has occurred (buf #%1 * %2)").arg(i+1).arg(WRITE_CATALOG_BUFFER_SIZE),"");return false;}

    }
    if (numRest > 0) {
    in->read(buf,numRest*sizeof(RECORD));
    if(in->fail()){QApplication::restoreOverrideCursor();error(QString("An reading buffer error has occurred (buf #%1 * %2)").arg(numBigBuffer+1).arg(numRest),"");return false;}
    outfile.write(buf,numRest*sizeof(RECORD));
    if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing error has occurred (buf #%1 * %2)").arg(numBigBuffer+1).arg(numRest),"");return false;}
    }


Catalog cat; QStringList inistr = QStringList(); QStringList str = QStringList(); QStringList catstr = QStringList();
for (int j = 0; j < ViewBufSize; j++) {
//-----------------------------------------------------------------------------------
in->read((char*)&inirecord,sizeof(RECORD ));
catstr << cat.getFieldsFromRecord(inirecord);
//qWarning("'%s' ",catstr.toAscii().data());
//------------------------------------------------------------------------------------
}
     for (int i = 0; i < model->rowCount(); ++i) {
      record = GetRecordFromFileds(i );
//-----------------------------------------------------------------------------------
str << cat.getFieldsFromRecord(record);
inistr << model->GetRowInitialValues(i);

//QMessageBox::information(this,"",QString("inistr '%1'\n str '%2'")
//	.arg(cat.getFieldsFromRecord(record)).arg(model->GetRowInitialValues(i)));
//qWarning("ini '%s' str '%s' ",inistr.toAscii().data(),str.toAscii().data());
//-----------------------------------------------------------------------------------
      outfile.write((char*)&record,sizeof(RECORD));
      if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing  buffer error has occurred (row %1)").arg(i+1),"");return false;}
     }

       SetProtocolInfo(catstr,inistr,str);
//	QMessageBox::information(this,"",QString("catSize %1 ViewBufSize %2 find %3  model->rowCount() %4 numberNewRecord %5")
//	.arg(catSize).arg(ViewBufSize).arg(scrollPosStart + ViewBufSize + 1).arg( model->rowCount()).arg(numberNewRecord));
    catSize = numberNewRecord;

	   if( scrollPosStart + ViewBufSize + 1 < catSize) {
	   in->seekg((scrollPosStart + ViewBufSize + 1)*sizeof(RECORD),ios::beg);
       outfile << in->rdbuf();
	}
       in->close();
       outfile.close();
	   
	if (catSize < setDoc->GetWorkSpaceSize()->GetValue().toInt())
		ViewBufSize = catSize;	 
	
       QDir dp(MainWindow::self->tempDirectory);
       dp.remove(MainWindow::self->GetOpenBinaryFileName());
       dp.rename ( filename,MainWindow::self->GetOpenBinaryFileName()); 
       QApplication::restoreOverrideCursor();
       QString name =  MainWindow::self->GetOpenBinaryFileName();
       delete in;
       in = new fstream (name.toAscii().data(),ios::binary | ios::in | ios::out);
       if (!(*in)) {QApplication::restoreOverrideCursor();error(QString("cannot open input file"),name);return false;}
       MainWindow::self->viewBufferModified = false;
       MainWindow::self->modifiedCatalog = true;
       MainWindow::self->modifiedProtocol = true;
       RestoreWorkSpaceAfterChangeIndex();
       
       QApplication::restoreOverrideCursor();
       return true;
       
}


void CatTable::SearchNumber(int number){
 if (catSize == ViewBufSize) {
	  scrollPosStart = 0;
    selectRow (number - 1);
    return;
  }
 
   int start = model->GetHeaderRowRecordNumber(0);
   int count = model->rowCount() -1;
   int finish =  start + (int)(((float)count/2));

//---------------------------------------------------------------------------
 //qWarning("start %d finish %d count %d number %d",start,finish,count,number);
//---------------------------------------------------------------------------
   if (!(number < start  || number > finish -1 )){

//      clearSelection ();
      bool findRecord = false;
	    int  findRecordRow = 0;
	    for ( int i = 0; i < finish; ++i) {
        if( (findRecordRow = model->GetHeaderRowRecordNumber(i) ) !=-1)
	      if ( number == findRecordRow  ) { findRecord = true; findRecordRow = i; break;}
	   }
	   if ( findRecord) selectRow(findRecordRow);//showRow (findRecordRow); 
	   else {
	    QMessageBox::information(this,mainWindowCaption(),
				     QString("Has not found record  (# %1)  ").arg(number));

	   }
     return;
   }



/*
  int h =  rowHeight (0);
  int  height =   visibleRegion().boundingRect ().height();
  int shift =  (int)( (float)height / (float)h/2. +0.5);

*/

if (maybeSave() ) { 

    int halfBuffer = (int)(((float)ViewBufSize/2)+0.5);
   
	  if ( number  <  halfBuffer  ) scrollPosStart =  0;
    else {
     scrollPosStart =  number - halfBuffer;
     if ( scrollPosStart + ViewBufSize  >= catSize-1) scrollPosStart = catSize - ViewBufSize;
    }

    InitBufferTable();
    selectRow (number  -1 - scrollPosStart);
   

}    
}
//--------------------------------------------------------------------------------------
void CatTable::RestoreWorkSpaceAfterChangeIndex(){
QApplication::setOverrideCursor(Qt::WaitCursor);
    CheckErrorCatalog();
    if (ViewBufSize >= catSize) scrollPosStart = 0;
    else if ( scrollPosStart + ViewBufSize  >= catSize) scrollPosStart = catSize - ViewBufSize;
      InitBufferTable();
      selectRow (0);
      QApplication::restoreOverrideCursor();
}  
//-----------------------------------------------------------------------------------------
void CatTable::EscTableLineEditor(){
  QPointer<TableLineEditor> lin = delegate->GetLineEdit();
  if (lin != 0) {
  QString next = lin->GetEditedValue();
  lin->setText(next);
  CloseOpenedTableLineEditor();
 }
}
void CatTable::CloseOpenedTableLineEditor(){
  QPointer<TableLineEditor> lin = delegate->GetLineEdit();
  if (lin != 0) {
  IProperty* prop = lin->GetProperty();
  QModelIndex index = prop->index;
  QString prev = prop->GetInitStr();
  QString next = lin->text();
  if (next.isEmpty ()) next = lin->GetEditedValue();
  lin->close();
  if (prev != next) {
  model->setData(index, next);
  MainWindow::self->viewBufferModified = true;
 }
  int row = index.row()+1; int col = index.column();
  if ( row > model->rowCount()-1) row = 0; 
  
  selectRow(row);clearSelection();
         QItemSelection sel(model->index(row,col, QModelIndex()),model->index(row,col,QModelIndex())); 
         selectModel->select (sel,QItemSelectionModel::Select);

 }

}
void CatTable::initFindDialog(bool indNew){
	if (!indNew && findDialog->isHidden ()) return;
 	QString infoCatalog = "";
    RECORD record;
    Catalog cat;
    in->clear();
    in->seekg(sizeof(RECORD),ios::beg);
    in->read((char*)&record,sizeof(RECORD ));
    QString timeRecordFirst = cat.getQStringISODate(record);
    in->seekg(-((int)sizeof(RECORD)),ios::end);
    in->read((char*)&record,sizeof(RECORD ));
    QString timeRecordLast = cat.getQStringISODate(record);
	findDialog->SetDatesNumbersToFinddialog(timeRecordFirst,
	            timeRecordLast,
	            catSize);
}

void CatTable::viewfind() {
if (!parentWindow->isCatalogLoaded()) return;
    if(findDialog != 0) {
		if (findDialog->isHidden () ) {
		delete findDialog; findDialog = 0;
		}
		else { findDialog->raise (); return;}
	}
    findDialog = new FindRecord(this);
	initFindDialog(true);
	findDialog->show();
	

}
void CatTable::buttonSaveArea(){
if (!parentWindow->isCatalogLoaded()) return;
CloseOpenedTableLineEditor();
if (MainWindow::self->viewBufferModified == false){
	    QMessageBox::information(this,mainWindowCaption(),
				     QString("The catalog view segment has not been modified."));
	    return;

	}
  if (maybeSave()) {
  
 //     RestoreWorkSpaceAfterChangeIndex();

  }
}   

void CatTable::buttonCancelArea(){
if (!parentWindow->isCatalogLoaded()) return;
CloseOpenedTableLineEditor();
   if (MainWindow::self->viewBufferModified == false){
	    QMessageBox::information(this,mainWindowCaption(),
				     QString("The catalog view segment has not been modified."));
	    return;

  } else {
        int ret = QMessageBox::warning(this, mainWindowCaption(),
                     "Do you want remove all catalog view segment changes?",
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No | QMessageBox::Escape);
        if (ret == QMessageBox::No)
            return;
       }
  RestoreIndex();

}

void CatTable::MarkParameterErrorCatalog(int row,int col,int count){   
	if (row < scrollPosStart  || row > scrollPosStart + model->rowCount() -1) {
     if (maybeSave() ) SearchNumber(row);
  }

 	bool findRecord = false;
	int  findRow = 0;
	for ( int i = 0; i < model->rowCount(); ++i) {
  if( (findRow = model->GetHeaderRowRecordNumber(i) ) !=-1)
	    if ( row == findRow  ) { findRecord  = true; findRow = i; break;}
	}
	if ( findRecord)  {
        selectRow (findRow);
        if (col != -1) {
         QModelIndex ind = QModelIndex();
         for (int i = 0; i < count; i++) {
         QModelIndex ind =  model->index(findRow,col+i, QModelIndex());
         IProperty * p = model->findProperty(ind) ;
         if(!p->GetChanged()) p->SetInvalid(TRUE);
         }
         QItemSelection sel1(model->index(findRow,0, QModelIndex()),model->index(findRow,NumCols-1, QModelIndex()));
         selectModel->select (sel1,QItemSelectionModel::Deselect);
         QItemSelection sel(model->index(findRow,col, QModelIndex()),model->index(findRow,col+count-1, QModelIndex())); 
         selectModel->select (sel,QItemSelectionModel::Select);

    }     
   resizeRowToContents(findRow);

	}
	else {
	    QMessageBox::information(this,mainWindowCaption(),
				     QString("Has not found the error record  (# %1)  ").arg(row));
	    return;

	}
 
}
//---------------------------------------------------------------------------
void CatTable::add(){
if (CheckErrorSortRange()) return;
     if(appendDialog != 0) {delete appendDialog; appendDialog = 0;}
     appendDialog = new AppendImpl(this,doc,AppendImpl::Tool);
     appendDialog->show();
     appendDialog->resize(MainWindow::self->addSize);
     appendDialog->move(MainWindow::self->addPos);
	 
}
//---------------------------------------------------------------------------
void CatTable::range(){
if (CheckErrorSortRange()) return;
    if (MainWindow::self->viewBufferModified) save();
     if(rangeDialog != 0) {delete rangeDialog; rangeDialog = 0;}
     rangeDialog = new RangeLimitImpl(this,doc,RangeLimitImpl::Tool);
     rangeDialog->show();
     rangeDialog->resize(MainWindow::self->rangeSize);
     rangeDialog->move(MainWindow::self->rangePos);
}
//---------------------------------------------------------------------------
bool CatTable::MergeGetSecondCatalogIndex(){
  QApplication::setOverrideCursor(Qt::WaitCursor);
  QString    name = MainWindow::self->GetOpenBinaryFileName();
  name.replace(openBinaryCatalogfileName(),openSecondBinaryCatalogfileName() );
  RECORD record;
  if (inSecond != 0) {    inSecond->close();  delete inSecond;}
  inSecond = new fstream (name.toAscii(),ios::binary | ios::in | ios::out);
  if (!(*inSecond)) {QApplication::restoreOverrideCursor();error(QString("cannot open input file"),name);return false;}
    inSecond->seekg(0,ios::beg);
    inSecond->read((char*)&catSizeSecond,sizeof(int));
  inSecond->seekg(sizeof(RECORD),ios::beg);
  double datetime = 0;
  if (!indexSecond.empty()) indexSecond.clear();
// IndexSecond has value = -(numCatalogRecord  + 1) !!!
  int numRec = 1;
    while(inSecond->read((char*)&record,sizeof(RECORD ))){
	    if(inSecond->fail()){QApplication::restoreOverrideCursor();error(QString("An reading error has occurred"),name);return false;}
	    datetime =  DateTime(record);
	    indexSecond.insert(make_pair(datetime,-numRec));
      numRec++;
	 }
 QApplication::restoreOverrideCursor();
return true;
//----------------------------------------------------------------------------
}
void CatTable::merge(){   //   opendialog == 0  catalDialog does not exist or closes
if (CheckErrorSortRange()) return;
     if(catalDialog != 0) {delete catalDialog; catalDialog = 0;}
     catalDialog = new CatalBaseImpl(this,doc,CatalBaseImpl::MergeCatalog,CatalBaseImpl::Tool);
     catalDialog->show();
     catalDialog->resize(MainWindow::self->catalImplSize);
     catalDialog->move(MainWindow::self->catalImplPos);
     
}
//---------------------------------------------------------------------------
void CatTable::comp(){
if (CheckErrorSortRange()) return;
     if(catalDialog != 0) {delete catalDialog; catalDialog = 0;}
     catalDialog = new CatalBaseImpl(this,doc,CatalBaseImpl::CompareCatalog,CatalBaseImpl::Tool);
     catalDialog->show();
     catalDialog->resize(MainWindow::self->catalImplSize);
     catalDialog->move(MainWindow::self->catalImplPos);

	
}
void CatTable::sample(){
     if (CheckErrorSortRange()) return;
     if(subCatalogDialog != 0) {delete subCatalogDialog; subCatalogDialog = 0;}
     subCatalogDialog = new SubCatalogImpl(this,doc,SubCatalogImpl::Tool);
	 subCatalogDialog->show();
     subCatalogDialog->resize(MainWindow::self->subCatalogSize);
     subCatalogDialog->move(MainWindow::self->subCatalogPos);

}
void CatTable::magnitude(){
if (CheckErrorSortRange()) return;

    TransfMagnImpl  dialog(this,doc,TransfMagnImpl::Tool);
     dialog.resize(MainWindow::self->trmagImplSize);
     dialog.move(MainWindow::self->trmagImplPos);
    if (dialog.exec()) {
    }
    MainWindow::self->trmagImplPos   =  dialog.pos();
    MainWindow::self->trmagImplSize  =  dialog.size();
 }

bool CatTable::CheckErrorSortRange(){
  maybeSave();
/*
  if (errorRange) {
  error(QString("Catalog contains invalid time range records"),"");
  return true;
  } else
*/

  if (errorSortRange) {
  MainWindow::self->bottomWindow->OpenProblemsProtocolPage();
  error(QString("Catalog is not sorted"),"");
  return true;
  } else  return false;
 
}
void CatTable::CheckDuplicate(){
     if (CheckErrorSortRange()) return;
     if(doubleDialog != 0) {delete doubleDialog; doubleDialog = 0;}
     doubleDialog = new DoubleBaseImpl(this,doc,DoubleBaseImpl::Tool);
     doubleDialog->show();
     doubleDialog->resize(MainWindow::self->doubleImplSize);
     doubleDialog->move(MainWindow::self->doubleImplPos);

}

void CatTable::ReplaceDuplicateMgnitude(){
/*
     in->clear(); inSecond->clear();
     RECORD record;
     DoubleShortMultiMap* indexMagnitude;
     indexMagnitude = &catalDialog->indexMagnitude;
     int selMagnitudeIndex = catalDialog->GetComboBoxSel()->currentIndex ();
     if (!indexMagnitude->empty())  {
        multimap<double,short>::iterator it;
        for (it = indexMagnitude->begin(); it!=indexMagnitude->end();it++)  {
        // pair<ItrMultiMap, ItrMultiMap> p =  index.equal_range(it->first) ;
         
          ItrMultiMap pos;
         // if((p.first)->second >= 0) pos = p.first;
         // else  pos = p.second;
          pos = index.find(it->first);
// qWarning("xx p1 %d p2 %d (%d) pos %d",(p.first)->second,(p.second)->second,it->second,pos->second);
          int inpos = pos->second +1;
          in->seekg(inpos*sizeof(RECORD),ios::beg);
          in->read((char*)&record,sizeof(RECORD ));
          record.m[selMagnitudeIndex] = it->second;
          in->seekg(inpos*sizeof(RECORD),ios::beg);
          in->write((char*)&record,sizeof(RECORD));
        }
         // print   indexMagnitude

          // print   indexMagnitude
     }
*/
}
void CatTable::MergeTwoCatalogs(CatalBaseImpl::TransformMag action) {

    if (action == CatalBaseImpl::Replace) ReplaceDuplicateMgnitude();
    WriteCatalogFromTwoCatatalgIndex(CatalBaseImpl::Merge);
}
                        
void CatTable::WriteCatalogFromTwoCatatalgIndex(CatalBaseImpl::ModeAction action) {
    in->clear();  inSecond->clear();
    RECORD record;
    QString filename  = MainWindow::self->GetOpenBinaryFileName();
    filename.replace( openBinaryCatalogfileName(), openTempBinaryCatalogfileName() );
    QApplication::setOverrideCursor(Qt::WaitCursor);
    ofstream outfile(filename.toAscii(), ios::binary | ios::trunc);
    if (!(outfile) ){ QApplication::restoreOverrideCursor();error(QString("cannot open temporary output file"),filename);return;}
    int numberNewRecord = index.size();
    outfile.seekp(0,ios::beg);
    outfile.write((char*)&numberNewRecord,sizeof(int));
    if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing error has occurred (catalog size)"),"");return;}

    in->seekg(sizeof(int),ios::beg);
    quint32 magic;
    in->read((char*)&magic,sizeof(quint32));
    if(in->fail()){QApplication::restoreOverrideCursor();error(QString("An reading error has occurred (rec %1)").arg(1),"");return;}
    outfile.write((char*)&magic,sizeof(quint32));
    if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing error has occurred (format indicator)"),"");return;}
    outfile.seekp(sizeof(RECORD),ios::beg);
    in->seekg(sizeof(RECORD),ios::beg);


    int numRec = 0;
    if (!errorvec.empty()) errorvec.clear();
    in->clear();
    inSecond->clear();
    ItrMultiMap pos;
    for (pos = index.begin();pos !=index.end();pos++) {
  if (pos->second >=0 ) {
	int inpos = pos->second +1;
	in->seekg(inpos*sizeof(RECORD),ios::beg);
	in->read((char*)&record,sizeof(RECORD ));
	if(in->eof()){QApplication::restoreOverrideCursor();error(QString("An expected eof of first catalog has occurred"),"");return;  }
	if(in->fail()){QApplication::restoreOverrideCursor();error(QString("An reading error of first has occurred (rec %1)").arg(inpos),"");return;}
if (action == CatalBaseImpl::Equivalence || action == CatalBaseImpl::Nonequivalence) record.inten = 1;
	outfile.write((char*)&record,sizeof(RECORD));
	   if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing  buffer error has occurred (rec %1)").arg(numRec+1),"");return;}
 }  else {
	int inpos = -pos->second;
	inSecond->seekg(inpos*sizeof(RECORD),ios::beg);
	inSecond->read((char*)&record,sizeof(RECORD ));
	if(inSecond->eof()){QApplication::restoreOverrideCursor();error(QString("An expected eof of second catalog has occurred"),"");return;  }
	if(inSecond->fail()){QApplication::restoreOverrideCursor();error(QString("An reading error of second has occurred (rec %1)").arg(inpos),"");return;}
if (action == CatalBaseImpl::Equivalence || action == CatalBaseImpl::Nonequivalence) record.inten = 2;
	outfile.write((char*)&record,sizeof(RECORD));
	   if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing error has occurred (rec %1)").arg(numRec+1),"");return;}

 }
   numRec++;
  }

       in->close();
       outfile.close();

       QDir dp(MainWindow::self->tempDirectory);
       dp.remove(MainWindow::self->GetOpenBinaryFileName());
       dp.rename ( filename,MainWindow::self->GetOpenBinaryFileName());
       QString name =  MainWindow::self->GetOpenBinaryFileName();
       delete in;
       in = new fstream (name.toAscii().data(),ios::binary | ios::in | ios::out);
       if (!(*in)) {QApplication::restoreOverrideCursor();error(QString("cannot open input file"),name);return;}

       ViewBufSize = setDoc->GetWorkSpaceSize()->GetValue().toInt();

       RestoreWorkSpaceAfterChangeIndex();
       MainWindow::self->viewBufferModified = false;
       QApplication::restoreOverrideCursor();

}
void CatTable::WriteCatFromIndex(){

    RECORD record;
    QString filename  = MainWindow::self->GetOpenBinaryFileName();
    filename.replace( openBinaryCatalogfileName(), openTempBinaryCatalogfileName() );
    QApplication::setOverrideCursor(Qt::WaitCursor);
    ofstream outfile(filename.toAscii(), ios::binary | ios::trunc);
    if (!(outfile) ){ QApplication::restoreOverrideCursor();error(QString("cannot open temporary output file"),filename);return;}
    int numberNewRecord = index.size();
    outfile.seekp(0,ios::beg);
    outfile.write((char*)&numberNewRecord,sizeof(int));
    if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing error has occurred (catalog size)"),"");return;}

    in->seekg(sizeof(int),ios::beg);
    quint32 magic;
    in->read((char*)&magic,sizeof(quint32));
    if(in->fail()){QApplication::restoreOverrideCursor();error(QString("An reading error has occurred (rec %1)").arg(1),"");return;}
    outfile.write((char*)&magic,sizeof(quint32));
    if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing error has occurred (format indicator)"),"");return;}
    outfile.seekp(sizeof(RECORD),ios::beg);
    in->seekg(sizeof(RECORD),ios::beg);


    int numRec = 0;
    if (!errorvec.empty()) errorvec.clear();
    in->clear();
    ItrMultiMap pos;
    for (pos = index.begin();pos !=index.end();pos++) {
	int inpos = pos->second +1;
	in->seekg(inpos*sizeof(RECORD),ios::beg);
	in->read((char*)&record,sizeof(RECORD ));
	if(in->eof()){QApplication::restoreOverrideCursor();error(QString("An expected eof catalog has occurred"),"");return;  }
	if(in->fail()){QApplication::restoreOverrideCursor();error(QString("An reading error has occurred (rec %1)").arg(inpos),"");return;}
	outfile.write((char*)&record,sizeof(RECORD));
	   if(outfile.fail()){QApplication::restoreOverrideCursor();error(QString("An writing buffer error has occurred (rec %1)").arg(numRec+1),"");return;}
	   numRec++;
    }

       in->close();
       outfile.close();
       QDir dp(MainWindow::self->tempDirectory);
       dp.remove(openBinaryCatalogfileName());
       dp.rename (openTempBinaryCatalogfileName(),openBinaryCatalogfileName());
       RestoreWorkSpaceAfterChangeIndex();
       QApplication::restoreOverrideCursor();
}
void CatTable::RestoreIndex(){
    ViewBufSize = setDoc->GetWorkSpaceSize()->GetValue().toInt();
    RestoreWorkSpaceAfterChangeIndex();
}
bool CatTable::writeFile(const QString &filename){
    in->clear();
    in->seekg(0,ios::beg);
    ofstream outfile(filename.toAscii(), ios::binary | ios::trunc);
    if (!(outfile) ){ QApplication::restoreOverrideCursor();error(QString("cannot open output file"),filename);return false;}
    outfile << in->rdbuf();
    outfile.close();

    return true;
}
void CatTable::CheckSortCatalog(){
  if (!errorvec.empty()) errorvec.clear();
  int numErr = 0;
//  int numCorr = 0;
//  int numCheck = 0;
  int numRec = 1;
  bool stopFind = false;
  ItrMultiMap pos;
  ItrMultiMap pos1;
  
  ItrMultiMap posStart = index.begin(); posStart++;
   for (pos = posStart;pos !=index.end();pos++)
    {
     pos1 = pos;pos1--;
    // qWarning("numRec %d    pos %d ",numRec+1,pos1->second+1);
     int dif = abs(pos->second -  pos1->second);
     if ( dif > 1) {
     QString tmp =   QString(errorSortPattern()).arg(numRec+1);
     tmp = tmp  + QString("@%1@ after: %2").arg(pos->second+1).arg(pos1->second+1);
     if ( numErr > setDoc->GetMaxErrorLines()->GetValue().toInt() ) {stopFind = true; break;}
     errorvec.push_back(tmp);
     numErr++;        
      
     if (stopFind) break;
    // qWarning("------------>numErr %d  order place %d after:%d ",pos->second+1,numRec+1,pos1->second+1);
    // qWarning("------------> %s ",tmp.toAscii().data());
    }
  numRec++;
  }
  sort(errorvec.begin(),errorvec.end(),recSortNumberComp());
  if (stopFind)  MainWindow::self->bottomWindow->comtable->InitCommentTable(&errorvec,ComTable::SortFirst);
  else MainWindow::self->bottomWindow->comtable->InitCommentTable(&errorvec,ComTable::SortAll);
  if (!errorvec.empty()) {
   errorSortRange = true; 
   MainWindow::self->bottomWindow->OpenProblemsProtocolPage();
  } else errorSortRange = false;
}
void CatTable::CheckSortButton() {
if (maybeSave()) CheckSortCatalog();
}
void CatTable::SortCatalog()  {
 if (maybeSave()) {
 if(errorSortRange) {
 MainWindow::self->bottomWindow->OpenProblemsProtocolPage();
 WriteCatFromIndex();
 MainWindow::self->modifiedCatalog = true;
 }
 }
}


///----------------------------------------------------------------------------------------------
void CatTable::SetProtocolInfo(QStringList &rec,QStringList &initbl,QStringList &tbl){
actionName = "*** Catalog Correction Records ***";
/*
for (int i = 0; i < tbl.size(); ++i) {
qWarning("'%s' '%s'",tbl.at(i).toAscii().data(),initbl.at(i).toAscii().data());
}

for (int i = 0; i < rec.size(); ++i) {

qWarning("'%s'",rec.at(i).toAscii().data());
}
*/
if (firstCorrection == true) {
numberCorrectionLine = 0;
      texted = MainWindow::self->bottomWindow->messageTab;
      correctioned = MainWindow::self->bottomWindow->correctionTab;
      MainWindow::self->bottomWindow->OpenMessagesProtocolPage();
     // texted->clear();
      texted->insertPlainText(prefixMessage() + "\n\n" + actionName + ":\n");
      correctioned->clear();
      correctioned->insertPlainText(prefixMessage() + "\n\n" + actionName + ":\n");
}
firstCorrection = false;
maxNumberCorrectionLine = MainWindow::self->GetSetDocument()->GetMaxCorrectionLines()->GetValue().toInt();
corrmess = "";
message  = "";

message += "\nCorrected block:\n";
message += rec.at(0) + "\n";
message += rec.at(rec.size() - 1) + "\n";

corrmess += "\nCorrected block:\n";
corrmess += rec.at(0) + "\n";
corrmess += rec.at(rec.size() - 1) + "\n";

//------ Removed ----------------------------
QStringList remList = QStringList();
for (int i = 0; i < rec.size(); ++i) {
    bool find = false;
    for (int j = 0; j < initbl.size(); ++j) 
      if(rec.at(i) == initbl.at(j)) {find = true; break;}
    if(!find) remList << rec.at(i);
}

if (!remList.isEmpty()) {
   corrmess += "Removed Recods:\n";
   for (int i = 0; i < remList.size(); ++i) {
     QString tmp = QString("%1: %2\n").arg(i+1).arg(remList.at(i));
     corrmess += tmp;
     bool find = false; int k = 0;
     for (int j = 0; j < rec.size(); ++j) 
         if(rec.at(j) == remList.at(i)) {find = true; k = j; break;}
     if(find) rec.removeAt (k);
   }

}

//--------------------------------------------

//----------- Added ---------------------
QStringList addList = QStringList();
for (int i = 0; i < initbl.size(); ++i) {
    bool find = false;
    for (int j = 0; j < rec.size(); ++j) 
      if(rec.at(j) == initbl.at(i)) {find = true; break;}

    if(!find) addList << tbl.at(i);
}


if (!addList.isEmpty()) {
   corrmess += "Added Recods:\n";
   for (int i = 0; i < addList.size(); ++i) {
     QString tmp = QString("%1: %2\n").arg(i+1).arg(addList.at(i));
     corrmess += tmp;
     bool find = false; int k = 0;
     for (int j = 0; j < initbl.size(); ++j) 
         if(tbl.at(j) == addList.at(i)) {find = true;k = j; break;}
     if(find) {tbl.removeAt (k);initbl.removeAt (k);}
   }

}
//--------------------------------------------
//----------- Correction fields ---------------------
QStringList corrList = QStringList();
for (int i = 0; i < tbl.size(); ++i) {
    bool find = false;
    for (int j = 0; j < rec.size(); ++j) 
      if(rec.at(j) == tbl.at(i)) {find = true; break;}

    if(!find) {corrList << initbl.at(i)+"@"+tbl.at(i);}
}
if (!corrList.isEmpty()) {
   corrmess += "\nCorrected Records:\n";
   QStringList tmp = QStringList();QStringList tmp1 = QStringList();
   QStringList recList = QStringList(); 
   for (int i = 0; i < corrList.size(); ++i) {
   QString corFields = QString();
        recList = corrList.at(i).split ('@');
        tmp = recList.at(0).split (' '); tmp1 = recList.at(1).split (' ');
        QString cor = GetStringToCorrectionProtocol(i+1,tmp,tmp1,corFields);
        corrmess += "\n" + corFields + "\n";
        corrmess += cor + "\n";
        corrmess += GetStringToCorrectionProtocol(i+1,tmp1,tmp,corFields);
        corrmess += "\n";
        

  }
}
//--------------------------------------------
texted->insertPlainText(message);
texted->ensureCursorVisible ();

numberCorrectionLine += corrmess.count('\n');
if (numberCorrectionLine > maxNumberCorrectionLine) {
        correctioned->insertPlainText(QString("Stop info: number of correction lines more than %1 ... \n").arg(maxNumberCorrectionLine));
         }
if (numberCorrectionLine <= maxNumberCorrectionLine) {
correctioned->insertPlainText(corrmess);
correctioned->ensureCursorVisible ();
}

}

QString CatTable::GetStringToCorrectionProtocol(int num,QStringList &listToProtocol,QStringList &listToCompare,
                                               QString & corFields){
corFields=QString();
QStringList &tmp  = listToProtocol;
QStringList &tmp1 = listToCompare;
    QString result;
    QTextStream txt(&result);
    txt.setFieldWidth(8);  txt << num;
    txt.setFieldWidth(2);  txt << ": ";
    corFields += result + " fields:  ";
//year
         if(tmp .at(0) == tmp1 .at(0)) { 
         txt.setFieldWidth(4);  txt << tmp.at(0);
         } else {
         txt.setFieldWidth(2);  txt << "< ";
         txt.setFieldWidth(4);  txt << tmp.at(0);
         txt.setFieldWidth(2);  txt << " >";
         corFields += "year, ";
         };
         txt.setFieldWidth(1); txt << ' ';
//month
         if(tmp .at(1) == tmp1 .at(1)) { 
         txt.setFieldWidth(2);  txt << tmp.at(1);
         } else {
         txt.setFieldWidth(2);  txt << "< ";
         txt.setFieldWidth(2);  txt << tmp.at(1);
         txt.setFieldWidth(2);  txt << " >";
         corFields += "month, ";
         };
         txt.setFieldWidth(1); txt << ' ';
//day
        if(tmp .at(2) == tmp1 .at(2)) { 
         txt.setFieldWidth(2);  txt << tmp.at(2);
         } else {
         txt.setFieldWidth(2);  txt << "< ";
         txt.setFieldWidth(2);  txt << tmp.at(2);
         txt.setFieldWidth(2);  txt << " >";
         corFields += "day, ";
         };
         txt.setFieldWidth(1); txt << "  ";
//hour
        if(tmp .at(3) == tmp1 .at(3)) { 
         txt.setFieldWidth(2);  txt << tmp.at(3);
         } else {
         txt.setFieldWidth(2);  txt << "< ";
         txt.setFieldWidth(2);  txt << tmp.at(3);
         txt.setFieldWidth(2);  txt << " >";
         corFields += "hour, ";
         };
         txt.setFieldWidth(1); txt << ' ';

//mn
        if(tmp .at(4) == tmp1 .at(4)) { 
         txt.setFieldWidth(2);  txt << tmp.at(4);
         } else {
         txt.setFieldWidth(2);  txt << "< ";
         txt.setFieldWidth(2);  txt << tmp.at(4);
         txt.setFieldWidth(2);  txt << " >";
         corFields += "minute, ";
         };
         txt.setFieldWidth(1); txt << ' ';

//sec
        if(tmp .at(5) == tmp1 .at(5)) { 
         txt.setFieldWidth(5);  txt << tmp.at(5);
         } else {
         txt.setFieldWidth(2);  txt << "< ";
         txt.setFieldWidth(5);  txt << tmp.at(5);
         txt.setFieldWidth(2);  txt << " >";
         corFields += "second, ";
         };
         txt.setFieldWidth(1); txt << "  ";

//lat
        if(tmp .at(6) == tmp1 .at(6)) { 
         txt.setFieldWidth(7);  txt << tmp.at(6);
         } else {
         txt.setFieldWidth(2);  txt << "< ";
         txt.setFieldWidth(7);  txt << tmp.at(6);
         txt.setFieldWidth(2);  txt << " >";
         corFields += "latitude, ";
         };
         txt.setFieldWidth(1); txt << "  ";

//lon
        if(tmp .at(7) == tmp1 .at(7)) { 
         txt.setFieldWidth(8);  txt << tmp.at(7);
         } else {
         txt.setFieldWidth(2);  txt << "< ";
         txt.setFieldWidth(8);  txt << tmp.at(7);
         txt.setFieldWidth(2);  txt << " >";
         corFields += "longitude, ";
         };
         txt.setFieldWidth(1); txt << "  ";

//dep
       if(tmp .at(8) == tmp1 .at(8)) { 
         txt.setFieldWidth(5);  txt << tmp.at(8);
         } else {
         txt.setFieldWidth(2);  txt << "< ";
         txt.setFieldWidth(5);  txt << tmp.at(8);
         txt.setFieldWidth(2);  txt << " >";
         corFields += "depth, ";
         };
         txt.setFieldWidth(1); txt << "  ";

//magn
for(int i = 0; i < 4; ++i) {
       if(tmp .at(9+i) == tmp1 .at(9+i)) { 
         txt.setFieldWidth(4);  txt << tmp.at(9+i);
         } else {
         txt.setFieldWidth(2);  txt << "< ";
         txt.setFieldWidth(4);  txt << tmp.at(9+i);
         txt.setFieldWidth(2);  txt << " >";
if (i == 0) corFields += "M1, ";
if (i == 1) corFields += "M2, ";
if (i == 2) corFields += "M3, ";
if (i == 3) corFields += "M4, ";
         };
         txt.setFieldWidth(1); txt << " ";

}
      txt.setFieldWidth(1); txt << " ";
      if(tmp .at(13) == tmp1 .at(13)) { 
         txt.setFieldWidth(2);  txt << tmp.at(13);
         } else {
         txt.setFieldWidth(2);  txt << "< ";
         txt.setFieldWidth(2);  txt << tmp.at(13);
         txt.setFieldWidth(2);  txt << " >";
         corFields += "intensity, ";
         };
     if(!corFields.isNull()) corFields.remove( corFields.length()-2, 2);
return  result;
}
//===============================================================================
//----------------------------------------------------------------------
void CatTable::TimeOutShot(){
progress->repaint();
}
//----------------------------------------------------------------------
void CatTable::buildProfile() {
Catalog cat;
if ( !cat.CopyFile(MainWindow::self->GetProjectBinaryCatalogFileName(),
MainWindow::self->GetOpenBinaryFileName()) ) return;
    QTextEdit* texted = MainWindow::self->bottomWindow->messageTab;
    MainWindow::self->bottomWindow->OpenMessagesProtocolPage();
    texted->clear();
    texted->insertPlainText("Build Project Messages:\n\n");

    QTextEdit* correctioned = MainWindow::self->bottomWindow->correctionTab;
    correctioned->clear();
    correctioned->insertPlainText("Build Project Messages:\n\n");

    RestoreIndex();

ProfileList*         prolist = MainWindow::self->prolist;
QString proFile =  MainWindow::self->GetCurrentProjectName();
if (proFile.isEmpty()) proFile = "unnamed";

QString proCatalog =  doc->GetProjectCatalogName()->GetValue();
texted->insertPlainText("Project File: " + proFile + "\n\n");
texted->insertPlainText(cat.GetCatalogInfoMessage(proCatalog));

correctioned->insertPlainText("Project File: " + proFile + "\n\n");
correctioned->insertPlainText(cat.GetCatalogInfoMessage(proCatalog));

QStringList list = doc->GetProjectActionList()->GetStringList();
QStringList listAction = projectActionList().split ('@');
QList<QWidget *> history;
TransfMagnImpl* trMag = 0;

for (int i = 0; i <  list.size(); i++){
 QString str = list.at(i);

if (str == listAction.at(0)) {
    trMag = new TransfMagnImpl(this,doc,TransfMagnImpl::Execute);
    history.append(trMag); 
}
if (str == listAction.at(1)) {
    if(doubleDialog != 0) {delete doubleDialog; doubleDialog = 0;}
    doubleDialog = new DoubleBaseImpl(this,doc,DoubleBaseImpl::Execute);
    history.append(doubleDialog); 

}

if (str == listAction.at(2))  {
    if(subCatalogDialog != 0) {delete subCatalogDialog; subCatalogDialog = 0;}
    subCatalogDialog = new SubCatalogImpl(this,doc,SubCatalogImpl::Execute);
    history.append(subCatalogDialog); 
}

if (str == listAction.at(3)) {
   if(catalDialog != 0) {delete catalDialog; catalDialog = 0;}
   catalDialog = new CatalBaseImpl(this,doc,CatalBaseImpl::MergeCatalog,CatalBaseImpl::Execute);
   history.append(catalDialog); 
}
if (str == listAction.at(4)) {
   if(appendDialog != 0) {delete appendDialog; appendDialog = 0;}
   appendDialog = new AppendImpl(this,doc,AppendImpl::Execute);
   history.append(appendDialog);
}
if (str == projectCompareTitle()) {
   if(catalDialog != 0) {delete catalDialog; catalDialog = 0;}
   catalDialog = new CatalBaseImpl(this,doc,CatalBaseImpl::CompareCatalog,CatalBaseImpl::Execute);
   history.append(catalDialog); 
}
if (str == projectRangeTitle()) {
   if(rangeDialog != 0) {delete rangeDialog; rangeDialog = 0;}
   rangeDialog = new RangeLimitImpl(this,doc,RangeLimitImpl::Execute);
   history.append(rangeDialog);
}

}
//----------------------------------------------------------------

	int num = history.size();
	QString progressTitle = "Project execution in progress ...\n";
	
if (progress != 0 ) {delete progress; progress = 0;}
progress = new QProgressDialog("Project execution in progress", "Cancel", 0,num,this);
	progress->setRange (0,num);
progress->setLabelText (progressTitle);
	progress->show();
	int historyCount = 0;
//----------------------------------------------------------------


QString prefix = prefixMessage();
bool exeCanceled = false;
foreach (QWidget * wd, history)  {
//QApplication::setOverrideCursor(Qt::WaitCursor);
  qApp->processEvents();

            if (progress->wasCanceled()) {
                exeCanceled = true;
                break;
        }

   QDialog * dlg = (QDialog *) wd;
      if (dynamic_cast<QDialog *>(dlg) == dynamic_cast<TransfMagnImpl *>(dlg)) {
         TransfMagnImpl * p =  (TransfMagnImpl *)dlg;
         prolist->GetListWidget()->GetModel()->SetActiveAction(listAction.at(0));
         texted->insertPlainText(prefix + listAction.at(0) + " ***\n\n");
texted->ensureCursorVisible ();
progress->setLabelText (progressTitle + listAction.at(0));
progress->setValue (historyCount);
         if (!p->execute()) break;

     }
      if (dynamic_cast<QDialog *>(dlg) == dynamic_cast<DoubleBaseImpl *>(dlg)) {
         DoubleBaseImpl * p =  (DoubleBaseImpl *)dlg;
         prolist->GetListWidget()->GetModel()->SetActiveAction(listAction.at(1));
         texted->insertPlainText(prefix + listAction.at(1) + " ***\n\n");
texted->ensureCursorVisible ();
progress->setLabelText (progressTitle + listAction.at(1));
progress->setValue (historyCount);
         if (!p->execute()) break;

     }
      if (dynamic_cast<QDialog *>(dlg) == dynamic_cast<SubCatalogImpl *>(dlg)) {
         SubCatalogImpl * p =  (SubCatalogImpl *)dlg;
         prolist->GetListWidget()->GetModel()->SetActiveAction(listAction.at(2));
         texted->insertPlainText("\n"+prefix + listAction.at(2) + " ***\n\n");
texted->ensureCursorVisible ();
progress->setLabelText (progressTitle + listAction.at(2));
progress->setValue (historyCount);
         if (!p->execute()) break;

     }
      if (dynamic_cast<QDialog *>(dlg) == dynamic_cast<CatalBaseImpl *>(dlg)) {
         CatalBaseImpl * p =  (CatalBaseImpl *)dlg;
         if (p->GetCatAct() == CatalBaseImpl::MergeCatalog) {
           prolist->GetListWidget()->GetModel()->SetActiveAction(listAction.at(3));
           texted->insertPlainText(prefix  + listAction.at(3) + " ***\n\n");
progress->setLabelText (progressTitle + listAction.at(3));
         }
         if (p->GetCatAct() == CatalBaseImpl::CompareCatalog) {
         prolist->GetListWidget()->GetModel()->SetActiveAction(projectCompareTitle());
         texted->insertPlainText(prefix + projectCompareTitle() + " ***\n\n");
progress->setLabelText (progressTitle +  projectCompareTitle());
         }
texted->ensureCursorVisible ();
progress->setValue (historyCount);
         if (!p->execute()) break;

         
     }
      if (dynamic_cast<QDialog *>(dlg) == dynamic_cast<AppendImpl *>(dlg)) {
         AppendImpl * p =  (AppendImpl *)dlg;
         prolist->GetListWidget()->GetModel()->SetActiveAction(listAction.at(4));
         texted->insertPlainText(prefix  + listAction.at(4) + " ***\n\n");
texted->ensureCursorVisible ();
progress->setLabelText (progressTitle + listAction.at(4));
progress->setValue (historyCount);
         if (!p->execute()) break;

     }
      if (dynamic_cast<QDialog *>(dlg) == dynamic_cast<RangeLimitImpl *>(dlg)) {
         RangeLimitImpl * p =  (RangeLimitImpl *)dlg;
         prolist->GetListWidget()->GetModel()->SetActiveAction(projectRangeTitle());
         texted->insertPlainText(prefix  + projectRangeTitle() + " ***\n\n");
texted->ensureCursorVisible ();
progress->setLabelText (progressTitle + projectRangeTitle());
progress->setValue (historyCount);
         if (!p->execute()) break;

     }
QTimer::singleShot(2000, this, SLOT(TimeOutShot()));
historyCount++;
 }

if (exeCanceled) {
if(trMag != 0) {delete trMag; trMag = 0;}
prolist->GetListWidget()->GetDelegate()->GetExecuteAction() = false;
prolist->repaint();
history.clear();
if ( !cat.CopyFile(MainWindow::self->GetProjectBinaryCatalogFileName(),
MainWindow::self->GetOpenBinaryFileName()) ) return;
    RestoreIndex();

    texted->insertPlainText("Canceled by user.\n");

    correctioned->insertPlainText("Canceled by user.\n");


QApplication::restoreOverrideCursor();
return;
}
progress->setValue (historyCount);
prolist->GetListWidget()->GetDelegate()->GetExecuteAction() = true;

prolist->repaint(); texted->repaint();
texted->ensureCursorVisible ();

correctioned->ensureCursorVisible ();
prolist->GetListWidget()->GetModel()->SetActiveAction("");

if(trMag != 0) {delete trMag; trMag = 0;}
history.clear();
QApplication::restoreOverrideCursor();

MainWindow::self->setCurrentFile("");
MainWindow::self->modifiedCatalog = true;
MainWindow::self->modifiedProtocol= true;
}




