#ifndef VIEW_CATALOG_IMPL_H
#define VIEW_CATALOG_IMPL_H
#include <QTableView>
#include <QModelIndex>
#include <fstream>
#include <iostream>
#include <map>
#include <vector>
#include <algorithm>
#include <functional>
#include <QList>
#include <QDateTime>
using namespace std;
#include "ui_viewcatalogbase.h"
#include "catalog.h"
#include "doubleimpl.h"
#include "catalimpl.h"
class PropertyModel;
class PropertyDelegate;
class QItemSelectionModel;
class QProgressDialog;
class FDocument;
class SettingDocument;
class DoubleBaseImpl;
class ViewCatalogImpl;
class QLabel;
class QDateTimeEdit;
class QSpinBox;
class QToolButton;
class UndoChange;
class UndoDelete;
class CatalBaseImpl;
class SubCatalogImpl;
class AppendImpl;
class RangeLimitImpl;
class FindRecord;
class CatTable : public QTableView
{
    Q_OBJECT
    typedef multimap<double,int> DoubleIntMultiMap;
    typedef multimap<double,short> DoubleShortMultiMap;
    typedef DoubleIntMultiMap::iterator ItrMultiMap;
    typedef vector<QString> ErrorVector;
    typedef ErrorVector::iterator ItrVector;
public:
    static CatTable *tSelf;
    int curUndoIndex;
    fstream* in;
    fstream* inSecond;
    QList<UndoChange *> undoList;
    CatTable(int numCols,int numRows,QWidget* parent);
    ~CatTable();
	void SetCatalog(FDocument*doc,SettingDocument*setDoc,MainWindow::AutoCheckErrors act);	
    bool InitBufferTable();
    bool InsertRowTable();
    QString GetRowRecordTime (int row);
    int  GetRowRecordNumber (int row);
    void ResizeHorizontalHeader(QHeaderView *hd);
    void ResizeRows();
    bool CheckErrorCatalog();
    bool &GetCheckRangeLimits() { return checkRangeLimits;}
    void SetOutOfRangeParameters();
    QStringList CheckErrorRecord(RECORD record,int numrec);
    void SearchNumber(int number);
    bool maybeSave();
    bool save();
    void cut();
    void copy();
    void paste();
    void del();
    void insert();
    void undo();
    void redo();
    void zoom();
    void sample();
    void add();
    void range();
    void magnitude();
    void buildProfile();
    RECORD GetRecordFromFileds(int row );
    int GetNumCols(){return NumCols;}
    void MarkParameterErrorCatalog(int row,int col,int count);
    void SetViewBufSize(int num);
    void WriteCatFromIndex();
    void WriteCatalogFromTwoCatatalgIndex(CatalBaseImpl::ModeAction action);
    void MergeTwoCatalogs(CatalBaseImpl::TransformMag action);
    void IntersectionTwoCatalogs();
    void EquivalenceTwoCatalogs();
    void UnequivalenceTwoCatalogs();
    void DifferenceTwoCatalogs();
    void ReplaceDuplicateMgnitude();
    void RestoreIndex();
    void CheckDuplicate();
    void merge();
    void comp();
    bool writeFile(const QString &fileName);
    void CheckSortCatalog();
    void CheckSortButton();
    void SortCatalog();
    bool CheckErrorSortRange();
    void CloseOpenedTableLineEditor();
    void EscTableLineEditor();
    DoubleIntMultiMap* GetRecordCatalogIndex() {return  &index;}
    DoubleIntMultiMap* GetRecordCatalogIndexSecond() {return  &indexSecond;}
    fstream* GetFstream(){return in;}
    fstream* GetFstreamSecond(){return inSecond;}
    bool MergeGetSecondCatalogIndex();
    int GetCatSize(){return catSize;}
    DoubleBaseImpl*  doubleDialog;
    CatalBaseImpl*   catalDialog;
    AppendImpl*      appendDialog;
    SubCatalogImpl*  subCatalogDialog;
    RangeLimitImpl*  rangeDialog;
	FindRecord*  findDialog;
    void SetErrorRange(bool b){errorRange = b;}
    QString GetCatalogInfo();
    void initFindDialog(bool indNew);
	void ClearCatalogTable();
protected:
    void contextMenuEvent(QContextMenuEvent *event);
    void SetContextMenu ( const QPoint & pos );
public slots:
   void verticalContextMenu ( const QPoint & pos );
   void clickedRowCol(QModelIndex  );
   void clickedVerticalHeader(int );
   void ButtonHome();
   void ButtonEnd();
   void ButtonDown();
   void ButtonUp();

   void buttonSaveArea();
   void buttonCancelArea();
   void TimeOutShot();
   void viewfind();
private:
                                                         // act == 0 del act ==1 just copy
    bool GetTableSelection(UndoDelete* undoSel,QItemSelection &isel,int act);
    void RemoveRowsDelLists();
    void RestoreWorkSpaceAfterChangeIndex();
    void SetEnableArrowKeys();
    void SetProtocolInfo(QStringList &cat,QStringList &initbl,QStringList &tbl);
    QString GetStringToCorrectionProtocol(int num,QStringList &listToProtocol,QStringList &listToCompare,
            QString &corFields);
    void RemoveOwnPoiners();
    FDocument* doc;
    SettingDocument* setDoc;
    bool errorRange;
    bool errorSortRange;
    int ViewBufSize;
    int NumCols;
    int scrollPosStart;  // viewed catalog part - first record
   // int scrollPosEnd;    // viewed catalog part last record
    int catSize,catSizeSecond;
    PropertyModel* model;
    PropertyDelegate *delegate;
    QItemSelectionModel* selectModel;
    ViewCatalogImpl* parentWindow;
    DoubleIntMultiMap index,indexSecond;
    ErrorVector errorvec;
    ErrorVector dupvec;
    UndoChange* selCopy;
    QTextEdit* texted;
    QString message;
    QTextEdit* correctioned;
    QString    corrmess;
    QString    actionName;
    bool firstCorrection;
    bool checkRangeLimits;
    QDateTime dateTimeFirstRecord;
    QDateTime dateTimeLastRecord;
    int numberCorrectionLine;
    int maxNumberCorrectionLine;
    QProgressDialog* progress;

};
//----------------------------------------------------------------------------
class ViewCatalogImpl : public QDialog, private Ui::ViewCatalogBase
{
    Q_OBJECT
	typedef vector<QString> ErrorVector;
	typedef ErrorVector::iterator ItrVector;   
public:
	ViewCatalogImpl(QWidget *parent,int numRows);
    ~ViewCatalogImpl();  
    CatTable* cattable;
    QToolButton*   GetToolButtonHome(){return toolButtonHome;}
    QToolButton*   GetToolButtonEnd(){return toolButtonEnd;}
    QToolButton*   GetToolButtonUp(){return toolButtonUp;}
    QToolButton*   GetToolButtonDown(){return toolButtonDown;}

	bool& isCatalogLoaded()  {return isCatLoaded;}
	QString GetCatalogInfo() {return cattable->GetCatalogInfo();}
    void zoom();
protected:
private slots:
    void escAction();
public slots:
private:
	FDocument* doc;
	SettingDocument* setDoc;
	QAction *escAct;
	bool isCatLoaded;
};


#endif
